-- this file can be deleted if it is not necessary.

local gen = require("generalLibrary")
local param = require("parameters")
local object = require("object")
local text = require("text")
local func = require("functions")
local flag = require("flag")
local counter = require("counter")

local onTurn = require("onTurn")
local tables = require("tables")

local delayedAction = require("delayedAction")

local register = {}

---&autoDoc onCityTaken
function register.onCityTaken(city,defender)
    --civ.ui.text("city taken separate file test")

	-- If two cities of Corsica are in Allied control, delete any remaining German units on island
	if (city == object.cAjaccio and defender == object.pAxis) 
	and (city == object.cBastia and defender == object.pAxis) then
		-- Delete all German units in Corsica 
		for unit in civ.iterateUnits() do
			if unit.owner == object.pAxis and gen.inPolygon(unit.location,tables.corsicaPolygon) then 
				print("Deleted " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
				civ.deleteUnit(unit)
			end	
		end	
		text.simple("Generalfeldmarschall Kesselring orders all remaining Axis forces in Corsica to flee to mainland Italy!","Axis forces Evacuate Corsica!",object.mAlbertKesselring)
	end	

	-- AJACCIO
	if city == object.cAjaccio and defender == object.pAxis then 
		gen.justOnce("Capture of Ajaccio",function()
			civ.playSound("Cheers2.wav")
			text.simple("The Allies enter the capital of Corsica, the city\n^of Ajaccio!\n^\r^With it's fall, General De Gaulle authorizes the\n^deployment of an extra regiment of Goumier\n^Marocain in the city!\n^\r^In addition, you can now transport infantry units\n^located in Ajaccio back to Palermo by pressing\n^on the 'shift' + '2' keys.","Ajaccio is captured!",object.mAlliesCaptureCity)
			gen.createUnit(object.uGoumierMarocain, object.pAllies, {{40,48,0}}, {count=1, randomize=false, veteran=false})
			counter.add("# Goumier Marocain",1)
			counter.add("# Total Free French",1)
		end)
	end	

	-- BARI
	if city == object.cBari then 
		gen.justOnce("Capture of Bari",function()
			civ.playSound("Cheers2.wav")
			text.simple("The capture of the capital of Puglia, the objective port city of Bari, nets the Allies an extra " .. param.cityOfBari .. " supplies! You should endeavour to clear the port of obstacles as quickly as possible so that you may build a dock installation in the city.","Bari is captured!",object.mAlliesCaptureCity)
			object.pAllies.money = object.pAllies.money + param.cityOfBari
		end)
	end

	-- BASTIA
	if city == object.cBastia then 
		gen.justOnce("Capture of Bastia",function()
			civ.playSound("Cheers2.wav")
			text.simple("The Corsican objective port city of Bastia has fallen to the Allies! The capture of its airfield, vital for the future Operation Dragoon, the invasion of southern France, will also provide you with an extra " .. param.bastiaAirfield .. " supplies per turn.","Bastia is captured!",object.m6thArmyGrp)
			flag.setTrue("Airfield Captured in Bastia") 
		end)
	end

	-- BOLOGNA
	if city == object.cBologna then 
		gen.justOnce("Capture of Bologna",function()
			civ.playSound("Cathedrl.wav")
			text.simple("The Allies capture the strategic city of Bologna,\n^headquarters of Army Gruppe Sud and central\n^command of the Gothic Line!\n^\r^As such, hence forth the Germans will no longer\n^benefit from the advantages of city walls!","Bologna falls to the Allies!",object.mAlliesCaptureCity)
		end)
	end	

	-- CATANIA
	if city == object.cCatania and defender == object.pAxis then 
		gen.justOnce("Capture of Catania",function()
			civ.playSound("Cheers2.wav")
			text.simple("The Allies finally enter the stronghold city of Catania! With it's fall, the capture of the ultimate prize, Messina, is one step closer! You should endeavour to clear the port of obstacles as quickly as possible so that you may build a dock installation in the city.","Catania is captured!",object.mAlliesCaptureCity)
		end)
	end	
	
	-- FIRENZE
	if city == object.cFirenze and defender == object.pAxis then 
		gen.justOnce("Capture of Firenze",function()
			civ.playSound("Cathedrl.wav")
			text.setLinesPerWindow(20)
				text.simple("The historic city of Firenze, the capital of Tuscana,\n^falls to the Allied powers, which nets your command\n^an extra " .. param.captureFirenze .. " supplies.\n^\r^Given its strategic position, building a Supply Depot\n^Improvement in the city may be beneficial to your\n^conduct of operations in the area.\n^\r^Finally, you may still deploy 12 ground units within the\n^confines of the city but no more than 4 of them may\n^be artillery units. Should you attempt to exceed this\n^this limit the excess artillery units will be redeployed in\n^an adjacent tile to the city.\n^\r^(NOTE: this was done to maintain play balance and\n^prevent the human player from stacking a pile of\n^artillery in this location to pommel surrounding\n^German defensive positions.)","Firenze is captured!",object.mAlliesCaptureCity)
			text.setLinesPerWindow(12)
			object.pAllies.money = object.pAllies.money + param.captureFirenze
			flag.setTrue("Firenze is captured") 
		end)
	end		

	-- FOGGIA
	if city == object.cFoggia and defender == object.pAxis then 
		gen.justOnce("Capture of Foggia",function()
			civ.playSound("Cheers2.wav")
			text.simple("The city of Foggia falls to the Allied powers. The\n^Foggia Airfield Complex previously maintained by the\n^Regia Aeronautica, will be an ideal site of operations\n^for the US Fifteenth Air Force.\n^\r^As such, one new P-51D Mustang and two B-24 Liberator\n^air groups will arrive in 2 turns in the city.\n^\r^The airfield complex will also provide you with an extra\n^" .. param.napoliAirfield .. " supplies per turn.","Foggia is captured!",object.mItalianAirfield)
			
			flag.setTrue("Foggia is captured") 
			flag.setTrue("Airfield Captured in Foggia") 
			-- Add 1 to # Cities For Italian Surrender counter
			counter.add("# Cities For Italian Surrender",1)	

			delayedAction.doInFuture("receiveNewAirGroups",{},civ.getTurn()+ 2 ,object.pAllies.id)
		end)
	end			

	-- LIVORNO
	if city == object.cLivorno and defender == object.pAxis then 
		gen.justOnce("Capture of Livorno",function()
			civ.playSound("Cathedrl.wav")
			text.simple("The important port city of Livorno falls to the Allied powers.\n^\r^The Joint Chiefs of Staff are pleased by this progress and\n^release an extra " .. param.captureLivorno .. " supplies to support your campaign.\n^\r^You should endeavour to clear the port of obstacles as\n^quickly as possible so that you may build a dock installation\n^in the city.\n^\r^Given its strategic position, building a Supply Depot\n^improvement in the city may be beneficial to your\n^conduct of operations in the area.","Livorno is captured!",object.mAlliesCaptureCity)
			-- Grant extra supplies
			object.pAllies.money = object.pAllies.money + param.captureLivorno
			flag.setTrue("Livorno is captured") 
		end)
	end	
	
	-- MESSINA
	-- SEE 'function secondUSArmoredWithdrawn(argTable)' BELOW 
	if city == object.cMessina and defender == object.pAxis then -- and flag.value("Sea Mines removed from Taranto") == true then
		gen.justOnce("Capture of Messina",function()
			-- civ.playSound("Cathedrl.wav")
			civ.playSound("LARGEXPL.wav")
			text.setLinesPerWindow(25)
				text.simple("The Allied powers enter the port of Messina, though find\n^that the Germans have destroyed all useful service buildings\n^in the city!\n^\r^On the other hand, with their departure, all the defensive\n^obstacles that blocked the Straits of Messina are cleared\n^now and Allied vessels can now transit freely through\n^this waterway.\n^\r^You should endeavour to build a dock installation in the\n^city as soon as possible, in order to benefit from the\n^extra supplies it provides.\n^\r^With the Sicilian campaign coming to an end, the Joint\n^Chief of staff also advise you that the US 1st Infantry\n^and 2nd Armored divisions will be withdrawn from this\n^theater next turn and sent to England in order to\n^prepare for the future invasion of France.","Messina Falls to Allies!",object.mAlliesInMessina)
			text.setLinesPerWindow(12)
			
			civ.removeImprovement(object.cMessina,object.iNewspaper)
			civ.removeImprovement(object.cMessina,object.iRailStation)
			civ.removeImprovement(object.cMessina,object.iDesignbureau)
			civ.removeImprovement(object.cMessina,object.iShoreBattery)
			flag.setTrue("Messina is captured") 
			flag.setTrue("Sea Mines removed from Messina") 
			
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis and unit.type == object.uSeaMines 
				-- and (unit.location.x >= 70 and unit.location.y >= 152 and unit.location.x <= 76 and unit.location.y <= 160 
				and (unit.location.x >= 69 and unit.location.y >= 153 and unit.location.x <= 77 and unit.location.y <= 161
				and unit.location.z == 0) then
				-- and (unit.location.x == 104 and unit.location.y == 158 and unit.location.z == 0) then
					civ.deleteUnit(unit)
					civ.ui.redrawTile(unit.location) 
					-- activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
				end
			end	
			delayedAction.doInFuture("2ndUSArmoredRemoved",{},civ.getTurn()+ 1 ,object.pAllies.id)
		end)
	end	
	
	-- NAPOLI
	if city == object.cNapoli and defender == object.pAxis then 
		gen.justOnce("Capture of Napoli",function()
			civ.playSound("LARGEXPL.wav")
			text.setLinesPerWindow(16)
			text.simple("The Allies enter the major port city of Napoli, which\n^grants your naval units an extra 2 MP's! The Joint Chiefs\n^of Staff are pleased by this progress and release an\n^extra " .. param.captureNapoli .. " supplies to support your campaign.\n^\r^On the other hand, in their haste to leave the city before\n^being surrounded, the Germans still managed to destroy\n^the factory, industrial farms and fishing port and thus its\n^inhabitants may well be in danger of starvation!\n^\r^You should endeavour to clear the port of obstacles as\n^quickly as possible so that you may build a dock installation\n^in the city. If you manage to capture Roma as well, then most\n^Allied reinforcements (not replacements) will arrive here.\n^\r^Given its strategic position, building a Supply Improvement\n^may be beneficial to your conduct of operations in the area...","Napoli is captured!",object.mNapoliDestruction)
			text.simple("Finally, the capture of the city's airbase convinces the\n^US 15th Air Force command to release an extra group\n^of B-17's stationed in Tunis for your usage.\n^\r^It will also provide you with an extra " .. param.napoliAirfield .. " supplies per turn.","USAAF Reinforcements",object.m15thAirForce)
			text.setLinesPerWindow(12)
			
			civ.removeImprovement(object.cNapoli,object.iFactory)
			civ.removeImprovement(object.cNapoli,object.iIndustrialFarming)
			civ.removeImprovement(object.cNapoli,object.iFishingPort)
			
			flag.setTrue("Airfield Captured in Napoli") 
			-- Grant extra supplies
			object.pAllies.money = object.pAllies.money + param.captureNapoli
			-- Grant extra B-17
			gen.createUnit(object.uB17Fortress, object.pAllies, {{0,136,0}}, {count=1, randomize=false, veteran=false})
			counter.add("# B-17",1)
			counter.add("# Total US Bombers",1)
			-- Add 2 to # Cities For Italian Surrender counter
			counter.add("# Cities For Italian Surrender",2)			
		end)
	end	

	-- OLBIA
	if city == object.cOlbia then 
		gen.justOnce("Capture of Olbia",function()
			civ.playSound("Cheers2.wav")
			text.simple("The capture of the port city of Olbia will now allow you to 'invade' Corsica!","Olbia is captured!",object.mAlliesCaptureCity)
			flag.setTrue("Ostia is captured") 
		end)
	end	

	-- PALERMO
	if city == object.cPalermo and defender == object.pAxis then
		gen.justOnce("Capture of Palermo",function()
			civ.playSound("Cathedrl.wav")
			text.simple("The capital of Sicily, the city of Palermo, falls to Allied\n^troops, which comes as a shock to the Italian regime and\n^its people.\n^\r^In the meantime, the Allies, which gain " .. param.capturePalermo .. " supplies in the\n^process, should endeavour to clear the port of obstacles as\n^quickly as possible so that they may build a dock installation.","Palermo is captured!", object.mPalermoSurrenders)
			text.simple("In a surprise to everyone, the Italian head of state, King Victor Emmanuel III\n^and the Gran Consiglio del Fascismo depose Mussolini, after having lost faith\n^in his leadership, who is immediately placed under arrest.\n^\r^Marshall Pietro Badoglio is named his successor and vows to his German\n^allies that Italy will continue to fight.\n^\r^Adolf Hitler in Berlin remains sceptical of the Italian resolve and orders\n^Feldmarshall Keitel to make preparations for Operation Achse, the quick\n^occupation of the peninsula, should Italy betray Germany and surrender\n^to the Allies.","Badoglio new Prime Minister!", object.mBadoglio)
			flag.setTrue("Mussolini Dismissed and Arrested")
			
			local randomNumber = math.random(4)
			-- SEE 'Mussolini Liberated from Captivity' in section BELOW
			-- Essentially, 6 to 9 turns after Mussolini is deposed, he is liberated by the Germans to form the Republic of Salo
			if randomNumber == 1 then
				delayedAction.doInFuture("mussoliniCaptured",{},civ.getTurn()+ 6 ,object.pAxis.id)
			elseif randomNumber == 2 then
				delayedAction.doInFuture("mussoliniCaptured",{},civ.getTurn()+ 7 ,object.pAxis.id)
			elseif randomNumber == 3 then
				delayedAction.doInFuture("mussoliniCaptured",{},civ.getTurn()+ 8 ,object.pAxis.id)
			else
				delayedAction.doInFuture("mussoliniCaptured",{},civ.getTurn()+ 9 ,object.pAxis.id)
			end
			
			-- gen.createUnit(object.uGIVeteran, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
			object.pAllies.money = object.pAllies.money + param.capturePalermo
			object.pAxis.leader.name = "Marshall Pietro Badoglio"
		end)
	end	
	
	-- PALERMO OR CATANIA
	-- If either Catania or Palermo is captured
	if (city == object.cCatania and defender == object.pAxis) or (city == object.cPalermo and defender == object.pAxis) then 
		gen.justOnce("Italians surrender en masse in Sicily",function()
			for unit in civ.iterateUnits() do 
				if tables.italiansToSurrenderSicily[unit.type.id] and (gen.inPolygon(unit.location,tables.sicilyPolygon)) then	
					-- gen.createUnit(object.uPrigionieridiguerra, object.pAxis, unit.location, {count=1, randomize=true, veteran=false})
					-- gen.deleteUnit(unit)
					gen.replaceUnit(unit,object.uPrigionieridiguerra)
				end 
			end	
			text.simple("The remaining Italian forces morale in Sicily is shattered and\n^most are prepared to surrender to the Allies at the first\n^opportunity!\n^\r^As such, each 'Prigionieri di guerra' you capture will be a wealth\n^of intelligence information and grant you an extra ".. param.capturePrigionieridiguerra .. " supplies as\n^a result.","Italian Morale Collapses in Sicily!",object.mItaliansSurrender)
			flag.setTrue("Italian Prisonners Can Be Captured")
		end)
	end	
	
	-- PESCARA
	if city == object.cPescara and flag.value("German 10th Army Cutoff") == false then 
		gen.justOnce("Capture of Pescara",function()
			civ.playSound("Telegraph.wav")
			text.simple("The capture of the city of Pescara threatens to expose\n^the Gustav Line to Allied outflanking maneuvers!\n^\r^Allied Intelligence have intercepted messages from\n^Feldmarshall Kesselring's headquarters ordering the\n^redeployment of German troops in the area to counter\n^this threat.","Pescara is captured!",object.mAlbertKesselring)
			flag.setTrue("Pescara is captured") 
			local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{106,80,0},{108,72,0}}, {randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
				gen.setMoved(unit)
				gen.setToFortified(unit)
			end			
			local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{103,79,0},{103,75,0}}, {randomize=false, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
				gen.setMoved(unit)
				gen.setToFortified(unit)
			end	
			
			local pescaraReactionLocationList = {{104,80,0},{103,85,0},{107,75,0}}			
				gen.createUnit(object.uPzkwVG, object.pAxis, pescaraReactionLocationList, {count=1, randomize=true, scatter=false, veteran=true})
				gen.createUnit(object.uPanzerGrenadier, object.pAxis, pescaraReactionLocationList, {count=1, randomize=true, scatter=false, veteran=true})
				gen.createUnit(object.uGrilleHsIG33, object.pAxis, pescaraReactionLocationList, {count=1, randomize=true, scatter=false, veteran=true})					
			for unit in civ.iterateUnits() do
				if (unit.location.x >= 103 and unit.location.y >= 75 and unit.location.x <= 107 and unit.location.y <= 85 and unit.location.z == 0) then
					unit.gotoTile = civ.getTile(107,83,0) 
				end
			end	
		end)
	end		

	-- REGGIO CALABRIA
	if city == object.cReggioCalabria and defender == object.pAxis then 
		gen.justOnce("Capture of Reggio Calabria",function()
			civ.playSound("Cheers2.wav")
			text.simple("The capture of the objective city of Reggio Calabria\n^further secures the Straits of Messina to Allied\n^shipping and as a consequnece an additional oil tanker\n^arrives in Valetta to support the Allied war effort.\n^\r^It also opens up the route for the conquest of Calabria,\n^whose towns of Castrovillari, Catanzaro and Crotone\n^will each net you between " .. param.provincialCapital .. " and " .. param.townOfCrotone .. " additional supplies.\n^\r^Finally, the well defended airbase near the town\n^of Crotone, can also serve as a point of transit for\n^your air units into southern Italy and the ocean tile\n^on (104,156) as a future naval fueling base.","Reggio Calabria is captured!",object.mAlliesCaptureCity)
			gen.createUnit(object.uOilTankerF, object.pAllies, {{40,180,0}}, {count=1, randomize=false, veteran=false})
			-- flag.setTrue("Reggio Calabria is captured") 
			counter.add("# Total Oil Tankers",1)
			counter.add("# Total Naval Units",1)			
			
			-- Add 1 to # Cities For Italian Surrender counter
			counter.add("# Cities For Italian Surrender",1)
		end)
	end		
	
	-- ROMA
	if city == object.cROMA and defender == object.pAxis then
		gen.justOnce("Capture of Rome",function()
			civ.playSound("Cathedrl.wav")
		
			-- Check if German 10th Army was previously cutoff
			-- If not cutoff, then provide Allies 450 supplies and set "German 10th Army Escaped" flag to true
			if flag.value("German 10th Army Cutoff") == false then
				text.setLinesPerWindow(25)
					text.simple("After a long struggle, the Eternal city falls, at last,\n^to the Allied powers, much to the acclaim of the local\n^populace, which wish nothing more then an end to\n^Mussolini's regime and the war itself.\n^\r^As the city can no longer serve as the major supply\n^route for German forces along the Tyrrhenian Italian\n^coast, the majority are forced to quickly abandon their\n^defensive positions, leaving only a scattered rearguard\n^in that area to slow down your progress.\n^\r^On the other hand, the capture of the Italian capital\n^prior to cutting off the escape route of the German 10th\n^Army has allowed them to begin the withdrawal of some\n^of their mobile forces along the Gustav Line, which will\n^be able to fight another day.\n^\r^All the same, the Allied High Command remains pleased\n^with your progress and therefore allocates a further " .. param.captureRoma .. "\n^supplies to your theater.\n^\r^Finally, if you can control the town of Fiumicino you\n^can force the surrender of the forces in Lido di Roma.","The Eternal City Falls to the Allies!", object.mAlliesEnterRome)
				text.setLinesPerWindow(12)
				object.pAllies.money = object.pAllies.money + param.captureRoma
				flag.setTrue("German 10th Army Escaped")
			-- If cutoff, then reduce number of German mobile reaction troops along Gothic Line but only give Allies 150 supplies
			elseif flag.value("German 10th Army Cutoff") == true then
				text.setLinesPerWindow(25)
					text.simple("After a long struggle, the Eternal city falls, at last,\n^to the Allied powers, much to the acclaim of the local\n^populace, which wish nothing more then an end to\n^Mussolini's regime and the war itself.\n^\r^As the city can no longer serve as the major supply\n^route for German forces along the Tyrrhenian Italian\n^coast, the majority are forced to quickly abandon their\n^defensive positions, leaving only a scattered rearguard\n^in that area to slow down your progress.\n^\r^Since you've already cutoff the German 10th Army's\n^escape route from the Gustav Line, at the expense of\n^theather supplies to support the push, there should be\n^less mobile forces to oppose you moving northward.\n^\r^All the same, the Allied High Command is very pleased\n^with your progress and therefore allocates an extra " .. math.floor(param.captureRoma / 3) .. "\n^supplies to your theater.\n^\r^Finally, if you can control the town of Fiumicino you\n^can force the surrender of the forces in Lido di Roma.","The Eternal City Falls to the Allies!", object.mAlliesEnterRome)				
				text.setLinesPerWindow(12)
				object.pAllies.money = object.pAllies.money + (param.captureRoma / 3)
			end

			text.simple("Finally, the capture of the city's surrounding airbases\n^convinces the US 15th Air Force command to release an\n^extra group of B-17's stationed in Tunis for your usage.\n^\r^It will also provide you with an extra " .. param.romaAirfield .. " supplies per turn.","USAAF Reinforcements",object.m15thAirForce)

			-- Delete all fort type German units, save bunkers, along Tyrrhenian coast south of Rome
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis and (tables.germanFortifiedStatus[unit.type.id] and unit.type ~= object.uBunker) and gen.inPolygon(unit.location,tables.tenthArmyWestpolygon1) then 
					print("Destroyed " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
					civ.deleteUnit(unit)
				end
			end
			
			-- Delete 37cm Flak, Kustenbatterie and Widerstandsnest from Civitavecchia hill defenses
			-- Reason: to reflect collapse of German southern defenses including around Rome
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis and (unit.type == object.u37cmFlak36 or unit.type == object.uKustenbatterie or unit.type == object.uWiderstandsnest) and unit.location == object.lhillCivitavecchia then 
					print("Destroyed " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
					civ.deleteUnit(unit)
				end
			end	

			-- Inflict damage on bunkers to reflect weakened state
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis and unit.type == object.uBunker and gen.inPolygon(unit.location,tables.tenthArmyWestpolygon1) then 
					if math.random(1,2) == 2 then
						print("Destroyed " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
						civ.deleteUnit(unit)
					else
						unit.damage = unit.damage + 16
					end
				end
			end

			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis and (tables.germanFortifiedStatus[unit.type.id] and unit.type ~= object.uBunker) and gen.inPolygon(unit.location,tables.tenthArmyWestpolygon2) then 
					print("Destroyed " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
					civ.deleteUnit(unit)
				end
			end

			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis and unit.type == object.uBunker and gen.inPolygon(unit.location,tables.tenthArmyWestpolygon2) then 
					unit.damage = unit.damage + 10
				end
			end
			
			flag.setTrue("Roma is captured")
			flag.setTrue("Airfield Captured in ROMA") 
			-- Add 2 to # Cities For Italian Surrender counter
			counter.add("# Cities For Italian Surrender",2)
			-- object.pAxis.leader.name = "Marshall Badoglio"
			
			-- Grant extra B-17
			gen.createUnit(object.uB17Fortress, object.pAllies, {{0,136,0}}, {count=1, randomize=false, veteran=false})
			counter.add("# B-17",1)
			counter.add("# Total US Bombers",1)
			
			delayedAction.doInFuture("activateCLNPartisans",{},civ.getTurn()+ 8 ,object.pAllies.id)
		end)
	end	

	-- TARANTO
	-- SEE 'function eightIndianArrives(argTable)' BELOW 
	if city == object.cTaranto and defender == object.pAxis then -- and flag.value("Sea Mines removed from Taranto") == true then
		gen.justOnce("Capture of Taranto",function()
			local randomNumber = math.random(3)
			if randomNumber == 1 then
				delayedAction.doInFuture("8thIndianSetToArrive",{},civ.getTurn()+ 1 ,object.pAllies.id)
			elseif randomNumber == 2 then
				delayedAction.doInFuture("8thIndianSetToArrive",{},civ.getTurn()+ 2 ,object.pAllies.id)
			elseif randomNumber == 3 then
				delayedAction.doInFuture("8thIndianSetToArrive",{},civ.getTurn()+ 3 ,object.pAllies.id)	
			end	

			civ.playSound("Cheers2.wav")		
			text.simple("The Allied powers enter the port of Taranto and the local Italian commanders are ordered to begin clearing the entrance of all sea mines, which should take " .. randomNumber .." weeks.","Allies Enter Taranto!",object.mAlliesCaptureCity)
		end)
	end		

	-- If one of the following cities is captured by Allies remove all Illustrious Class CV and there air components
	-- Reason: Once Allied forces had secured major airfields on Italian mainland no longer required Carrier support
	if (city == object.cFoggia and defender == object.pAxis) 
	or (city == object.cNapoli and defender == object.pAxis) 
	or (city == object.cROMA and defender == object.pAxis) then
		-- Delete Illustrious Class CV, Seafire Mk.II and Fairey Baaracuda
		gen.justOnce("Italian airfield secured",function()
			civ.playSound("Telegraph.wav")
			text.simple("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. onTurn.commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. onTurn.commander15thArmy_Base .. ", (15th Army Group)\n^\r^Now that you have secured a major airfield north of the\n^Italian surrender line, you should be able to redeploy most\n^of your airforces nearer to the front lines in order to\n^provide closer air support.\n^\r^As a consequence, the Allied High Command is of the opinion\n^that you no longer require the support of the Illustrious Class\n^Carrier Groups and their air components and therefore starting\n^next turn they will all be reassigned to other theaters.","Italian Mainland Airfield Secured!",object.mItalianAirfield)
			delayedAction.doInFuture("withdrawCarrierGroups",{},civ.getTurn()+ 1 ,object.pAllies.id)
		end)
	end	
	
	-- TRIESTE
	if city == object.cTrieste and defender == object.pAxis then
		gen.justOnce("Capture of Trieste",function()
			civ.playSound("Cathedrl.wav")
			text.simple("The important objective city of Trieste falls to the\n^Allied powers.\n^\r^The Combined Chiefs of Staff are pleased by this\n^event as it prevents the city from falling into Tito's\n^communists forces and should help secure the\n^Western Allies southern flank from further 'red'\n^incursions.","Trieste is captured!",object.mAlliesCaptureCity)
			flag.setTrue("Trieste is captured") 
		end)
	end		

	-- VERONA
	if city == object.cVerona and defender == object.pAxis then
		gen.justOnce("Capture of Verona",function()
			civ.playSound("Cathedrl.wav")
			text.simple("The important objective and railyard junction city\n^of Verona falls to the Allied powers.\n^\r^The German garrison of the town of Salo, if any are\n^still remaining, flee the town to attempt to escape\n^back to Germany.","Verona is captured!",object.mAlliesCaptureCity)
			-- flag.setTrue("Trieste is captured") 

			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis and (unit.type == object.u37cmFlak36 or unit.type == object.uWiderstandsnest) and unit.location == object.ltownSalo then 
					print("Destroyed " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
					civ.deleteUnit(unit)
				end
			end	
		end)
	end	



end -- END register.onCityTaken ==============================
---&endAutoDoc

-- ==============================
-- DELAYED ACTION FUNCTIONS
-- ==============================

-- Allied Carrier Groups are withdrawn one turn after any mainland airfield is secured (Foggia, Napoli or Roma)
-- SEE delayedAction.doInFuture("withdrawCarrierGroups", ABOVE as they are related when you want to delay an action
local function carrierBattleGroupWithdrawn(argTable)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAllies and (unit.type == object.uIllustriousClassCV) then 
			print("Deleted " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
			civ.deleteUnit(unit)			
		end	
		if unit.owner == object.pAllies and (unit.type == object.uSeafireMkII) then 
			print("Deleted " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
			civ.deleteUnit(unit)
		end	

		if unit.owner == object.pAllies and (unit.type == object.uFaireyBarracuda) then 
			print("Deleted " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
			civ.deleteUnit(unit)
		end			

	end
	text.simple("All remaining Royal Navy carriers along with their air components are reassigned to other theaters!","Royal Navy Carriers Reassigned!",object.mHMSIllustrious)
	flag.setTrue("Seafire MkII carrier aircraft withdrawn")
	flag.setTrue("Fairey Barracuda carrier aircraft withdrawn")

	counter.subtract("# Seafire MkII",2)
	counter.subtract("# Total UK Fighters",2)	
	counter.subtract("# Fairey Barracuda",2)
	counter.subtract("# Total UK Bombers",2)	

	counter.subtract("# Illustrious Class CV",2)
	counter.subtract("# Total Naval Units",2) -- CV's don't count against Total Naval units	
end 
delayedAction.makeFunctionDelayable("withdrawCarrierGroups",carrierBattleGroupWithdrawn)

-- Extra Allied Air Groups arrive two turns after Foggia mainland airfield is secured
-- SEE delayedAction.doInFuture("receiveNewAirGroups", ABOVE as they are related when you want to delay an action
local function newAirGroupsArrive(argTable)
	text.simple("The USAAF 98th, 376th Bomb and 82nd Fighter Groups of the US Fifteenth Air Force deploys in Foggia!","USAAF Reinforcements",object.m15thAirForce)
	gen.createUnit(object.uB24Liberator, object.pAllies, {{111,107,0}}, {count=2, randomize=false, veteran=false})
	gen.createUnit(object.uP51DMustang, object.pAllies, {{111,107,0}}, {count=1, randomize=false, veteran=false})
	
	counter.add("# P-51",1)
	counter.add("# Total US Fighters",1)
	counter.add("# B-24",2)
	counter.add("# Total US Bombers",2)	
	
end 
delayedAction.makeFunctionDelayable("receiveNewAirGroups",newAirGroupsArrive)

-- 2nd US Armored Division is withdrawn 1 turn after Allies enter Messina
-- SEE delayedAction.doInFuture("2ndUSArmoredRemoved", ABOVE as they are related when you want to delay an action
local function secondUSArmoredWithdrawn(argTable)
	text.simple("The US 2nd Armored Division is withdrawn from the theater and sent to England to prepare for the invasion of France!","Supreme Allied Command",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAllies and tables.usArmoredToBeRemoved[unit.type.id] then
			civ.deleteUnit(unit)
			civ.ui.redrawTile(unit.location) 
			-- activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
		end
	end	
	flag.setFalse("US Armored Division is Active")

	counter.subtract("# Armored Infantry",2)
	counter.subtract("# M5 Stuart",1)
	counter.subtract("# M4A1",3)
	counter.subtract("# M12 155mm HMC",1)
	counter.subtract("# Total US Armored",6)
	counter.subtract("# Total US Artillery",1)
end 
delayedAction.makeFunctionDelayable("2ndUSArmoredRemoved",secondUSArmoredWithdrawn)

-- 8th Indian Division arrives up to 3 turns after Taranto is captured
-- SEE delayedAction.doInFuture("8thIndianSetToArrive", ABOVE as they are related when you want to delay an action
local function eightIndianArrives(argTable)
	-- Remove see mines from port of Taranto
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 118 and unit.location.y >= 136 and unit.location.x <= 120 and unit.location.y <= 138 and unit.location.z == 0) then
			civ.deleteUnit(unit)
			civ.ui.redrawTile(unit.location) 
			-- activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
		end
	end
	flag.setTrue("Sea Mines removed from Taranto") 
	
	gen.createUnit(object.uIndianInfantry, object.pAllies, {{120,138,0}}, {count=3, randomize=false, veteran=false})
	text.simple("The sea mines have been removed from the port of Taranto and the 8th Indian Infantry Division arrives in the city!","8th Indian Division Arrives!",object.monexone)
	flag.setTrue("Indian Infantry units arrive")
	
	counter.add("# Indian Infantry",3)
	counter.add("# Total Indian",3)	
	
end 
delayedAction.makeFunctionDelayable("8thIndianSetToArrive",eightIndianArrives)

-- CNL Partisans activated 8 turns after Roma is captured
-- SEE delayedAction.doInFuture("activateCLNPartisans", ABOVE as they are related when you want to delay an action
local function italianPartisansActivated(argTable)
	flag.setTrue("Italian CLN Front Activated")
end 
delayedAction.makeFunctionDelayable("activateCLNPartisans",italianPartisansActivated)

-- Mussolini Liberated from Captivity
-- SEE delayedAction.doInFuture("mussoliniCaptured",ABOVE as they are related when you want to delay an action
-- Liberation of Mussolini leads to recruitment of extra Blackshirts units
local function mussoliniLiberated(argTable)
	civ.playSound("InfantryGer.wav")
	text.simple(object.granSassoRaid,"Gran Sasso Raid", object.mMussoliniLiberated)
	gen.createUnit(object.uWiderstandsnest, object.pAxis, {{103,7,0}}, {count=1, randomize=false, veteran=false})
	gen.createUnit(object.u37cmFlak36, object.pAxis, {{103,7,0}}, {count=1, randomize=false, veteran=false})
	gen.createUnit(object.uParacadutisto, object.pAxis, {{103,7,0}}, {count=1, randomize=false, veteran=true})
	flag.setTrue("Republic of Salo Created")
	
end 
delayedAction.makeFunctionDelayable("mussoliniCaptured",mussoliniLiberated)


return register
