-- This file can be deleted if it is not necessary.
--
local gen = require("generalLibrary")
local param = require("parameters")
local object = require("object")
local text = require("text")
local func = require("functions")

local flag = require("flag")
local counter = require("counter")
local delayedAction = require("delayedAction")

local tables = require("tables")

-- This function is only run when a unit is killed in standard civ ii combat.  If you need an effect to happen even if a unit is 'defeated' by another unit in an event, use onUnitDefeated.lua

local onUnitKilled = {}

---&autoDoc onUnitKilled
-- This will only run when a unit is killed in combat (i.e. not when an event
-- 'kills' a unit)
-- note that if the aggressor loses, aggressor.location will not work
-- note: use loserLocation instead of loser.location, since loser.location doesn't work if the attacker loses
-- (the game returns a 'tile' off the map)

function onUnitKilled.onUnitKilled(loser,winner,aggressor,victim,loserLocation,winnerVetStatus,loserVetStatus)
    if _global.eventTesting then
        -- civ.ui.text(loser.type.name.." killed by "..winner.type.name.." separate file test killed in combat.")
    end
	
	-- ===============================
	-- COUNT TOTAL ALLIED AND AXIS UNITS KILLED AND DISPLAY ON STATUS DASHBOARD (see consolidatedEvents.lua, key '1')
	-- Do not count munitions
	
	-- ALLIES: FOR EACH ALLIED UNIT KIA GIVE BONUS SUPPLIES TO GERMANS
	if loser.owner == object.pAllies and not tables.munitionsToBeDestroyed[loser.type.id] then
		counter.add("# Total Allied Killed",1)
		-- Provide bonus to Axis treasury for each Allied unit destroyed
		object.pAxis.money = object.pAxis.money + param.alliedUnitKilled
	end

	-- AXIS
	-- Axis Total
	if loser.owner == object.pAxis and not tables.munitionsToBeDestroyed[loser.type.id] then	
		counter.add("# Total Axis Killed",1)
	end

	-- Germans
	if loser.owner == object.pAxis 
	and (tables.german_Air_Units[loser.type.id]
	or tables.german_Artillery_Gun_Units[loser.type.id]
	or tables.german_Artillery_AA_Units[loser.type.id]
	or tables.german_Fortifications_Units[loser.type.id]
	or tables.german_Ground_Armor_Units[loser.type.id]
	or tables.german_Ground_Units[loser.type.id]
	or tables.german_Ground_SS_Units[loser.type.id]
	or tables.german_Naval_Units[loser.type.id])
	and not tables.munitionsToBeDestroyed[loser.type.id] then	
		counter.add("# Total Germans Killed",1)
	end	
	
	-- Italians
	if loser.owner == object.pAxis 
	and (tables.italian_Air_Units[loser.type.id] 
	or tables.italian_Artillery_Units[loser.type.id] 
	or tables.italian_Ground_Units[loser.type.id] 
	or tables.italian_Naval_Units[loser.type.id]) 
	and not tables.munitionsToBeDestroyed[loser.type.id] then	
		counter.add("# Total Italians Killed",1)
	end
	

	-- ===============================
	-- RIVER CROSSING BLOWN AFTER 'DESTROYING' AXIS PIONIERE UNIT 
	-- (see discreteEvents.lua tileData.flagGetValue(unit.location,"riggedBridge"), which has replaced this Pioniere blown bridge event)
	-- Northern River crossing with Crossing Temp 'resource' added to base terrain tile
	-- if loser.type == object.uPioniere and loser.location.terrain == object.tCrossingTemp then
		-- civ.playSound("LARGEXPL.wav")
		-- gen.setTerrainType(loser.location,14)
		-- local tileToChange = civ.getTile(loser.location.x,loser.location.y,loser.location.z)
		-- tileToChange.terrain = object.tBlownBridge2
		-- gen.removeRailroad(loser.location)
		-- gen.removeRoad(loser.location)
		
		-- text.simple("The bridge on the river crossing located on tile (" .. text.coordinates(loserLocation) .. ") has been destroyed by enemy pioniere!","Bridge Destroyed",object.mBridgeDestroyed)
	
	-- -- Southern River crossing with base terrain tile only
	-- elseif loser.type == object.uPioniere and loser.location.baseTerrain == object.bRiverCrossing then 
		-- civ.playSound("LARGEXPL.wav")
		-- gen.setTerrainType(loser.location,11)
		-- local tileToChange = civ.getTile(loser.location.x,loser.location.y,loser.location.z)
		-- tileToChange.terrain = object.tBlownBridge1
		-- gen.removeRailroad(loser.location)
		-- gen.removeRoad(loser.location)
		
		-- text.simple("The bridge on the river crossing located on tile (" .. text.coordinates(loserLocation) .. ") has been destroyed by enemy pioniere!","Bridge Destroyed",object.mBridgeDestroyed)	
		-- --text.simple("The river crossing is destroyed at" .. loserLocation.location .. " !","Bridge Destroyed",object.mBridgeDestroyed)
		-- -- civ.ui.text("The river crossing is destroyed "..loser.location.." !")
		-- -- civ.ui.text("The river crossing is destroyed at ".. tostring(text.coordinates(tile)) .." !")
		-- -- civ.ui.text("The river crossing is destroyed ".. {{loser.location.x, loser.location.y, loser.location.z}} .." !""Bridge Destroyed",object.mBridgeDestroyed)
	-- end

	
	-- IF AXIS DESTROY PONTOON BRIDGE REVERT TERRAIN TILE BACK TO RIVERBED
	-- Temperate Riverbed
	if loser.type == object.uPontoonBridge and loser.location.terrain == object.tCrossing then
		civ.playSound("LARGEXPL.wav")
		gen.setTerrainType(loser.location,14) -- Temperate Riverbed
		local tileToChange = civ.getTile(loser.location.x,loser.location.y,loser.location.z)
		tileToChange.terrain = tileToChange.baseTerrain:getTerrain(0)		
		text.simple("The pontoon bridge on the river crossing located on tile (" .. text.coordinates(loserLocation) .. ") has been destroyed by enemy action!","Pontoon Bridge Destroyed!",object.mBridgeDestroyed)
	-- Riverbed
	elseif loser.type == object.uPontoonBridge then 	
		civ.playSound("LARGEXPL.wav")
		gen.setTerrainType(loser.location,11) -- Riverbed
		text.simple("The pontoon bridge on the river crossing located on tile (" .. text.coordinates(loserLocation) .. ") has been destroyed by enemy action!","Pontoon Bridge Destroyed!",object.mBridgeDestroyed)
	end		
	
	-- IF DOCKS UNIT DESTROYED BY AXIS RESET THE CITY DOCKS FLAGS' AND TERRAIN RESOURCE ICON
	if loser.type == object.uDocks then
		-- BARI
		if loser.location == object.lBari then
			text.simple("The docks located in " .. object.cBari.name .. " have been destroyed by enemy action and therefore no longer provide its extra supplies!\n^\r^It can be rebuilt by an engineer if you send one to the city.","Docks Destroyed in " .. object.cBari.name .. "!",object.monexone)
			local tileToChange = civ.getTile(121,125,0)
			tileToChange.terrain = tileToChange.baseTerrain:getTerrain(0)
			flag.setFalse("Docks being built in Bari")
			flag.setFalse("Docks built in Bari")
		-- CATANIA
		elseif loser.location == object.lCatania then
			text.simple("The docks located in " .. object.cCatania.name .. " have been destroyed by enemy action and therefore no longer provide its extra supplies!\n^\r^It can be rebuilt by an engineer if you send one to the city.","Docks Destroyed in " .. object.cCatania.name .. "!",object.monexone)
			local tileToChange = civ.getTile(60,162,0)
			tileToChange.terrain = tileToChange.baseTerrain:getTerrain(0)
			flag.setFalse("Docks being built in Catania")
			flag.setFalse("Docks built in Catania")
		-- LIVORNO
		elseif loser.location == object.lLivorno then
			text.simple("The docks located in " .. object.cLivorno.name .. " have been destroyed by enemy action and therefore no longer provide its extra supplies!\n^\r^It can be rebuilt by an engineer if you send one to the city.","Docks Destroyed in " .. object.cLivorno.name .. "!",object.monexone)
			local tileToChange = civ.getTile(76,34,0)
			tileToChange.terrain = tileToChange.baseTerrain:getTerrain(0)
			flag.setFalse("Docks being built in Livorno")
			flag.setFalse("Docks built in Livorno")
		-- MESSINA
		elseif loser.location == object.lMessina then
			text.simple("The docks located in " .. object.cMessina.name .. " have been destroyed by enemy action and therefore no longer provide its extra supplies!\n^\r^It can be rebuilt by an engineer if you send one to the city.","Docks Destroyed in " .. object.cMessina.name .. "!",object.monexone)
			local tileToChange = civ.getTile(71,155,0)
			tileToChange.terrain = tileToChange.baseTerrain:getTerrain(0)
			flag.setFalse("Docks being built in Messina")
			flag.setFalse("Docks built in Messina")
		-- NAPOLI
		elseif loser.location == object.lNapoli then
			text.simple("The docks located in " .. object.cNapoli.name .. " have been destroyed by enemy action and therefore no longer provide its extra supplies!\n^\r^It can be rebuilt by an engineer if you send one to the city.","Docks Destroyed in " .. object.cNapoli.name .. "!",object.monexone)
			local tileToChange = civ.getTile(86,106,0)
			tileToChange.terrain = tileToChange.baseTerrain:getTerrain(0)
			flag.setFalse("Docks being built in Napoli")
			flag.setFalse("Docks built in Napoli")
		-- PALERMO
		elseif loser.location == object.lPalermo then
			text.simple("The docks located in " .. object.cPalermo.name .. " have been destroyed by enemy action and therefore no longer provide its extra supplies!\n^\r^It can be rebuilt by an engineer if you send one to the city.","Docks Destroyed in " .. object.cPalermo.name .. "!",object.monexone)
			local tileToChange = civ.getTile(45,141,0)
			tileToChange.terrain = tileToChange.baseTerrain:getTerrain(0)
			flag.setFalse("Docks being built in Palermo")
			flag.setFalse("Docks built in Palermo")
		-- TARANTO
		elseif loser.location == object.lTaranto then
			text.simple("The docks located in " .. object.cTaranto.name .. " have been destroyed by enemy action and therefore no longer provide its extra supplies!\n^\r^It can be rebuilt by an engineer if you send one to the city.","Docks Destroyed in " .. object.cTaranto.name .. "!",object.monexone)
			local tileToChange = civ.getTile(120,136,0)
			tileToChange.terrain = tileToChange.baseTerrain:getTerrain(0)
			flag.setFalse("Docks being built in Taranto")
			flag.setFalse("Docks built in Taranto")
		-- TUNIS
		elseif loser.location == object.lTunis then
			text.simple("The docks located in " .. object.cTunis.name .. " have been destroyed by enemy action and therefore no longer provide its extra supplies!\n^\r^It can be rebuilt by an engineer if you send one to the city.","Docks Destroyed in " .. object.cTunis.name .. "!",object.monexone)
			local tileToChange = civ.getTile(2,136,0)
			tileToChange.terrain = tileToChange.baseTerrain:getTerrain(0)
			flag.setFalse("Docks being built in Tunis")
			flag.setFalse("Docks built in Tunis")
		-- VALETTA
		elseif loser.location == object.lValetta then
			text.simple("The docks located in " .. object.cValetta.name .. " have been destroyed by enemy action and therefore no longer provide its extra supplies!\n^\r^It can be rebuilt by an engineer if you send one to the city.","Docks Destroyed in " .. object.cValetta.name .. "!",object.monexone)
			local tileToChange = civ.getTile(39,181,0)
			tileToChange.terrain = tileToChange.baseTerrain:getTerrain(0)
			flag.setFalse("Docks being built in Valetta")
			flag.setFalse("Docks built in Valetta")
		end	
	end	
	
	-- EACH ITALIAN PRIGIONERI DI GUERRA CAPTURED GRANTS EXTRA SUPPLIES
	if loser.type == object.uPrigionieridiguerra and flag.value("Italian Prisonners Can Be Captured") == true then
		-- civ.playSound("LARGEXPL.wav")
		text.simple("The " .. loser.type.name .. " located on tile (" .. text.coordinates(loserLocation) .. ") has been captured by Allied forces and ".. param.capturePrigionieridiguerra .. " supplies are added to your theater depots!","" .. loser.type.name .. "Italian Prisonners Captured!",object.mItaliansSurrender)
		object.pAllies.money = object.pAllies.money + param.capturePrigionieridiguerra
	end
	
	-- EACH ITALIAN BATTLESHIP SUNK GRANTS EXTRA SUPPLIES
	if loser.type == object.uLittorioClassBB then
		civ.playSound("LARGEXPL.wav")
		text.simple("The " .. loser.type.name .. " located on tile (" .. text.coordinates(loserLocation) .. ") has been sunk by Allied forces and ".. param.sunkLittoriBB .. " supplies are added to your theather depots!","" .. loser.type.name .. " Battleship Sunk!",object.mCapitalShipSunk)
		object.pAllies.money = object.pAllies.money + param.sunkLittoriBB
	end	

	-- FLAK SHELLS DELETED WHEN VICTORIOUS IN BATTLE
	-- This code will delete a Flak shell if it is victorious in a battle. It is required because the 'Destroyed after attacking' rules.txt flag seems to prevent the shell from attacking domain 1 units otherwise, and thus needs to be omitted.
	if winner.type == object.uLightFlak or winner.type == object.uHeavyFlak then
		civ.ui.text("The flak shell was victorious at " .. text.coordinates(loserLocation) .. "!")
		civ.deleteUnit(winner)
	end
	
	-- ANZIO 'ANNIE' MESSAGE FIRST TIME IT FIRES
	if loser.owner == object.pAxis and loser.type == object.uHeavyShell 
	or winner.owner == object.pAxis and winner.type == object.uHeavyShell 
	then
		gen.justOnce("Anzio Annie K5 Rail Gun Deployed",function()
			civ.playSound("LARGEXPL.wav")
			text.simple("In response to the latest Allied invasion moves, the Germans bring in a pair of their infamous Krupp K5 rail guns to pommel their positions!","Anzio Annie Krupp K5 Railgun!",object.mAnzioAnnie)
		end)
	end	

	-- GERMAN AXIS UNITS ELIMINATED IN SICILY
	-- If German units stationed in Sicily are eliminated prior to turn #10, rebuild them in a damaged state in Italy with a delay of 6 turns
	-- NOTE: This is done to represent the fact that most German units managed to escape the island before the Allies were able to capture Messina
	-- SEE 'function fallschirmjagerRebuilt(argTable)' BELOW 
	-- if civ.getTurn() <= 9 and loser.type == object.uFallschirmjager and flag.value("Sicily is Conquered") == false then
		-- delayedAction.doInFuture("rebuildFallschirmjager",{},civ.getTurn()+ 3 ,object.pAxis.id)
	-- end	
	-- SEE 'function motInfanterieRebuilt(argTable)' BELOW
	if civ.getTurn() <= 9 and loser.type == object.uMotInfanterie and flag.value("Sicily is Conquered") == false then
		delayedAction.doInFuture("rebuildMotInfanterie",{},civ.getTurn()+ 4 ,object.pAxis.id)
	end	
	-- SEE 'function panzerGrenadierRebuilt(argTable)' BELOW
	if civ.getTurn() <= 9 and loser.type == object.uPanzerGrenadier and flag.value("Sicily is Conquered") == false then
		delayedAction.doInFuture("rebuildPanzerGrenadier",{},civ.getTurn()+ 5 ,object.pAxis.id)
	end	
	-- SEE 'function sdkFz72AARebuilt(argTable)' BELOW
	if civ.getTurn() <= 9 and loser.type == object.uSdkFz72AA and flag.value("Sicily is Conquered") == false then
		delayedAction.doInFuture("rebuildSdkFz72AA",{},civ.getTurn()+ 5 ,object.pAxis.id)
	end	
	-- SEE 'function sdkFz231Rebuilt(argTable)' BELOW
	if civ.getTurn() <= 9 and loser.type == object.uSdkFz231 and flag.value("Sicily is Conquered") == false then
		delayedAction.doInFuture("rebuildSdkFz231",{},civ.getTurn()+ 5 ,object.pAxis.id)
	end	
	-- SEE 'function stugIIIGRebuilt(argTable)' BELOW
	if civ.getTurn() <= 9 and loser.type == object.uStugIIIG and flag.value("Sicily is Conquered") == false then
		delayedAction.doInFuture("rebuildStugIIIG",{},civ.getTurn()+ 5 ,object.pAxis.id)
	end	
	-- SEE 'function pzkwIVFRebuilt(argTable)' BELOW
	if civ.getTurn() <= 9 and loser.type == object.uPzkwIVF and flag.value("Sicily is Conquered") == false then
		delayedAction.doInFuture("rebuildPzkwIVF",{},civ.getTurn()+ 5 ,object.pAxis.id)
	end		
	-- SEE 'function pzkwVIERebuilt(argTable)' BELOW
	if civ.getTurn() <= 9 and loser.type == object.uPzkwVIE and flag.value("Sicily is Conquered") == false then
		delayedAction.doInFuture("rebuildPzkwVIE",{},civ.getTurn()+ 6 ,object.pAxis.id)
	end	

	-- SEE 'function fallschirmjagerRebuilt_2(argTable)' BELOW
	-- Each Fallschirmjager unit eliminated along Gustav Line after turn #10 and Roma isn't captured, rebuild them in a damaged state along Gothic Line with a delay of 12 turns	
	-- if civ.getTurn() >= 10 and loser.type == object.uFallschirmjager and flag.value("Roma is captured") == false then
		-- delayedAction.doInFuture("rebuildFallschirmjager_2",{},civ.getTurn()+ 12 ,object.pAxis.id)
	-- end	

	-- SEE 'function railroadyardRebuilt(argTable)' BELOW
	-- Rebuild each destroyed Axis railroad yard with a delay of 4 to 6 turns
	-- Active Railroad yards play an important role in determining the Axis Replacement rates per turn (see AXIS REPLACEMENTS, onTurn.lua)
	if loser.type == object.uRailroadYard then
		local randomNumber = math.random(3)

		if randomNumber == 1 then
			delayedAction.doInFuture("rebuildRailRoadYard",{},civ.getTurn()+ 4 ,object.pAxis.id)
		elseif randomNumber == 2 then
			delayedAction.doInFuture("rebuildRailRoadYard",{},civ.getTurn()+ 5 ,object.pAxis.id)
		elseif randomNumber == 3 then
			delayedAction.doInFuture("rebuildRailRoadYard",{},civ.getTurn()+ 6 ,object.pAxis.id)	
		end	
		civ.playSound("LARGEXPL.wav")
		text.simple("The " .. object.uRailroadYard.name .." located at (" .. text.coordinates(loser.location) ..") was destroyed by an Allied raid. The Deutsche Reichsbahn (German rail service) will attempt to repair the damage or reroute the rail line to an alternate location, which it is estimated could take " .. 3 + randomNumber .. " weeks!\n^\r^The Allied High Command releases ".. param.destroyedRailYard .. " supplies\r^as a reward for you efforts!","Railroad Yard Destroyed!",object.mBombedRailYard)
		object.pAllies.money = object.pAllies.money + param.destroyedRailYard
		
	end			

	-- SEE REBUILD SS UNITS (argTable)'s BELOW 
	-- Rebuild each destroyed SS unit with a delay of 6 or more turns if 16th SS PzGr Div is triggered 
	if civ.getTurn() >= 21 and loser.type == object.uSSMotInfanterie and flag.value("16th SS PzGr Div triggered") == true then
		delayedAction.doInFuture("rebuildSSMotInfanterie",{},civ.getTurn()+ 6 ,object.pAxis.id)
	end	
	if civ.getTurn() >= 21 and loser.type == object.uSSSdkFz72AA and flag.value("16th SS PzGr Div triggered") == true then
		delayedAction.doInFuture("rebuildSSSdkFz72AA",{},civ.getTurn()+ 6 ,object.pAxis.id)
	end
	if civ.getTurn() >= 21 and loser.type == object.uSSSdkFz2342 and flag.value("16th SS PzGr Div triggered") == true then
		delayedAction.doInFuture("rebuildSSSdkFz2342",{},civ.getTurn()+ 6 ,object.pAxis.id)
	end	
	if civ.getTurn() >= 21 and loser.type == object.uSSPzkwIV and flag.value("16th SS PzGr Div triggered") == true then
		delayedAction.doInFuture("rebuildSSPzkwIV",{},civ.getTurn()+ 8 ,object.pAxis.id)
	end	
	if civ.getTurn() >= 21 and loser.type == object.uSSHummel and flag.value("16th SS PzGr Div triggered") == true then
		delayedAction.doInFuture("rebuildSSHummel",{},civ.getTurn()+ 10 ,object.pAxis.id)
	end	
	
	-- ALLIED BEACHHEAD DESTROYED
		if loser.type == object.uBeachhead then
		civ.playSound("LARGEXPL.wav")
		text.simple("The USN " .. loser.type.name .. " unit located on tile (" .. text.coordinates(loserLocation) .. ")\n^has been destroyed by enemy forces!\n^\r^The loss of the beachhead costs the Allies " .. param.beachheadDestroyed .. " supplies\n^in lost resources and becomes one less destination for\n^you to disembark your troops onto.","USN " .. loser.type.name .. " Destroyed!",object.mDUKWBeachead)
		object.pAllies.money = object.pAllies.money - param.beachheadDestroyed
	end	

	-- ALLIED BATTLESHIPS AND CV'S SUNK
		if loser.type == object.uQEClassBB or loser.type == object.uIllustriousClassCV then
		civ.playSound("LARGEXPL.wav")
		text.simple("The Royal Navy " .. loser.type.name .. " unit located on tile (" .. text.coordinates(loserLocation) .. ")\n^has been sunk by enemy forces!\n^\r^The loss of a capital ship is another drain on England's strained\n^resources and as such " .. param.CapitalShipSunk .. " supplies are taken from the Meditterranean\n^theater to compensate for the loss for the future Normandy\n^amphibious operation.","Royal Navy " .. loser.type.name .. " Sunk!",object.mCapitalShipSunk)
		object.pAllies.money = object.pAllies.money - param.CapitalShipSunk
	end	
	
	-- ALLIED GENERALS - WOUNDED IN ACTION
	-- Gen. Montgomery
	if civ.getTurn() >= 1 and loser.type == object.uGenMontgomery then
		local randomNumber = math.random(4)
		
		if randomNumber == 1 then
			delayedAction.doInFuture("genMontgomeryWounded",{},civ.getTurn()+ 6 ,object.pAllies.id)
		elseif randomNumber == 2 then
			delayedAction.doInFuture("genMontgomeryWounded",{},civ.getTurn()+ 7 ,object.pAllies.id)
		elseif randomNumber == 3 then
			delayedAction.doInFuture("genMontgomeryWounded",{},civ.getTurn()+ 8 ,object.pAllies.id)	
		elseif randomNumber == 4 then
			delayedAction.doInFuture("genMontgomeryWounded",{},civ.getTurn()+ 9 ,object.pAllies.id)
		end	
		civ.ui.text(object.uGenMontgomery.name .. " was wounded in combat near (" .. text.coordinates(loserLocation) .. ") and is evacuated to a field hospital. His recovery is expected to take " .. 5 + randomNumber .. " weeks!")
		print(object.uGenMontgomery.name .. " wounded Random number is " .. randomNumber .. "!")
	end	

	-- Gen. Leese
	if civ.getTurn() >= 1 and loser.type == object.uGenLeese then
		local randomNumber = math.random(4)
		
		if randomNumber == 1 then
			delayedAction.doInFuture("genLeeseWounded",{},civ.getTurn()+ 6 ,object.pAllies.id)
		elseif randomNumber == 2 then
			delayedAction.doInFuture("genLeeseWounded",{},civ.getTurn()+ 7 ,object.pAllies.id)
		elseif randomNumber == 3 then
			delayedAction.doInFuture("genLeeseWounded",{},civ.getTurn()+ 8 ,object.pAllies.id)	
		elseif randomNumber == 4 then
			delayedAction.doInFuture("genLeeseWounded",{},civ.getTurn()+ 9 ,object.pAllies.id)
		end	
		civ.ui.text(object.uGenLeese.name .. " was wounded in combat near (" .. text.coordinates(loserLocation) .. ") and is evacuated to a field hospital. His recovery is expected to take " .. 5 + randomNumber .. " weeks!")
		print(object.uGenLeese.name .. " wounded Random number is " .. randomNumber .. "!")
	end		

	-- Gen. McCreery
	if civ.getTurn() >= 1 and loser.type == object.uGenMcCreery then
		local randomNumber = math.random(4)
		
		if randomNumber == 1 then
			delayedAction.doInFuture("genMcCreeryWounded",{},civ.getTurn()+ 6 ,object.pAllies.id)
		elseif randomNumber == 2 then
			delayedAction.doInFuture("genMcCreeryWounded",{},civ.getTurn()+ 7 ,object.pAllies.id)
		elseif randomNumber == 3 then
			delayedAction.doInFuture("genMcCreeryWounded",{},civ.getTurn()+ 8 ,object.pAllies.id)	
		elseif randomNumber == 4 then
			delayedAction.doInFuture("genMcCreeryWounded",{},civ.getTurn()+ 9 ,object.pAllies.id)
		end	
		civ.ui.text(object.uGenMcCreery.name .. " was wounded in combat near (" .. text.coordinates(loserLocation) .. ") and is evacuated to a field hospital. His recovery is expected to take " .. 5 + randomNumber .. " weeks!")
		print(object.uGenMcCreery.name .. " wounded Random number is " .. randomNumber .. "!")
	end	

	-- Gen. Patton
	if civ.getTurn() >= 1 and loser.type == object.uGenPatton then
		local randomNumber = math.random(4)
		
		if randomNumber == 1 then
			delayedAction.doInFuture("genPattonWounded",{},civ.getTurn()+ 6 ,object.pAllies.id)
		elseif randomNumber == 2 then
			delayedAction.doInFuture("genPattonWounded",{},civ.getTurn()+ 7 ,object.pAllies.id)
		elseif randomNumber == 3 then
			delayedAction.doInFuture("genPattonWounded",{},civ.getTurn()+ 8 ,object.pAllies.id)	
		elseif randomNumber == 4 then
			delayedAction.doInFuture("genPattonWounded",{},civ.getTurn()+ 9 ,object.pAllies.id)
		end	
		civ.ui.text(object.uGenPatton.name .. " was wounded in combat near (" .. text.coordinates(loserLocation) .. ") and is evacuated to a field hospital. His recovery is expected to take " .. 5 + randomNumber .. " weeks!")
		print(object.uGenPatton.name .. " wounded Random number is " .. randomNumber .. "!")
	end	

	-- Gen. Clark
	if civ.getTurn() >= 1 and loser.type == object.uGenClark then
		local randomNumber = math.random(4)
		
		if randomNumber == 1 then
			delayedAction.doInFuture("genClarkWounded",{},civ.getTurn()+ 6 ,object.pAllies.id)
		elseif randomNumber == 2 then
			delayedAction.doInFuture("genClarkWounded",{},civ.getTurn()+ 7 ,object.pAllies.id)
		elseif randomNumber == 3 then
			delayedAction.doInFuture("genClarkWounded",{},civ.getTurn()+ 8 ,object.pAllies.id)	
		elseif randomNumber == 4 then
			delayedAction.doInFuture("genClarkWounded",{},civ.getTurn()+ 9 ,object.pAllies.id)
		end	
		civ.ui.text(object.uGenClark.name .. " was wounded in combat near (" .. text.coordinates(loserLocation) .. ") and is evacuated to a field hospital. His recovery is expected to take " .. 5 + randomNumber .. " weeks!")
		print(object.uGenClark.name .. " wounded Random number is " .. randomNumber .. "!")
	end		

	-- Gen. Truscott
	if civ.getTurn() >= 1 and loser.type == object.uGenTruscott then
		local randomNumber = math.random(4)
		
		if randomNumber == 1 then
			delayedAction.doInFuture("genTruscottWounded",{},civ.getTurn()+ 6 ,object.pAllies.id)
		elseif randomNumber == 2 then
			delayedAction.doInFuture("genTruscottWounded",{},civ.getTurn()+ 7 ,object.pAllies.id)
		elseif randomNumber == 3 then
			delayedAction.doInFuture("genTruscottWounded",{},civ.getTurn()+ 8 ,object.pAllies.id)	
		elseif randomNumber == 4 then
			delayedAction.doInFuture("genTruscottWounded",{},civ.getTurn()+ 9 ,object.pAllies.id)
		end	
		civ.ui.text(object.uGenTruscott.name .. " was wounded in combat near (" .. text.coordinates(loserLocation) .. ") and is evacuated to a field hospital. His recovery is expected to take " .. 5 + randomNumber .. " weeks!")
		print(object.uGenTruscott.name .. " wounded Random number is " .. randomNumber .. "!")
	end	
	
end -- END onUnitKilled.onUnitKilled ==============================
---&endAutoDoc

-- ==============================
-- DELAYED ACTION FUNCTIONS
-- ==============================

-- REBUILD GERMAN UNITS DESTROYED IN SICILY
-- SEE delayedAction.doInFuture("rebuildFallschirmjager", ABOVE as they are related when you want to delay an action
-- local function fallschirmjagerRebuilt(argTable) -- (written by Prof. Garfield)
	-- local placementLocations = {{91,93,0},{94,92,0},{88,94,0},{88,86,0},{90,90,0}}
	-- -- local placementLocations = {{50,168,0}} -- For testing purposes
	-- for key,tileCoord in pairs(placementLocations) do
		-- if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uFallschirmjager) then
			-- placementLocations[key] = nil
		-- end
	-- end
	-- gen.makeArrayOneToN(placementLocations) -- I'm pretty sure this line is unnecessary
	-- local unit = gen.createUnit(object.uFallschirmjager, object.pAxis,placementLocations , {randomize=false, veteran=true})[1]
	-- if unit then -- if the unit was created, it will be a Fallschirmjger, but we have to check the unit was actually created, and is not nil
		-- gen.setToNoOrders(unit)
		-- gen.setToFortified(unit)
		-- gen.placeFortress(unit.location)
		-- gen.moveSpent = 255
		-- unit.damage = unit.damage + 15
	-- end
-- end 
-- delayedAction.makeFunctionDelayable("rebuildFallschirmjager",fallschirmjagerRebuilt)

local function motInfanterieRebuilt(argTable)
	local unit = gen.createUnit(object.uMotInfanterie, object.pAxis, {{88,106,0},{111,107,0},{99,121,0},{92,104,0}}, {count=1, randomize=true, scatter=true, veteran=true})[1]
	-- local unit = gen.createUnit(object.uMotInfanterie, object.pAxis, {{50,168,0}}, {count=1, randomize=true, scatter=true, veteran=false})[1]  -- For testing purposes
	if unit then -- if the unit was created, it will be a MotInfanterie, but we have to check the unit was actually created, and is not nil
		unit.moveSpent = 255
		gen.setToNoOrders(unit)
		unit.damage = unit.damage + 5
	end
end 
delayedAction.makeFunctionDelayable("rebuildMotInfanterie",motInfanterieRebuilt)		

local function panzerGrenadierRebuilt(argTable)
	local unit = gen.createUnit(object.uPanzerGrenadier, object.pAxis, {{88,106,0},{111,107,0},{99,121,0},{92,104,0}}, {count=1, randomize=true, scatter=true, veteran=true})[1]
	--local unit = gen.createUnit(object.uPanzerGrenadier, object.pAxis, {{50,168,0}}, {count=1, randomize=true, scatter=true, veteran=false})[1]  -- For testing purposes
	if unit then -- if the unit was created, it will be a PanzerGrenadier, but we have to check the unit was actually created, and is not nil
		unit.moveSpent = 255
		gen.setToNoOrders(unit)
		unit.damage = unit.damage + 10
	end
end 
delayedAction.makeFunctionDelayable("rebuildPanzerGrenadier",panzerGrenadierRebuilt)	

local function sdkFz72AARebuilt(argTable)
	local unit = gen.createUnit(object.uSdkFz72AA, object.pAxis, {{88,106,0},{111,107,0},{88,86,0},{92,104,0}}, {count=1, randomize=true, scatter=true, veteran=true})[1]
	--local unit = gen.createUnit(object.uSdkFz72AA, object.pAxis, {{50,168,0}}, {count=1, randomize=true, scatter=true, veteran=false})[1]  -- For testing purposes
	if unit then -- if the unit was created, it will be a SdkFz72AA, but we have to check the unit was actually created, and is not nil
		unit.moveSpent = 255
		gen.setToNoOrders(unit)
		unit.damage = unit.damage + 10
	end
end 
delayedAction.makeFunctionDelayable("rebuildSdkFz72AA",sdkFz72AARebuilt)	

local function sdkFz231Rebuilt(argTable)
	local unit = gen.createUnit(object.uSdkFz231, object.pAxis, {{88,106,0},{111,107,0},{88,86,0},{92,104,0}}, {count=1, randomize=true, scatter=true, veteran=true})[1]
	--local unit = gen.createUnit(object.uSdkFz231, object.pAxis, {{50,168,0}}, {count=1, randomize=true, scatter=true, veteran=false})[1]  -- For testing purposes
	if unit then -- if the unit was created, it will be a SdkFz231, but we have to check the unit was actually created, and is not nil
		unit.moveSpent = 255
		gen.setToNoOrders(unit)
		unit.damage = unit.damage + 10
	end
end 
delayedAction.makeFunctionDelayable("rebuildSdkFz231",sdkFz231Rebuilt)	

local function stugIIIGRebuilt(argTable)
	local unit = gen.createUnit(object.uStugIIIG, object.pAxis, {{88,106,0},{111,107,0},{99,121,0},{92,104,0}}, {count=1, randomize=true, scatter=true, veteran=true})[1]
	-- local unit = gen.createUnit(object.uStugIIIG, object.pAxis, {{50,168,0}}, {count=1, randomize=true, scatter=true, veteran=false})[1]  -- For testing purposes
	if unit then -- if the unit was created, it will be a StugIIIG, but we have to check the unit was actually created, and is not nil
		unit.moveSpent = 255
		gen.setToNoOrders(unit)
		unit.damage = unit.damage + 10
	end
end 
delayedAction.makeFunctionDelayable("rebuildStugIIIG",stugIIIGRebuilt)	

local function pzkwIVFRebuilt(argTable)
	local unit = gen.createUnit(object.uPzkwIVF, object.pAxis, {{88,106,0},{111,107,0},{99,121,0},{92,104,0}}, {count=1, randomize=true, scatter=true, veteran=true})[1]
	-- local unit = gen.createUnit(object.uPzkwIVF, object.pAxis, {{50,168,0}}, {count=1, randomize=true, scatter=true, veteran=false})[1]  -- For testing purposes
	if unit then -- if the unit was created, it will be a PzkwIVF, but we have to check the unit was actually created, and is not nil
		unit.moveSpent = 255
		gen.setToNoOrders(unit)
		unit.damage = unit.damage + 10
	end
end 
delayedAction.makeFunctionDelayable("rebuildPzkwIVF",pzkwIVFRebuilt)	

local function pzkwVIERebuilt(argTable)
	local unit = gen.createUnit(object.uPzkwVIE, object.pAxis, {{88,106,0},{111,107,0},{88,86,0},{92,104,0}}, {count=1, randomize=true, scatter=true, veteran=true})[1]
	-- local unit = gen.createUnit(object.uPzkwVIE, object.pAxis, {{50,168,0}}, {count=1, randomize=true, scatter=true, veteran=false})[1]  -- For testing purposes
	if unit then -- if the unit was created, it will be a PzkwVIE, but we have to check the unit was actually created, and is not nil
		unit.moveSpent = 255
		gen.setToNoOrders(unit)
		unit.damage = unit.damage + 20
	end
end 
delayedAction.makeFunctionDelayable("rebuildPzkwVIE",pzkwVIERebuilt)	

-- REBUILD FALLSCHIRMJAGER DESTROYED ALONG GUSTAV LINE
-- SEE delayedAction.doInFuture("rebuildFallschirmjager_2", ABOVE as they are related when you want to delay an action
-- local function fallschirmjagerRebuilt_2(argTable) 
	-- local placementLocations = {{91,37,0},{106,44,0},{88,34,0},{106,48,0},{95,31,0},{103,39,0},{93,39,0},{107,37,0}}
	-- for key,tileCoord in pairs(placementLocations) do
		-- if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uFallschirmjager) then
			-- placementLocations[key] = nil
		-- end
	-- end
	-- gen.makeArrayOneToN(placementLocations) -- I'm pretty sure this line is unnecessary
	-- local unit = gen.createUnit(object.uFallschirmjager, object.pAxis,placementLocations , {randomize=true, veteran=true})[1]
	-- if unit then -- if the unit was created, it will be a Fallschirmjger, but we have to check the unit was actually created, and is not nil
		-- gen.setToNoOrders(unit)
		-- gen.setToFortified(unit)
		-- gen.placeFortress(unit.location)
		-- gen.moveSpent = 255
		-- unit.damage = unit.damage + 15
	-- end
-- end 
-- delayedAction.makeFunctionDelayable("rebuildFallschirmjager_2",fallschirmjagerRebuilt_2)

-- REBUILD RAILROAD YARDS
local function railroadyardRebuilt(argTable)
	-- local activeUnit = civ.getActiveUnit()
	local placementLocations = {{98,4,0},{106,14,0},{136,20,0},{84,2,0},{100,30,0},{89,39,0},{96,24,0},{75,1,0}}
	for key,tileCoord in pairs(placementLocations) do
		if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uRailroadYard) then
			placementLocations[key] = nil
		end
	end
	gen.makeArrayOneToN(placementLocations) -- I'm pretty sure this line is unnecessary
	gen.createUnit(object.uRailroadYard, object.pAxis,placementLocations , {randomize=true, veteran=true})--[1]
	text.simple("Allied intelligence have detected increased railroad\n^activity in northern Italy!\n^\r^The destruction of each railroad yard facility reduces\n^the Axis replacement rate!","New Railroad Yard Online!",object.mRailyard)

end 
delayedAction.makeFunctionDelayable("rebuildRailRoadYard",railroadyardRebuilt)	

-- REBUILD SS UNITS
local function motInfanterieSSRebuilt(argTable)
	local unit = gen.createUnit(object.uSSMotInfanterie, object.pAxis, {{80,76,0},{89,39,0},{100,30,0},{106,14,0}}, {count=1, randomize=false, veteran=false})
end 
delayedAction.makeFunctionDelayable("rebuildSSMotInfanterie",motInfanterieSSRebuilt)		

local function sdkFz72AASSRebuilt(argTable)
	local unit = gen.createUnit(object.uSSSdkFz72AA, object.pAxis, {{80,76,0},{89,39,0},{100,30,0},{106,14,0}}, {count=1, randomize=false, veteran=false})
end 
delayedAction.makeFunctionDelayable("rebuildSSSdkFz72AA",sdkFz72AASSRebuilt)

local function sdkFz2342SSRebuilt(argTable)
	local unit = gen.createUnit(object.uSSSdkFz2342, object.pAxis, {{80,76,0},{89,39,0},{100,30,0},{106,14,0}}, {count=1, randomize=false, veteran=false})
end 
delayedAction.makeFunctionDelayable("rebuildSSSdkFz2342",sdkFz2342SSRebuilt)	

local function pzkwIVSSRebuilt(argTable)
	local unit = gen.createUnit(object.uSSPzkwIV, object.pAxis, {{80,76,0},{89,39,0},{100,30,0},{106,14,0}}, {count=1, randomize=false, veteran=false})
end 
delayedAction.makeFunctionDelayable("rebuildSSPzkwIV",pzkwIVSSRebuilt)

local function hummelSSRebuilt(argTable)
	local unit = gen.createUnit(object.uSSHummel, object.pAxis, {{80,76,0},{89,39,0},{100,30,0},{106,14,0}}, {count=1, randomize=false, veteran=false})
end 
delayedAction.makeFunctionDelayable("rebuildSSHummel",hummelSSRebuilt)		

-- ALLIED GENERALS RETURN AFTER BEING WOUNDED
local function genMontgomeryReturns(argTable)
	if flag.value("General Montgomery Reassigned") == false then
		gen.createUnit(object.uGenMontgomery, object.pAllies, {{40,180,0},{59,169,0},{73,155,0},{120,138,0},{88,106,0}}, {count=1, randomize=false, veteran=false})
		civ.ui.text(object.uGenMontgomery.name .. " returns from convalescence to resume his command of the British 8th Army!")
	end
end 
delayedAction.makeFunctionDelayable("genMontgomeryWounded",genMontgomeryReturns)	

local function genLeeseReturns(argTable)
	if flag.value("General Leese Reassigned") == false then
		gen.createUnit(object.uGenLeese, object.pAllies, {{40,180,0},{59,169,0},{73,155,0},{120,138,0},{88,106,0}}, {count=1, randomize=false, veteran=false})
		civ.ui.text(object.uGenLeese.name .. " returns from convalescence to resume his command of the British 8th Army!")
	end
end 
delayedAction.makeFunctionDelayable("genLeeseWounded",genLeeseReturns)

local function genMcCreeryReturns(argTable)
	gen.createUnit(object.uGenMcCreery, object.pAllies, {{40,180,0},{59,169,0},{73,155,0},{120,138,0},{88,106,0}}, {count=1, randomize=false, veteran=false})
	civ.ui.text(object.uGenMcCreery.name .. " returns from convalescence to resume his command of the British 8th Army!")
end 
delayedAction.makeFunctionDelayable("genMcCreeryWounded",genMcCreeryReturns)


local function genPattonReturns(argTable)
	if flag.value("Patton dismissed") == false then
		gen.createUnit(object.uGenPatton, object.pAllies, {{0,136,0},{44,140,0},{88,106,0}}, {count=1, randomize=false, veteran=false})
		civ.ui.text(object.uGenPatton.name .. " returns from convalescence to take command of the American 5th Army!")
	end
end 
delayedAction.makeFunctionDelayable("genPattonWounded",genPattonReturns)

local function genClarkReturns(argTable)
	if flag.value("General Clark Reassigned") == false then
		gen.createUnit(object.uGenClark, object.pAllies, {{0,136,0},{44,140,0},{88,106,0}}, {count=1, randomize=false, veteran=false})
		civ.ui.text(object.uGenClark.name .. " returns from convalescence to take command of the American 5th Army!")
	end
end 
delayedAction.makeFunctionDelayable("genClarkWounded",genClarkReturns)

local function genTruscottReturns(argTable)
	gen.createUnit(object.uGenTruscott, object.pAllies, {{0,136,0},{44,140,0},{88,106,0}}, {count=1, randomize=false, veteran=false})
	civ.ui.text(object.uGenTruscott.name .. " returns from convalescence to take command of the American 5th Army!")
end 
delayedAction.makeFunctionDelayable("genTruscottWounded",genTruscottReturns)
 
 -- END DELAYED ACTIONS ==============================

return onUnitKilled
