

local versionNumber = 1
local fileModified = true -- set this to true if you change this file for your scenario
-- if another file requires this file, it checks the version number to ensure that the
-- version is recent enough to have all the expected functionality
-- if you set fileModified to true, the error generated if this file is out of date will
-- warn you that you've modified this file


local munitions = require("munitions")

local munitionSettings = {}
local object = require("object")
---@module "generalLibrary"
local gen = require("generalLibrary"):minVersion(1)
gen.versionFunctions(munitionSettings,versionNumber,fileModified,"MechanicsFiles".."\\".."munitionsSettings.lua")
local param = require("parameters")

-- This disables the workaround where air units
-- are left with 1 movement point, so they expend their range unit of range
-- (which wouldn't happen if their moveSpent was changed by event so they have
-- no more movement for the turn)
-- This is now corrected, but the option is enabled by default for backward compatibility
munitions.airCanHaveZeroMovement()


--specificationTable[unitType.id]={
--
-- goldCost = integer or nil
--      amount of gold it costs to generate a unit
--      absent means 0
-- minTreasury = integer or nil
--      minimum amount of gold in treasury to generate a unit
--      absent means refer to gold cost
--      (tribe will generate and be set to 0 if treasury is less than goldCost)
-- treasuryFailMessage = string or nil
--      A message to be displayed if the unit fails to spawn a unit due to an 
--      insufficient treasury
--      nil means no message

-- There are three ways to specify move costs, in full movement points,
-- in "atomic" movement points, and as a fraction of total movement points for the
-- unit type.  Use only one kind per unit type

-- moveCost = integer or nil
--      movement points to be expended generating the unit
--      "full" movement points
--      absent means 0
-- minMove = integer or nil
--      minimum remaining movement points to be allowed to generate a unit
--      "full" movement points
--      absent means any movement points for land/sea, 2 "atomic" for air units
-- postGenMinMove = integer or nil
--      a unit will be left with at least this many movement points after
--      the generation function
--      absent means 0 for land/sea, 1 "atomic" for air units
-- moveCostAtomic = integer or nil
--      movement points to be expended generating the unit
--      refers to the unit.moveSpent movement points
--      absent means 0
-- minMoveAtomic = integer or nil
--      minumum remaining movement points to be allowed to generate a unit
--      referes to the unit.moveSpent movement points
--      absent means any movement points for land, 2 "atomic" for air units
-- postGenMinMoveAtomic = integer or nil
--      a unit will be left with at least this many movement points after
--      the generation function
--      absent means 0 for land/sea, 1 "atomic" for air units (unless munitions.airCanHaveZeroMovement() is run)
-- moveCostFraction = number in [0,1] or nil
--      fraction of unit's total movement points expended generating the unit
--      round up to nearest "atomic" movement point
--      absent means 0
-- minMoveFraction = number in [0,1] or nil
--      fraction of unit's total movement points that must remain to be allowed
--      to generate a unit
--      absent means any movement points for land, 2 "atomic" for air units
--      round up to nearest "atomic" movement point
-- postGenMinMoveFraction = number in [0,1] or nil
--      a unit will be left with at least this many movement points after
--      the generation function, round up to nearest "atomic" move point
--      absent means 0 for land/sea, 1 "atomic" for air units(unless munitions.airCanHaveZeroMovement() is run)
-- roundFractionFull = bool or nil
--      fractional movement cost and minimum are rounded up to full movement point
--      instead of atomic movement point
--      nil/false means don't
-- roundFractionFullDown = bool or nil
--      fractional cost and minimum are rounded down to full move point
--      nil/false means don't
-- minMoveFailMessage = string or nil
--      a message to be displayed if a unit is not generated due to insufficient
--      movement points.
--      nil means no message

-- allowedTerrainTypes = table of integers or nil
--      a unit may only be generated if the tile it is standing on
--      corresponds to one of numbers in the table
--      nil means the unit can be generated on any terrain type
-- terrainTypeFailMessage = string or nil
--      message to be displayed if a unit is not generated due to standing
--      on the incorrect terrain

-- requiredTech = tech object or nil
--      the generating civ must have this technology in order to generate
--      the unit
--      nil means no requirement
-- techFailMessage = string or nil
--      message to be displayed if a unit is not generated due to not having
--      the correct technology

-- payload = boolean or nil
--      if true, unit must have a home city in order to generate munitions
--      and generating munitions sets the home city to NONE
-- payloadFailMessage = string or nil
--      message to be displayed if a unit is not generated due to the 
--      payload restriction
-- payloadRestrictionCheck = nil or function(unit)-->boolean
--      If function returns false, the home city is not valid for giving the
--      unit a payload.  This will be checked when the unit is activated, when
--      the unit is given a new home city with the 'h' key and when the unit
--      tries to generate a munition
--      nil means no restriction
-- payloadRestrictionMessage = nil or string
--      message to show if a unit fails the payloadRestrictionCheck
-- reArmingTileFunction = nil or function(tile)--> city or false
--      if function returns false for the tile the unit is on, the unit will not re-arm
--      if the function returns a city, the unit will rearm using that city as the home city
--      nil means it will attempt to re-arm in any city, and use that city as the home
--      as long as the city has extra support available
--      no effect if not a payload unit
-- noPayloadActivationMessage = nil or function(unit)-->string or nil
--      If this unit is activated without a home city, determine if a
--      message should be shown to warn the player.
--      No message will be shown if a unit is not a payload unit
--      Default is message shown when activated in a city
--      If you don't want a message under any circumstances, provide a function 
--      function(unit) return nil end
--      
--

-- canGenerateFunction = nil or function(unit)-->boolean 
--      This function applied to the generating unit must return true in order
--      for a unit to be spawned.  All other conditions still apply.
--      any failure message should be part of canGenerateFunction
--      absent means no extra conditions

-- generateUnitFunction = nil or function(unit)-->table of unit
--      This function creates the unit or units to be generated
--      and returns a table containing those units
--      Ignore any specifications prefixed with * if this is used
--      nil means use other specifications

--*generatedUnitType = unitType
--      The type of unit to be generated
--      can't be nil unless generateUnitFunction is used instead

--*giveVeteran = bool or nil
--      generated unit(s) given veteran status if true
--      nil or false means don't give vet status
--      if true, overrides copyVeteranStatus

--*copyVeteranStatus = bool or nil
--      generated unit(s) copy veteran status of generating unit if true
--      nil or false means don't give vet status

--*setHomeCityFunction = nil or function(generatingUnit)-->cityObject
--      determines what home city the spawned unit should have
--      nil means a home city of NONE

--*numberToGenerate = nil or number or thresholdTable or function(generatingUnit)-->number
--      if nil, generate 1 unit in all circumstances
--      if integer, generate that many units (generate 0 if number less than 0)
--      if number not integer, generate floor(number), and 1 more with
--      probability number-floor(number)
--      if thresholdTable, use remaining hp as the key, to get the number to create
--      if function, get the number as the returned value of the function 

-- activate = bool or nil
--      Activates one of the generated units if true.  If generateUnitFunction was used,
--      the unit at index 1 is activated, if index 1 has a value.  (if not, any unit in
--      the list might be chosen)
--      
--  successMessage = string or nil
--  message to show if a unit (or units) is created


-- This is the primary attack specification table
local primaryAttackTable = {}
--[[
primaryAttackTable[gen.original.uBomber.id] = {
    generatedUnitType = gen.original.uCruiseMsl,
    moveCost = 4,
}
--]]

	-- Allied Artillery unit types
	-- American
	primaryAttackTable[object.uM2A1105mmArt.id] = {goldCost = param.mm105ShellCost, minTreasury = param.mm105ShellCost, moveCost = 1,
		payload = false, 
		generatedUnitType = object.u105mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uM2A1105mmArt.name.." units can only fire "..object.u105mmShell.name.." if there are at least ".. param.mm105ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uM2A1105mmArt.name.." units can only fire if they have at least 1 movement points."}	

	primaryAttackTable[object.uM1155mmArt.id] = {goldCost = param.mm155ShellCost, minTreasury = param.mm155ShellCost, moveCost = 1,
		payload = false, 
		generatedUnitType = object.u155mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uM1155mmArt.name.." units can only fire "..object.u155mmShell.name.." if there are at least ".. param.mm155ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uM1155mmArt.name.." units can only fire if they have at least 1 movement points."}	

	primaryAttackTable[object.uUSBofors40mmAA.id] = {goldCost = param.LightFlakCost, minTreasury = param.LightFlakCost, moveCost = 1,
		payload = false, 
		generatedUnitType = object.uLightFlak, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uUSBofors40mmAA.name.." units can only fire "..object.uLightFlak.name.." if there are at least ".. param.LightFlakCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uUSBofors40mmAA.name.." units can only fire if they have at least 1 movement points."}	
		
	-- British
	primaryAttackTable[object.u25pdrMKIIArt.id] = {goldCost = param.mm105ShellCost, minTreasury = param.mm105ShellCost, moveCost = 1,
		payload = false, 
		generatedUnitType = object.u105mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.u25pdrMKIIArt.name.." units can only fire "..object.u105mmShell.name.." if there are at least ".. param.mm105ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.u25pdrMKIIArt.name.." units can only fire if they have at least 1 movement points."}	

	primaryAttackTable[object.u55inchArt.id] = {goldCost = param.mm155ShellCost, minTreasury = param.mm155ShellCost, moveCost = 1,
		payload = false, 
		generatedUnitType = object.u155mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.u55inchArt.name.." units can only fire "..object.u155mmShell.name.." if there are at least ".. param.mm155ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.u55inchArt.name.." units can only fire if they have at least 1 movement points."}	

	primaryAttackTable[object.uUKBofors40mmAA.id] = {goldCost = param.LightFlakCost, minTreasury = param.LightFlakCost, moveCost = 1,
		payload = false, 
		generatedUnitType = object.uLightFlak, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uUKBofors40mmAA.name.." units can only fire "..object.uLightFlak.name.." if there are at least ".. param.LightFlakCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uUKBofors40mmAA.name.." units can only fire if they have at least 1 movement points."}	
		
	-- Allied Minors
	primaryAttackTable[object.uIndian25pdrArt.id] = {goldCost = param.mm105ShellCost, minTreasury = param.mm105ShellCost, moveCost = 1,
		payload = false, 
		generatedUnitType = object.u105mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uIndian25pdrArt.name.." units can only fire "..object.u105mmShell.name.." if there are at least ".. param.mm105ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uIndian25pdrArt.name.." units can only fire if they have at least 1 movement points."}	
	
	primaryAttackTable[object.uPolish25pdrArt.id] = {goldCost = param.mm105ShellCost, minTreasury = param.mm105ShellCost, moveCost = 1,
		payload = false, 
		generatedUnitType = object.u105mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uPolish25pdrArt.name.." units can only fire "..object.u105mmShell.name.." if there are at least ".. param.mm105ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uPolish25pdrArt.name.." units can only fire if they have at least 1 movement points."}	
	
	primaryAttackTable[object.uFreeFrench105Art.id] = {goldCost = param.mm105ShellCost, minTreasury = param.mm105ShellCost, moveCost = 1,
		payload = false, 
		generatedUnitType = object.u105mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uFreeFrench105Art.name.." units can only fire "..object.u105mmShell.name.." if there are at least ".. param.mm105ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uFreeFrench105Art.name.." units can only fire if they have at least 1 movement points."}		
			
	-- Allied Mobile Artillery unit types
	-- American
	primaryAttackTable[object.uM7105mmHMC.id] = {goldCost = param.mm105ShellCost, minTreasury = param.mm105ShellCost, moveCost = 5,
		payload = false, 
		generatedUnitType = object.u105mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uM7105mmHMC.name.." units can only fire "..object.u105mmShell.name.." if there are at least ".. param.mm105ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uM7105mmHMC.name.." units can only fire if they have at least 1 movement points."}	

	primaryAttackTable[object.uM12155mmHMC.id] = {goldCost = param.mm155ShellCost, minTreasury = param.mm155ShellCost, moveCost = 4,
		payload = false, 
		generatedUnitType = object.u155mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uM12155mmHMC.name.." units can only fire "..object.u155mmShell.name.." if there are at least ".. param.mm155ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uM12155mmHMC.name.." units can only fire if they have at least 1 movement points."}	
	
	-- British
	primaryAttackTable[object.uBishop25pdr.id] = {goldCost = param.mm105ShellCost, minTreasury = param.mm105ShellCost, moveCost = 3,
		payload = false, 
		generatedUnitType = object.u105mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uBishop25pdr.name.." units can only fire "..object.u105mmShell.name.." if there are at least ".. param.mm105ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uBishop25pdr.name.." units can only fire if they have at least 1 movement points."}	
	
	-- Allied Minors
	primaryAttackTable[object.uCdnM7105mm.id] = {goldCost = param.mm105ShellCost, minTreasury = param.mm105ShellCost, moveCost = 5,
		payload = false, 
		generatedUnitType = object.u105mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uCdnM7105mm.name.." units can only fire "..object.u105mmShell.name.." if there are at least ".. param.mm105ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uCdnM7105mm.name.." units can only fire if they have at least 1 movement points."}	

	primaryAttackTable[object.uNZ25pdrArt.id] = {goldCost = param.mm105ShellCost, minTreasury = param.mm105ShellCost, moveCost = 5,
		payload = false, 
		generatedUnitType = object.u105mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uNZ25pdrArt.name.." units can only fire "..object.u105mmShell.name.." if there are at least ".. param.mm105ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uNZ25pdrArt.name.." units can only fire if they have at least 1 movement points."}	

	primaryAttackTable[object.uSASexton.id] = {goldCost = param.mm105ShellCost, minTreasury = param.mm105ShellCost, moveCost = 5,
		payload = false, 
		generatedUnitType = object.u105mmShell, giveVeteran = false, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uSASexton.name.." units can only fire "..object.u105mmShell.name.." if there are at least ".. param.mm105ShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uSASexton.name.." units can only fire if they have at least 1 movement points."}	
		
	-- Allied Naval unit types
	-- British
	primaryAttackTable[object.uQEClassBB.id] = {goldCost = param.HeavyShellCost, minTreasury = param.HeavyShellCost, moveCost = 10,
		payload = false, 
		generatedUnitType = object.uHeavyShell, giveVeteran = false, copyVeteranStatus = true, minMove = 3, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uQEClassBB.name.." units can only fire "..object.uHeavyShell.name.." if there are at least ".. param.HeavyShellCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uQEClassBB.name.." units can only fire if they have at least 3 movement points.",
		generatedUnitType = object.uHeavyShell, copyVeteranStatus = true, numberToGenerate = 1, activate = true}	

	-- Allied Bomber Air unit types
	-- American
	primaryAttackTable[object.uB17Fortress.id] = {goldCost = param.HeavyBombloadCost, minTreasury = param.HeavyBombloadCost, moveCost = 56, postGenMinMove = 0,
		payload = true, 
		payloadFailMessage = object.uB17Fortress.name.." units can only fire "..object.uHeavyBombload.name.." once per sortie.  Return to base to rearm.",
		treasuryFailMessage = object.uB17Fortress.name.." units can only fire "..object.uHeavyBombload.name.." if there are at least ".. param.HeavyBombloadCost .." supplies in your theater depots.",
		generatedUnitType = object.uHeavyBombload, copyVeteranStatus = true, numberToGenerate = 1, activate = true, reArmingTileFunction = nil}
		
	primaryAttackTable[object.uB24Liberator.id] = {goldCost = param.HeavyBombloadCost, minTreasury = param.HeavyBombloadCost, moveCost = 44, postGenMinMove = 0,
		payload = true, 
		payloadFailMessage = object.uB24Liberator.name.." units can only fire "..object.uHeavyBombload.name.." once per sortie.  Return to base to rearm.",
		treasuryFailMessage = object.uB24Liberator.name.." units can only fire "..object.uHeavyBombload.name.." if there are at least ".. param.HeavyBombloadCost .." supplies in your theater depots.",
		generatedUnitType = object.uHeavyBombload, copyVeteranStatus = true, numberToGenerate = 1, activate = true, reArmingTileFunction = nil}

	primaryAttackTable[object.uB25Mitchell.id] = {goldCost = param.MediumBombloadCost, minTreasury = param.MediumBombloadCost, moveCost = 38, postGenMinMove = 0,
		payload = true, 
		payloadFailMessage = object.uB25Mitchell.name.." units can only fire "..object.uMediumBombload.name.." once per sortie.  Return to base to rearm.",
		treasuryFailMessage = object.uB25Mitchell.name.." units can only fire "..object.uMediumBombload.name.." if there are at least ".. param.MediumBombloadCost .." supplies in your theater depots.",
		generatedUnitType = object.uMediumBombload, copyVeteranStatus = true, numberToGenerate = 1, activate = true, reArmingTileFunction = nil}
		
	primaryAttackTable[object.uB26Marauder.id] = {goldCost = param.MediumBombloadCost, minTreasury = param.MediumBombloadCost, moveCost = 32, postGenMinMove = 0,
		payload = true, 
		payloadFailMessage = object.uB26Marauder.name.." units can only fire "..object.uMediumBombload.name.." once per sortie.  Return to base to rearm.",
		treasuryFailMessage = object.uB26Marauder.name.." units can only fire "..object.uMediumBombload.name.." if there are at least ".. param.MediumBombloadCost .." supplies in your theater depots.",
		generatedUnitType = object.uMediumBombload, copyVeteranStatus = true, numberToGenerate = 1, activate = true, reArmingTileFunction = nil}

	primaryAttackTable[object.uA36Apache.id] = {goldCost = param.LightBombloadCost, minTreasury = param.LightBombloadCost, moveCost = 18, postGenMinMove = 0,
		payload = false, 
        generatedUnitType = object.uLightBombload, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true,
		minMoveFailMessage = object.uA36Apache.name.." units can only fire bombs if they have full (1) movement points.",
		treasuryFailMessage = object.uA36Apache.name.." units can only fire "..object.uLightBombload.name.." if there are at least ".. param.LightBombloadCost .." supplies in your theater depots."}

		
	-- British	
	primaryAttackTable[object.uBostonIII.id] = {goldCost = param.MediumBombloadCost, minTreasury = param.MediumBombloadCost, moveCost = 27, postGenMinMove = 0,
		payload = true, 
		payloadFailMessage = object.uBostonIII.name.." units can only fire "..object.uMediumBombload.name.." once per sortie.  Return to base to rearm.",
		treasuryFailMessage = object.uBostonIII.name.." units can only fire "..object.uMediumBombload.name.." if there are at least ".. param.MediumBombloadCost .." supplies in your theater depots.",
		generatedUnitType = object.uMediumBombload, copyVeteranStatus = true, numberToGenerate = 1, activate = true, reArmingTileFunction = nil}	

	primaryAttackTable[object.uMartinBaltimore.id] = {goldCost = param.LightBombloadCost, minTreasury = param.LightBombloadCost, moveCost = 30, postGenMinMove = 0,
		payload = true, 
		payloadFailMessage = object.uMartinBaltimore.name.." units can only fire "..object.uLightBombload.name.." once per sortie.  Return to base to rearm.",
		treasuryFailMessage = object.uMartinBaltimore.name.." units can only fire "..object.uLightBombload.name.." if there are at least ".. param.LightBombloadCost .." supplies in your theater depots.",
		generatedUnitType = object.uLightBombload, copyVeteranStatus = true, numberToGenerate = 1, activate = true, reArmingTileFunction = nil}

	primaryAttackTable[object.uVickersWellington.id] = {goldCost = param.MediumBombloadCost, minTreasury = param.MediumBombloadCost, moveCost = 60, postGenMinMove = 0,
		payload = true, 
		payloadFailMessage = object.uVickersWellington.name.." units can only fire "..object.uMediumBombload.name.." once per sortie.  Return to base to rearm.",
		treasuryFailMessage = object.uVickersWellington.name.." units can only fire "..object.uMediumBombload.name.." if there are at least ".. param.MediumBombloadCost .." supplies in your theater depots.",
		generatedUnitType = object.uMediumBombload, copyVeteranStatus = true, numberToGenerate = 1, activate = true, reArmingTileFunction = nil}
	
	primaryAttackTable[object.uHurricaneIIc.id] = {goldCost = param.LightBombloadCost, minTreasury = param.LightBombloadCost, moveCost = 18, postGenMinMove = 0,
		payload = false, 
        generatedUnitType = object.uLightBombload, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true,
		minMoveFailMessage = object.uHurricaneIIc.name.." units can only fire bombs if they have full (1) movement points.",
		treasuryFailMessage = object.uHurricaneIIc.name.." units can only fire "..object.uLightBombload.name.." if there are at least ".. param.LightBombloadCost .." supplies in your theater depots."}

	primaryAttackTable[object.uFaireyBarracuda.id] = {goldCost = param.LightBombloadCost, minTreasury = param.LightBombloadCost, moveCost = 28, postGenMinMove = 0,
		payload = false, 
        generatedUnitType = object.uLightBombload, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true,
		minMoveFailMessage = object.uFaireyBarracuda.name.." units can only fire bombs if they have full (1) movement points.",
		treasuryFailMessage = object.uFaireyBarracuda.name.." units can only fire "..object.uLightBombload.name.." if there are at least ".. param.LightBombloadCost .." supplies in your theater depots."}

	-- primaryAttackTable[object.uSunderlandIII.id] = {goldCost = param.LightBombloadCost, minTreasury = param.LightBombloadCost, moveCost = 44, postGenMinMove = 0,
		-- payload = false, 
        -- generatedUnitType = object.uLightBombload, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true,
		-- minMoveFailMessage = object.uSunderlandIII.name.." units can only fire bombs if they have full (1) movement points.",
		-- treasuryFailMessage = object.uSunderlandIII.name.." units can only fire "..object.uLightBombload.name.." if there are at least ".. param.LightBombloadCost .." supplies in your theater depots."}
	
	
	-- This is the secondary attack specification table
local secondaryAttackTable = {}
--[[
secondaryAttackTable[gen.original.uBomber.id] = {
    generatedUnitType = gen.original.uCruiseMsl,
    moveCost = 4,
}
--]]

	secondaryAttackTable[object.uFaireyBarracuda.id] = {goldCost = param.TorpedoCost, minTreasury = param.TorpedoCost, moveCost = 28, postGenMinMove = 0,
		payload = false, 
        generatedUnitType = object.uTorpedo, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true,
		minMoveFailMessage = object.uFaireyBarracuda.name.." units can only fire torpedoes if they have full (1) movement points.",
		treasuryFailMessage = object.uFaireyBarracuda.name.." units can only fire "..object.uTorpedo.name.." if there are at least ".. param.TorpedoCost .." supplies in your theater depots."}

	-- Allied Naval unit types
	-- British
	secondaryAttackTable[object.uTribalClassDD.id] = {goldCost = param.TorpedoCost, minTreasury = param.TorpedoCost, moveCost = 12,
		payload = false, 
		generatedUnitType = object.uTorpedo, giveVeteran = false, copyVeteranStatus = true, minMove = 3, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uTribalClassDD.name.." units can only fire "..object.uTorpedo.name.." if there are at least ".. param.TorpedoCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uTribalClassDD.name.." units can only fire if they have at least 3 movement points."}	
		
	secondaryAttackTable[object.uSClass.id] = {goldCost = param.TorpedoCost, minTreasury = param.TorpedoCost, moveCost = 6,
		payload = false, 
		generatedUnitType = object.uTorpedo, giveVeteran = false, copyVeteranStatus = true, minMove = 4, numberToGenerate = 1, activate = true, 
		treasuryFailMessage = object.uSClass.name.." units can only fire "..object.uTorpedo.name.." if there are at least ".. param.TorpedoCost .." supplies in your theater depots.",
		minMoveFailMessage = object.uSClass.name.." units can only fire if they have at least 4 movement points."}	

	secondaryAttackTable[object.uSunderlandIII.id] = {goldCost = param.LightBombloadCost, minTreasury = param.LightBombloadCost, moveCost = 50, postGenMinMove = 0,
		payload = false, 
        generatedUnitType = object.uLightBombload, copyVeteranStatus = true, minMove = 1, numberToGenerate = 1, activate = true,
		minMoveFailMessage = object.uSunderlandIII.name.." units can only fire bombs if they have full (1) movement points.",
		treasuryFailMessage = object.uSunderlandIII.name.." units can only fire "..object.uLightBombload.name.." if there are at least ".. param.LightBombloadCost .." supplies in your theater depots."}
	

local function primaryAttack(generatingUnit)
    return munitions.spawnUnit(generatingUnit,primaryAttackTable,gen.getActivationFunction())
end
munitionSettings.primaryAttack = primaryAttack

local function secondaryAttack(generatingUnit)
    return munitions.spawnUnit(generatingUnit,secondaryAttackTable,gen.getActivationFunction())
end
munitionSettings.secondaryAttack = secondaryAttack

local function activationReArm(unit)
    munitions.activationReArm(unit,primaryAttackTable,secondaryAttackTable)
end
munitionSettings.activationReArm = activationReArm


local function afterProductionReArm()
    munitions.afterProductionReArm(primaryAttackTable,secondaryAttackTable)
end
munitionSettings.afterProductionReArm = afterProductionReArm


local function payloadRestrictionCheck(carryingUnit)
    munitions.payloadRestrictionCheck(carryingUnit,primaryAttackTable)
    munitions.payloadRestrictionCheck(carryingUnit,secondaryAttackTable)
end
munitionSettings.payloadRestrictionCheck = payloadRestrictionCheck

local function keyPressPayloadRestrictionCheck(carryingUnit)
    civ.sleep(100)
    payloadRestrictionCheck(carryingUnit)
end
munitionSettings.keyPressPayloadRestrictionCheck = keyPressPayloadRestrictionCheck

local function onProdPayloadRestrictionCheck(carryingUnit)
    munitions.onProdPayloadRestrictionCheck(carryingUnit,primaryAttackTable)
    munitions.onProdPayloadRestrictionCheck(carryingUnit,secondaryAttackTable)
end
munitionSettings.onProdPayloadRestrictionCheck = onProdPayloadRestrictionCheck
local fileFound, discreteEvents = gen.requireIfAvailable("discreteEventsRegistrar") 
if fileFound then
    function discreteEvents.onActivateUnit(unit,source,rep)
        activationReArm(unit)
    end
    function discreteEvents.onAfterProduction(turn,tribe)
        afterProductionReArm()   
    end
    function discreteEvents.onCityProduction(city,prod)
        if civ.isUnit(prod) then
            onProdPayloadRestrictionCheck(prod)
        end
    end
end

return munitionSettings
