local gen = require("generalLibrary")
local param = require("parameters")
local object = require("object")
local text = require("text")
local func = require("functions")
local flag = require("flag")
local counter = require("counter")

local discreteEvents = require("discreteEventsRegistrar")

-- local gustavLineLocations  = {{80,76,0},{88,106,0},{107,83,0},{104,86,0},{88,96,0},{91,93,0},{100,86,0}}

-- ======================================================
-- TABLE for MUNITIONS TO BE DESTROYED AT BEGINNING OF EACH TURN
-- ======================================================
-- see onTurn.lua
-- see onTribeBeginTurn.lua
local munitionsToBeDestroyed = {}
munitionsToBeDestroyed[object.uHeavyShell.id] = 1 -- "Heavy Shell"
munitionsToBeDestroyed[object.u105mmShell.id] = 1 -- "105mm Shell"
munitionsToBeDestroyed[object.u155mmShell.id] = 1 -- "155mm Shell"
munitionsToBeDestroyed[object.uLightBombload.id] = 1 -- "Light Bombload"
munitionsToBeDestroyed[object.uMediumBombload.id] = 1 -- "Medium Bombload"
munitionsToBeDestroyed[object.uHeavyBombload.id] = 1 -- "Heavy Bombload"
munitionsToBeDestroyed[object.uLightFlak.id] = 1 -- "Light Flak"
munitionsToBeDestroyed[object.uHeavyFlak.id] = 1 -- "Heavy Flak"
munitionsToBeDestroyed[object.uMortarShells.id] = 1 -- "Small Arms"
munitionsToBeDestroyed[object.uTorpedo.id] = 1 -- "Torpedo"

-- see onTribeBeginTurn.lua
local alliedAirUnit = {}
alliedAirUnit[object.uB24Liberator.id] = true 
alliedAirUnit[object.uB25Mitchell.id] = true 
alliedAirUnit[object.uB26Marauder.id] = true 
alliedAirUnit[object.uB17Fortress.id] = true 
alliedAirUnit[object.uMartinBaltimore.id] = true 
alliedAirUnit[object.uVickersWellington.id] = true 
alliedAirUnit[object.uBostonIII.id] = true 

-- ======================================================
-- TABLE for Allied ground units that are eligible to recevive HP transfers
-- ======================================================
-- see consolidatedEvents.lua
-- UK Eligible
local ukEligibleUnitArt = {}
ukEligibleUnitArt[object.uUKBofors40mmAA.id] = true 
ukEligibleUnitArt[object.u25pdrMKIIArt.id] = true 
ukEligibleUnitArt[object.u55inchArt.id] = true 
ukEligibleUnitArt[object.u6pdrAT.id] = true 

local ukEligibleUnitInf = {}
ukEligibleUnitInf[object.uBritishVeteran.id] = true 
ukEligibleUnitInf[object.uBritishPIAT.id] = true 
ukEligibleUnitInf[object.uBritishMG.id] = true 
ukEligibleUnitInf[object.uCommandos.id] = true 
ukEligibleUnitInf[object.u1stAirborne.id] = true 
ukEligibleUnitInf[object.uBritishRegular.id] = true 

local ukEligibleUnitMot = {}
ukEligibleUnitMot[object.uBritishMotorTroops.id] = true 
ukEligibleUnitMot[object.uDaimlerAC.id] = true
ukEligibleUnitMot[object.uShermanIIaBde.id] = true
ukEligibleUnitMot[object.uChurchillMkIV.id] = true
ukEligibleUnitMot[object.uShermanVCFirefly.id] = true
ukEligibleUnitMot[object.uShermanIIa.id] = true 
ukEligibleUnitMot[object.uCromwellMkVII.id] = true 
ukEligibleUnitMot[object.uChurchillAVRE.id] = true 
ukEligibleUnitMot[object.uBishop25pdr.id] = true 
-- CW armor
ukEligibleUnitMot[object.uCdnShermanIII.id] = true 
ukEligibleUnitMot[object.uSAShermanIII.id] = true 
ukEligibleUnitMot[object.uPolishShermanIII.id] = true 
ukEligibleUnitMot[object.uNZShermanIII.id] = true 
-- CW mobile artillery
ukEligibleUnitMot[object.uCdnM7105mm.id] = true 
ukEligibleUnitMot[object.uSASexton.id] = true 

-- US Eligible
local usEligibleUnitArt = {}
usEligibleUnitArt[object.uM2A1105mmArt.id] = true
usEligibleUnitArt[object.uM1155mmArt.id] = true
usEligibleUnitArt[object.uM157mmAT.id] = true 

local usEligibleUnitInf = {}
usEligibleUnitInf[object.uUS1stInfantry.id] = true 
usEligibleUnitInf[object.uGIVeteran.id] = true 
usEligibleUnitInf[object.uGIRegular.id] = true 
usEligibleUnitInf[object.uUSBazooka.id] = true 
usEligibleUnitInf[object.uRanger.id] = true 
usEligibleUnitInf[object.u82ndAirborne.id] = true
usEligibleUnitInf[object.uUSBofors40mmAA.id] = true
usEligibleUnitInf[object.uUS10thMountain.id] = true 
usEligibleUnitInf[object.uM10Wolverine.id] = true 

local usEligibleUnitMot = {}
usEligibleUnitMot[object.uArmoredInfantry.id] = true 
usEligibleUnitMot[object.uM8Greyhound.id] = true 
usEligibleUnitMot[object.uM5Stuart.id] = true 
usEligibleUnitMot[object.uM4A1Sherman.id] = true
usEligibleUnitMot[object.uM4A1ShermanInd.id] = true  
usEligibleUnitMot[object.uM4A376WSherman.id] = true 
usEligibleUnitMot[object.uM10Wolverine.id] = true 
usEligibleUnitMot[object.uM7105mmHMC.id] = true
usEligibleUnitMot[object.uM12155mmHMC.id] = true
-- FF Armor
usEligibleUnitMot[object.uFreeFrenchM10.id] = true 

-- ======================================================
-- TABLE to check for Allied Artillery that is subject to supply rules
-- ======================================================
-- see keyPressSettings.lua
-- see onActivateUnit.lua
local alliedArtillery = {}
alliedArtillery[object.uM2A1105mmArt.id] = true
alliedArtillery[object.uM1155mmArt.id] = true
alliedArtillery[object.uM7105mmHMC.id] = true
alliedArtillery[object.uM12155mmHMC.id] = true
alliedArtillery[object.u25pdrMKIIArt.id] = true 
alliedArtillery[object.u55inchArt.id] = true 
alliedArtillery[object.uBishop25pdr.id] = true 
alliedArtillery[object.uCdnM7105mm.id] = true 
alliedArtillery[object.uSASexton.id] = true 
alliedArtillery[object.uIndian25pdrArt.id] = true
alliedArtillery[object.uPolish25pdrArt.id] = true
alliedArtillery[object.uFreeFrench105Art.id] = true 
alliedArtillery[object.uNZ25pdrArt.id] = true 

-- ======================================================
-- TABLE for Italian converting to PdG if Palermo or Catania are captured
-- ======================================================
-- onCityTaken.lua
local italiansToSurrenderSicily = {}
italiansToSurrenderSicily[object.uFanteriaVeterana.id] = 1 -- "Fanteria Italiana"
italiansToSurrenderSicily[object.uBersaglieri.id] = 1 -- "Bersaglieri"
italiansToSurrenderSicily[object.uReggimentoCostiero.id] = 1 -- "Divisione Costiera"
-- italiansToSurrenderSicily[object.uC200Saetta.id] = 1 -- "C.200 Saetta"
-- italiansToSurrenderSicily[object.uSM79Sparviero.id] = 1 -- "SM.79 Sparviero"
italiansToSurrenderSicily[object.uM1340.id] = 1 -- "M13/40"
italiansToSurrenderSicily[object.uSemovente7518.id] = 1 -- "Semovente 75/18"
italiansToSurrenderSicily[object.uAB41.id] = 1 -- "AB 41"
italiansToSurrenderSicily[object.u2065AA.id] = 1 -- "20/65 AA."
italiansToSurrenderSicily[object.u7546AA.id] = 1 -- "75/46 AA."
italiansToSurrenderSicily[object.u7527Art.id] = 1 -- "75/27 Art."
italiansToSurrenderSicily[object.u10528Art.id] = 1 -- "105/28 Art."
italiansToSurrenderSicily[object.u14940Art.id] = 1 -- "149/40 Art."
italiansToSurrenderSicily[object.uMarines.id] = 1 -- "Marines"
italiansToSurrenderSicily[object.uFanteriaMotorizzata.id] = 1 -- "Fanteria Motorizzata"

-- ======================================================
-- TABLES for preventing Italian units from moving after Sicily is conquered
-- ======================================================
-- see onActivateUnit.lua
local italian_VivaIlRe = {}
italian_VivaIlRe[object.uFanteriaVeterana.id] = true -- "Fanteria Italiana"
italian_VivaIlRe[object.uBersaglieri.id] = true -- "Bersaglieri"
italian_VivaIlRe[object.uReggimentoCostiero.id] = true -- "Divisione Costiera"
italian_VivaIlRe[object.uC200Saetta.id] = true -- "C.200 Saetta"
italian_VivaIlRe[object.uSM79Sparviero.id] = true -- "SM.79 Sparviero"
italian_VivaIlRe[object.uM1340.id] = true -- "M13/40"
italian_VivaIlRe[object.uSemovente7518.id] = true -- "Semovente 75/18"
italian_VivaIlRe[object.uAB41.id] = true -- "AB 41"
italian_VivaIlRe[object.u7546AA.id] = true -- "75/46 AA."
italian_VivaIlRe[object.u7527Art.id] = true -- "75/27 Art."
italian_VivaIlRe[object.u14940Art.id] = true -- "149/40 Art."
italian_VivaIlRe[object.uMarines.id] = true -- "Marines"
italian_VivaIlRe[object.uFanteriaMotorizzata.id] = true -- "Fanteria Motorizzata"
italian_VivaIlRe[object.u2065AA.id] = true -- "20/65 AA."
italian_VivaIlRe[object.uC202Folgore.id] = true -- "C.202 Folgore"
italian_VivaIlRe[object.uZ1007Alcione.id] = true -- "Z.1007 Alcione"
italian_VivaIlRe[object.u10528Art.id] = true -- "105/28 Art."

-- ======================================================
-- TABLES for Italian units to be destroyed upon Surrender event
-- ======================================================
-- see onTurn.lua
local italiansToBeDestroyed_Level1 = {}
italiansToBeDestroyed_Level1[object.uFanteriaVeterana.id] = 1 -- "Fanteria Italiana"
italiansToBeDestroyed_Level1[object.uBersaglieri.id] = 1 -- "Bersaglieri"
italiansToBeDestroyed_Level1[object.uReggimentoCostiero.id] = 1 -- "Divisione Costiera"
italiansToBeDestroyed_Level1[object.uPrigionieridiguerra.id] = 1 -- "Divisione Costiera"
italiansToBeDestroyed_Level1[object.uM1340.id] = 1 -- "M13/40"
italiansToBeDestroyed_Level1[object.uSemovente7518.id] = 1 -- "Semovente 75/18"
italiansToBeDestroyed_Level1[object.uAB41.id] = 1 -- "AB 41"
italiansToBeDestroyed_Level1[object.u7546AA.id] = 1 -- "75/46 AA."
italiansToBeDestroyed_Level1[object.u7527Art.id] = 1 -- "75/27 Art."
italiansToBeDestroyed_Level1[object.u14940Art.id] = 1 -- "149/40 Art."

local italiansToBeDestroyed_Level2 = {}
italiansToBeDestroyed_Level2[object.uMarines.id] = 1 -- "Marines"
italiansToBeDestroyed_Level2[object.uFanteriaMotorizzata.id] = 1 -- "Fanteria Motorizzata"
italiansToBeDestroyed_Level2[object.u2065AA.id] = 1 -- "20/65 AA."

local italiansToBeDestroyed_Level3 = {}
italiansToBeDestroyed_Level1[object.uC200Saetta.id] = 1 -- "C.200 Saetta"
italiansToBeDestroyed_Level3[object.uC202Folgore.id] = 1 -- "C.202 Folgore"
italiansToBeDestroyed_Level1[object.uSM79Sparviero.id] = 1 -- "SM.79 Sparviero"
italiansToBeDestroyed_Level3[object.uZ1007Alcione.id] = 1 -- "Z.1007 Alcione"
italiansToBeDestroyed_Level3[object.u10528Art.id] = 1 -- "105/28 Art."

local italianNavyToBeDestroyed = {}
italianNavyToBeDestroyed[object.uSellaClassDD.id] = 1 -- "Sella Class DD"
italianNavyToBeDestroyed[object.uGiusannoClassCL.id] = 1 -- "Giusanno CL"
italianNavyToBeDestroyed[object.uLittorioClassBB.id] = 1 -- "Littorio BB"
italianNavyToBeDestroyed[object.uSottomarino.id] = 1 -- "Sottomarino"

-- ======================================================
-- TABLES for Evacuating German units from Sicily
-- ======================================================
-- see onTurn.lua
local germansToBeEvacuatedSicily = {}
germansToBeEvacuatedSicily[object.uFallschirmjager.id] = 1
germansToBeEvacuatedSicily[object.uMotInfanterie.id] = 1
germansToBeEvacuatedSicily[object.uPanzerGrenadier.id] = 1
germansToBeEvacuatedSicily[object.uSdkFz231.id] = 1
germansToBeEvacuatedSicily[object.uSdkFz72AA.id] = 1
germansToBeEvacuatedSicily[object.uGrilleHsIG33.id] = 1
germansToBeEvacuatedSicily[object.uStugIIIG.id] = 1
germansToBeEvacuatedSicily[object.uPzkwIVF.id] = 1
germansToBeEvacuatedSicily[object.uPzkwVIE.id] = 1
germansToBeEvacuatedSicily[object.uBunker.id] = 1

local germansAirToBeEvacuated = {}
germansAirToBeEvacuated[object.uMe109G.id] = 1
germansAirToBeEvacuated[object.uFW190A.id] = 1
germansAirToBeEvacuated[object.uMe110.id] = 1
germansAirToBeEvacuated[object.uDo217E.id] = 1
germansAirToBeEvacuated[object.uJu88.id] = 1
germansAirToBeEvacuated[object.uEliteJagdgeschwader.id] = 1

-- ======================================================
-- POLYGON TABLES
-- ======================================================
-- Set coordinates of each reaction zone
-- see onTurn.lua
-- see consolidatedEvents.lua
local anzioPolygonCoordinates = {{76,76},{78,74},{79,75},{79,77},{81,79},{81,81},{83,83},{79,87},{78,86},{79,85},{77,83},{78,82},{77,81},{78,80},{76,78},doesNotCrossThisX=2}
local cassinoPolygonCoordinates = {{87,95},{92,90},{96,94},{91,99},{87,99},doesNotCrossThisX=2}
local cecinaPolygonCoordinates = {{76,36},{77,37},{79,37},{79,41},{74,46},{72,44},{75,41},{75,39},{76,38},doesNotCrossThisX=2}
local civitavecchiaPolygonCoordinates = {{73,59},{75,57},{77,59},{78,60},{73,59},{75,57},{78,60},{78,68},{76,70},{75,69},{75,67},{75,61},doesNotCrossThisX=2}
local fanoPolygonCoordinates = {{104,46},{106,44},{106,46},{107,47},{107,49},{109,51},{109,53},{110,54},{108,56},{104,52},doesNotCrossThisX=2}
local firenzePolygonCoordinates = {{80,34},{82,32},{92,32},{99,39},{99,43},{93,49},{87,49},{80,42},doesNotCrossThisX=2}
local firenzuolaPolygonCoordinates = {{89,35},{95,35},{95,41},{89,41},doesNotCrossThisX=0}
local foggiaPolygonCoordinates = {{98,106},{110,94},{111,95},{111,97},{117,103},{117,117},{109,117},doesNotCrossThisX=2}
local forliPolygonCoordinates = {{97,37},{98,36},{103,41},{103,45},{97,45},doesNotCrossThisX=2}
local massaPolygonCoordinates = {{78,24},{82,24},{82,32},{78,36},{76,34},{77,33},{77,31},{78,30},doesNotCrossThisX=2}
local napoliPolygonCoordinates = {{87,103},{88,102},{92,102},{95,105},{95,107},{92,110},{90,110},{90,108},{87,105},doesNotCrossThisX=2}
local ortonaPolygonCoordinates = {{101,91},{107,85},{107,89},{108,90},{105,93},{101,93},doesNotCrossThisX=2}
local pracchiaPolygonCoordinates = {{83,27},{86,24},{88,26},{88,36},{86,36},{83,33},doesNotCrossThisX=2}
local riminiPolygonCoordinates = {{104,42},{106,40},{106,46},{107,47},{107,49},{109,51},{107,53},{104,50},doesNotCrossThisX=2}
local salernoPolygonCoordinates = {{90,112},{91,111},{93,111},{95,109},{98,112},{98,120},{93,125},{90,122},{93,119},{93,113},{92,112},doesNotCrossThisX=2}
local terracinaPolygonCoordinates = {{80,90},{82,88},{86,92},{86,94},{84,94},doesNotCrossThisX=2}

-- Set coordinates for other zones
local corsicaPolygon = {{46,40},{48,40},{49,41},{51,41},{52,42},{52,44},{54,44},{57,41},{58,42},{52,48},{53,49},{49,53},{47,53},{43,57},{41,57},{40,58},{38,58},{38,56},{37,55},{37,53},{40,50},{40,44},{43,41},{45,41},doesNotCrossThisX=2}
-- local etnaPolygon = {{63,155},{65,157},{63,159},{61,157},doesNotCrossThisX=2} -- old polygon
local etnaPolygon = {{63,157},{64,158},{63,159},{62,160},{62,158},doesNotCrossThisX=2}

local sardiniaPolygon = {{21,59},{21,61},{22,62},{34,62},{36,64},{36,70},{35,71},{35,73},{29,79},{29,81},{25,85},{25,87},{18,94},{16,94},{16,92},{12,88},{11,89},{11,91},{9,93},{7,93},{7,91},{5,91},{6,90},{6,88},{5,87},{3,87},{6,86},{6,84},{4,84},{10,80},{12,80},{13,79},{13,75},{15,73},{15,71},{17,69},{17,67},{18,66},{18,64},{17,63},doesNotCrossThisX=0}
local sicilyPolygon = {{33,135},{37,135},{37,139},{41,139},{42,138},{44,138},{44,140},{45,141},{45,143},{48,146},{52,146},{56,150},{62,150},{63,151},{65,151},{67,153},{69,153},{70,152},{72,152},{74,154},{72,156},{70,156},{68,158},{66,158},{60,164},{60,170},{59,171},{57,171},{53,175},{46,168},{46,162},{45,161},{43,161},{39,157},{39,155},{38,154},{38,152},{36,150},{36,148},{29,141},{31,139},{31,137},doesNotCrossThisX=2}
local southernFrancePolygon = {{20,0},{48,0},{51,3},{51,5},{53,7},{52,8},{52,12},{49,9},{48,10},{46,8},{45,9},{44,8},{42,10},{41,9},{40,10},{38,8},{37,9},{36,8},{33,11},{31,9},{30,10},{28,8},{26,8},{26,6},{25,5},{23,5},{22,4},{24,4},{24,2},doesNotCrossThisX=2}
local southernItalyPolygon = {{94,124},{96,124},{104,116},{114,116},{115,117},{117,117},{122,124},{122,136},{103,155},{103,157},{101,159},{91,159},{88,162},{80,162},{79,163},{75,163},{74,162},{74,158},{75,157},{79,157},{80,156},{93,143},{94,142},doesNotCrossThisX=2}

local milanoPolygon = {{68,0},{78,0},{78,2},{75,5},{67,5},{67,1},doesNotCrossThisX=2}
local torinoPolygon = {{58,0},{66,0},{66,4},{62,8},{57,3},{57,1},doesNotCrossThisX=0}
local milanoTorinoPolygon = {{56,0},{80,0},{70,10},{56,10},doesNotCrossThisX=2}
local triestePolygon = {{134,28},{136,26},{137,27},{139,25},{139,39},{133,45},{131,45},{130,46},{129,45},{129,39},{131,37},{131,35},{132,34},{132,32},{133,33},{135,33},{136,32},{136,28},doesNotCrossThisX=2}

-- local gustavLinePolygon = {{86,94},{90,94},{93,91},{95,93},{100,88},{100,86},{104,88},{107,85},{107,83},{107,89},{108,90},{107,91},{105,91},{103,93},{99,93},{97,95},{93,95},{88,100},{86,98},{87,97},{87,95},doesNotCrossThisX=2}
local gustavLinePolygon = {{86,94},{89,91},{93,91},{95,93},{100,88},{100,86},{104,86},{107,83},{107,89},{108,90},{107,91},{105,91},{103,93},{99,93},{97,95},{93,95},{88,100},{86,98},{87,97},{87,95},doesNotCrossThisX=2}
-- local gothicLinePolygon = {{78,26},{82,30},{86,30},{93,37},{93,43},{98,48},{104,48},{106,46},{109,49},{109,51},{108,52},{102,52},{100,50},{96,50},{92,46},{92,40},{88,36},{84,36},{82,36},{77,31},{78,30},{103,47},{94,40},{94,42},{99,47},{91,41},{97,51},{99,51},{100,52},{101,47},doesNotCrossThisX=2}
-- local gothicLinePolygon = {{78,26},{82,30},{86,30},{93,37},{93,43},{98,48},{104,48},{106,46},{109,49},{109,51},{108,52},{102,52},{100,50},{96,50},{92,46},{92,42},{90,42},{90,38},{88,36},{84,36},{82,36},{77,31},{78,30},{103,47},{94,40},{94,42},{99,47},{91,41},{97,51},{99,51},{100,52},{101,47},doesNotCrossThisX=2}
local gothicLinePolygon = {{78,24},{82,24},{103,45},{105,45},{107,47},{107,49},{109,51},{108,52},{106,52},{105,51},{104,50},{98,50},{96,48},{92,48},{90,46},{90,40},{87,37},{86,38},{83,35},{82,36},{77,31},{78,30},doesNotCrossThisX=2}

local gothicLineTotalPolygon = {{76,22},{81,17},{104,40},{106,40},{106,46},{107,47},{107,51},{103,47},{99,47},{83,31},{79,31},{78,30},{78,24},doesNotCrossThisX=0}

local tenthArmyWestpolygon1 = {{79,81},{81,79},{92,90},{88,94},{86,92},{83,89},{83,85},doesNotCrossThisX=2}
local tenthArmyWestpolygon2 = {{88,96},{91,93},doesNotCrossThisX=2}
local italianSurrenderPolygon = {{78,24},{106,24},{106,46},{107,47},{107,49},{109,51},{109,53},{110,54},{110,56},{111,57},{111,65},{110,66},{110,68},{109,69},{109,71},{108,72},{108,74},{107,75},{107,89},{109,91},{109,93},{111,95},{111,97},{118,104},{120,104},{122,106},{122,108},{121,109},{115,109},{115,113},{117,115},{115,115},{113,113},{112,114},{111,113},{110,114},{108,112},{106,114},{105,113},{96,122},{95,121},{94,122},{92,122},{91,121},{93,119},{93,113},{92,112},{88,112},{87,111},{89,111},{90,110},{90,108},{88,106},{86,106},{84,106},{87,105},{87,99},{86,98},{87,97},{87,95},{86,94},{84,94},{80,90},{80,88},{78,86},{78,84},{77,83},{77,79},{76,78},{76,76},{77,75},{77,73},{76,72},{76,70},{75,69},{75,67},{76,66},{76,64},{75,63},{75,61},{72,58},{74,58},{74,56},{73,55},{73,51},{72,50},{73,49},{73,45},{72,44},{75,41},{75,39},{76,38},{76,36},{75,35},{76,34},{77,33},{77,31},{78,30},doesNotCrossThisX=0}
local italianSurrenderPolygon2 = {{62,2},{98,2},{107,11},{125,11},{137,23},{137,27},{131,27},{126,22},{118,22},{109,31},{107,31},{107,41},{106,42},{106,46},{107,47},{107,49},{109,51},{109,53},{110,54},{110,56},{111,57},{111,65},{110,66},{110,68},{109,69},{109,71},{108,72},{108,74},{107,75},{107,89},{109,91},{109,93},{111,95},{111,97},{118,104},{120,104},{122,106},{122,108},{121,109},{115,109},{115,113},{117,115},{115,115},{113,113},{112,114},{111,113},{110,114},{108,112},{106,114},{105,113},{96,122},{94,122},{92,122},{91,121},{93,119},{93,113},{92,112},{88,112},{87,111},{89,111},{90,110},{90,108},{87,105},{86,106},{84,106},{87,103},{87,99},{86,98},{87,97},{87,95},{86,94},{84,94},{80,90},{80,88},{78,86},{78,84},{77,83},{77,79},{76,78},{76,76},{77,75},{77,73},{76,72},{76,70},{75,69},{75,67},{76,66},{76,64},{75,63},{75,61},{72,58},{74,58},{74,56},{73,55},{73,51},{72,50},{73,49},{73,45},{72,44},{75,41},{75,39},{76,38},{76,36},{75,35},{77,33},{77,31},{78,30},{78,24},{76,22},{76,20},{75,19},{75,17},{73,15},{73,13},{70,10},{66,10},{62,6},doesNotCrossThisX=2} 
local poValleyPolygon = {{62,0},{98,0},{111,13},{137,13},{137,27},{131,27},{127,23},{117,23},{115,25},{111,25},{107,21},{79,21},{73,15},{73,13},{70,10},{62,10},{62,6},doesNotCrossThisX=2}

local talamonePocketpolygon = {{74,52},{76,52},{76,56},{74,56},doesNotCrossThisX=2}

local italySouthGustavLinePolygon = {{87,101},{91,101},{96,96},{100,96},{102,94},{104,94},{106,92},{108,92},{111,95},{111,97},{122,108},{121,109},{115,109},{115,113},{117,115},{117,117},{120,120},{120,122},{123,125},{123,127},{126,130},{126,134},{129,137},{129,139},{130,140},{130,142},{122,142},{120,140},{120,136},{118,134},{112,134},{105,141},{103,141},{102,142},{102,144},{104,146},{104,148},{105,149},{105,153},{103,155},{103,157},{101,159},{98,156},{92,156},{91,157},{91,159},{88,162},{84,162},{83,161},{81,161},{79,163},{75,163},{73,161},{73,159},{75,157},{79,157},{80,156},{82,156},{85,153},{89,153},{90,152},{90,150},{91,149},{91,147},{93,145},{93,143},{94,142},{94,136},{96,134},{96,130},{92,126},{92,120},{93,119},{93,113},{90,110},{90,108},{87,105},doesNotCrossThisX=2}

-- ======================================================
-- TABLE for list of towns associated to ocean BH tiles and Beachhead units
-- ======================================================
-- onEnterTile.lua
local townNames = {
	[gen.getTileID(78,84,0)] = "Anzio",
	[gen.getTileID(56,172,0)] = "Avola",
	[gen.getTileID(75,39,0)] = "Cecina",
	[gen.getTileID(75,67,0)] = "Civitavecchia",
	[gen.getTileID(103,157,0)] = "Crotone",
	[gen.getTileID(109,51,0)] = "Fano",
	[gen.getTileID(46,164,0)] = "Gela",
	[gen.getTileID(80,156,0)] = "Giola Tauro",
	[gen.getTileID(43,161,0)] = "Licata",
	[gen.getTileID(31,139,0)] = "Marsala",
	[gen.getTileID(79,27,0)] = "Massa",
	[gen.getTileID(126,130,0)] = "Monopoli",
	[gen.getTileID(41,139,0)] = "Monreale",
	[gen.getTileID(128,138,0)] = "Ostuni",
	[gen.getTileID(90,110,0)] = "Pompei",
	[gen.getTileID(88,154,0)] = "Pizzo",	
	[gen.getTileID(93,113,0)] = "Salerno",	
	[gen.getTileID(111,95,0)] = "Termoli",
	[gen.getTileID(82,92,0)] = "Terracina",	
}

-- ======================================================
-- TABLE for establishing fuel costs for moving ground units
-- ======================================================
-- onEnterTile.lua
local fuelReliantUnit = {}
fuelReliantUnit[object.uM4A1Sherman.id] = true

-- ======================================================
-- TABLE for excluding engineer and leaders from seasonal movement penalties
-- ======================================================
-- see onActivateUnit.lua
local excludeFromSeasonalTerrainPenalties = {}
excludeFromSeasonalTerrainPenalties[object.uEngineer.id] = true
excludeFromSeasonalTerrainPenalties[object.uDUKW.id] = true
-- excludeFromSeasonalTerrainPenalties[object.uUSTruck.id] = true
-- excludeFromSeasonalTerrainPenalties[object.uUKTruck.id] = true
-- excludeFromSeasonalTerrainPenalties[object.uPontoonBridge.id] = true
-- excludeFromSeasonalTerrainPenalties[object.uSupplyHub.id] = true
excludeFromSeasonalTerrainPenalties[object.uGenPatton.id] = true
excludeFromSeasonalTerrainPenalties[object.uGenClark.id] = true
excludeFromSeasonalTerrainPenalties[object.uGenTruscott.id] = true
excludeFromSeasonalTerrainPenalties[object.uGenMontgomery.id] = true
excludeFromSeasonalTerrainPenalties[object.uGenLeese.id] = true
excludeFromSeasonalTerrainPenalties[object.uGenMcCreery.id] = true

-- see onActivateUnit.lua
local mechanizedSeasonalTerrainPenalties = {}
mechanizedSeasonalTerrainPenalties[object.uM8Greyhound.id] = true
-- mechanizedSeasonalTerrainPenalties[object.uDUKW.id] = true
mechanizedSeasonalTerrainPenalties[object.uArmoredInfantry.id] = true
mechanizedSeasonalTerrainPenalties[object.uM5Stuart.id] = true
mechanizedSeasonalTerrainPenalties[object.uM4A1Sherman.id] = true
mechanizedSeasonalTerrainPenalties[object.uM4A376WSherman.id] = true
mechanizedSeasonalTerrainPenalties[object.uM10Wolverine.id] = true
mechanizedSeasonalTerrainPenalties[object.uM7105mmHMC.id] = true
mechanizedSeasonalTerrainPenalties[object.uM12155mmHMC.id] = true
mechanizedSeasonalTerrainPenalties[object.uUSTruck.id] = true
mechanizedSeasonalTerrainPenalties[object.uDaimlerAC.id] = true
mechanizedSeasonalTerrainPenalties[object.uShermanIIaBde.id] = true
mechanizedSeasonalTerrainPenalties[object.uChurchillMkIV.id] = true
mechanizedSeasonalTerrainPenalties[object.uShermanVCFirefly.id] = true
mechanizedSeasonalTerrainPenalties[object.uUKTruck.id] = true
mechanizedSeasonalTerrainPenalties[object.uBritishMotorTroops.id] = true
mechanizedSeasonalTerrainPenalties[object.uShermanIIa.id] = true
mechanizedSeasonalTerrainPenalties[object.uBishop25pdr.id] = true
mechanizedSeasonalTerrainPenalties[object.uCdnShermanIII.id] = true
mechanizedSeasonalTerrainPenalties[object.uCdnM7105mm.id] = true
mechanizedSeasonalTerrainPenalties[object.uCdnMotorTroops.id] = true
mechanizedSeasonalTerrainPenalties[object.uSAMotInf.id] = true
mechanizedSeasonalTerrainPenalties[object.uSAShermanIII.id] = true
mechanizedSeasonalTerrainPenalties[object.uSASexton.id] = true
mechanizedSeasonalTerrainPenalties[object.uSAMotorTroops.id] = true
mechanizedSeasonalTerrainPenalties[object.uPolishShermanIII.id] = true
mechanizedSeasonalTerrainPenalties[object.uFreeFrenchInfantry.id] = true
mechanizedSeasonalTerrainPenalties[object.uFreeFrenchM10.id] = true
mechanizedSeasonalTerrainPenalties[object.uNZMotInf.id] = true
mechanizedSeasonalTerrainPenalties[object.uNZShermanIII.id] = true
mechanizedSeasonalTerrainPenalties[object.uUS1stInfantry.id] = true
mechanizedSeasonalTerrainPenalties[object.uM4A1ShermanInd.id] = true
mechanizedSeasonalTerrainPenalties[object.uCromwellMkVII.id] = true
mechanizedSeasonalTerrainPenalties[object.uChurchillAVRE.id] = true

-- ======================================================
-- TABLE for checking units that don'thave impassable terrain flag
-- ======================================================
-- see onEnterTile.lua
local impassableTerrainUnit = {}
impassableTerrainUnit[object.uM8Greyhound.id] = true 
impassableTerrainUnit[object.uArmoredInfantry.id] = true 
impassableTerrainUnit[object.uM5Stuart.id] = true 
impassableTerrainUnit[object.uM4A1Sherman.id] = true 
impassableTerrainUnit[object.uM4A376WSherman.id] = true 
impassableTerrainUnit[object.uM10Wolverine.id] = true 
impassableTerrainUnit[object.uM7105mmHMC.id] = true 
impassableTerrainUnit[object.uM12155mmHMC.id] = true 
impassableTerrainUnit[object.uUSBofors40mmAA.id] = true 
impassableTerrainUnit[object.uM2A1105mmArt.id] = true
impassableTerrainUnit[object.uM1155mmArt.id] = true 
impassableTerrainUnit[object.uUSTruck.id] = true 
impassableTerrainUnit[object.uDaimlerAC.id] = true
impassableTerrainUnit[object.uShermanIIaBde.id] = true 
impassableTerrainUnit[object.uChurchillMkIV.id] = true 
impassableTerrainUnit[object.uShermanVCFirefly.id] = true
impassableTerrainUnit[object.uUKTruck.id] = true 
impassableTerrainUnit[object.uBritishMotorTroops.id] = true
impassableTerrainUnit[object.uShermanIIa.id] = true 
impassableTerrainUnit[object.uBishop25pdr.id] = true 
impassableTerrainUnit[object.uUKBofors40mmAA.id] = true
impassableTerrainUnit[object.u25pdrMKIIArt.id] = true
impassableTerrainUnit[object.u55inchArt.id] = true 
impassableTerrainUnit[object.uCdnMotInf.id] = true
impassableTerrainUnit[object.uCdnShermanIII.id] = true 
impassableTerrainUnit[object.uCdnM7105mm.id] = true 
impassableTerrainUnit[object.uCdnMotorTroops.id] = true
impassableTerrainUnit[object.uSAMotInf.id] = true
impassableTerrainUnit[object.uSAShermanIII.id] = true
impassableTerrainUnit[object.uSASexton.id] = true 
impassableTerrainUnit[object.uSAMotorTroops.id] = true 
impassableTerrainUnit[object.uIndian25pdrArt.id] = true
impassableTerrainUnit[object.uPolishShermanIII.id] = true
impassableTerrainUnit[object.uPolish25pdrArt.id] = true
impassableTerrainUnit[object.uFreeFrenchInfantry.id] = true
impassableTerrainUnit[object.uFreeFrench105Art.id] = true 
impassableTerrainUnit[object.uFreeFrenchM10.id] = true 
impassableTerrainUnit[object.uNZMotInf.id] = true
impassableTerrainUnit[object.uNZShermanIII.id] = true
impassableTerrainUnit[object.uNZ25pdrArt.id] = true 
impassableTerrainUnit[object.uM4A1ShermanInd.id] = true 
impassableTerrainUnit[object.uCromwellMkVII.id] = true 
impassableTerrainUnit[object.uChurchillAVRE.id] = true

-- ======================================================
-- TABLES for limiting unit stacking
-- ======================================================
-- see onEnterTile.lua

local terrainOccupancyLimit = {}
terrainOccupancyLimit[object.bBeach.type] = 5 -- 4
terrainOccupancyLimit[object.bRough.type] = 5 -- 4
terrainOccupancyLimit[object.bTemperateGrassland.type] = 6 -- 5
terrainOccupancyLimit[object.bForest.type] = 5 -- 4
terrainOccupancyLimit[object.bHills.type] = 5 --4
terrainOccupancyLimit[object.bApennines.type] = 3 -- 3
terrainOccupancyLimit[object.bGrassland.type] = 6 -- 5
terrainOccupancyLimit[object.bImpassable.type] = 1
terrainOccupancyLimit[object.bSwamp.type] = 3
terrainOccupancyLimit[object.bTownCenter.type] = 15 -- 12 -- or math.huge
terrainOccupancyLimit[object.bOcean.type] = 6 -- or math.huge
terrainOccupancyLimit[object.bRiverbed.type] = 4 -- 3
terrainOccupancyLimit[object.bTemperateHills.type] = 5 -- 4
terrainOccupancyLimit[object.bTemperateRough.type] = 5 -- 4
terrainOccupancyLimit[object.bTemperateRiverbed.type] = 4 -- 3
terrainOccupancyLimit[object.bRiverCrossing.type] = 5 -- 4

local excludeFromTerrainOccupancyLimit = {}
excludeFromTerrainOccupancyLimit[object.uEngineer.id] = true
excludeFromTerrainOccupancyLimit[object.uDUKW.id] = true
excludeFromTerrainOccupancyLimit[object.uRanger.id] = true
excludeFromTerrainOccupancyLimit[object.uUSTruck.id] = true
excludeFromTerrainOccupancyLimit[object.uCommandos.id] = true
excludeFromTerrainOccupancyLimit[object.uUKTruck.id] = true
excludeFromTerrainOccupancyLimit[object.uPontoonBridge.id] = true
excludeFromTerrainOccupancyLimit[object.uRadarStation.id] = true
excludeFromTerrainOccupancyLimit[object.uGurkhas.id] = true
excludeFromTerrainOccupancyLimit[object.uSupplyHub.id] = true
excludeFromTerrainOccupancyLimit[object.uGenPatton.id] = true
excludeFromTerrainOccupancyLimit[object.uGenClark.id] = true
excludeFromTerrainOccupancyLimit[object.uGenTruscott.id] = true
excludeFromTerrainOccupancyLimit[object.uGenMontgomery.id] = true
excludeFromTerrainOccupancyLimit[object.uGenLeese.id] = true
excludeFromTerrainOccupancyLimit[object.uGenMcCreery.id] = true

-- ======================================================
-- TABLES for limiting unit stacking
-- ======================================================
-- see onEnterTile.lua
local terrainTypes = {}
terrainTypes[object.bBeach.type] = true
terrainTypes[object.bRough.type] = true
terrainTypes[object.bTemperateGrassland.type] = true
terrainTypes[object.bForest.type] = true
terrainTypes[object.bHills.type] = true
terrainTypes[object.bApennines.type] = true
terrainTypes[object.bGrassland.type] = true
-- terrainTypes[object.bImpassable.type] = true
terrainTypes[object.bSwamp.type] = true
-- terrainTypes[object.bTownCenter.type] = true
-- terrainTypes[object.bOcean.type] = true
terrainTypes[object.bRiverbed.type] = true
terrainTypes[object.bTemperateHills.type] = true
terrainTypes[object.bTemperateRough.type] = true
terrainTypes[object.bTemperateRiverbed.type] = true
terrainTypes[object.bRiverCrossing.type] = true

-- ======================================================
-- TABLES for units eligible for leader bonuses
-- ======================================================
-- see onActivateUnit.lua

local ukEligibleUnit = {}
-- British
ukEligibleUnit[object.uBritishVeteran.id] = true 
ukEligibleUnit[object.uBritishRegular.id] = true 
ukEligibleUnit[object.uBritishPIAT.id] = true 
ukEligibleUnit[object.uBritishMG.id] = true 
ukEligibleUnit[object.uCommandos.id] = true 
ukEligibleUnit[object.u1stAirborne.id] = true 
ukEligibleUnit[object.uGurkhas.id] = true
ukEligibleUnit[object.uBritishMotorTroops.id] = true 
ukEligibleUnit[object.uShermanIIa.id] = true 
ukEligibleUnit[object.uShermanIIaBde.id] = true 
ukEligibleUnit[object.uChurchillMkIV.id] = true 
ukEligibleUnit[object.uShermanVCFirefly.id] = true 
-- Canadian
ukEligibleUnit[object.uCdnMotInf.id] = true 
ukEligibleUnit[object.uCdnShermanIII.id] = true    
ukEligibleUnit[object.uCdnMotorTroops.id] = true   
ukEligibleUnit[object.uCdnInfantry.id] = true 
-- Indian
ukEligibleUnit[object.uGurkhas.id] = true 
ukEligibleUnit[object.uIndianInfantry.id] = true 
ukEligibleUnit[object.u4thIndianDiv.id] = true 
-- New Zealand
ukEligibleUnit[object.uNZMotInf.id] = true 
ukEligibleUnit[object.uNZShermanIII.id] = true 
-- Polish
ukEligibleUnit[object.uPolishInfantry.id] = true 
ukEligibleUnit[object.uPolishShermanIII.id] = true 
-- South African
ukEligibleUnit[object.uSAMotInf.id] = true 
ukEligibleUnit[object.uSAShermanIII.id] = true 
ukEligibleUnit[object.uSAMotorTroops.id] = true  
-- Munitions    
ukEligibleUnit[object.u105mmShell.id] = true 
ukEligibleUnit[object.u155mmShell.id] = true 

local usEligibleUnit = {}
-- American
usEligibleUnit[object.uUS1stInfantry.id] = true 
usEligibleUnit[object.uGIVeteran.id] = true 
usEligibleUnit[object.uGIRegular.id] = true 
usEligibleUnit[object.uUSBazooka.id] = true 
usEligibleUnit[object.uRanger.id] = true 
usEligibleUnit[object.u82ndAirborne.id] = true 
usEligibleUnit[object.uUS10thMountain.id] = true 
usEligibleUnit[object.uArmoredInfantry.id] = true 
usEligibleUnit[object.uM5Stuart.id] = true 
usEligibleUnit[object.uM4A1Sherman.id] = true 
usEligibleUnit[object.uM4A1ShermanInd.id] = true 
usEligibleUnit[object.uM4A376WSherman.id] = true 
usEligibleUnit[object.uM10Wolverine.id] = true 
-- Free French
usEligibleUnit[object.uInfanterieColoniale.id] = true
usEligibleUnit[object.uGoumierMarocain.id] = true    
usEligibleUnit[object.uTirailleurAlgerien.id] = true 
usEligibleUnit[object.uFreeFrenchInfantry.id] = true 
usEligibleUnit[object.uFreeFrenchM10.id] = true 
-- Brazilian
usEligibleUnit[object.uBraz1stInf.id] = true
-- Munitions    
usEligibleUnit[object.u105mmShell.id] = true 
usEligibleUnit[object.u155mmShell.id] = true      

-- ======================================================
-- TABLE for removing US 2nd Armored Division after Sicilian conquest
-- ======================================================
-- see onCityTaken.lua
local usArmoredToBeRemoved = {}
usArmoredToBeRemoved[object.uArmoredInfantry.id] = 1 
usArmoredToBeRemoved[object.uM5Stuart.id] = 1 
usArmoredToBeRemoved[object.uM4A1Sherman.id] = 1 
usArmoredToBeRemoved[object.uM12155mmHMC.id] = 1 

-- ======================================================
-- TABLE for removing 1st FF Mechanized Division 
-- ======================================================
-- see onTribeBeginTurn.lua
local ffMechanizedToBeRemoved = {}
ffMechanizedToBeRemoved[object.uFreeFrenchM10.id] = 1 
ffMechanizedToBeRemoved[object.uFreeFrenchInfantry.id] = 1 
-- ffMechanizedToBeRemoved[object.uFreeFrench105Art.id] = 1 

-- ======================================================
-- TABLE for allowing Allied ground unit to be teleported to Corsica/Sardegna
-- ======================================================
-- see consolidatedEvents.lua
local alliedSardegnaUnit = {}
alliedSardegnaUnit[object.uGIVeteran.id] = true 
alliedSardegnaUnit[object.uGIRegular.id] = true 
alliedSardegnaUnit[object.uRanger.id] = true 
alliedSardegnaUnit[object.uGoumierMarocain.id] = true 

-- ======================================================
-- TABLE for resetting Widerstandsnest, Fallschirmjager, Bunker, Artillery, etc Type Units to Fortified Status
-- ======================================================
-- see onTribeTurnEnd.lua
local germanFortifiedStatus = {}
germanFortifiedStatus[object.uWiderstandsnest.id] = true
germanFortifiedStatus[object.uFallschirmjager.id] = true
germanFortifiedStatus[object.uBunker.id] = true
germanFortifiedStatus[object.uKasematte.id] = true
germanFortifiedStatus[object.uKustenbatterie.id] = true
germanFortifiedStatus[object.uKruppK5.id] = true
germanFortifiedStatus[object.u105cmleFH18.id] = true 
germanFortifiedStatus[object.u15cmsIG33.id] = true 
germanFortifiedStatus[object.u37cmFlak36.id] = true 
germanFortifiedStatus[object.u88cmFlak37.id] = true 

-- ======================================================
-- TABLES FOR TESTING of possible AI disbandment of units
-- ======================================================
-- see tables.lua

local italian_Air_Units = {}
italian_Air_Units[object.uC200Saetta.id] = 1 -- "C.200 Saetta"
italian_Air_Units[object.uC202Folgore.id] = 1 -- "C.202 Folgore"
italian_Air_Units[object.uSM79Sparviero.id] = 1 -- "SM.79 Sparviero"
italian_Air_Units[object.uZ1007Alcione.id] = 1 -- "Z.1007 Alcione"

local italian_Artillery_Units = {}
italian_Artillery_Units[object.uAB41.id] = 1 -- "AB 41"
italian_Artillery_Units[object.u2065AA.id] = 1 -- "20/65 AA."
italian_Artillery_Units[object.u7546AA.id] = 1 -- "75/46 AA."
italian_Artillery_Units[object.u7527Art.id] = 1 -- "75/27 Art."
italian_Artillery_Units[object.u10528Art.id] = 1 -- "105/28 Art."
italian_Artillery_Units[object.u14940Art.id] = 1 -- "149/40 Art."

local italian_Ground_Units = {}
italian_Ground_Units[object.uFanteriaVeterana.id] = 1 -- "Fanteria Italiana"
italian_Ground_Units[object.uBersaglieri.id] = 1 -- "Bersaglieri"
italian_Ground_Units[object.uMSVN.id] = 1 -- "MSVN"
italian_Ground_Units[object.uMarines.id] = 1 -- "Marines"
italian_Ground_Units[object.uParacadutisto.id] = 1 -- "Paracadutisto"
italian_Ground_Units[object.uReggimentoCostiero.id] = 1 -- "Divisione Costiera"
italian_Ground_Units[object.uPrigionieridiguerra.id] = 1 -- "Prigionieri di guerra"
italian_Ground_Units[object.uFanteriaMotorizzata.id] = 1 -- "Fanteria Motorizzata"
italian_Ground_Units[object.uM1340.id] = 1 -- "M13/40"
italian_Ground_Units[object.uSemovente7518.id] = 1 -- "Semovente 75/18"

local italian_Naval_Units = {}
italian_Naval_Units[object.uSellaClassDD.id] = 1 -- "Sella Class DD"
italian_Naval_Units[object.uGiusannoClassCL.id] = 1 -- "Giusanno CL"
italian_Naval_Units[object.uLittorioClassBB.id] = 1 -- "Littorio BB"
italian_Naval_Units[object.uSottomarino.id] = 1 -- "Sottomarino"

local german_Air_Units = {}
german_Air_Units[object.uMe109G.id] = 1
german_Air_Units[object.uFW190A.id] = 1
german_Air_Units[object.uMe110.id] = 1
german_Air_Units[object.uDo217E.id] = 1
german_Air_Units[object.uJu88.id] = 1
german_Air_Units[object.uEliteJagdgeschwader.id] = 1

local german_Artillery_Gun_Units = {}
german_Artillery_Gun_Units[object.u105cmleFH18.id] = 1 -- "105cm leFH18"
german_Artillery_Gun_Units[object.u15cmsIG33.id] = 1 -- "15cm sIG33"

local german_Artillery_AA_Units = {}
german_Artillery_AA_Units[object.u37cmFlak36.id] = 1 -- "37cm Flak36."
german_Artillery_AA_Units[object.u88cmFlak37.id] = 1 -- "88cm Flak37"

local german_Fortifications_Units = {}
german_Fortifications_Units[object.uWiderstandsnest.id] = 1
german_Fortifications_Units[object.uFallschirmjager.id] = 1
german_Fortifications_Units[object.uBunker.id] = 1
german_Fortifications_Units[object.uKasematte.id] = 1
german_Fortifications_Units[object.uKustenbatterie.id] = 1
german_Fortifications_Units[object.uKruppK5.id] = 1

local german_Ground_Armor_Units = {}
german_Ground_Armor_Units[object.uPzkwVG.id] = 1
german_Ground_Armor_Units[object.uPzkwVIE.id] = 1
german_Ground_Armor_Units[object.uGrilleHsIG33.id] = 1
german_Ground_Armor_Units[object.uStugIIIG.id] = 1
german_Ground_Armor_Units[object.uPzkwIVF.id] = 1
german_Ground_Armor_Units[object.uPzkwIVH.id] = 1
german_Ground_Armor_Units[object.uSdkFz231.id] = 1
german_Ground_Armor_Units[object.uSdkFz72AA.id] = 1

local german_Ground_Units = {}
german_Ground_Units[object.uInfanterieVeteranen.id] = 1
german_Ground_Units[object.uGebirgsjager.id] = 1
german_Ground_Units[object.uPanzerGrenadier.id] = 1
german_Ground_Units[object.uPanzerschreck.id] = 1
german_Ground_Units[object.uMotInfanterie.id] = 1

local german_Ground_SS_Units = {}
german_Ground_SS_Units[object.uSSSdkFz72AA.id] = 1
german_Ground_SS_Units[object.uSSPanzergrenadier.id] = 1
german_Ground_SS_Units[object.uSSPanther.id] = 1
german_Ground_SS_Units[object.uSSHummel.id] = 1
german_Ground_SS_Units[object.uSSPzkwIV.id] = 1
german_Ground_SS_Units[object.uSSMotInfanterie.id] = 1
german_Ground_SS_Units[object.uSSSdkFz2342.id] = 1

local german_Naval_Units = {}
german_Naval_Units[object.uTypeIXC.id] = 1 -- "Type IXC"

local german_Sardinia_Units = {}
german_Sardinia_Units[object.uMotInfanterie.id] = 1
german_Sardinia_Units[object.uStugIIIG.id] = 1
german_Sardinia_Units[object.uGrilleHsIG33.id] = 1
german_Sardinia_Units[object.uParacadutisto.id] = 1

local tables = {}
-- local register = {}


function tables.onTribeTurnBegin(turn,tribe)

	-- FOR GAME TESTING PURPOSES:
	-- Count number of Axis units at beginning and end of Tribe turn
	-- Reason: to try and ascertain if AI is disbanding some of its own units without a valid cause

	-- TO ACTIVATE AND USE THIS FUNCTION FROM THIS FILE YOU MUST DO THE FOLLOWING (otherwise use onTribeTurnBegin.lua):
	-- In events.lua 'local function doOnTribeTurnBegin(turn,tribe)' add this line = tables.onTribeTurnBegin(turn,tribe)
	
	if flag.value("Display Axis Testing Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
		text.setLinesPerWindow(21)
		if turn >= 2 and tribe == object.pAxis then
			local totalAxisUnits = 0
			local totalGermanAirUnits = 0
			local totalGermanNavalUnits = 0
			local totalGermanArmorUnits = 0
			local totalGermanArtilleryGunUnits = 0
			local totalGermanArtilleryAAUnits = 0
			local totalGermanFallschirmjagerUnits = 0
			local totalGermanWiderstandsnestUnits = 0
			local totalGermanFortificationUnits = 0
			local totalGermanGroundUnits = 0
			local totalGermanCorsicaGroundUnits = 0
			local totalGermanSardininaGroundUnits = 0
			
			local totalGermanGothicLineUnits = 0
			local totalGermanGoLKasematteUnits = 0
			local totalGermanGoLWiderstandsnestUnits = 0
			local totalGermanGoLFallschirmjagerUnits = 0
			local totalGermanGoLArtillery105GunUnits = 0
			local totalGermanGoLArtillery155GunUnits = 0
			local totalGermanGoLArtilleryAAUnits = 0
			
			local totalGermanGustavLineUnits = 0
			local totalGermanGLWiderstandsnestUnits = 0
			local totalGermanGLFallschirmjagerUnits = 0
			local totalGermanGLBunkerUnits = 0
			local totalGermanGLArtillery105GunUnits = 0
			local totalGermanGLArtillery155GunUnits = 0
			local totalGermanGLArtilleryAAUnits = 0
			
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis and unit.type ~= object.uSeaMines then
					totalAxisUnits = totalAxisUnits + 1
				end
				if unit.owner == object.pAxis and tables.german_Air_Units[unit.type.id] then
					totalGermanAirUnits = totalGermanAirUnits + 1
				end
				if unit.owner == object.pAxis and unit.type == object.uTypeIXC then
					totalGermanNavalUnits = totalGermanNavalUnits + 1
				end
				if unit.owner == object.pAxis and tables.german_Ground_Armor_Units[unit.type.id] then
					totalGermanArmorUnits = totalGermanArmorUnits + 1
				end
				if unit.owner == object.pAxis and tables.german_Artillery_Gun_Units[unit.type.id] then
					totalGermanArtilleryGunUnits = totalGermanArtilleryGunUnits + 1
				end
				if unit.owner == object.pAxis and tables.german_Artillery_AA_Units[unit.type.id] then
					totalGermanArtilleryAAUnits = totalGermanArtilleryAAUnits + 1
				end
				if unit.owner == object.pAxis and tables.german_Fortifications_Units[unit.type.id] then
					totalGermanFortificationUnits = totalGermanFortificationUnits + 1
				end
				if unit.owner == object.pAxis and unit.type == object.uFallschirmjager then
					totalGermanFallschirmjagerUnits = totalGermanFallschirmjagerUnits + 1
				end	
				if unit.owner == object.pAxis and unit.type == object.uWiderstandsnest then
					totalGermanWiderstandsnestUnits = totalGermanWiderstandsnestUnits + 1
				end
				if unit.owner == object.pAxis and tables.german_Ground_Units[unit.type.id] then
					totalGermanGroundUnits = totalGermanGroundUnits + 1
				end

				if unit.owner == object.pAxis and tables.german_Sardinia_Units[unit.type.id] and gen.inPolygon(unit.location,corsicaPolygon) then
					totalGermanCorsicaGroundUnits = totalGermanCorsicaGroundUnits + 1
				end 			
				if unit.owner == object.pAxis and tables.german_Sardinia_Units[unit.type.id] and gen.inPolygon(unit.location,sardiniaPolygon) then
					totalGermanSardininaGroundUnits = totalGermanSardininaGroundUnits + 1
				end 
				
				if unit.owner == object.pAxis and gen.inPolygon(unit.location,gustavLinePolygon) and flag.value("Gustav Line is breached") == false then
					if unit.type == object.uWiderstandsnest then
						totalGermanGLWiderstandsnestUnits = totalGermanGLWiderstandsnestUnits + 1
					end
					if unit.type == object.uFallschirmjager then
						totalGermanGLFallschirmjagerUnits = totalGermanGLFallschirmjagerUnits + 1
					end
					if unit.type == object.uBunker then
						totalGermanGLBunkerUnits = totalGermanGLBunkerUnits + 1
					end
					if unit.type == object.u105cmleFH18 then
						totalGermanGLArtillery105GunUnits = totalGermanGLArtillery105GunUnits + 1
					end
					if unit.type == object.u15cmsIG33 then
						totalGermanGLArtillery155GunUnits = totalGermanGLArtillery155GunUnits + 1
					end				
					if tables.german_Artillery_AA_Units[unit.type.id] then
						totalGermanGLArtilleryAAUnits = totalGermanGLArtilleryAAUnits + 1
					end	
					totalGermanGustavLineUnits = totalGermanGLWiderstandsnestUnits + totalGermanGLFallschirmjagerUnits + totalGermanGLBunkerUnits + totalGermanGLArtillery105GunUnits + totalGermanGLArtillery155GunUnits +totalGermanGLArtilleryAAUnits
					
				end 			

				if unit.owner == object.pAxis and gen.inPolygon(unit.location,gothicLineTotalPolygon) and flag.value("Gothic Line Construction Begun") == true then
					if unit.type == object.uKasematte then
						totalGermanGoLKasematteUnits = totalGermanGoLKasematteUnits + 1
					end
					if unit.type == object.uWiderstandsnest then
						totalGermanGoLWiderstandsnestUnits = totalGermanGoLWiderstandsnestUnits + 1
					end
					if unit.type == object.uFallschirmjager then
						totalGermanGoLFallschirmjagerUnits = totalGermanGoLFallschirmjagerUnits + 1
					end
					if unit.type == object.u105cmleFH18 then
						totalGermanGoLArtillery105GunUnits = totalGermanGoLArtillery105GunUnits + 1
					end
					if unit.type == object.u15cmsIG33 then
						totalGermanGoLArtillery155GunUnits = totalGermanGoLArtillery155GunUnits + 1
					end				
					if tables.german_Artillery_AA_Units[unit.type.id] then
						totalGermanGoLArtilleryAAUnits = totalGermanGoLArtilleryAAUnits + 1
					end	
					totalGermanGothicLineUnits = totalGermanGoLKasematteUnits + totalGermanGoLWiderstandsnestUnits + totalGermanGoLFallschirmjagerUnits + totalGermanGoLArtillery105GunUnits + totalGermanGoLArtillery155GunUnits +totalGermanGoLArtilleryAAUnits
					
				end 
				
			end	

			text.simple("AXIS TESTING MESSAGE:\n^Units on the map at the beginning of the Axis' turn!\n^\r^There are total of "..tostring(totalAxisUnits).." Axis units\n^There are "..tostring(totalGermanAirUnits).." German Air units\n^There are "..tostring(totalGermanNavalUnits).." German Naval units\n^There are "..tostring(totalGermanArmorUnits).." German Armor units\n^There are "..tostring(totalGermanArtilleryGunUnits).." German Artillery units\n^There are "..tostring(totalGermanArtilleryAAUnits).." German Flak units\n^There are "..tostring(totalGermanFallschirmjagerUnits).." German Fallschirmjager units\n^There are "..tostring(totalGermanWiderstandsnestUnits).." German Widerstandsnest units\n^There are "..tostring(totalGermanFortificationUnits).." German Fortification units\n^There are "..tostring(totalGermanGroundUnits).." German Infantry units\n^\r^There are "..tostring(totalGermanCorsicaGroundUnits).." German units in Corsica\n^There are "..tostring(totalGermanSardininaGroundUnits).." German units in Sardinia\n^\r^There are "..tostring(totalGermanGustavLineUnits).." German units in Gustav Line\n^There are "..tostring(totalGermanGLArtillery105GunUnits).." German 105 Artillery units\n^There are "..tostring(totalGermanGLArtillery155GunUnits).." German 155 Artillery units\n^There are "..tostring(totalGermanGLArtilleryAAUnits).." German Flak units\n^There are "..tostring(totalGermanGLFallschirmjagerUnits).." German Fallschirmjager units\n^There are "..tostring(totalGermanGLWiderstandsnestUnits).." German Widerstandsnest units\n^There are "..tostring(totalGermanGLBunkerUnits).." German Bunker units\n^\r^There are "..tostring(totalGermanGothicLineUnits).." German units in Gothic Line\n^There are "..tostring(totalGermanGoLKasematteUnits).." German Kasematte units\n^There are "..tostring(totalGermanGoLArtillery105GunUnits).." German 105 Artillery units\n^There are "..tostring(totalGermanGoLArtillery155GunUnits).." German 155 Artillery units\n^There are "..tostring(totalGermanGoLArtilleryAAUnits).." German Flak units\n^There are "..tostring(totalGermanGoLFallschirmjagerUnits).." German Fallschirmjager units\n^There are "..tostring(totalGermanGoLWiderstandsnestUnits).." German Widerstandsnest units","Total Axis Units Report!",object.m10thArmy)
		end	
		text.setLinesPerWindow(12)
	end
end


function tables.onTribeTurnEnd(turn,tribe)

	-- FOR GAME TESTING PURPOSES:
	-- Count number of Axis units at beginning and end of Tribe turn
	-- Reason: to try and ascertain if AI is disbanding some of its own units without a valid cause
	
	-- TO ACTIVATE AND USE THIS FUNCTION FROM THIS FILE YOU MUST DO THE FOLLOWING (otherwise use onTribeTurnEnd.lua):
	-- In events.lua 'local function doOnTribeTurnEnd(turn,tribe)' add this line = tables.onTribeTurnEnd(turn,tribe)
	
	if flag.value("Display Axis Testing Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
		text.setLinesPerWindow(21)
		if turn >= 2 and tribe == object.pAxis then
			local totalAxisUnits = 0
			local totalGermanAirUnits = 0
			local totalGermanNavalUnits = 0
			local totalGermanArmorUnits = 0
			local totalGermanArtilleryGunUnits = 0
			local totalGermanArtilleryAAUnits = 0
			local totalGermanFallschirmjagerUnits = 0
			local totalGermanWiderstandsnestUnits = 0
			local totalGermanFortificationUnits = 0
			local totalGermanGroundUnits = 0
			local totalGermanCorsicaGroundUnits = 0
			local totalGermanSardininaGroundUnits = 0

			local totalGermanGothicLineUnits = 0	
			local totalGermanGoLKasematteUnits = 0	
			local totalGermanGoLWiderstandsnestUnits = 0	
			local totalGermanGoLFallschirmjagerUnits = 0	
			local totalGermanGoLArtillery105GunUnits = 0	
			local totalGermanGoLArtillery155GunUnits = 0	
			local totalGermanGoLArtilleryAAUnits = 0	

			local totalGermanGustavLineUnits = 0
			local totalGermanGLWiderstandsnestUnits = 0
			local totalGermanGLFallschirmjagerUnits = 0
			local totalGermanGLBunkerUnits = 0
			local totalGermanGLArtillery105GunUnits = 0
			local totalGermanGLArtillery155GunUnits = 0
			local totalGermanGLArtilleryAAUnits = 0
			
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis and unit.type ~= object.uSeaMines then
					totalAxisUnits = totalAxisUnits + 1
				end
				if unit.owner == object.pAxis and tables.german_Air_Units[unit.type.id] then
					totalGermanAirUnits = totalGermanAirUnits + 1
				end
				if unit.owner == object.pAxis and unit.type == object.uTypeIXC then
					totalGermanNavalUnits = totalGermanNavalUnits + 1
				end				
				if unit.owner == object.pAxis and tables.german_Ground_Armor_Units[unit.type.id] then
					totalGermanArmorUnits = totalGermanArmorUnits + 1
				end
				if unit.owner == object.pAxis and tables.german_Artillery_Gun_Units[unit.type.id] then
					totalGermanArtilleryGunUnits = totalGermanArtilleryGunUnits + 1
				end
				if unit.owner == object.pAxis and tables.german_Artillery_AA_Units[unit.type.id] then
					totalGermanArtilleryAAUnits = totalGermanArtilleryAAUnits + 1
				end
				if unit.owner == object.pAxis and tables.german_Fortifications_Units[unit.type.id] then
					totalGermanFortificationUnits = totalGermanFortificationUnits + 1
				end
				if unit.owner == object.pAxis and unit.type == object.uFallschirmjager then
					totalGermanFallschirmjagerUnits = totalGermanFallschirmjagerUnits + 1
				end	
				if unit.owner == object.pAxis and unit.type == object.uWiderstandsnest then
					totalGermanWiderstandsnestUnits = totalGermanWiderstandsnestUnits + 1
				end
				if unit.owner == object.pAxis and tables.german_Ground_Units[unit.type.id] then
					totalGermanGroundUnits = totalGermanGroundUnits + 1
				end

				if unit.owner == object.pAxis and tables.german_Sardinia_Units[unit.type.id] and gen.inPolygon(unit.location,corsicaPolygon) then
					totalGermanCorsicaGroundUnits = totalGermanCorsicaGroundUnits + 1
				end 			
				if unit.owner == object.pAxis and tables.german_Sardinia_Units[unit.type.id] and gen.inPolygon(unit.location,sardiniaPolygon) then
					totalGermanSardininaGroundUnits = totalGermanSardininaGroundUnits + 1
				end 
				
				if unit.owner == object.pAxis and gen.inPolygon(unit.location,gustavLinePolygon) and flag.value("Gustav Line is breached") == false then
					if unit.type == object.uWiderstandsnest then
						totalGermanGLWiderstandsnestUnits = totalGermanGLWiderstandsnestUnits + 1
					end
					if unit.type == object.uFallschirmjager then
						totalGermanGLFallschirmjagerUnits = totalGermanGLFallschirmjagerUnits + 1
					end
					if unit.type == object.uBunker then
						totalGermanGLBunkerUnits = totalGermanGLBunkerUnits + 1
					end
					if unit.type == object.u105cmleFH18 then
						totalGermanGLArtillery105GunUnits = totalGermanGLArtillery105GunUnits + 1
					end
					if unit.type == object.u15cmsIG33 then
						totalGermanGLArtillery155GunUnits = totalGermanGLArtillery155GunUnits + 1
					end				
					if tables.german_Artillery_AA_Units[unit.type.id] then
						totalGermanGLArtilleryAAUnits = totalGermanGLArtilleryAAUnits + 1
					end	
					totalGermanGustavLineUnits = totalGermanGLWiderstandsnestUnits + totalGermanGLFallschirmjagerUnits + totalGermanGLBunkerUnits + totalGermanGLArtillery105GunUnits + totalGermanGLArtillery155GunUnits +totalGermanGLArtilleryAAUnits
					
				end 			
				
				if unit.owner == object.pAxis and gen.inPolygon(unit.location,gothicLineTotalPolygon) and flag.value("Gothic Line Construction Begun") == true then
					if unit.type == object.uKasematte then
						totalGermanGoLKasematteUnits = totalGermanGoLKasematteUnits + 1
					end
					if unit.type == object.uWiderstandsnest then
						totalGermanGoLWiderstandsnestUnits = totalGermanGoLWiderstandsnestUnits + 1
					end
					if unit.type == object.uFallschirmjager then
						totalGermanGoLFallschirmjagerUnits = totalGermanGoLFallschirmjagerUnits + 1
					end
					if unit.type == object.u105cmleFH18 then
						totalGermanGoLArtillery105GunUnits = totalGermanGoLArtillery105GunUnits + 1
					end
					if unit.type == object.u15cmsIG33 then
						totalGermanGoLArtillery155GunUnits = totalGermanGoLArtillery155GunUnits + 1
					end				
					if tables.german_Artillery_AA_Units[unit.type.id] then
						totalGermanGoLArtilleryAAUnits = totalGermanGoLArtilleryAAUnits + 1
					end	
					totalGermanGothicLineUnits = totalGermanGoLKasematteUnits + totalGermanGoLWiderstandsnestUnits + totalGermanGoLFallschirmjagerUnits + totalGermanGoLArtillery105GunUnits + totalGermanGoLArtillery155GunUnits +totalGermanGoLArtilleryAAUnits

				end 

			end	

			-- text.simple("AXIS TESTING MESSAGE:\n^Units on the map at the end of the Axis' turn!\n^\r^There are total of "..tostring(totalAxisUnits).." Axis units\n^There are "..tostring(totalGermanAirUnits).." German Air units\n^There are "..tostring(totalGermanNavalUnits).." German Naval units\n^There are "..tostring(totalGermanArmorUnits).." German Armor units\n^There are "..tostring(totalGermanArtilleryGunUnits).." German Artillery units\n^There are "..tostring(totalGermanArtilleryAAUnits).." German Flak units\n^There are "..tostring(totalGermanFallschirmjagerUnits).." German Fallschirmjager units\n^There are "..tostring(totalGermanWiderstandsnestUnits).." German Widerstandsnest units\n^There are "..tostring(totalGermanFortificationUnits).." German Fortification units\n^There are "..tostring(totalGermanGroundUnits).." German Infantry units\n^\r^There are "..tostring(totalGermanCorsicaGroundUnits).." German units in Corsica\n^There are "..tostring(totalGermanSardininaGroundUnits).." German units in Sardinia\n^\r^There are "..tostring(totalGermanGustavLineUnits).." German units in Gustav Line\n^There are "..tostring(totalGermanGLArtillery105GunUnits).." German 105 Artillery units\n^There are "..tostring(totalGermanGLArtillery155GunUnits).." German 155 Artillery units\n^There are "..tostring(totalGermanGLArtilleryAAUnits).." German Flak units\n^There are "..tostring(totalGermanGLFallschirmjagerUnits).." German Fallschirmjager units\n^There are "..tostring(totalGermanGLWiderstandsnestUnits).." German Widerstandsnest units","Total Axis Units Report!",object.m10thArmy)
			text.simple("AXIS TESTING MESSAGE:\n^Units on the map at the beginning of the Axis' turn!\n^\r^There are total of "..tostring(totalAxisUnits).." Axis units\n^There are "..tostring(totalGermanAirUnits).." German Air units\n^There are "..tostring(totalGermanNavalUnits).." German Naval units\n^There are "..tostring(totalGermanArmorUnits).." German Armor units\n^There are "..tostring(totalGermanArtilleryGunUnits).." German Artillery units\n^There are "..tostring(totalGermanArtilleryAAUnits).." German Flak units\n^There are "..tostring(totalGermanFallschirmjagerUnits).." German Fallschirmjager units\n^There are "..tostring(totalGermanWiderstandsnestUnits).." German Widerstandsnest units\n^There are "..tostring(totalGermanFortificationUnits).." German Fortification units\n^There are "..tostring(totalGermanGroundUnits).." German Infantry units\n^\r^There are "..tostring(totalGermanCorsicaGroundUnits).." German units in Corsica\n^There are "..tostring(totalGermanSardininaGroundUnits).." German units in Sardinia\n^\r^There are "..tostring(totalGermanGustavLineUnits).." German units in Gustav Line\n^There are "..tostring(totalGermanGLArtillery105GunUnits).." German 105 Artillery units\n^There are "..tostring(totalGermanGLArtillery155GunUnits).." German 155 Artillery units\n^There are "..tostring(totalGermanGLArtilleryAAUnits).." German Flak units\n^There are "..tostring(totalGermanGLFallschirmjagerUnits).." German Fallschirmjager units\n^There are "..tostring(totalGermanGLWiderstandsnestUnits).." German Widerstandsnest units\n^There are "..tostring(totalGermanGLBunkerUnits).." German Bunker units\n^\r^There are "..tostring(totalGermanGothicLineUnits).." German units in Gothic Line\n^There are "..tostring(totalGermanGoLKasematteUnits).." German Kasematte units\n^There are "..tostring(totalGermanGoLArtillery105GunUnits).." German 105 Artillery units\n^There are "..tostring(totalGermanGoLArtillery155GunUnits).." German 155 Artillery units\n^There are "..tostring(totalGermanGoLArtilleryAAUnits).." German Flak units\n^There are "..tostring(totalGermanGoLFallschirmjagerUnits).." German Fallschirmjager units\n^There are "..tostring(totalGermanGoLWiderstandsnestUnits).." German Widerstandsnest units","Total Axis Units Report!",object.m10thArmy)
		end	
		text.setLinesPerWindow(12)
	end
end

-- ========================================================
-- REQUIRED SECTION TO BE ABLE TO USE TABLES IN OTHER FILES
-- ========================================================

tables.munitionsToBeDestroyed = munitionsToBeDestroyed
tables.alliedAirUnit = alliedAirUnit

tables.ukEligibleUnitArt = ukEligibleUnitArt
tables.ukEligibleUnitInf = ukEligibleUnitInf
tables.ukEligibleUnitMot = ukEligibleUnitMot
tables.usEligibleUnitArt = usEligibleUnitArt
tables.usEligibleUnitInf = usEligibleUnitInf
tables.usEligibleUnitMot = usEligibleUnitMot

tables.alliedArtillery = alliedArtillery

tables.italiansToSurrenderSicily = italiansToSurrenderSicily
tables.italian_VivaIlRe = italian_VivaIlRe

tables.italiansToBeDestroyed_Level1 = italiansToBeDestroyed_Level1
tables.italiansToBeDestroyed_Level2 = italiansToBeDestroyed_Level2
tables.italiansToBeDestroyed_Level3 = italiansToBeDestroyed_Level3
tables.italianNavyToBeDestroyed = italianNavyToBeDestroyed

tables.germansToBeEvacuatedSicily = germansToBeEvacuatedSicily
tables.germansAirToBeEvacuated = germansAirToBeEvacuated
tables.germanFortifiedStatus = germanFortifiedStatus

tables.alliedSardegnaUnit = alliedSardegnaUnit

tables.townNames = townNames
tables.fuelReliantUnit = fuelReliantUnit

tables.excludeFromSeasonalTerrainPenalties = excludeFromSeasonalTerrainPenalties
tables.mechanizedSeasonalTerrainPenalties = mechanizedSeasonalTerrainPenalties

tables.impassableTerrainUnit = impassableTerrainUnit
tables.terrainOccupancyLimit = terrainOccupancyLimit
tables.excludeFromTerrainOccupancyLimit = excludeFromTerrainOccupancyLimit
tables.terrainTypes = terrainTypes

tables.ukEligibleUnit = ukEligibleUnit
tables.usEligibleUnit = usEligibleUnit

tables.usArmoredToBeRemoved = usArmoredToBeRemoved
tables.ffMechanizedToBeRemoved = ffMechanizedToBeRemoved

tables.anzioPolygonCoordinates = anzioPolygonCoordinates
tables.cassinoPolygonCoordinates = cassinoPolygonCoordinates
tables.cecinaPolygonCoordinates = cecinaPolygonCoordinates
tables.civitavecchiaPolygonCoordinates = civitavecchiaPolygonCoordinates
tables.fanoPolygonCoordinates = fanoPolygonCoordinates
tables.firenzePolygonCoordinates = firenzePolygonCoordinates
tables.firenzuolaPolygonCoordinates = firenzuolaPolygonCoordinates
tables.foggiaPolygonCoordinates = foggiaPolygonCoordinates
tables.forliPolygonCoordinates = forliPolygonCoordinates
tables.massaPolygonCoordinates = massaPolygonCoordinates
tables.napoliPolygonCoordinates = napoliPolygonCoordinates
tables.ortonaPolygonCoordinates = ortonaPolygonCoordinates
tables.pracchiaPolygonCoordinates = pracchiaPolygonCoordinates
tables.riminiPolygonCoordinates = riminiPolygonCoordinates
tables.salernoPolygonCoordinates = salernoPolygonCoordinates
tables.terracinaPolygonCoordinates = terracinaPolygonCoordinates

tables.corsicaPolygon = corsicaPolygon
tables.etnaPolygon = etnaPolygon
tables.sardiniaPolygon = sardiniaPolygon
tables.sicilyPolygon = sicilyPolygon
tables.southernFrancePolygon = southernFrancePolygon
tables.southernItalyPolygon = southernItalyPolygon

tables.milanoPolygon = milanoPolygon
tables.torinoPolygon = torinoPolygon
tables.milanoTorinoPolygon = milanoTorinoPolygon
tables.triestePolygon = triestePolygon

tables.gustavLinePolygon = gustavLinePolygon
tables.gothicLinePolygon = gothicLinePolygon
tables.gothicLineTotalPolygon = gothicLineTotalPolygon
tables.tenthArmyWestpolygon1 = tenthArmyWestpolygon1
tables.tenthArmyWestpolygon2 = tenthArmyWestpolygon2
tables.talamonePocketpolygon = talamonePocketpolygon

tables.italianSurrenderPolygon = italianSurrenderPolygon
tables.italianSurrenderPolygon2 = italianSurrenderPolygon2
tables.italySouthGustavLinePolygon = italySouthGustavLinePolygon
tables.poValleyPolygon = poValleyPolygon

tables.italian_Air_Units = italian_Air_Units
tables.italian_Artillery_Units = italian_Artillery_Units
tables.italian_Ground_Units = italian_Ground_Units
tables.italian_Naval_Units = italian_Naval_Units

tables.german_Air_Units = german_Air_Units
tables.german_Artillery_Gun_Units = german_Artillery_Gun_Units
tables.german_Artillery_AA_Units = german_Artillery_AA_Units
tables.german_Fortifications_Units = german_Fortifications_Units
tables.german_Ground_Units = german_Ground_Units
tables.german_Ground_Armor_Units = german_Ground_Armor_Units
tables.german_Ground_SS_Units = german_Ground_SS_Units
tables.german_Naval_Units = german_Naval_Units

tables.german_Sardinia_Units = german_Sardinia_Units

return tables
-- return register