
local gen = require("generalLibrary")
local param = require("parameters")
local object = require("object")
local text = require("text")
local func = require("functions")
local flag = require("flag")
local counter = require("counter")
local tables = require("tables")

local delayedAction = require("delayedAction")

local civlua = require("civluaModified")
local tileData = require("tileData")
tileData.defineFlag("riggedBridge")
tileData.defineFlag("placedMinenfeld")


-- local procurement = require("procurement")

-- CALENDAR FUNCTION - Begin ====================================
local yearName = { 
[0] = "1943",
[1] = "1944",
[2] = "1945"
}

local monthName = { 
[0] = "January",
[1] = "February",
[2] = "March",
[3] = "April",
[4] = "May",
[5] = "June",
[6] = "July",
[7] = "August",
[8] = "September",
[9] = "October",
[10] = "November",
[11] = "December"
}

local weekName = { 
[0] = "Week I",
[1] = "Week II",
[2] = "Week III",
[3] = "Week IV"
}

local seasonName = { 
[0] = "Winter", -- "January"
[1] = "Winter", -- "February"
[2] = "Spring", -- "March"
[3] = "Summer", -- "April"
[4] = "Summer", -- "May"
[5] = "Summer", -- "June"
[6] = "Summer", -- "July"
[7] = "Summer", -- "August"
[8] = "Summer", -- "September"
[9] = "Fall", -- "October"
[10] = "Fall", -- "November"
[11] = "Winter", -- "December"
}

local function getYearNumber (alternateTurn)
    local turn = alternateTurn or civ.getTurn()
	-- 1943
	if turn <= 24 then
		return 0
	-- 1944
	elseif turn <= 72 then
		return 1
	-- 1945
	else
		return 2
	end
end

local function getMonthNumber (alternateTurn)
	local turn = alternateTurn or civ.getTurn()
	-- January
	if (turn >= 25 and turn <= 28) or (turn >= 73 and turn <= 76) then
		return 0
	-- February
	elseif (turn >= 29 and turn <= 32) or (turn >= 77 and turn <= 80) then
		return 1
	-- March
	elseif (turn >= 33 and turn <= 36) or (turn >= 81 and turn <= 84) then
		return 2
	-- April
	elseif (turn >= 37 and turn <= 40) or (turn >= 85 and turn <= 88) then
		return 3
	-- May
	elseif (turn >= 41 and turn <= 44) or (turn >= 89 and turn <= 92) then
		return 4	
	-- June
	elseif (turn >= 45 and turn <= 48) then
		return 5
	-- July
	elseif (turn >= 1 and turn <=4) or (turn >= 49 and turn <=52) then
		return 6
	-- August
	elseif (turn >= 5 and turn <= 8) or (turn >= 53 and turn <= 56) then
		return 7
	-- September
	elseif (turn >= 9 and turn <= 12) or (turn >= 57 and turn <= 60) then
		return 8
	-- October
	elseif (turn >= 13 and turn <= 16) or (turn >= 61 and turn <= 64) then
		return 9
	-- November
	elseif (turn >= 17 and turn <= 20) or (turn >= 65 and turn <= 68) then
		return 10
	-- December
	elseif (turn >= 21 and turn <= 24) or (turn >= 69 and turn <= 72) then
		return 11		
	end
end

-- Each table corresponds to the turn number under which the Week I, II, III and IV are set to display
local weekOneNumbers = {1,5,9,13,17,21,25,29,33,37,41,45,49,53,57,61,65,69,73,77,81,85,89}
local weekTwoNumbers = {2,6,10,14,18,22,26,30,34,38,42,46,50,54,58,62,66,70,74,78,82,86,90}
local weekThreeNumbers = {3,7,11,15,19,23,27,31,35,39,43,47,51,55,59,63,67,71,75,79,83,87}
local weekFourNumbers = {4,8,12,16,20,24,28,32,36,40,44,48,52,56,60,64,68,72,76,80,84,88}

-- Function used to check if turn # is in week table above
local function contains(table, val)
   for i=1,#table do
      if table[i] == val then 
         return true
      end
   end
   return false
end

local function getWeekNumber (alternateTurn)
	local turn =alternateTurn or civ.getTurn()
	-- WEEK I
	if contains(weekOneNumbers, turn) then
		return 0
	-- WEEK II
	elseif contains(weekTwoNumbers, turn) then
		return 1
	-- WEEK III
	elseif contains(weekThreeNumbers, turn) then
		return 2
	-- WEEK IV
	elseif contains(weekFourNumbers, turn) then 
		return 3
	end
end

-- Use this code if you don't need to call the displayWeekMonthYear from another lua file
-- local function displayWeekMonthYear ()
	-- return weekName[getWeekNumber()] .. " of " .. monthName[getMonthNumber()] .. ", " .. yearName[getYearNumber()] 
-- end

-- Use this code if you need to call displayWeekMonthYear in the consolidatedEvents.lua file
local onTurn = {}
-- function onTurn.displayWeekMonthYear ()
	-- return weekName[getWeekNumber()] .. " of " .. monthName[getMonthNumber()] .. ", " .. yearName[getYearNumber()] 
-- end

function onTurn.displayWeekMonthYear(alternateTurn)
    local turn = alternateTurn or civ.getTurn()
    return weekName[getWeekNumber(alternateTurn)] .. " of " .. monthName[getMonthNumber(alternateTurn)] .. ", " .. yearName[getYearNumber(alternateTurn)] .. ", " .. seasonName[getMonthNumber(alternateTurn)] 
end


--[[ Much simpler calendar solution option using modulo operator by Knigttime
local function displayWeekMonthYear (alternateTurn)
	local turn =alternateTurn or civ.getTurn()
	local weekNumber = (turn - 1) % 4
	local monthNumber = (((turn - 1) // 4) + 6) % 12
	local yearNumber = ((turn - 1) + 24) // 48
	return weekName[weekNumber] .. " of " .. monthName[monthNumber] .. ", " .. yearName[yearNumber]
end
--]]

-- END ====================================

		
-- PARTIAL UNIT.TYPE DELETION FUNCTION ================ (by Prof. Garfield)
local function withdrawTroops(unitType,quantity,divisionName)
    local deletedUnits = 0
    local unitTypeStillInTheatre = false
    local messageTitle = "Supreme Allied Command"
    for unit in civ.iterateUnits() do
        if unit.type == unitType and deletedUnits < quantity then
            civ.deleteUnit(unit)
            deletedUnits = deletedUnits +1
        elseif unit.type == unitType and deletedUnits == quantity then
            unitTypeStillInTheatre = true
        end
    end
    if deletedUnits == 0 then
        -- no message, since no units were deleted
        return
    elseif deletedUnits == 1 then
        if unitTypeStillInTheatre then
            text.simple(text.substitute("The %STRING1 Division withdraws 1 regiment!",{divisionName}),messageTitle)
            return
        else
            text.simple(text.substitute("The %STRING1 Division withdraws its last regiment!",{divisionName}),messageTitle)
            return
        end
    else
        if unitTypeStillInTheatre then
            text.simple(text.substitute("The %STRING1 Division withdraws %STRING2 regiment!",{divisionName, deletedUnits}),messageTitle)
            return
        else
            text.simple(text.substitute("The %STRING1 Division withdraws its last %STRING2 regiments!",{divisionName,deletedUnits}),messageTitle)
            return
        end
    end
end

-- ============================================================
-- CHECK TILE OWNERSHIP FUNCTION ================ (by Prof. Garfield)
-- Used in GOTHIC LINE and AXIS REPLACEMENTS sections below
-- Allows code to check if tile is owned by specific power before being able to create a unit on it
local function filterTilesByOwner(tileList,tribe)
    local filtered = {}
    local index = 1
    -- use ipairs to maintain order, in case
    -- randomization is not desired sometimes
    -- (but tileList must have integer indices)
    for _,tile in ipairs(tileList) do
        tile = gen.toTile(tile)
        if tile.owner ~= tribe then -- (modified by tootall: used ~= to check that tile is NOT owned by selected tribe - Reason: because in this scenario a tile in Axis 'territory' could also be owned by the 'Barbarians' tribe)
            filtered[index] = tile
            index = index+1
        end
    end
    return filtered
end

-- ============================================================
-- AXIS INVASION REACTION FUNCTIONS (POLYGON SET UPS) - Begin 
-- Set minimum number of Allied units located in zones required to trigger Axis reaction forces
local minInvasionInAnzio = 1
local minInvasionInCassino = 1
local minInvasionInCecina = 1
local minInvasionInCivitavecchia = 1
local minInvasionInFano = 1
local minInvasionInFirenze = 1
local minInvasionInFirenzuola = 1
local minInvasionInFoggia = 1
local minInvasionInForli = 1
local minInvasionInMassa = 1
local minInvasionInNapoli = 3
local minInvasionInOrtona = 1
local minInvasionInPracchia = 1
local minInvasionInRimini = 1
local minInvasionInSalerno = 3
local minInvasionInTerracina = 1

local minInvasionInItalianSurrender = 21 -- Special variable used to determine Italian surrender

-- Set coordinates of each reaction zone (see tables.lua)


-- Set tiles within each reaction zones
local function inAnzio(tile)
    return gen.inPolygon(tile,tables.anzioPolygonCoordinates)
end

local function inCassino(tile)
    return gen.inPolygon(tile,tables.cassinoPolygonCoordinates)
end

local function inCecina(tile)
    return gen.inPolygon(tile,tables.cecinaPolygonCoordinates)
end

local function inCivitavecchia(tile)
    return gen.inPolygon(tile,tables.civitavecchiaPolygonCoordinates)
end

local function inFano(tile)
    return gen.inPolygon(tile,tables.fanoPolygonCoordinates)
end

local function inFirenze(tile)
    return gen.inPolygon(tile,tables.firenzePolygonCoordinates)
end

local function inFirenzuola(tile)
    return gen.inPolygon(tile,tables.firenzuolaPolygonCoordinates)
end

local function inFoggia(tile)
    return gen.inPolygon(tile,tables.foggiaPolygonCoordinates)
end

local function inForli(tile)
    return gen.inPolygon(tile,tables.forliPolygonCoordinates)
end

local function inMassa(tile)
    return gen.inPolygon(tile,tables.massaPolygonCoordinates)
end

local function inNapoli(tile)
    return gen.inPolygon(tile,tables.napoliPolygonCoordinates)
end

local function inOrtona(tile)
    return gen.inPolygon(tile,tables.ortonaPolygonCoordinates)
end

local function inPracchia(tile)
    return gen.inPolygon(tile,tables.pracchiaPolygonCoordinates)
end

local function inRimini(tile)
    return gen.inPolygon(tile,tables.riminiPolygonCoordinates)
end

local function inSalerno(tile)
    return gen.inPolygon(tile,tables.salernoPolygonCoordinates)
end

local function inTerracina(tile)
    return gen.inPolygon(tile,tables.terracinaPolygonCoordinates)
end

local function inItalianSurrender(tile)
    return gen.inPolygon(tile,tables.italianSurrenderPolygon)
end

-- Type of Allied units within zones which may trigger reaction forces
local invasionUnitType = {} 
invasionUnitType[object.uGIVeteran.id] = 1
invasionUnitType[object.uGIRegular.id] = 1
invasionUnitType[object.uUSBazooka.id] = 1
invasionUnitType[object.uRanger.id] = 1
invasionUnitType[object.u82ndAirborne.id] = 1
invasionUnitType[object.uArmoredInfantry.id] = 1
invasionUnitType[object.uM5Stuart.id] = 1
invasionUnitType[object.uM4A1Sherman.id] = 1
invasionUnitType[object.uM4A1ShermanInd.id] = 1
invasionUnitType[object.uM10Wolverine.id] = 1
invasionUnitType[object.uM7105mmHMC.id] = 1
invasionUnitType[object.uM2A1105mmArt.id] = 1
invasionUnitType[object.uBritishVeteran.id] = 1
invasionUnitType[object.uBritishRegular.id] = 1
invasionUnitType[object.uBritishPIAT.id] = 1
invasionUnitType[object.uCommandos.id] = 1
invasionUnitType[object.u1stAirborne.id] = 1
invasionUnitType[object.uBritishMotorTroops.id] = 1
invasionUnitType[object.uShermanIIaBde.id] = 1
invasionUnitType[object.uShermanIIa.id] = 1
invasionUnitType[object.uChurchillMkIV.id] = 1
invasionUnitType[object.uBishop25pdr.id] = 1
invasionUnitType[object.u25pdrMKIIArt.id] = 1
invasionUnitType[object.uCdnInfantry.id] = 1
invasionUnitType[object.uCdnShermanIII.id] = 1
invasionUnitType[object.uGurkhas.id] = 1
invasionUnitType[object.uIndianInfantry.id] = 1
invasionUnitType[object.u4thIndianDiv.id] = 1
invasionUnitType[object.uIndian25pdrArt.id] = 1
invasionUnitType[object.uPolishInfantry.id] = 1
invasionUnitType[object.uPolishShermanIII.id] = 1
invasionUnitType[object.uPolish25pdrArt.id] = 1
invasionUnitType[object.uInfanterieColoniale.id] = 1
invasionUnitType[object.uGoumierMarocain.id] = 1
invasionUnitType[object.uTirailleurAlgerien.id] = 1
invasionUnitType[object.uFreeFrenchInfantry.id] = 1
invasionUnitType[object.uFreeFrench105Art.id] = 1
invasionUnitType[object.uFreeFrenchM10.id] = 1
invasionUnitType[object.uNZMotInf.id] = 1
invasionUnitType[object.uNZShermanIII.id] = 1
invasionUnitType[object.uNZ25pdrArt.id] = 1

-- Count Allied units in zones to determine if they trigger Axis reaction forces 
local function unitsInRegion(regionFn) 
    local value = 0
    for unit in civ.iterateUnits() do
        if invasionUnitType[unit.type.id] and regionFn(unit) then
            value = value + invasionUnitType[unit.type.id]
			print("Successfully counted invasion forces")
        end
		
    end
    return value

end

-- Check to see if minimum number of Allied units was met in each specific reaction zone
local function invasionUnitsInAnzio()
    return  minInvasionInAnzio <= unitsInRegion(inAnzio)
end

local function invasionUnitsInCassino()
    return  minInvasionInCassino <= unitsInRegion(inCassino)
end

local function invasionUnitsInCecina()
    return  minInvasionInCecina <= unitsInRegion(inCecina)
end

local function invasionUnitsInCivitavecchia()
    return  minInvasionInCivitavecchia <= unitsInRegion(inCivitavecchia)
end

local function invasionUnitsInFano()
    return  minInvasionInFano <= unitsInRegion(inFano)
end

local function invasionUnitsInFirenze()
    return  minInvasionInFirenze <= unitsInRegion(inFirenze)
end

local function invasionUnitsInFirenzuola()
    return  minInvasionInFirenzuola <= unitsInRegion(inFirenzuola)
end

local function invasionUnitsInFoggia()
    return  minInvasionInFoggia <= unitsInRegion(inFoggia)
end

local function invasionUnitsInForli()
    return  minInvasionInForli <= unitsInRegion(inForli)
end

local function invasionUnitsInMassa()
    return  minInvasionInMassa <= unitsInRegion(inMassa)
end

local function invasionUnitsInNapoli()
    -- return  minInvasionInNapoli <= unitsInRegion(inNapoli)
	return  unitsInRegion(inNapoli) >= minInvasionInNapoli 
end

local function invasionUnitsInOrtona()
    return  minInvasionInOrtona <= unitsInRegion(inOrtona)
end

local function invasionUnitsInPracchia()
    return  minInvasionInPracchia <= unitsInRegion(inPracchia)
end

local function invasionUnitsInRimini()
    return  minInvasionInRimini <= unitsInRegion(inRimini)
end

local function invasionUnitsInSalerno()
    -- return  minInvasionInSalerno <= unitsInRegion(inSalerno)
	return  unitsInRegion(inSalerno) >= minInvasionInSalerno 
end

local function invasionUnitsInTerracina()
    return  minInvasionInTerracina <= unitsInRegion(inTerracina)
end

local function invasionUnitsInItalianSurrender()
    return  minInvasionInItalianSurrender <= unitsInRegion(inItalianSurrender)
end

-- END ==============================

-- INITIAL AFHQ and 15TH ARMY COMMANDER NAMES (can change overtime)
local commanderAFHQ_Base = "Dwight D. Eisenhower"
local commanderAFHQ_1 = "Dwight D. Eisenhower"
local commanderAFHQ_2 = "Henry M. Wilson"
local commanderAFHQ_3 = "Harold H. Alexander"
local commander15thArmy_Base = "Harold H. Alexander"
local commander15thArmy_1 = "Harold H. Alexander"
local commander15thArmy_2 = "Mark W. Clark"
local jointChiefofStaff = "Alan F. Brooke"


-- This file can be deleted from the template if it is not needed
-- local onTurn = {}

---&autoDoc onTurn
function onTurn.onTurn(turn)
    if _global.eventTesting then
        civ.ui.text("on turn test separate file")
    end

	-- ============================================================
	-- SET COMMANDER NAMES BASED ON TURN #
	-- AFHQ
	if turn >= 1 and turn <= 26 then
		commanderAFHQ_Base = commanderAFHQ_1
	elseif turn >= 27 and turn <= 69 then
		commanderAFHQ_Base = commanderAFHQ_2
	elseif turn >= 70 and turn <= 92 then
		commanderAFHQ_Base = commanderAFHQ_3
	end
	
	-- 15th Army Group
	if turn >= 1 and turn <= 69 then
		commander15thArmy_Base = commander15thArmy_1
	elseif turn >= 70 and turn <= 92 then
		commander15thArmy_Base = commander15thArmy_2
	end	

	-- ===============================================================================================
	-- DISPLAY GAME TURN + INTRODUCTORY TEXT
	if turn == 1 then 
		-- civ.playSound("LiliMarleneShort.wav")
		-- civ.playSound("ddayaddressshort.wav")
		-- civ.ui.text(func.splitlines(displayYearMonthWeek() .. ":\n^\r^Game turn #" .. turn .. " of 89"))
		-- civ.ui.text(onTurn.displayWeekMonthYear() .. " (turn " .. turn .. " of 89)")
		-- civ.ui.text("Week I, 9 -10 July, 1943, Summer (turn " .. turn .. " of 89)")
		
		civ.playSound("Telegraph.wav")
		text.simple("****************** MISSION STATEMENT ******************\n^\r^Date: Week I, 9-10 July, 1943, Summer\n^\r^From: ".. jointChiefofStaff .. ", (CSS/CIGS)\n^To: ".. commanderAFHQ_Base .. ", (AFHQ)\n^\r^You have been placed in command of Allied Forces HQ in\n^the Mediterranean and are tasked with bringing about\n^the capitulation of Fascist Italy, the conquest of the\n^Italian Peninsula in order to secure the Mediterranean\n^for Allied shipping and seizing the Po Valley to serve\n^as a final base of operations against Nazi Germany itself.\n^\r^As the first step, the Allied High Command has directed you\n^to carry out Operation Husky, the amphibious invasion of\n^Sicily. The conquest of the island is considered to be an\n^ABSOLUTE precursor to any invasion of the mainland,\n^since it will secure the vital airbases necessary to support\n^that operation.\n^\r^You are free to select any Sicilian beaches for your invasion\n^forces, but need to invade before the end of week II of July\n^(next turn). Do NOT attempt to land forces on the Italian\n^mainland as the Royal Navy is not prepared to support any\n^such invasion at this time.\n^\r^You begin Operation Husky with a total of 10 Landing Craft\n^(LC), which will be withdrawn at the beginning of week III of July\n^for repairs and refurbishing in order to be ready for the next\n^amphibious operation. Thus you should disembark all your\n^troops from your LC's by the end of of week II of July (turn 2),\n^otherwise any unloaded units will return to port in Tunis.\n^\r^Your LC's play a vital role as not only are they the sole means\n^for conducting amphibious invasions on enemy beaches but are\n^required for creating 'Beachhead' units on BH ocean tiles. If\n^possible, it is highly recommended you return your LC's to Valetta,\n^as soon as they have off-loaded their troops, for safe keeping.\n^\r^Until you liberate a port and clear it of the sea mines that obstruct\n^their entrances, your reinforcments will only be able to be unloaded\n^on beaches from Liberty Ships located on Beachhead units.\n^\r^For Operation Husky it is highly recommended, though not\n^required, you set up 2 separate beachheads to quickly bolster\n^your ability to bring in reinforcements onto the island.\n^\r^General Montgomery is in command of the British Eight Army\n^and General Patton in command of the American Seventh Army.\n^Both leaders are aboard their own Leander Class cruisers\n^located with their respective armies. Take full advantage of their\n^leadership abilities to increase your units combat strengths.\n^\r^Though Allied intelligence doesn't believe the Italians are\n^quite ready to commit their main battle fleet, it has detected\n^the presence of smaller Axis naval forces in and around Sicily.\n^As such, make sure to protect your vital transports at all times!\n^\r^Only once you have captured all 5 of Sicily's major cities\n^will your initial mission be considered complete and be given\n^the green light to proceed with your next amphibious operation:\n^the invasion of Italy itself.\n^\r^The Allied High Command believes you have the necessary\n^forces at your disposal to complete this first objective within\n^the next 8 weeks.\n^\r^But be aware that time is not on your side, as it is feared that\n^every extra week required to complete your conquest of\n^the island will only embolden a growing number of hardline\n^Italian Fascists to continue the struggle by the side of their\n^Axis partner!","Mission Statement: Operation Husky!",object.mSicily)
		
		-- PRESENT UNIT STACKING LIMITS GAME OPTION
		text.setLinesPerWindow(25)
			text.simple(object.stackingLimits,"TIP: Stacking Limits",object.mGameConcept2)
		text.setLinesPerWindow(12)
		-- PRESENT NAVAL UNITS AT SEA PENALTIES GAME OPTION
		civ.playSound("Calmsea.wav")
		-- text.simple("Starting next turn, any naval unit that BEGINS its turn at sea\n^will be considered out of supply, unless it is within 3 tiles of\n^a fully supplied oil tanker vessel (Oil Tanker (F)).\n^\r^This is considered an optional feature for those players who\n^desire an extra level of challenge in the game.\n^\r^You may at any time opt to turn this feature off by going to the\n^'Game Tips and Concept' menu (key '6') and selecting the\n^'Set Ships At Sea Penalty Option' box.\n^\r^The next message box will provide more details on the feature.","Game Concept - Ships At Sea Penalty",object.mGameConcept2)
		text.simple("Starting next turn, any naval unit that BEGINS its turn at sea\n^will be considered out of supply, unless it is within 3 tiles of\n^a fully supplied oil tanker vessel (Oil Tanker (F)).\n^\r^The next message box will provide more details on the feature.","Game Concept - Ships At Sea Penalty",object.mGameConcept2)
		text.simple(object.oilTankers,"TIP: Oil Tanker units",object.mResupplyAtSea)
		-- PRESENT GAME MENUS	
		text.setLinesPerWindow(25)
			text.simple(object.gameMenus,"CONCEPT: Game Menus",object.mGameConcept2)
		text.setLinesPerWindow(12)
		-- RESET TUNIS TERRAIN TO TOWN CENTER ALONG WITH FACTORY RESOURCE
		gen.setTerrainType({0,136,0},8) -- Town Center
		local tileToChange = civ.getTile(0, 136, 0)
		tileToChange.terrain = object.tFactory	

		-- CREATE TEMPORARY SUPPLY HUB UNIT ON ISLAND NE OF PANTELLERIA
		text.simple("The Allies have placed a temporary Supply Hub, with\n^a supply range of 8 tiles, on the island north of Pantelleria\n^to support any possible invasion near Trapani!\n^\r^It will be disbanded at the beginning of the week of\n^July II (turn 2).\n^\r^Valetta, with a Supply Depot improvement, has a\n^supply range of 12 tiles which extends into south-eastern\n^Sicily, as delimited by the dotted green asterik line.","Supply Hub - Pantelleria",object.mAFHQ)
		gen.createUnit(object.uSupplyHub, object.pAllies, {{21,147,0}}, {randomize=false, veteran=false})

	end

	if turn >= 2 then 
		-- civ.ui.text(func.splitlines(displayYearMonthWeek() .. ":\n^\r^Game turn #" .. turn .. " of 89"))
		if  flag.value("Game ends March I, 1945") == false then
			text.simple(onTurn.displayWeekMonthYear() .. " (turn " .. turn .. " of 89)","Calendar Date",object.monexone)
		else
			text.simple(onTurn.displayWeekMonthYear() .. " (turn " .. turn .. " of 81)","Calendar Date",object.monexone)
		end	
	end

	-- =======================================
	-- TURN # TO CHANGE SEASONS
	if turn == 15 then 
		civ.playSound("Storm.wav") 
		text.setLinesPerWindow(20)	
			text.simple(object.fallSeason,"Fall Season!",object.mFall)
		text.setLinesPerWindow(12)	
		flag.setFalse("Summer Months")
		flag.setTrue("FallSpring Months")
	end 
	if turn == 21 then 
		civ.playSound("Winter.wav") 
		text.setLinesPerWindow(22)	
			text.simple(object.winterSeason,"Winter Season!",object.mWinter)
		text.setLinesPerWindow(12)		
		flag.setFalse("FallSpring Months")
		flag.setTrue("Winter Months")
	end 
	if turn == 33 then 
		civ.playSound("Storm.wav") 
		text.setLinesPerWindow(20)	
			text.simple(object.springSeason,"Spring Season!",object.mSpring)
		text.setLinesPerWindow(12)	
		flag.setFalse("Winter Months")
		flag.setTrue("FallSpring Months")
	end 
	if turn == 39 then 
		civ.playSound("Spring.wav") 
		text.simple("Summer returns to the land and your forces no longer suffer any of the ill effects from the previous two seasons.","Summer Season!",object.mSummer)
		flag.setFalse("FallSpring Months")
		flag.setTrue("Summer Months") 
	end 
	if turn == 63 then 
		civ.playSound("Storm.wav")  
		text.setLinesPerWindow(20)	
			text.simple(object.fallSeason,"Fall Season!",object.mFall)
		text.setLinesPerWindow(12)	
		flag.setFalse("Summer Months")
		flag.setTrue("FallSpring Months")
	end 
	if turn == 69 then 
		civ.playSound("Winter.wav")  
		text.setLinesPerWindow(22)	
			text.simple(object.winterSeason,"Winter Season!",object.mWinter)
		text.setLinesPerWindow(12)
		flag.setFalse("FallSpring Months")
		flag.setTrue("Winter Months")
	end 
	if turn == 81 then 
		civ.playSound("Storm.wav") 
		text.setLinesPerWindow(20)	
			text.simple(object.springSeason,"Spring Season!",object.mSpring)
		text.setLinesPerWindow(12)		
		flag.setFalse("Winter Months")
		flag.setTrue("FallSpring Months")
	end 
	if turn == 87 then 
		civ.playSound("Spring.wav") 
		text.simple("Summer returns to the land and your forces no longer suffer any of the ill effects from the previous two seasons.","Summer Season!",object.mSummer) 
		flag.setFalse("FallSpring Months")
		flag.setTrue("Summer Months") 
	end 

	-- ===============================================================================================
	-- RANDOMLY SET RIVER CROSSING TILES TO BE RIGGED TO BLOW BRIDGES AT THE BEGINNING OF THE SCENARIO
	if turn == 1 then 
	-- (also check out -- RIVER CROSSING BLOWN WHEN ALLIED UNIT ENTERS RIVER CROSSING TILE THAT HAS BEEN RIGGED TO BLOW in discreteEvents.lua )
	-- NEED TO ADD THREE LINES BELOW IN HEADER OF THIS 'onTurn.lua' FILE
	-- local civlua = require("civluaModified")
	-- local tileData = require("tileData")
	-- tileData.defineFlag("riggedBridge")
		
		local messageText = "Bridges in Sicily: " .. onTurn.displayWeekMonthYear() .. " \n^"
		for tile in civlua.iterateTiles() do
			-- Add Blown Bridges for river crossing tiles IN Sicily		
			if tile.baseTerrain == object.bRiverCrossing and gen.inPolygon(tile,tables.sicilyPolygon) then
				if math.random(1,3) == 3 then -- Want a 33% chance that a river crossing will be rigged to blow
					tileData.flagSetTrue(tile,"riggedBridge")
					messageText = messageText .. 
					"Rigged Bridge tile in Sicily: (" .. tile.x .. "," .. tile.y .. "," .. tile.z .. ") is " .. tostring(tileData.flagGetValue(tile,"riggedBridge")) .. "\n^" 
				end
			end
		end
		print(func.splitlines(messageText))
	end 	

	if turn == 2 then -- turn 2		
		local messageText = "Bridges in Italy south of Gustav Line: " .. onTurn.displayWeekMonthYear() .. " \n^"
		for tile in civlua.iterateTiles() do
			-- Add Blown Bridges for river crossing tile in Italy South of Gustav Line
			-- if tile.baseTerrain == object.bRiverCrossing and (not gen.inPolygon(tile,tables.sicilyPolygon)) then
			if tile.baseTerrain == object.bRiverCrossing and gen.inPolygon(tile,tables.italySouthGustavLinePolygon) then
				if math.random(1,4) == 4 then -- Want a 25% chance that a river crossing will be rigged to blow
					tileData.flagSetTrue(tile,"riggedBridge")
					messageText = messageText .. 
					"Rigged Bridge tile in Italy south of Gustav Line: (" .. tile.x .. "," .. tile.y .. "," .. tile.z .. ") is " .. tostring(tileData.flagGetValue(tile,"riggedBridge")) ..  "\n^"
				end	
			end
		end
		print(func.splitlines(messageText))		
		
	end

	if turn == 3 then -- turn 3		
		local messageText = "Bridges in Italy north of Gustav Line: " .. onTurn.displayWeekMonthYear() .. " \n^" 
		for tile in civlua.iterateTiles() do
			-- Add Blown Bridges for river crossing tile NOT IN Corsica, Sardinia, Sicily, Gustav Line or Gothic Line
			-- if tile.baseTerrain == object.bRiverCrossing and ((not gen.inPolygon(tile,tables.corsicaPolygon)) and (not gen.inPolygon(tile,tables.sardiniaPolygon)) and (not gen.inPolygon(tile,tables.sicilyPolygon)) and (not gen.inPolygon(tile,tables.gustavLinePolygon)) and (not gen.inPolygon(tile,tables.italySouthGustavLinePolygon)) ) then
			if tile.baseTerrain == object.bRiverCrossing and ((gen.inPolygon(tile,tables.italianSurrenderPolygon2)) and (not gen.inPolygon(tile,tables.gustavLinePolygon)) and (not gen.inPolygon(tile,tables.gothicLinePolygon)) ) then
				if math.random(1,4) == 4 then -- Want a 25% chance that a river crossing will be rigged to blow
					tileData.flagSetTrue(tile,"riggedBridge")
					messageText = messageText .. 
					"Rigged Bridge tile in Italy north of Gustav Line: (" .. tile.x .. "," .. tile.y .. "," .. tile.z .. ") is " .. tostring(tileData.flagGetValue(tile,"riggedBridge")) ..  "\n^"
				end	
			end
		end
		print(func.splitlines(messageText))		
		
	end

	if turn == 3 then -- turn 3		
		local messageText = "Bridges within Gustav Line: " .. onTurn.displayWeekMonthYear() .. " \n^"
		for tile in civlua.iterateTiles() do
			-- Add Blown Bridges for river crossing tile within Gustav Line
			-- if tile.baseTerrain == object.bRiverCrossing and (not gen.inPolygon(tile,tables.sicilyPolygon)) then
			if tile.baseTerrain == object.bRiverCrossing and gen.inPolygon(tile,tables.gustavLinePolygon) then
				if math.random(1,1) == 1 then -- Want a 100% chance that a river crossing will be rigged to blow
					tileData.flagSetTrue(tile,"riggedBridge")
					messageText = messageText .. 
					"Rigged Bridge tile in Gustav Line: (" .. tile.x .. "," .. tile.y .. "," .. tile.z .. ") is " .. tostring(tileData.flagGetValue(tile,"riggedBridge")) ..  "\n^"
				end	
			end
		end
		print(func.splitlines(messageText))		
		
	end

	if turn == 3 then -- turn 3		
		local messageText = "Bridges within Gothic Line: " .. onTurn.displayWeekMonthYear() .. " \n^"
		for tile in civlua.iterateTiles() do
			-- Add Blown Bridges for river crossing tile within Gothic Line
			if tile.baseTerrain == object.bRiverCrossing and gen.inPolygon(tile,tables.gothicLinePolygon) then
				if math.random(1,1) == 1 then -- Want a 100% chance that a river crossing will be rigged to blow
					tileData.flagSetTrue(tile,"riggedBridge")
					messageText = messageText .. 
					"Rigged Bridge tile in Gothic Line: (" .. tile.x .. "," .. tile.y .. "," .. tile.z .. ") is " .. tostring(tileData.flagGetValue(tile,"riggedBridge")) ..  "\n^"
				end	
			end
		end
		print(func.splitlines(messageText))		
		
	end

	-- RANDOMLY SET BEACH AND OTHER TILES TO BE RIGGED WITH MINENFELD AT THE BEGINNING OF THE SCENARIO
	if turn == 1 then 
	-- (also check out -- WHEN ALLIED UNIT ENTERS MINENFELD TILE in discreteEvents.lua )
	-- NEED TO ADD THREE LINES BELOW IN HEADER OF THIS 'onTurn.lua' FILE
	-- local civlua = require("civluaModified")
	-- local tileData = require("tileData")
	-- tileData.defineFlag("placedMinenfeld")
		
		local messageText = "Sicily Beaches: " .. onTurn.displayWeekMonthYear() .. " \n^"
		for tile in civlua.iterateTiles() do
			-- Add Minenfeld on beach tiles IN Sicily
			if tile.baseTerrain == object.bBeach and gen.inPolygon(tile,tables.sicilyPolygon) then
				if math.random(1,3) == 3 then -- Want a 33% chance that a tile will be rigged with land mines
					tileData.flagSetTrue(tile,"placedMinenfeld")
					messageText = messageText .. 
					"Placed Minenfeld on beach tile in Sicily: (" .. tile.x .. "," .. tile.y .. "," .. tile.z .. ") is " .. tostring(tileData.flagGetValue(tile,"placedMinenfeld")) ..  "\n^"
				end	
			end
		end	
		print(func.splitlines(messageText))
		
		local messageText = "Sicily Inland: " .. onTurn.displayWeekMonthYear() .. " \n^"
		for tile in civlua.iterateTiles() do
			-- Add Minenfeld on rough/grassland tiles with roads/highways IN Sicily
			if (tile.baseTerrain == object.bGrassland or tile.baseTerrain == object.bRough) and (gen.hasRoad(tile) or gen.hasRailroad(tile)) and gen.inPolygon(tile,tables.sicilyPolygon) then
				if math.random(1,11) == 11 then -- Want a 9% chance that a tile will be rigged with land mines
					tileData.flagSetTrue(tile,"placedMinenfeld")
					messageText = messageText ..  
					"Placed Minenfeld road tile in Sicily: (" .. tile.x .. "," .. tile.y .. "," .. tile.z .. ") is " .. tostring(tileData.flagGetValue(tile,"placedMinenfeld")) ..  "\n^"
				end	
			end
		end	
		print(func.splitlines(messageText))
	end	

	if turn == 2 then -- turn 2
		local messageText = "Southern Italy: " .. onTurn.displayWeekMonthYear() .. " \n^"
		for tile in civlua.iterateTiles() do
			-- Add Minenfeld on beach/rough/grassland tiles with roads/highways IN Southern Italy
			if (tile.baseTerrain == object.bBeach or tile.baseTerrain == object.bGrassland or tile.baseTerrain == object.bRough) and (gen.hasRoad(tile) or gen.hasRailroad(tile)) and gen.inPolygon(tile,tables.italySouthGustavLinePolygon) then
				if math.random(1,15) == 15 then -- Want a 6% chance that a tile will be rigged with land mines
					tileData.flagSetTrue(tile,"placedMinenfeld")
					messageText = messageText ..  
					"Placed Minenfeld road tile in Southern Italy: (" .. tile.x .. "," .. tile.y .. "," .. tile.z .. ") is " .. tostring(tileData.flagGetValue(tile,"placedMinenfeld")) ..  "\n^"
				end	
			end
		end
		print(func.splitlines(messageText))

		local messageText = "Gustav Line: " .. onTurn.displayWeekMonthYear() .. " \n^"
		for tile in civlua.iterateTiles() do
			-- Add Minenfeld IN Gustav Line Defensive Zone (equals roughly 65 tiles)
			if tile.baseTerrain ~= object.bApennines and gen.inPolygon(tile,tables.gustavLinePolygon) then
				if math.random(1,5) == 5 then -- Want a 20% chance that a tile will be rigged with land mines
					tileData.flagSetTrue(tile,"placedMinenfeld")
					messageText = messageText ..  
					"Placed Minenfeld in Gustav Line Zone: (" .. tile.x .. "," .. tile.y .. "," .. tile.z .. ") is " .. tostring(tileData.flagGetValue(tile,"placedMinenfeld")) ..  "\n^"
				end	
			end
		end
		print(func.splitlines(messageText))

	end
	
	if turn == 3 then -- turn 3
		local messageText = "Central Italy: " .. onTurn.displayWeekMonthYear() .. " \n^"
		for tile in civlua.iterateTiles() do
			-- Add Minenfeld on beach/rough/grassland tiles with roads/highways IN Central Italy
			if (tile.baseTerrain == object.bBeach or tile.baseTerrain == object.bGrassland or tile.baseTerrain == object.bRough) and (gen.hasRoad(tile) or gen.hasRailroad(tile)) and gen.inPolygon(tile,tables.italianSurrenderPolygon) and (not gen.inPolygon(tile,tables.italySouthGustavLinePolygon)) then
				if math.random(1,10) == 10 then -- Want a 10% chance that a tile will be rigged with land mines
					tileData.flagSetTrue(tile,"placedMinenfeld")
					messageText = messageText ..  
					"Placed Minenfeld road tile in Central Italy: (" .. tile.x .. "," .. tile.y .. "," .. tile.z .. ") is " .. tostring(tileData.flagGetValue(tile,"placedMinenfeld")) ..  "\n^"
				end	
			end
		end
		print(func.splitlines(messageText))

		local messageText = "Gothic Line: " .. onTurn.displayWeekMonthYear() .. " \n^"
		for tile in civlua.iterateTiles() do
			-- Add Minenfeld IN Gothic Line Defensive Zone (equals roughly 110 tiles)
			if tile.baseTerrain ~= object.bApennines and gen.inPolygon(tile,tables.gothicLinePolygon) then
				if math.random(1,5) == 5 then -- Want a 20% chance that a tile will be rigged with land mines
					tileData.flagSetTrue(tile,"placedMinenfeld")
					messageText = messageText ..  
					"Placed Minenfeld in Gothic Line Zone: (" .. tile.x .. "," .. tile.y .. "," .. tile.z .. ") is " .. tostring(tileData.flagGetValue(tile,"placedMinenfeld")) ..  "\n^"
				end	
			end
		end
		print(func.splitlines(messageText))

	end

	-- ===============================================================================================
	-- METHOD 1: DELETE UNUSED ALLIED MUNITIONS AT THE BEGINNING OF EACH TURN
	-- if turn >= 1 then 
		-- for unit in civ.iterateUnits() do 
			-- if (unit.type == object.uHeavyShell or unit.type == object.u105mmShell or unit.type == object.u155mmShell or unit.type == object.uLightBombload or unit.type == object.u500lbBomb or unit.type == object.uFlak) 
			-- and unit.owner == object.pAllies  then -- or unit.owner == object.pAxis) then	
				-- civ.deleteUnit(unit) 
			-- end 
		-- end
	-- end 
	
	-- METHOD 2: DELETE UNUSED ALLIED MUNITIONS USING THE MUNITIONS TABLE AT THE BEGINNING OF EACH TURN (also see onTribeTurnBegin.lua for deleting AXIS AI munitions)
	if turn >= 2 then 
		--local activeUnit = civ.getActiveUnit()
		for activeUnit in civ.iterateUnits() do
		   if activeUnit.owner == object.pAllies and tables.munitionsToBeDestroyed[activeUnit.type.id] then 
				print("Destroyed " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z)
				civ.deleteUnit(activeUnit)
				
			end
		end
	end
	
	-- =====================================================================================================================
	-- TEST - EXAMPLES OF CODE
	
	if turn == 1 then
	
		-- civ.ui.text("The turn is "..tostring(turn)..".")

		-- local tileToChange = civ.getTile(121,105,0)
		-- tileToChange.terrain = tileToChange.baseTerrain:getTerrain(0)

		-- TEST NOT EQUAL OPERATOR
		-- and unit.type ~= object.uC47Dakota

		-- TEST for re-activating tribes
		-- in Lua console enter 'civ.game.activeTribes = 255'

		-- TEST for adding/removing 
		-- gen.removeAirbase({109,107,0})
		-- gen.placeAirbaseForce({108,106,0})	
		-- gen.placeFortressForce({91,93,0})
		-- gen.placeRoad({109,135,0})
		-- gen.removeRoad({110,136,0})
		-- gen.setTerrainType({53,49,0},15)
		-- gen.setTerrainType({48,52,0},15)
		-- gen.setTerrainType({40,50,0},15)
		-- gen.setTerrainType({57,161,0},11)
		-- gen.setTerrainType({60,154,0},11)
		-- gen.setTerrainType({137,183,0},10)
		
		-- TEST for changing terrain tile resource type
		-- local tileToChange = civ.getTile(99, 85, 0)
		-- tileToChange.terrain = object.tCrossingTemp
		-- local tileToChange = civ.getTile(103, 71, 0)
		-- tileToChange.terrain = object.tCrossingTemp
		
		-- local tileToChange = civ.getTile(39, 179, 0)		
		-- tileToChange.terrain = object.tSupplyDepot
		-- gen.placeRoad({39, 179, 0})
		-- local tileToChange = civ.getTile(2, 136, 0)		
		-- tileToChange.terrain = object.tSupplyDepot

		-- TEST for removing resource from a terrain tile
		-- local tileToChange = civ.getTile(loser.location.x,loser.location.y,loser.location.z)
		-- tileToChange.terrain = tileToChange.baseTerrain:getTerrain(0)

		-- TEST for checking unit location based on terrain or resource type tile
		-- activeUnit.location.baseTerrain == object.bTownCenter -- Terrain type
		-- activeUnit.location.terrain == object.tTownBeach -- Resource type
	
		-- TEST for adding/removing city improvements
		-- civ.removeImprovement(object.cValetta,object.iSupplyDepot)
		-- civ.addImprovement(object.cTunis,object.iSupplyDepot)
		-- civ.hasImprovement(object.cTunis,object.iSupplyDepot)
		-- civ.addImprovement(object.cReggioCalabria,object.iShoreBattery)
		-- object.cTunis:addImprovement(object.object.iSupplyDepot)	

		-- TEST for homing a unit to a specific city
		-- gen.createUnit(object.uShermanIIaBde, object.pAllies, {{40,180,0}}, {count=2, homeCity=object.cValetta, randomize=false, veteran=false})		
	
		-- TEST for switching city control to another power
		-- if turn == 1 then
			-- object.cTrapani.owner = object.pAllies
		-- end
		
		-- TEST for adding money to treasury
		-- object.pAllies.money = object.pAllies.money + 1000
	
		-- TEST for hiding or revealing a tile (aka explored/unexplored)
		-- gen.coverTile({119,27,0},object.pAllies)
		-- gen.revealTile({119,27,0},object.pAllies)
	
		-- TEST - CHANGING A LEADER'S NAME (note: this will also change the save game's name)
		-- if turn == 26 then
			-- object.pAllies.leader.name = "Henry M. Wilson"
			-- text.simple("Field Marshall " .. object.pAllies.leader.name.. " replaces General Eisenhower as Supreme Allied Commander Mediterranean (AFHQ) based in Algiers and therefore will exercise strategic control over the campaign in Italy.","Henry M. Wilson", object.mHenryWilson)
		-- end
		
		-- TEST - To loop through different types of conditions
		-- for possibleBeachhead in gen.nearbyUnits(unit.location, 1) do
		-- for unit in activeUnit.location.units do
		-- for unit in civ.iterateUnits() do 
		-- for unit in object.ltownAncona.unit do
		-- for city in civ.iterateCities() do 
		-- while( a < 1 ) do

		-- TEST -- If want to replace a unit with another 
		-- gen.replaceUnit(unit,object.uBritishVeteran)
		
		-- TEST -- for creating a city
			-- civ.createCity(object.pAllies, {{16,148,0},})

		-- TEST -- For Technologies (to give, check or take)
		-- civ.giveTech(object.pAllies, object.aEarlyRadarSystem)
		-- civ.hasTech(object.pAllies, object.aEarlyRadarSystem)
		-- civ.takeTech(object.pAllies, object.aEarlyRadarSystem)
		
		-- TEST -- for checking regular, non-city, tile ownership
		-- text.simple("Print Hex 100,132: " .. tostring(object.lHex100_132.owner) .. "\n^Print Hex 104,140: " .. tostring(object.lHex104_140.owner) .. "\n^Print Hex 112,134: " .. tostring(object.lHex112_134.owner) .. "")
		-- print("Print Bonifacio: " .. tostring(object.ltownBonifacio.owner))
		
		-- FOR TESTING UPDATING COUNTER VALUES:
		-- ADDING:
		-- gen.createUnit(object.uP40Warhawk, object.pAllies, {{0,136,0}}, {count=1, randomize=false, veteran=false})
		-- counter.add("# P-40",1)
		-- counter.add("# Total US Fighters",1)		

        -- SUBTRACTING:
		-- local a = 0
        -- for unit in civ.iterateUnits() do 
            -- if unit.type == object.uP40Warhawk then
                -- if a < 1 then
                    -- civ.deleteUnit(unit)
                    -- a = a + 1
                -- end
            -- end    
        -- end 
		-- counter.subtract("# P-40",1)
		-- counter.subtract("# Total US Fighters",1)

		-- ==============================================================
		-- TURN ONE - UNIT PLACEMENT JUST FOR TESTING PURPOSES		
		
		-- local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{48,166,0},{49,167,0},{49,171,0},{50,172,0}}, {count=2, randomize=true, scatter=true, veteran=false})[1]
		-- if unit then
			-- gen.setToNoOrders(unit)
			-- gen.setToFortified(unit)
			-- gen.placeFortress(unit.location)
			-- unit.moveSpent = 255
		-- end	
		
	end

	-- ===========================================================
	-- SUPPLY HUB
	-- If a Supply Hub's damage is 9 or greater at the beginning of a turn delete the unit 
	-- x amount of Supplies are returned to the theater depots (aka Treasury)
	-- if turn >= 1 then
		-- for unit in civ.iterateUnits() do 
			-- if unit.type == object.uSupplyHub and unit.damage >= 9 then	
				-- civ.ui.text("The " ..tostring(unit.type.name).. " at (".. unit.x .. "," .. unit.y .. "," .. unit.z ..") was dispanded! It's remaining 50 supplies were redistributed to the theater depots.")
				-- object.pAllies.money = object.pAllies.money + 50
				-- civ.deleteUnit(unit)
				-- civ.ui.redrawTile(unit.location)
	
				-- -- .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z ..
			-- end 
		-- end
	-- end
	
	-- Each Supply Hub unit receives 2 HP damage per turn to reflect its usage by Allied troops
	-- The Supply Hub can be resupplied by a Truck unit as part of the HP Transfer function
	-- if turn >= 1 then
		-- for unit in civ.iterateUnits() do 
			-- if unit.type == object.uSupplyHub then	
				-- unit.damage = unit.damage + 2 
			-- end 
		-- end
	-- end
	
	-- Subtract supplies from Theater Depot each turn for each active Supply Hub unit
	if turn >= 1 then	
		local totalSupplyHubs = 0
		for unit in civ.iterateUnits() do
			if unit.owner == object.pAllies then
				if unit.type == object.uSupplyHub then
					totalSupplyHubs = totalSupplyHubs + 1
					print("Subtracted " .. param.costPerSupplyHub .. " supplies for Supply Hub(s) at " .. unit.x .. "," .. unit.y .. "," .. unit.z)
				end
			end
		end		
		if totalSupplyHubs > 0 then
			object.pAllies.money = object.pAllies.money - (totalSupplyHubs * param.costPerSupplyHub)
		end
	end
	
	-- ===========================================================
	-- EXTRA SUPPLIES PER DOCKS UNIT
	-- Each Allied Docks unit provides an extra number of supplies per month to the Allied Theater Depot (aka Treasury)
	if turn >= 1 then
		local totalDocks = 0
		for unit in civ.iterateUnits() do
			if unit.owner == object.pAllies then
				if unit.type == object.uDocks then
					totalDocks = totalDocks + 1
					print("Received " .. param.suppliesPerDocks .. " supplies for Docks at " .. unit.x .. "," .. unit.y .. "," .. unit.z)
				end
			end
		end	
		if totalDocks > 0 then
			object.pAllies.money = object.pAllies.money + (totalDocks * param.suppliesPerDocks)
		end
	end

	-- ===========================================================
	-- EXTRA SUPPLIES PER MAJOR AIRFIELD CONTROLLED
	if turn >= 1 then
		local totalAirfields = 0
		if object.cBari.owner == object.pAllies then
			totalAirfields = totalAirfields + param.bastiaAirfield
		end
		if object.cFoggia.owner == object.pAllies then
			totalAirfields = totalAirfields + param.foggiaAirfield
		end
		if object.cNapoli.owner == object.pAllies then
			totalAirfields = totalAirfields + param.napoliAirfield
		end
		if object.cROMA.owner == object.pAllies then
			totalAirfields =  totalAirfields + param.romaAirfield
		end
		object.pAllies.money = object.pAllies.money + totalAirfields
	end
	
	-- ===========================================================
	-- SUPPLIES LOST DUE TO CASUALTY RATES	
	
	 if turn >= 1 and counter.value("# Total Allied Killed") >= 50 then
		gen.justOnce("Allies suffer fifty casualties",function()
			civ.playSound("Telegraph.wav")
			civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^You have reached a threshold of 50 combat casualties in your\n^campaign, which is beginning to place an extra burden on your\n^overall resources due to the increased need for hospital and\n^training facilities required to maintain the effectivness of\n^your forces.\n^\r^Moving forward, supplies will be subtracted from your theater\n^depots based on your Allies to Axis losses percentage ratio.\n^The higher the loss percentage the greater the amount of supplies\n^to be deducted.\n^\r^You can always review the latest Allies to Axis losses ratio\n^in your weekly status menu."))	
		end)	 
	 end
	
	if turn >= 1 and counter.value("# Total Allied Killed") >= 50 then
	-- if turn >= 1 then	
		if math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) >= 70 then
			object.pAllies.money = object.pAllies.money - param.casualties70Plus
			if flag.value("Display Axis Testing Messages") == true then
				civ.ui.text("Supply Losses: " .. param.casualties70Plus .." due to casualty rate")
			end
		elseif (math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) >= 60)then
			object.pAllies.money = object.pAllies.money - param.casualties60Plus
			if flag.value("Display Axis Testing Messages") == true then
				civ.ui.text("Supply Losses: " .. param.casualties60Plus .." due to casualty rate")
			end
		elseif (math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) >= 50) then
			object.pAllies.money = object.pAllies.money - param.casualties50Plus
			if flag.value("Display Axis Testing Messages") == true then
				civ.ui.text("Supply Losses: " .. param.casualties50Plus .." due to casualty rate")
			end
		elseif (math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) >= 40) then
			object.pAllies.money = object.pAllies.money - param.casualties40Plus
			if flag.value("Display Axis Testing Messages") == true then
				civ.ui.text("Supply Losses: " .. param.casualties40Plus .." due to casualty rate")
			end
		elseif (math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) >= 26)then
			object.pAllies.money = object.pAllies.money - param.casualties26Plus
			if flag.value("Display Axis Testing Messages") == true then
				civ.ui.text("Supply Losses: " .. param.casualties26Plus .." due to casualty rate")
			end
		elseif math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) < 26 then
			if flag.value("Display Axis Testing Messages") == true then
				civ.ui.text("Supply Losses: 0 due to casualty rate")
			end
		end	
	end
	
	-- ===========================================================
	-- FOR TESTING PURPOSES
	-- SUPPLY DEPOT (SD) IMPROVEMENT: Damaged Allied ground units that begin turn in city with a SD receive a bonus HP heal amount
	if turn >= 1 then
		local numberUnitsHealed = 0
		for unit in civ.iterateUnits() do 
			-- if unit.owner == object.pAllies and unit.type.domain == 0 and unit.location == object.lPalermo and civ.hasImprovement(object.cPalermo,object.iSupplyDepot) then
			if unit.owner == object.pAllies and unit.type.domain == 0  
			and (
			(unit.location == object.lBari and civ.hasImprovement(object.cBari,object.iSupplyDepot)) or 
			(unit.location == object.lBologna and civ.hasImprovement(object.cBologna,object.iSupplyDepot)) or
			(unit.location == object.lCatania and civ.hasImprovement(object.cCatania,object.iSupplyDepot)) or
			(unit.location == object.lFirenze and civ.hasImprovement(object.cFirenze,object.iSupplyDepot)) or
			(unit.location == object.lFoggia and civ.hasImprovement(object.cFoggia,object.iSupplyDepot)) or
			(unit.location == object.lGenova and civ.hasImprovement(object.cGenova,object.iSupplyDepot)) or
			(unit.location == object.lLivorno and civ.hasImprovement(object.cLivorno,object.iSupplyDepot)) or
			(unit.location == object.lMessina and civ.hasImprovement(object.cMessina,object.iSupplyDepot)) or
			(unit.location == object.lNapoli and civ.hasImprovement(object.cNapoli,object.iSupplyDepot)) or
			(unit.location == object.lPalermo and civ.hasImprovement(object.cPalermo,object.iSupplyDepot)) or
			(unit.location == object.lPescara and civ.hasImprovement(object.cPescara,object.iSupplyDepot)) or
			(unit.location == object.lRimini and civ.hasImprovement(object.cRimini,object.iSupplyDepot)) or
			(unit.location == object.lROMA and civ.hasImprovement(object.cROMA,object.iSupplyDepot)) or			
			(unit.location == object.lTaranto and civ.hasImprovement(object.cTaranto,object.iSupplyDepot)) or
			(unit.location == object.lTunis and civ.hasImprovement(object.cTunis,object.iSupplyDepot)) or
			(unit.location == object.lValetta and civ.hasImprovement(object.cValetta,object.iSupplyDepot))
			)
			then

				if unit.damage > 0 then
					local healAmount = param.supplyDepotHeal -- math.random(0, 2)	
					local formerHitPoints = unit.hitpoints
					if unit.damage > healAmount then
						unit.damage = unit.damage - healAmount
						text.simple("The " .. unit.type.name .. " at (" .. text.coordinates(unit.location) .. ") healed an extra " .. param.supplyDepotHeal .. " points due to the Supply Depot; hit points changed from " .. formerHitPoints .. " to " .. unit.hitpoints .. ".","" .. unit.type.name .. " Healed!",object.monexone)	
						numberUnitsHealed = numberUnitsHealed + 1
					else
						unit.damage = 0
					end
				end

			end	
		end
	end		

	-- ===========================================================	
	-- REMOVE Waco Gliders that aren't located in cities or airbase tiles
	if turn >= 1 then
		for unit in civ.iterateUnits() do 
			-- if unit.type == object.uWacoGlider and ((not gen.hasAirbase(unit.location)) and	(not (unit.location.baseTerrain == object.bTownCenter ))) then
			if unit.type == object.uWacoGlider and ((not gen.hasAirbase(unit.location)) and	(unit.location.baseTerrain ~= object.bTownCenter)) then
				text.simple("The Waco Gliders on tile (" .. unit.x .. "," .. unit.y .. "," .. unit.z .. ") were damaged beyond repair and therefore scrapped!","Waco Glider Scrapped!",object.monexone)
				civ.deleteUnit(unit)
			end
		end
	end	
	
	-- REMOVE EXTRA SEA MINES PLACED TO PREVENT EARLY ALLIED INVASION ATTEMPTS OF ITALIAN MAINLAND 
	if turn == 3 then
		for unit in civ.iterateUnits() do 
			if unit.owner == object.pAxis and unit.type == object.uSeaMines 
			and (unit.location == object.lseamine1 
			or unit.location == object.lseamine2 
			or unit.location == object.lseamine3
			or unit.location == object.lseamine4
			or unit.location == object.lseamine5
			or unit.location == object.lseamine6)
			then	
				civ.deleteUnit(unit)
			end 
		end
	end	
	
	-- ===========================================================
	-- ADVANCED RADAR RESEARCH & SYSTEM TECHNOLOGIES
	--	Display message if Allies discover Advanced Radar Research tech
	if civ.hasTech(object.pAllies, object.aAdvRadarResearch) then
		gen.justOnce("Allies Discover Advanced Radar Research", function ()
			text.simple("The Allies have discovered the 'Advanced Radar Research' technology! This allows you to research the 'Advanced Radar System' advance which will on average double the accuracy of all your radar capable units.","Allies Discover Advanced Radar Research!",object.monexone)
		end)
	end	

	--	Display message if Allies discover Advanced Radar System tech
	if civ.hasTech(object.pAllies, object.aAdvRadarSystem) then
		gen.justOnce("Allies Discover Advanced Radar System", function ()
			text.simple("The Allies have discovered the 'Advanced Radar System' technology! This will on average double the accuracy of all of your radar capable units.","Allies Discover Advanced Radar System!",object.monexone)
		end)
	end	
	
	-- ===========================================================
	-- LUFTFLOTTE 2 RECEIVES EXTRA SUPPLIES EACH TIME ALLIES SPEND OVER 1000 SUPPLIES ON REPLACEMENTS AND MUNITIONS
	if turn >= 1 then
		if (counter.value("# Supplies Spent in Campaign") + counter.value("# Supplies Spent on Munitions"))	> counter.value("# Total Allied Supplies Spent") then
			object.pAxis.money = object.pAxis.money - param.bonusGermanAir	
			counter.add("# Total Allied Supplies Spent",1000)
			if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
				text.simple("AXIS MESSAGE:\n^\r^Luftflotte 2 receives " .. param.bonusGermanAir .." bonus supplies in response\n^to Allied war effort!","Luftflotte 2 Bonus Supplies!",object.mLuftflotte2)	
			end
		end	
	end

	-- FOR TESTING: Display current German treasury supply level
	if turn >= 1 and flag.value("Display Axis Testing Messages") == true then
		text.simple("ON TURN: The German theater depots currently have " .. object.pAxis.money .. " supplies!","German Theater Supplies!",object.m10thArmy)
	end

	-- EXTRA GERMAN AIR UNITS WHEN REACH 1000 SUPPLIES
	if turn >= 1 then
		if (flag.value("Gustav Line is breached") == false and flag.value("German 10th Army Cutoff") == false) and object.pAxis.money >= param.extraGermanAir then 
			local randomNumber = math.random(6)
			
			if randomNumber == 1 then
				gen.createUnit(object.uMe109G, object.pAxis, {{80,76,0},{91,65,0},{107,83,0},{111,107,0},{89,39,0}}, {count=1, randomize=true, scatter=true, veteran=false})
				object.pAxis.money = object.pAxis.money - param.extraGermanAir_Me109
			elseif randomNumber == 2 then
				gen.createUnit(object.uMe109G, object.pAxis, {{80,76,0},{91,65,0},{107,83,0},{111,107,0},{89,39,0}}, {count=1, randomize=true, scatter=true, veteran=false})
				object.pAxis.money = object.pAxis.money - param.extraGermanAir_Me109
			elseif randomNumber == 3 then
				gen.createUnit(object.uFW190A, object.pAxis, {{80,76,0},{91,65,0},{107,83,0},{111,107,0},{89,39,0}}, {count=1, randomize=true, scatter=true, veteran=false})
				object.pAxis.money = object.pAxis.money - param.extraGermanAir_FW190
			elseif randomNumber == 4 then
				gen.createUnit(object.uMe110, object.pAxis, {{80,76,0},{91,65,0},{107,83,0},{111,107,0},{89,39,0}}, {count=1, randomize=true, scatter=true, veteran=false})
				object.pAxis.money = object.pAxis.money - param.extraGermanAir_Me110
			elseif randomNumber == 5 then
				gen.createUnit(object.uDo217E, object.pAxis, {{80,76,0},{91,65,0},{107,83,0},{111,107,0},{89,39,0}}, {count=2, randomize=true, scatter=true, veteran=false})	
				object.pAxis.money = object.pAxis.money - param.extraGermanAir_Do217
			elseif randomNumber == 6 then
				gen.createUnit(object.uJu88, object.pAxis, {{80,76,0},{91,65,0},{107,83,0},{111,107,0},{89,39,0}}, {count=2, randomize=true, scatter=true, veteran=false})	
				object.pAxis.money = object.pAxis.money - param.extraGermanAir_Ju88
			end	
			if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
				text.simple("AXIS MESSAGE:\n^\r^Luftflotte 2 receives reinforcements in Italy (Gustav: ".. randomNumber ..")!","Luftflotte 2 Reinforcements!",object.mLuftflotte2)	
			end
		elseif (flag.value("Gustav Line is breached") == true or flag.value("German 10th Army Cutoff") == true) and object.pAxis.money >= param.extraGermanAir then
			local randomNumber = math.random(6)
			
			if randomNumber == 1 then
				gen.createUnit(object.uMe109G, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {count=1, randomize=true, scatter=true, veteran=false})
				object.pAxis.money = object.pAxis.money - param.extraGermanAir_Me109
			elseif randomNumber == 2 then
				gen.createUnit(object.uMe109G, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {count=1, randomize=true, scatter=true, veteran=false})
				object.pAxis.money = object.pAxis.money - param.extraGermanAir_Me109
			elseif randomNumber == 3 then
				gen.createUnit(object.uFW190A, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {count=1, randomize=true, scatter=true, veteran=false})
				object.pAxis.money = object.pAxis.money - param.extraGermanAir_FW190
			elseif randomNumber == 4 then
				gen.createUnit(object.uMe110, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {count=1, randomize=true, scatter=true, veteran=false})
				object.pAxis.money = object.pAxis.money - param.extraGermanAir_Me110
			elseif randomNumber == 5 then
				gen.createUnit(object.uDo217E, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {count=2, randomize=true, scatter=true, veteran=false})	
				object.pAxis.money = object.pAxis.money - param.extraGermanAir_Do217
			elseif randomNumber == 6 then
				gen.createUnit(object.uJu88, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {count=2, randomize=true, scatter=true, veteran=false})	
				object.pAxis.money = object.pAxis.money - param.extraGermanAir_Ju88
			end	
			if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
				text.simple("AXIS MESSAGE:\n^\r^Luftflotte 2 receives reinforcements in Italy  (Gothic: ".. randomNumber ..")!","Luftflotte 2 Reinforcements!",object.mLuftflotte2)	
			end
		end
	end	

	if turn >= 1 and flag.value("Display Axis Testing Messages") == true then
		text.simple("ON TURN: The German theater depots currently have " .. object.pAxis.money .. " supplies!","German Theater Supplies!",object.m10thArmy)
	end

	-- ====================================================================
	-- ALLIES 
	-- ====================================================================	
	-- ALLIED - REINFORCEMENTS PLACEMENT
	
	-- FOR PRINT OUTS OF TOWN TILES OWNED BY WHAT POWER
	-- if turn >= 1 then
		-- -- civ.ui.text("This ia a text print out test!")
		-- print("Print Bonifacio: " .. tostring(object.ltownBonifacio.owner))
		-- print("Print Cassino: " .. tostring(object.ltownCassino.owner))
		-- print("Print Formia: " .. tostring(object.ltownFormia.owner))
		-- print("Print Giola Tauro: " .. tostring(object.ltownGiolaTauro.owner))
		-- print("Print Massa: " .. tostring(object.ltownMassa.owner))
		-- print("Print Ortona: " .. tostring(object.ltownOrtona.owner))
		-- print("Print Pistoia: " .. tostring(object.ltownPistoia.owner))
		-- print("Print Salerno: " .. tostring(object.ltownSalerno.owner))
		-- print("Print Salo: " .. tostring(object.ltownSalo.owner))
		-- print("Print Sciacci: " .. tostring(object.ltownSciacci.owner))
		-- print("Print Transport: " .. tostring(object.ltransportPoint.owner)) -- tile (129,185,0) where truck transported Allied units are "temporarily" relocated
		
	-- end

	-- SPECIAL BULLETIN NOTICE THAT ALL LANDING CRAFT WILL BE WITHDRAWN AT THE START OF TURN 3
	if turn == 2 then -- July II, 1943
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^At the beginning of next turn, the week of " .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ", all\n^Operation Husky landing craft will be withdrawn from operations and\n^be returning to port for refurbishing!\n^\r^As such, you MUST unload any cargo from these vessels prior to the\n^end of THIS turn, otherwise they will be returned to Tunis!\n^\r^Notice: This does not include landing craft which have been converted\n^to Beachhead units!"))
	end	

	-- On turn 2 disband temporary Supply Hub located on island of Pantelleria
	if turn == 2 then
		for unit in civ.iterateUnits() do 
			if unit.type == object.uSupplyHub and unit.location == object.lPantelleriaNorth then	
				civ.ui.text("The temporary " ..tostring(unit.type.name).. " at (".. unit.x .. "," .. unit.y .. "," .. unit.z ..") was dispanded!")
				civ.deleteUnit(unit)
				civ.ui.redrawTile(unit.location)
			end 
		end
		gen.setTerrainType({21,147,0},10)
	end	

	-- REMAINING 82ND US AIRBORNE REGIMENTS ARRIVE
	if turn == 2 then -- July II, 1943
		text.simple("The 504th and 325Gth regiments of the US 82nd Airborne Division deploy on the Pantelleria island airbase!","American Reinforcements",object.mAFHQ)
		-- Setting the terrain to Ocean (type = 10) allows designer to place Waco Gliders (which is a domain = 2 (sea) unit) on a land airbase tile	
			gen.setTerrainType(object.lAirbasePantelleria,10) 
			gen.createUnit(object.uWacoGlider, object.pAllies, {{18,148,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uC47Dakota, object.pAllies, {{18,148,0}}, {count=1, randomize=false, veteran=false})
		-- Immediately after placing Waco Glider on airbase tile, reset terrain to its original 'Grassland' (type = 6)
			gen.setTerrainType(object.lAirbasePantelleria,6) 
			gen.createUnit(object.u82ndAirborne, object.pAllies, {{18,148,0}}, {count=2, randomize=false, veteran=false})
				
		-- Update the 82nd Airborne unit max total allowed
		counter.add("# 82nd Airborne",2)
		
		-- Update the US Infantry total allowed
		counter.add("# Total US Infantry",2)
	end

	-- REMAINING 1ST UK AIRBORNE REGIMENTS ARRIVE
	if turn == 2 then -- July II, 1943
		text.simple("The 2nd and 4th Parachute Brigades of the British 1st Airborne Division deploy on Valetta island airbase!","British Reinforcements",object.mAFHQ)
		-- Setting the terrain to Ocean (type = 10) allows designer to place Waco Gliders (which is a domain = 2 (sea) unit) on a land airbase tile
			gen.setTerrainType(object.lAirbaseValetta,10) 
			gen.createUnit(object.uWacoGlider, object.pAllies, {{39,177,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uC47Dakota, object.pAllies, {{39,177,0}}, {count=1, randomize=false, veteran=false})
		-- Immediately after placing Waco Glider on airbase tile, reset terrain to its original 'Grassland' (type = 6)
			gen.setTerrainType(object.lAirbaseValetta,6) 
			gen.createUnit(object.u1stAirborne, object.pAllies, {{39,177,0}}, {count=2, randomize=false, veteran=false})
		
		-- Update the 1st Airborne unit max total allowed
		counter.add("# 1st Airborne",2)
		counter.add("# Total UK Infantry",2)
	end

	-- USAAF 47TH AND 2686TH BOMBARDMENT WINGS ARRIVE 
	if turn == 2 then -- July II, 1943
		text.simple("The USAAF 47th and 2686th Bombardment Wings of the Northwest African Strategic Air Force deploy in Tunis!","USAAF Reinforcements",object.mAFHQ)
		gen.createUnit(object.uB25Mitchell, object.pAllies, {{0,136,0}}, {count=2, randomize=false, veteran=false})
		gen.createUnit(object.uB26Marauder, object.pAllies, {{0,136,0}}, {count=2, randomize=false, veteran=false})

		counter.add("# B-25",2)
		counter.add("# B-26",2)
		counter.add("# Total US Bombers",4)
	end

	-- RAF TACTICAL BOMBER GROUP ARRIVES 
	if turn == 2 then -- July II, 1943
		text.simple("The RAF Tactical Bomber Force of the Northwest African Strategic Air Force deploys in Valetta!","RAF Reinforcements",object.mAFHQ)
		gen.createUnit(object.uBostonIII, object.pAllies, {{40,180,0}}, {count=1, randomize=false, veteran=false})
		gen.createUnit(object.uVickersWellington, object.pAllies, {{40,180,0}}, {count=3, randomize=false, veteran=false})

		counter.add("# Boston III",1)
		counter.add("# Vickers Wellington",3)
		counter.add("# Total UK Bombers",4)
	end
	
	-- ALL LANDING CRAFT ARE WITHDRAWN
	if turn == 3 then -- July III, 1943
		
		-- Calculate how many landing craft are still in play at begining of 4th turn and add those that were withrawn to Valetta to determine the number that will be available for 2nd invasion
		local totalLandingCraft = 0
		local landingCraftFor2ndInvasion = 0
		local a = 0
		for unit in civ.iterateUnits() do
			if unit.owner == object.pAllies and unit.type == object.uLandingCraft then
				totalLandingCraft = totalLandingCraft + 1
			end
		end	
		landingCraftFor2ndInvasion = (totalLandingCraft + counter.value("# Landing Craft Withdrawn during 1st Invasion")) - param.availableLandingCraft2ndInvasion
		
		while( a < landingCraftFor2ndInvasion )	do
			counter.add("# Landing Craft Available for 2nd Invasion",1)
			a = a + 1
		end
		print("Total Landing Craft: There will be " .. tostring(counter.value("# Landing Craft Available for 2nd Invasion")) .. " available for the next invasion")
		
		-- Then teleport any unit still on board landing craft to Tunis...
		for unit in civ.iterateUnits() do
			if unit.owner == object.pAllies 
			and unit.type.domain == 0 
			and unit.location.baseTerrain == object.bOcean
			and (gen.unitTypeOnTile(unit.location,object.uLandingCraft)) then
				civ.teleportUnit(unit, object.lTunis)
			end
		end
		-- and then remove all remaining landing craft from the map
		for activeUnit in civ.iterateUnits() do
		   if activeUnit.owner == object.pAllies and activeUnit.type == object.uLandingCraft then 
				civ.deleteUnit(activeUnit)
			end
		end
		text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " all landing craft for Operation Husky have returned to port!\n^\r^A total of " .. tostring(counter.value("# Landing Craft Available for 2nd Invasion")) .. " will be available for the next invasion.","Special Bulletin",object.mAFHQ)

	end
	
	-- SEE commandDecision.lua FOR HANDLING (DEPLOYMENT AND WITHDRAWAL) OF 2ND INVASION LANDING CRAFT	

	-- USAAF 5TH BOMBARDMENT WING ARRIVES 
	if turn == 3 then -- July III, 1943
		text.simple("The USAAF 5th Bombardment Wing of the Northwest African Strategic Air Force deploys in Tunis!","USAAF Reinforcements",object.mAFHQ)
		gen.createUnit(object.uB17Fortress, object.pAllies, {{0,136,0}}, {count=2, randomize=false, veteran=false})

		counter.add("# B-17",2)
		counter.add("# Total US Bombers",2)
	end
	
	-- 4TH GOUMIERS MOUNTAIN REGIMENT ARRIVES
	if turn == 3 then -- July III, 1943
		text.simple("The 4th Goumiers Marocain regiment deploys in Tunis!","Free French Reinforcements",object.mAFHQ)
		gen.createUnit(object.uGoumierMarocain, object.pAllies, {{0,136,0}}, {randomize=false, veteran=false})
		flag.setTrue("Free French units arrive")
		flag.setTrue("Goumier Marocain units arrive")

		counter.add("# Goumier Marocain",1)
		counter.add("# Total Free French",1)
	end

	-- 9TH US INFANTRY DIVISION ARRIVES
	if turn == 4 then -- July IV, 1943
		text.simple("The US 9th Infantry Division and the attached units of the US Provisional Corps deploy in Tunis!","American Reinforcements",object.mAFHQ)
		gen.createUnit(object.uGIVeteran, object.pAllies, {{0,136,0}}, {count=3, randomize=false, veteran=false})
		gen.createUnit(object.uUSTruck, object.pAllies, {{0,136,0}}, {count=1, randomize=false, veteran=false})
		gen.createUnit(object.uM10Wolverine, object.pAllies, {{0,136,0}}, {count=1, randomize=false, veteran=false})
		gen.createUnit(object.uM4A1ShermanInd, object.pAllies, {{0,136,0}}, {count=1, randomize=false, veteran=false})
		gen.createUnit(object.uM8Greyhound, object.pAllies, {{0,136,0}}, {count=1, randomize=false, veteran=false})
		gen.createUnit(object.uM2A1105mmArt, object.pAllies, {{0,136,0}}, {count=2, randomize=false, veteran=false})
		gen.createUnit(object.uM1155mmArt, object.pAllies, {{0,136,0}}, {count=1, randomize=false, veteran=false})
		gen.createUnit(object.uM157mmAT, object.pAllies, {{0,136,0}}, {count=1, randomize=false, veteran=false})
		gen.createUnit(object.uUSBazooka, object.pAllies, {{0,136,0}}, {count=1, randomize=false, veteran=false})
		gen.createUnit(object.uLibertyShip, object.pAllies, {{0,136,0}}, {count=1, randomize=false, veteran=false})

		counter.add("# GI Veteran",3)
		counter.add("# US Bazooka",1)
		counter.add("# M8 Greyhound",1)
		counter.add("# M10 Wolverine",1)
		counter.add("# M4A1 Ind",1)
		counter.add("# US Truck",1)
		counter.add("# M2A1 105mm Art",2)
		counter.add("# M1 155mm Art",1)
		counter.add("# M1 57mm AT",1)
		
		counter.add("# Total US Infantry",4)
		counter.add("# Total US Armored",4)
		counter.add("# Total US Artillery",4)
		
		counter.add("# Liberty Ship",1)
		counter.add("# Total Naval Units",1)			
		
	end

	-- PATTON: SLAPPING INCIDENT (see onTribeTurnBegin.lua)
	if turn >= 4 and flag.value("Sicily is Conquered") == true then 
		gen.justOnce("Patton Slapping Incident",function()
			text.setLinesPerWindow(15)
				text.simple("****************** COMMAND DECISION #".. counter.add("Command Decision #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^General Eisenhower it has come to your attention that on two\n^separate occasions that General Patton physically slapped\n^American soldiers in front of staff and the wounded while visiting\n^army field hospitals. Should these events become known back in\n^the states, the public outcry demanding the general's removal\n^may become difficult to avoid.\n^\r^Though you believe the general indispensable to the allied war\n^effort due to his aggressive and winning leadership in the bitter\n^battles which are yet to come, you must make a decision:\n^\r^You can demand he apologize to all the concerned parties in\n^person and retain his services in the coming invasion of the Italian\n^mainland, which may well help shorten the campaign, but assume\n^the cost of the political backlash home.\n^Result: Keep Gen. Patton, lose " .. param.pattonReprimandCost .. " supplies AND shorten the\n^game by 8 TURNS.\n^\r^OR\n^\r^You can demand he apologize to all the concerned parties in person\n^but quietly reassign him to England, where his services may still\n^be used in a future role, in the hope that the backlash will subside\n^overtime.\n^\r^Result: Receive " .. param.clarkTakesCommandCost .. " supplies and Gen. Clark takes over command.","Patton Incident",object.mGeorgePatton)
			text.setLinesPerWindow(12)
			
			local pattonReprimandCost = param.pattonReprimandCost
			
			if object.pAllies.money >= pattonReprimandCost then
				local menuOptions = {[1]="No, do not remove Gen. Patton (costs ".. param.pattonReprimandCost .. " supplies).", [2]="Yes, reassign Gen. Patton to England (receive ".. param.clarkTakesCommandCost .. " supplies)."}
				local menuText = text.substitute("Do you wish to maintain or reassigned General Patton (for %STRING1). How shall you proceed?",
				-- {activeUnit.type.name, text.money(pattonReprimandCost),crossingUpgradeMoveCost1})
				{text.money(pattonReprimandCost)})
				local menuTitle = "Reassign General Patton?"
				
				local menuImage = civ.ui.loadImage("Images\\GeorgePatton.bmp")
				local choice = text.menu(menuOptions,menuText,menuTitle,menuImage) -- If you want to include an image in the pop up box
				-- local choice = text.menu(menuOptions,menuText,menuTitle) -- If you don't want to include an image in the pop up box
				
				if choice == 1 then
					text.simple("General Eisenhower you opted to maintain General Patton\n^in his post and therefore he will take command of the newly\n^constituted 5th Army in its invasion of the Italian mainland!\n^\r^As a consequence, ".. pattonReprimandCost .." supplies are subtracted from the\n^theater depots as payment for the political backlash.\n^\r^Note: the game is shortened by 8 turns to reflect Patton's\n^greater leadership abilities which means it should end on\n^turn #81 (March I, 1945).","Patton Remains in Command!",object.mGeorgePatton)
					object.pAllies.money = object.pAllies.money - pattonReprimandCost
					flag.setTrue("Game ends March I, 1945")
				elseif choice == 2 then
					-- activeUnit.owner.money = activeUnit.owner.money + pattonReprimandCost
					for unit in civ.iterateUnits() do 
						if unit.type == object.uGenPatton then	
							civ.deleteUnit(unit)
						end
					end	
					text.simple("General Eisenhower you opted to reassign General Patton\n^to England.\n^\r^As such, General Mark Clark immediately succeeds him\n^and will take command of the newly constituted 5th Army set\n^to invade mainland Italy.\n^\r^As another consequence, ".. param.clarkTakesCommandCost .." supplies are added to the\n^theater depots as the political backlash has been avoided!","Mark Clark Takes Over Command!",object.mMarkClark)
					gen.createUnit(object.uGenClark, object.pAllies, {{44,140,0},{73,155,0}}, {randomize=false, veteran=false})
					object.pAllies.money = object.pAllies.money + param.clarkTakesCommandCost
					flag.setTrue("Patton dismissed")
				end
			
			elseif object.pAllies.money <= pattonReprimandCost then
				text.simple(commanderAFHQ_Base .. " the cost of ".. pattonReprimandCost .." supplies\n^exceeds the amount currently in the theater depots\n^and therefore you don't have the political capital\n^to prevent General Patton from being automatically\n^reassigned to England!\n^\r^General Mark Clark immediately succeeds him and ".. param.clarkTakesCommandCost .."\n^supplies are added to the theater depots!","Mark Clark Takes Over Command!",object.mMarkClark)
				object.pAllies.money = object.pAllies.money + param.clarkTakesCommandCost
				for unit in civ.iterateUnits() do 
					if unit.type == object.uGenPatton then	
						civ.deleteUnit(unit)
					end
				end	
				gen.createUnit(object.uGenClark, object.pAllies, {{44,140,0},{73,155,0}}, {randomize=false, veteran=false})
				flag.setTrue("Patton dismissed")

			end
		end)		
	end	
	
	-- 78TH BRITISH INFANTRY DIVISION ARRIVES	
	if turn == 5 then -- Aug I, 1943
		text.simple("The British 78th Infantry Division deploys in Valetta!","British Reinforcements",object.mAFHQ)
		gen.createUnit(object.uBritishRegular, object.pAllies, {{40,180,0}}, {count=3, randomize=false, veteran=false})

		counter.add("# British Regular",3)
		counter.add("# Total UK Infantry",3)
	end

	-- 2ND US ARMORED DIVISION WITHDRAWS -- the division is withdrawn 1 turn after the capture of Messina (see onCityTaken.lua)
	-- 1ST US INFANTRY DIVISION WITHDRAWS -- the division is withdrawn, along with the 2nd Armored Division, 1 turn after the capture of Messina
	if flag.value("Messina is captured") == true then -- 
        gen.justOnce("1st US Infantry Division withdraws",function()
			withdrawTroops(object.uUS1stInfantry,3,"1st US Infantry")

			counter.subtract("# 1st Infantry",3)
			counter.subtract("# Total US Infantry",3)
			
			flag.setTrue("1st Infantry units withdrawn")			
		end)	
    end	
	
	-- 36TH US INFANTRY DIVISION ARRIVES
	if turn == 9 then -- Sept I, 1943
		if flag.value("Sea Mines removed from Palermo") == true then
			text.simple("The US 36th Infantry Division deploys in Palermo!","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{44,140,0}}, {count=3, randomize=false, veteran=false})
			
			counter.add("# GI Regular",3)
			counter.add("# Total US Infantry",3)
		else
			text.simple("The US 36th Infantry Division deploys in Tunis!","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{0,136,0}}, {count=3, randomize=false, veteran=false})
			
			counter.add("# GI Regular",3)
			counter.add("# Total US Infantry",3)
		end	
	end
	
	-- 46TH BRITISH INFANTRY DIVISION ARRIVES
	if turn == 9 then -- Sept I, 1943
		if flag.value("Sea Mines removed from Catania") == true then
			text.simple("The British 46th Infantry Division deploys in Catania!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uBritishRegular, object.pAllies, {{62,162,0}}, {count=3, randomize=false, veteran=false})

			counter.add("# British Regular",3)
			counter.add("# Total UK Infantry",3)
		else
			text.simple("The British 46th Infantry Division deploys in Valetta!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uBritishRegular, object.pAllies, {{40,180,0}}, {count=3, randomize=false, veteran=false})

			counter.add("# British Regular",3)
			counter.add("# Total UK Infantry",3)
		end	
	end	
	
	-- 56TH BRITISH INFANTRY DIVISION ARRIVES
	if turn == 10 then -- Sept II, 1943
		if flag.value("Sea Mines removed from Catania") == true then
			text.simple("The British 56th Infantry Division and the attached units of the British V Corps deploy in Catania!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uBritishRegular, object.pAllies, {{62,162,0}}, {count=3, randomize=false, veteran=false})
			gen.createUnit(object.uBritishPIAT, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uBritishMG, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uDaimlerAC, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uUKTruck, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.u25pdrMKIIArt, object.pAllies, {{62,162,0}}, {count=2, randomize=false, veteran=false})
			gen.createUnit(object.u55inchArt, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.u6pdrAT, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uLibertyShip, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
			
			counter.add("# British Regular",3)
			counter.add("# British PIAT",1)
			counter.add("# British MG",1)
			counter.add("# Daimler AC",1)
			counter.add("# UK Truck",1)
			counter.add("# 25pdr MKII Art",2)
			counter.add("# 55inch Art",1)
			counter.add("# 6pdr AT",1)
			
			counter.add("# Total UK Infantry",5)
			counter.add("# Total UK Armored",2)
			counter.add("# Total UK Artillery",4)

			counter.add("# Liberty Ship",1)
			counter.add("# Total Naval Units",1)

		else
			text.simple("The British 56th Infantry Division and the attached units of the British V Corps deploy in Valetta!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uBritishRegular, object.pAllies, {{40,180,0}}, {count=3, randomize=false, veteran=false})
			gen.createUnit(object.uBritishPIAT, object.pAllies, {{40,180,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uBritishMG, object.pAllies, {{40,180,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uDaimlerAC, object.pAllies, {{40,180,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uUKTruck, object.pAllies, {{40,180,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.u25pdrMKIIArt, object.pAllies, {{40,180,0}}, {count=2, randomize=false, veteran=false})
			gen.createUnit(object.u55inchArt, object.pAllies, {{40,180,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.u6pdrAT, object.pAllies, {{40,180,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uLibertyShip, object.pAllies, {{40,180,0}}, {count=1, randomize=false, veteran=false})
			
			counter.add("# British Regular",3)
			counter.add("# British PIAT",1)
			counter.add("# British MG",1)
			counter.add("# Daimler AC",1)
			counter.add("# UK Truck",1)
			counter.add("# 25pdr MKII Art",2)
			counter.add("# 55inch Art",1)
			counter.add("# 6pdr AT",1)
			
			counter.add("# Total UK Infantry",5)
			counter.add("# Total UK Armored",2)
			counter.add("# Total UK Artillery",4)

			counter.add("# Liberty Ship",1)
			counter.add("# Total Naval Units",1)
		end	
	end		

	-- INDIAN 8TH INFANTRY DIVISION = arrives up to 3 turns after capture of city of Taranto (see onCityTaken.lua)

	-- 34TH US INFANTRY DIVISION ARRIVES
	if turn == 11 then -- Sept III, 1943
		if flag.value("Sea Mines removed from Palermo") == true then
			text.simple("The US 34th Infantry Division deploys in Palermo!","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{44,140,0}}, {count=3, randomize=false, veteran=false})
			counter.add("# GI Regular",3)
			counter.add("# Total US Infantry",3)
		else
			text.simple("The US 34th Infantry Division deploys in Tunis!","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{0,136,0}}, {count=3, randomize=false, veteran=false})
			counter.add("# GI Regular",3)
			counter.add("# Total US Infantry",3)
		end	
	end	

	-- 7TH UK ARMORED DIVISION ARRIVES
	local uk7thArmValettaLocationList = {{40,180,0}}
	local uk7thArmCataniaLocationList = {{62,162,0}}
	if turn == 11 then -- Sept III, 1943
		if flag.value("Allies at War") == true then -- 2nd condition to apply if true
			local uk7thArmLocationList = uk7thArmValettaLocationList
			local uk7thArmLocationDesc = "Valetta"
			if flag.value("Sea Mines removed from Catania") == true then -- 1st condition to apply if true
				uk7thArmLocationList = uk7thArmCataniaLocationList
				uk7thArmLocationDesc = "Catania"
			end
			gen.justOnce("7th UK Armored Division arrives",function()
				gen.createUnit(object.uShermanIIa, object.pAllies, uk7thArmLocationList, {count=3, randomize=false, veteran=false})
				gen.createUnit(object.uBritishMotorTroops, object.pAllies, uk7thArmLocationList, {count=2, randomize=false, veteran=false})
				gen.createUnit(object.uBishop25pdr, object.pAllies, uk7thArmLocationList, {count=1, randomize=false, veteran=false})	
				text.simple("The British 7th Armored Division deploys in " .. uk7thArmLocationDesc .. "!","British Reinforcements",object.mAFHQ)
				flag.setTrue("British Armored Division Deployed")

				counter.add("# Sherman IIa",3)
				counter.add("# Bren Carrier",2)
				counter.add("# Bishop 25pdr",1)
				
				counter.add("# Total UK Armored",5)
				counter.add("# Total UK Artillery",1)
			end)
		end
	end	

	-- 2ND AMPHIBIOUS INVASION OPTION TIME LIMIT MESSAGE
	if turn == 12 and flag.value("2nd Invasion Launched") == false then -- Sept IV, 1943
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Next Week I of October 1943, will be your last chance to\n^launch any of the three 2nd amphibious invasion options.\n^\r^Failure to do so, within this timeframe, will only leave\n^you with the reduced final 3rd invasion option, set to begin no\n^earlier than WEEK I of November 1943 (turn#17)."))
	end
	
	-- 2ND AMPHIBIOUS INVASION OPTION NO LONGER POSSIBLE MESSAGE
	if turn == 14 and flag.value("2nd Invasion Launched") == false then -- Oct II, 1943
		-- civ.playSound("Telegraph.wav")
		counter.add("# Landing Craft Available for 3rd Invasion",3)
		civ.ui.text("It is no longer possible to activate the 2nd invasion options!")
		flag.setTrue("2nd Invasion Launched")
	end	
	
	-- 2ND NEW ZEALAND DIVISION ARRIVES
	local nzDivTarantoLocationList = {{120,138,0}}
	local nzDivCataniaLocationList = {{62,162,0}}
	if turn == 14 then -- Oct II, 1943
		if flag.value("Sea Mines removed from Catania") == true then	
			local nzDivLocationList = nzDivCataniaLocationList
			local nzDivLocationDesc = "Catania"
			if flag.value("Sea Mines removed from Taranto") == true then
				nzDivLocationList = nzDivTarantoLocationList
				nzDivLocationDesc = "Taranto"
			end
			gen.justOnce("2nd NZ Division arrives",function()
				gen.createUnit(object.uNZShermanIII, object.pAllies, nzDivLocationList, {count=3, randomize=false, veteran=false})
				gen.createUnit(object.uNZMotInf, object.pAllies, nzDivLocationList, {count=2, randomize=false, veteran=false})	
				gen.createUnit(object.uNZ25pdrArt, object.pAllies, nzDivLocationList, {count=1, randomize=false, veteran=false})
				text.simple("The New Zealand 2nd Division deploys in " .. nzDivLocationDesc .. "!","British Reinforcements",object.mAFHQ)
				flag.setTrue("New Zealand units arrive")

				counter.add("# NZ Mot Inf",2)
				counter.add("# NZ Sherman",3)
				counter.add("# NZ 25pdr Art",1)
				
				counter.add("# Total New Zealand",6)
			end)
		end
	end

	-- 3RD AMPHIBIOUS INVASION OPTION BECOMES AVAILABLE MESSAGE
	if turn == 17 then -- Nov I, 1943
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^As of this week up till the end of Week IV of January 1944, you will\n^have the option to launch a 3rd amphibious invasion along the\n^Italian Adriatic or Tyrrhenian coastlines.\n^\r^Depending on which option you select, a maximum of " .. counter.value("# Landing Craft Available for 3rd Invasion") .. " ".. object.uLandingCraft.name .."\n^will be made available to you either in the port of Bari, Napoli or Ostia.\n^\r^You can activate the operation by going to the Command Decision\n^menu (keyboard menu 5) and selecting the 'Activate 3rd Invasion'\n^box."))
		flag.setTrue("3rd Invasion Available")
	end
	
	-- SEE commandDecision.lua FOR HANDLING (DEPLOYMENT AND WITHDRAWAL) OF 3RD INVASION LANDING CRAFT	

	-- 1ST US ARMORED DIVISION ARRIVES
	local us1stArmTunisLocationList = {{0,136,0}}
	local us1stArmPalermoLocationList = {{44,140,0}}
	if turn == 17 then -- Nov I, 1943
		if flag.value("Allies at War") == true then -- 2nd condition to apply if true
			local us1stArmLocationList = us1stArmTunisLocationList
			local us1stArmLocationDesc = "Tunis"
			if flag.value("Sea Mines removed from Palermo") == true then -- 1st condition to apply if true
				us1stArmLocationList = us1stArmPalermoLocationList
				us1stArmLocationDesc = "Palermo"
			end
			gen.justOnce("1st US Armored Division arrives",function()
				gen.createUnit(object.uM4A1Sherman, object.pAllies, us1stArmLocationList, {count=2, randomize=false, veteran=false})
				gen.createUnit(object.uM4A376WSherman, object.pAllies, us1stArmLocationList, {count=1, randomize=false, veteran=false})
				gen.createUnit(object.uM5Stuart, object.pAllies, us1stArmLocationList, {count=1, randomize=false, veteran=false})	
				gen.createUnit(object.uArmoredInfantry, object.pAllies, us1stArmLocationList, {count=2, randomize=false, veteran=false})
				gen.createUnit(object.uM12155mmHMC, object.pAllies, us1stArmLocationList, {count=1, randomize=false, veteran=false})
				gen.createUnit(object.uLibertyShip, object.pAllies, us1stArmLocationList, {count=1, randomize=false, veteran=false})	
				text.simple("The US 1st Armored Division deploys in " .. us1stArmLocationDesc .. "!","American Reinforcements",object.mAFHQ)
				flag.setTrue("US Armored Division is Active")
				flag.setTrue("M4A3 76(W) enters US armor inventory")	
				
				counter.add("# Armored Infantry",2)
				counter.add("# M5 Stuart",1)
				counter.add("# M4A1",2)
				counter.add("# M4A3 (76)",1)
				counter.add("# M12 155mm HMC",1)
				
				counter.add("# Total US Armored",6)
				counter.add("# Total US Artillery",1)

				counter.add("# Liberty Ship",1)
				counter.add("# Total Naval Units",1)
			end)
		end
	end	

    -- On turn #19 1ST UK AIRBORNE WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)
	-- On turn #19 325G and 505 Rgt OF 82ND AIRBORNE WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)

	-- 1ST SSF REGIMENT ARRIVES
	if turn == 20 then -- Nov IV, 1943
		if flag.value("Sea Mines removed from Palermo") == true then
			text.simple("The 1st Special Services Force Regiment deploys in Palermo!","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uRanger, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
			counter.add("# US Ranger",1)
			counter.add("# Total US Infantry",1)
		else
			text.simple("The 1st Special Services Force Regiment deploys in Tunis!","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uRanger, object.pAllies, {{0,136,0}}, {count=1, randomize=false, veteran=false})
			counter.add("# US Ranger",1)
			counter.add("# Total US Infantry",1)
		end	
	end		

	-- On turn #21 9TH US INFANTRY DIVISION WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section) 
	
	-- 1ST ITALIAN MOTORIZED INFANTRY DIVISION ARRIVES	
	if turn >= 21 and turn < 22 and flag.value("Italy Surrendered") == true then -- not before Dec I, 1943
		gen.justOnce("1st Italian Motorized Infantry Division arrives",function()
			text.simple("The Italian Monarchist regime of King Victor Emmanuel III deploys its first combat unit in support of the Allied war effort, the 1st Motorized Infantry Division, in Taranto!","Italian Monarchist Reinforcements",object.mItalianMonarchist)
			gen.createUnit(object.uFanteriaMotorizzata, object.pAllies, {{120,138,0}}, {count=2, randomize=false, veteran=false})
		end)
	end	
	
	-- SPECIAL BULLETIN NOTICE THAT 7TH UK ARMORED DIVISION TO BE WITHDRAWN AT THE START OF TURN 22
	if turn == 21 then -- Dec I, 1943
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^At the beginning of next turn, " .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ",\n^the British 7th Armored Division will be withdrawn from the theater.\n^\r^As such, you should plan to adjust your frontline accordingly\n^if so required!"))
	end	

	-- On turn #22 7TH BRITISH ARMORED DIVISION WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section) 
	
	-- 1ST BRITISH INFANTRY DIVISION ARRIVES
	if turn == 22 then -- Dec II, 1943
		if flag.value("Sea Mines removed from Messina") == true then
			text.simple("The British 1st Infantry Division deploys in Messina!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uBritishVeteran, object.pAllies, {{73,155,0}}, {count=3, randomize=false, veteran=false})
			counter.add("# British Veteran",3)
			counter.add("# Total UK Infantry",3)
		else
			text.simple("The British 1st Infantry Division deploys in Valetta!","British Reinforcements",object.mAFHQ)			
			gen.createUnit(object.uBritishVeteran, object.pAllies, {{40,180,0}}, {count=3, randomize=false, veteran=false})
			counter.add("# British Veteran",3)
			counter.add("# Total UK Infantry",3)
		end	
	end	

	-- 2ND MOROCCAN INFANTRY DIVISION ARRIVES
	if turn == 22 then -- Dec II, 1943
		text.simple("The 2nd Moroccan Infantry Division deploys in Palermo!","Free French Reinforcements",object.mAFHQ)
		gen.createUnit(object.uTirailleurAlgerien, object.pAllies, {{44,140,0},{0,136,0}}, {count=3, randomize=false, veteran=false})
		flag.setTrue("Free French units arrive")
		flag.setTrue("Tirailleur Algérien units arrive")
		counter.add("# Tirailleur Algerien",3)
		counter.add("# Total Free French",3)
	end	

	-- SPECIAL BULLETIN NOTICE THAT 2 UK DIVISIONS TO BE WITHDRAWN AT THE START OF TURN 23
	if turn == 22 then -- Dec II, 1943
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^At the beginning of next turn, " .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ", \n^2 British Divisions (6 '" .. object.uBritishRegular.name .. "' regiments) will be\n^withdrawn from the theater.\n^\r^As such, you should plan to adjust your frontline accordingly\n^if so required!"))
	end	

    -- On turn #23 50TH AND 51ST BRITISH INFANTRY DIVISIONS WITHDRAW (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)


	-- INDIAN 4TH INFANTRY DIVISION ARRIVES
	if turn == 23 then -- Dec III, 1943
		if flag.value("Sea Mines removed from Taranto") == true then
			gen.createUnit(object.u4thIndianDiv, object.pAllies, {{120,138,0}}, {count=3, randomize=false, veteran=false})
			gen.createUnit(object.uGurkhas, object.pAllies, {{120,138,0}}, {count=1, randomize=false, veteran=false})
			text.simple("The elite 4th Indian Infantry Division arrives in the port of Taranto!","Indian Reinforcements",object.mAFHQ)
			flag.setTrue("4th Indian Infantry units arrive")
			flag.setTrue("Gurkhas units arrived")

			counter.add("# 4th Indian Div",3)
			counter.add("# Gurkhas",1)
			counter.add("# Total Indian",4)
		else
			gen.createUnit(object.u4thIndianDiv, object.pAllies, {{62,162,0}}, {count=3, randomize=false, veteran=false})
			gen.createUnit(object.uGurkhas, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
			text.simple("The elite 4th Indian Infantry Division arrives in the port of Catania!","Indian Reinforcements",object.mAFHQ)
			flag.setTrue("4th Indian Infantry units arrive")
			flag.setTrue("Gurkhas units arrived")

			counter.add("# 4th Indian Div",3)
			counter.add("# Gurkhas",1)
			counter.add("# Total Indian",4)
		end
	end
	
	-- SPECIAL BULLETIN NOTICE THAT GEN. MONTGOMERY IS TO BE REPLACED BY GEN. LEESE THIS TURN
	if turn == 24 then -- Dec IV, 1943 (#24)
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Effective immediately, " .. object.uGenMontgomery.name .. " is being reassigned to\n^England where he is to be given command of the 21st Army Group\n^consisting of all Allied ground forces that will be participating\n^in Operation Overlord.\n^\r^He is to be suceeded by " .. object.uGenLeese.name .. " who will take immediate\n^command of the British Eight Army in Italy!"))
		for unit in civ.iterateUnits() do 
			if unit.type == object.uGenMontgomery then	
				civ.deleteUnit(unit)
				-- gen.replaceUnit(unit,object.uGenLeese)
			end
		end		
		flag.setTrue("General Montgomery Reassigned")
		gen.createUnit(object.uGenLeese, object.pAllies, {{88,106,0},{73,155,0},{40,180,0}}, {randomize=false, veteran=false})
	end		

	-- MARLENE DIETRICH ENTERTAINS THE TROOPS WITH THE SONG 'LILI MARLENE'
	if turn == 24 then -- Dec IV, 1943 (#24)
		civ.playSound("LiliMarleneShort.wav")
		text.setLinesPerWindow(25)
			text.simple("It's Christmas time and the troops of both sides, wherever possible,\n^gather around a radio set to listen to the most popular song of the\n^war, 'Lili Marlene' by the German composer Norbert Schultze and\n^singer Lale Anderson.\n^\r^Originally banned by the Reich's Propaganda Minister, Joseph\n^Goebbels, as being too anti-war, going so far as to jailing Schultze,\n^he was ultimately compelled to relent and let it air due to the\n^clamour for it by the troops.\n^\r^Marlene Dietrich, who worked tirelessly during world war two\n^entertaining allied troops, also recorded both a German version\n^and an English version of the song. Compared to the Anderson\n^recordings, with their strong, marching tempo, Dietrichs versions,\n^which begin with the melancholy strains of an accordion, are slower,\n^sweeter, and more wistful.","Marlene Dietrich Entertains the Troops!",object.mMarleneDietrich)
		text.setLinesPerWindow(12)
	end

	-- 5TH CANADIAN ARMORED DIVISION ARRIVES
	local cdnDivMessinaLocationList = {{73,155,0}}
	local cdnDivCataniaLocationList = {{62,162,0}}
	if turn == 25 then -- Jan I, 1944 (#25)
		if flag.value("Sea Mines removed from Catania") == true then	
			local cdnDivLocationList = cdnDivCataniaLocationList
			local cdnDivLocationDesc = "Catania"
			if flag.value("Sea Mines removed from Messina") == true then
				cdnDivLocationList = cdnDivMessinaLocationList
				cdnDivLocationDesc = "Messina"
			end
			gen.justOnce("5th Canadian Division arrives",function()
				gen.createUnit(object.uCdnShermanIII, object.pAllies, cdnDivLocationList, {count=3, randomize=false, veteran=false})
				gen.createUnit(object.uCdnMotInf, object.pAllies, cdnDivLocationList, {count=1, randomize=false, veteran=false})	
				gen.createUnit(object.uCdnMotorTroops, object.pAllies, cdnDivLocationList, {count=1, randomize=false, veteran=false})
				gen.createUnit(object.uCdnM7105mm, object.pAllies, cdnDivLocationList, {count=1, randomize=false, veteran=false})
				text.simple("The Canadian 5th Armored Division deploys in " .. cdnDivLocationDesc .. "!","Canadian Reinforcements",object.mAFHQ)
				flag.setTrue("Cdn 5th Armored units arrive")

				counter.add("# Cdn Sherman",3)
				counter.add("# Cdn Mot Inf",1)
				counter.add("# Cdn Bren Carrier",1)
				counter.add("# Cdn M7 105mm",1)
				
				counter.add("# Total Canadian",6)
			end)
		end
	end

	-- POLISH 3RD CARPATHIAN INFANTRY DIVISION ARRIVES
	if turn == 25 then -- Jan I, 1944
		if flag.value("Sea Mines removed from Messina") == true then
			gen.createUnit(object.uPolishInfantry, object.pAllies, {{73,155,0}}, {count=3, randomize=false, veteran=false})
			text.simple("The 3rd Polish Carpathian Infantry Division arrives in the port of Messina!","Polish Reinforcements",object.mAFHQ)
			flag.setTrue("Polish units arrive")
			counter.add("# Polish Infantry",3)
			counter.add("# Total Polish",3)
		else
			gen.createUnit(object.uPolishInfantry, object.pAllies, {{62,162,0}}, {count=3, randomize=false, veteran=false})
			text.simple("The 3rd Polish Carpathian Infantry Division arrives in the port of Catania!","Polish Reinforcements",object.mAFHQ)
			flag.setTrue("Polish units arrive")
			counter.add("# Polish Infantry",3)
			counter.add("# Total Polish",3)
		end
	end
	
	-- 3RD ALGERIAN & 9TH COLONIAL INFANTRY DIVISIONS AND 1 GOUMIERS REGIMENT ARRIVES
	if turn == 25 then -- Jan I, 1944
		text.simple("The 3rd Algerien and 9th Colonial Infantry Divisions along with 1 new Goumiers regiment deploy in Palermo!","Free French Reinforcements",object.mAFHQ)
		gen.createUnit(object.uInfanterieColoniale, object.pAllies, {{44,140,0},{0,136,0}}, {count=3, randomize=false, veteran=false})
		gen.createUnit(object.uGoumierMarocain, object.pAllies, {{44,140,0},{0,136,0}}, {count=1, randomize=false, veteran=false})
		gen.createUnit(object.uTirailleurAlgerien, object.pAllies, {{44,140,0},{0,136,0}}, {count=3, randomize=false, veteran=false})
		gen.createUnit(object.uFreeFrench105Art, object.pAllies, {{44,140,0},{0,136,0}}, {count=1, randomize=false, veteran=false})
		gen.createUnit(object.uLibertyShip, object.pAllies, {{44,140,0},{0,136,0}}, {count=1, randomize=false, veteran=false})
		
		counter.add("# Infanterie Coloniale",3)
		counter.add("# Goumier Marocain",1)
		counter.add("# Tirailleur Algerien",3)
		counter.add("# Free French 105Art",1)
		
		counter.add("# Total Free French",8)
				
		counter.add("# Liberty Ship",1)
		counter.add("# Total Naval Units",1)	
	end	
	
	-- FIELD MARSHALL HENRY H. WILSON ASSUMES COMMAND OF AFHQ
	if turn == 26 then
		civ.playSound("Telegraph.wav")
		-- object.pAllies.leader.name = "Henry M. Wilson"
		commanderAFHQ_Base = "Henry M. Wilson"
		
		text.setLinesPerWindow(16)
			text.simple("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Effective immediately, Field Marshall " .. commanderAFHQ_Base.. " has\n^been appointed as Supreme Allied Commander Mediterranean\n^(AFHQ), based in Algiers, and therefore will heretofore\n^exercise strategic control over the campaign in Italy.\n^\r^General Eisenhower has been promoted as commander to\n^Supreme Allied Heaquarters Allied Forces (SHAEF), and\n^been tasked with the preparations for the upcoming\n^Operation Overlord; the invasion of France.","Henry M. Wilson",object.mHenryWilson)
		text.setLinesPerWindow(12)

	end
	
	-- On turn #27 4TH BRITISH TANK BRIGADE WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)

	-- 3RD GOUMIERS MOUNTAIN REGIMENT ARRIVES
	if turn == 28 then -- Jan IV, 1944
		text.simple("The 3rd Goumiers Marocain regiment deploys in Palermo!","Free French Reinforcements",object.mAFHQ)
		gen.createUnit(object.uGoumierMarocain, object.pAllies, {{44,140,0},{0,136,0}}, {randomize=false, veteran=false})
		counter.add("# Goumier Marocain",1)
		counter.add("# Total Free French",1)
	end	

	-- 3RD AMPHIBIOUS INVASION OPTION NO LONGER AVAILABLE  
	if turn == 29 and flag.value("3rd Invasion Available") == true and flag.value("3rd Invasion Launched") == false then -- Feb I, 1944
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^If you haven't already done so, as of this week, you will\n^no longer have the option to launch a 3rd amphibious invasion\n^along the Italian Adriatic or Tyrrhenian coastlines.\n^\r^This will remain true for the remainder of the campaign!"))
		flag.setTrue("3rd Invasion Launched")
		flag.setTrue("3rd Invasion Not-Launched")
	end	
	
	-- POLISH 5TH KRESOWA INFANTRY DIVISION AND 2ND TANK BRIGADE ARRIVES
	if turn == 30 then -- Feb II, 1944
		if flag.value("Sea Mines removed from Messina") == true then
			gen.createUnit(object.uPolishShermanIII, object.pAllies, {{73,155,0}}, {count=2, randomize=false, veteran=false})
			gen.createUnit(object.uPolishInfantry, object.pAllies, {{73,155,0}}, {count=3, randomize=false, veteran=false})
			gen.createUnit(object.uPolish25pdrArt, object.pAllies, {{73,155,0}}, {count=1, randomize=false, veteran=false})
			text.simple("The Polish 5th Kresowa Infantry Division and 2nd Tank Brigade arrive in the port of Messina!","Polish Reinforcements",object.mAFHQ)
			counter.add("# Polish Sherman",2)
			counter.add("# Polish Infantry",3)
			counter.add("# Polish 25pdr Art",1)

			counter.add("# Total Polish",6)
		else
			gen.createUnit(object.uPolishShermanIII, object.pAllies, {{62,162,0}}, {count=2, randomize=false, veteran=false})
			gen.createUnit(object.uPolishInfantry, object.pAllies, {{62,162,0}}, {count=3, randomize=false, veteran=false})
			gen.createUnit(object.uPolish25pdrArt, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
			text.simple("The Polish 5th Kresowa Infantry Division and 2nd Tank Brigade arrive in the port of Catania!","Polish Reinforcements",object.mAFHQ)			
			counter.add("# Polish Sherman",2)
			counter.add("# Polish Infantry",3)
			counter.add("# Polish 25pdr Art",1)

			counter.add("# Total Polish",6)
		end
	end	

	-- 4TH BRITISH INFANTRY DIVISION ARRIVES
	if turn == 32 then -- Feb IV, 1944
		if flag.value("Sea Mines removed from Messina") == true then
			text.simple("The British 4th Infantry Division deploys in Messina!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uBritishRegular, object.pAllies, {{73,155,0}}, {count=3, randomize=false, veteran=false})
			counter.add("# British Regular",3)
			counter.add("# Total UK Infantry",3)
		else
			text.simple("The British 4th Infantry Division deploys in Valetta!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uBritishRegular, object.pAllies, {{40,180,0}}, {count=3, randomize=false, veteran=false})
				counter.add("# British Regular",3)
			counter.add("# Total UK Infantry",3)
		end	
	end	

	-- 4TH MOROCCAN INFANTRY DIVISION ARRIVES
	if turn == 32 then -- Feb IV, 1944
		text.simple("The 4th Moroccan Infantry Division deploys in Palermo!","Free French Reinforcements",object.mAFHQ)
		gen.createUnit(object.uTirailleurAlgerien, object.pAllies, {{44,140,0},{0,136,0}}, {count=3, randomize=false, veteran=false})
		gen.createUnit(object.uFreeFrench105Art, object.pAllies, {{44,140,0},{0,136,0}}, {count=1, randomize=false, veteran=false})
		counter.add("# Tirailleur Algerien",3)
		counter.add("# Free French 105Art",1)
		
		counter.add("# Total Free French",4)
	end	

	-- On turn #33 BRITISH BB SQD's WITHDRAW (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)

	-- SPECIAL BULLETIN NOTICE THAT 2 UK DIVISIONS AND BRITISH V CORPS TO BE WITHDRAWN AT THE START OF TURN 35
	if turn == 34 then -- Mar II, 1944
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^At the beginning of next turn, the " .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ",\n^the following units of 2 British Divisions along with\n^elements of the British V Corps will be withdrawn from the\n^theater and reassigned to 21st Army Group in England, in\n^preparation for Operation Overlord:\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 6 " .. object.uBritishRegular.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 1 " .. object.uBritishMG.name .. " regiment\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 1 " .. object.uDaimlerAC.name .. " battalion\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 2 " .. object.u25pdrMKIIArt.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 1 " .. object.u55inchArt.name .. " regiment\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 1 " .. object.u6pdrAT.name .. " regiment\n^\r^Be forwarned, that up to " .. param.witdrawalMechanizedPenalty .. " supplies will be subracted for 'each'\n^unit that is missing from the total at the time of withdrawal.\n^\r^As such, you should plan to adjust your frontline accordingly\n^if so required!"))
	end	

    -- On turn #35 46TH AND 56TH BRITISH INFANTRY DIVISIONS WITHDRAW (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)
	
	-- On turn #35 504 Rgt OF 82ND AIRBORNE WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)
	
	-- 6TH UK ARMORED DIVISION ARRIVES
	local uk6thArmValettaLocationList = {{40,180,0}}
	local uk6thArmCataniaLocationList = {{62,162,0}}
	if turn == 35 then -- Mar III, 1944
		if flag.value("Allies at War") == true then -- 2nd condition to apply if true
			local uk6thArmLocationList = uk6thArmValettaLocationList
			local uk6thArmLocationDesc = "Valetta"
			if flag.value("Sea Mines removed from Catania") == true then -- 1st condition to apply if true
				uk6thArmLocationList = uk6thArmCataniaLocationList
				uk6thArmLocationDesc = "Catania"
			end
			gen.justOnce("6th UK Armored Division arrives",function()
				gen.createUnit(object.uCromwellMkVII, object.pAllies, uk6thArmLocationList, {count=2, randomize=false, veteran=false})
				gen.createUnit(object.uShermanVCFirefly, object.pAllies, uk6thArmLocationList, {count=1, randomize=false, veteran=false})
				gen.createUnit(object.uBritishMotorTroops, object.pAllies, uk6thArmLocationList, {count=2, randomize=false, veteran=false})
				gen.createUnit(object.uBishop25pdr, object.pAllies, uk6thArmLocationList, {count=1, randomize=false, veteran=false})	
				text.simple("The British 6th Armored Division deploys in " .. uk6thArmLocationDesc .. "!","British Reinforcements",object.mAFHQ)
				
				-- Run bat file to switch to UK 6th Arm Div badges
				text.simple("At this time, save and exit your game. Then run the BOI_1943.bat file and select option '4. Load 6th Arm Div Graphics'.","UK 6th Armored Arrives!",object.mAFHQ)		
				
				flag.setTrue("British Armored Division Deployed")
				flag.setTrue("Sherman Firefly enters UK armor inventory")

				counter.add("# Cromwell Mk VII",2)
				counter.add("# Sherman Firefly",1)
				counter.add("# Bren Carrier",2)
				counter.add("# Bishop 25pdr",1)
				
				counter.add("# Total UK Armored",5)
				counter.add("# Total UK Artillery",1)
			end)
		end
	end
	
	-- 85TH US INFANTRY DIVISION ARRIVES
	if turn == 37 then -- Apr I, 1944
		if flag.value("Sea Mines removed from Palermo") == true then
			text.simple("The US 85th Infantry Division deploys in Palermo","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{44,140,0}}, {count=3, randomize=false, veteran=false})
			counter.add("# GI Regular",3)
			counter.add("# Total US Infantry",3)
		else
			text.simple("The US 85th Infantry Division deploys in Tunis","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{0,136,0}}, {count=3, randomize=false, veteran=false})
			counter.add("# GI Regular",3)
			counter.add("# Total US Infantry",3)
		end	
	end	

	-- 1ST FREE FRENCH MECHANIZED DIVISION ARRIVES - OPTIONAL EMERGENCY NAVAL TRANSPORT OPERATION
	if turn == 37 then -- Apr I, 1944
		gen.justOnce("Free French 1st Mechanized Division",function()
			
			local ff1stDivisionTranportCost = param.ff1stDivisionTranportCost
			
			if object.pAllies.money >= ff1stDivisionTranportCost then
				local menuOptions = {[1]="Yes, transport division to Napoli (costs ".. param.ff1stDivisionTranportCost .. " supplies).", [2]="No, deploy division in Palermo."}
				-- local menuOptions = {[1]="Yes, tranport division to Napoli.", [2]="No, deploy division in Palermo."}
				local menuText = object.operationTranport1stFF
				local menuTitle = "Tranport 1st FF Division?"
				
				local menuImage = civ.ui.loadImage("Images\\LibertyShip.bmp")
				local choice = text.menu(menuOptions,menuText,menuTitle,menuImage) -- If you want to include an image in the pop up box
				-- local choice = text.menu(menuOptions,menuText,menuTitle) -- If you don't want to include an image in the pop up box
				
				if choice == 1 then 
					if flag.value("Sea Mines removed from Napoli") == true then
						text.simple("In a special transport operation the 1st Free French Mechanized Division is quickly tranported to Napoli!","Free French Reinforcements",object.mAFHQ)
						object.pAllies.money = object.pAllies.money - param.ff1stDivisionTranportCost
						gen.createUnit(object.uFreeFrenchInfantry, object.pAllies, {{88,106,0}}, {count=3, randomize=false, veteran=false})
						gen.createUnit(object.uFreeFrenchM10, object.pAllies, {{88,106,0}}, {count=2, randomize=false, veteran=false})
						gen.createUnit(object.uFreeFrench105Art, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=false})
						counter.add("# Free French Infantry",3)
						counter.add("# FreeFrenchM10",2)
						counter.add("# Free French 105Art",1)
						
						counter.add("# Total Free French",6)
					else
						text.simple("The port of Napoli is not clear of mines and therefore the 1st Free French Mechanized Division deploys in Palermo!","Free French Reinforcements",object.mAFHQ)
						gen.createUnit(object.uFreeFrenchInfantry, object.pAllies, {{44,140,0}}, {count=3, randomize=false, veteran=false})
						gen.createUnit(object.uFreeFrenchM10, object.pAllies, {{44,140,0}}, {count=2, randomize=false, veteran=false})
						gen.createUnit(object.uFreeFrench105Art, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
						counter.add("# Free French Infantry",3)
						counter.add("# FreeFrenchM10",2)
						counter.add("# Free French 105Art",1)
						
						counter.add("# Total Free French",6)
					end
				elseif choice == 2 then
					text.simple("The 1st Free French Mechanized Division deploys in Palermo!","Free French Reinforcements",object.mAFHQ)
					gen.createUnit(object.uFreeFrenchInfantry, object.pAllies, {{44,140,0}}, {count=3, randomize=false, veteran=false})
					gen.createUnit(object.uFreeFrenchM10, object.pAllies, {{44,140,0}}, {count=2, randomize=false, veteran=false})
					gen.createUnit(object.uFreeFrench105Art, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
					counter.add("# Free French Infantry",3)
					counter.add("# FreeFrenchM10",2)
					counter.add("# Free French 105Art",1)
					
					counter.add("# Total Free French",6)
				end
			
			elseif object.pAllies.money <= ff1stDivisionTranportCost then
				text.simple("The 1st Free French Mechanized Division deploys in Palermo!","Free French Reinforcements",object.mAFHQ)
				gen.createUnit(object.uFreeFrenchInfantry, object.pAllies, {{44,140,0}}, {count=3, randomize=false, veteran=false})
				gen.createUnit(object.uFreeFrenchM10, object.pAllies, {{44,140,0}}, {count=2, randomize=false, veteran=false})
				gen.createUnit(object.uFreeFrench105Art, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
				counter.add("# Free French Infantry",3)
				counter.add("# FreeFrenchM10",2)
				counter.add("# Free French 105Art",1)
				
				counter.add("# Total Free French",6)

			end
		end)		
	end	

	-- INDIAN 10TH INFANTRY DIVISION ARRIVES
	if turn == 38 then -- Apr II, 1944
		if flag.value("Sea Mines removed from Taranto") == true then
			gen.createUnit(object.uIndianInfantry, object.pAllies, {{120,138,0}}, {count=3, randomize=false, veteran=false})
			gen.createUnit(object.uIndian25pdrArt, object.pAllies, {{120,138,0}}, {count=1, randomize=false, veteran=false})
			text.simple("The 10th Indian Infantry Division arrives in the port of Taranto!","Indian Reinforcements",object.mAFHQ)
			flag.setTrue("10th Indian Infantry units arrive")
			counter.add("# Indian Infantry",3)
			counter.add("# Indian 25pdr Art",1)

			counter.add("# Total Indian",4)
		else
			gen.createUnit(object.u4thIndianDiv, object.pAllies, {{62,162,0}}, {count=3, randomize=false, veteran=false})
			gen.createUnit(object.uIndian25pdrArt, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
			text.simple("The 10th Indian Infantry Division arrives in the port of Catania!","Indian Reinforcements",object.mAFHQ)
			flag.setTrue("10th Indian Infantry units arrive")
			counter.add("# Indian Infantry",3)
			counter.add("# Indian 25pdr Art",1)

			counter.add("# Total Indian",4)
		end
	end	
	
	-- SPECIAL BULLETIN NOTICE THAT ROYAL NAVY WILL WITHDRAW BATTLESHIPS AT THE START OF TURN 37
	if turn == 39 then -- Apr III, 1944
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^In the next 2 turns, on May I 1944 (turn #41), the Royal Navy\n^will withdraws its remaining battleship divisions from the Italian\n^theater in order to go support the upcoming Operation Overlord,\n^the invasion of northern France!"))
	end

	-- 88TH US INFANTRY DIVISION ARRIVES
	if turn == 39 then -- Apr III, 1944
		if flag.value("Sea Mines removed from Napoli") == true and flag.value("Roma is captured") == true then
			text.simple("The US 88th Infantry Division deploys in Napoli","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{88,106,0}}, {count=3, randomize=false, veteran=false})
			counter.add("# GI Regular",3)
			counter.add("# Total US Infantry",3)
		else
			text.simple("The US 88th Infantry Division deploys in Palermo","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{44,140,0}}, {count=3, randomize=false, veteran=false})
			counter.add("# GI Regular",3)
			counter.add("# Total US Infantry",3)
		end	
	end	

	-- 25TH BRITISH TANK BRIGADE ARRIVES
	if turn == 39 then -- Apr III, 1944
		if flag.value("Sea Mines removed from Napoli") == true and flag.value("Roma is captured") == true then
			text.simple("The British 25th Tank Brigade deploys in Napoli!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uShermanIIaBde, object.pAllies, {{88,106,0}}, {count=2, randomize=false, veteran=false})
			counter.add("# Sherman IIa Bde",2)
			counter.add("# Total UK Armored",2)
		else
			text.simple("The British 25th Tank Brigade deploys in Messina!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uShermanIIaBde, object.pAllies, {{73,155,0}}, {count=2, randomize=false, veteran=false})
			counter.add("# Sherman IIa Bde",2)
			counter.add("# Total UK Armored",2)
		end	
	end	
	
	-- 6TH SOUTH AFRICAN ARMORED DIVISION ARRIVES
	local saDivTarantoLocationList = {{120,138,0}}
	local saDivCataniaLocationList = {{62,162,0}}
	if turn == 40 then -- Apr IV, 1944
		if flag.value("Sea Mines removed from Catania") == true then	
			local saDivLocationList = saDivCataniaLocationList
			local saDivLocationDesc = "Catania"
			if flag.value("Sea Mines removed from Taranto") == true then
				saDivLocationList = saDivTarantoLocationList
				saDivLocationDesc = "Taranto"
			end
			gen.justOnce("6th SA Armored Division arrives",function()
				gen.createUnit(object.uSAShermanIII, object.pAllies, saDivLocationList, {count=3, randomize=false, veteran=false})
				gen.createUnit(object.uSAMotInf, object.pAllies, saDivLocationList, {count=1, randomize=false, veteran=false})	
				gen.createUnit(object.uSAMotorTroops, object.pAllies, saDivLocationList, {count=1, randomize=false, veteran=false})
				gen.createUnit(object.uSASexton, object.pAllies, saDivLocationList, {count=1, randomize=false, veteran=false})
				text.simple("The South African 6th Armored Division deploys in " .. saDivLocationDesc .. "!","South African Reinforcements",object.mAFHQ)
				flag.setTrue("South African units arrive")
				counter.add("# SA Sherman",3)
				counter.add("# SA Mot Inf",1)
				counter.add("# SA Bren Carrier",1)
				counter.add("# SA Sexton",1)
				
				counter.add("# Total South African",6)
			end)
		end
	end	

	-- SPECIAL BULLETIN NOTICE THAT ALLIES MAY NOW LAUNCH MAJOR OFFENSIVE AT THE START OF TURN 41
	if turn == 40 then -- Apr IV, 1944 (#41)
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Starting next week, the Allies will have the option to\n^launch 2 major offensive operations during the campaign:\n^Diadem and Grapeshot.\n^\r^If you proceed, your munitions (105mm and 155mm Shells,\n^Light/Medium and Heavy Bombloads), and all your ground troops\n^will benefit from a +2 AF to all their attacks for 4 consecutive\n^weeks.\n^\r^This is in addition to any other attack bonuses they may also be\n^eligible for. It costs 500 supplies to launch each operation and\n^they both can only be conducted during the following timeframes:\n^\r^You may launch Diadem any time you desire provided you\n^have the required supplies. Grapeshot may only be launched\n^16 weeks AFTER Diadem has ended.\n^\r^You may activate the offensives from the Command Decisions\n^menu (key '5')."))
	end	

	-- 9TH AND 21TH BRITISH TANK BRIGADES ARRIVE
	if turn == 42 then -- May II, 1944
		if flag.value("Sea Mines removed from Napoli") == true and flag.value("Roma is captured") == true then
			text.simple("The British 9th and 21st Tank Brigades deploy in Napoli!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uShermanIIaBde, object.pAllies, {{88,106,0}}, {count=2, randomize=false, veteran=false})
			gen.createUnit(object.uChurchillMkIV, object.pAllies, {{88,106,0}}, {count=2, randomize=false, veteran=false})
			flag.setTrue("Churchill MkIV enters UK armor inventory")
			counter.add("# Sherman IIa Bde",2)
			counter.add("# Churchill Mk IV",2)
			counter.add("# Total UK Armored",4)
		elseif flag.value("Sea Mines removed from Messina") == true then
			text.simple("The British 9th and 21st Tank Brigades deploy in Messina!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uShermanIIaBde, object.pAllies, {{73,155,0}}, {count=2, randomize=false, veteran=false})
			gen.createUnit(object.uChurchillMkIV, object.pAllies, {{73,155,0}}, {count=2, randomize=false, veteran=false})
			flag.setTrue("Churchill MkIV enters UK armor inventory")
			counter.add("# Sherman IIa Bde",2)
			counter.add("# Churchill Mk IV",2)
			counter.add("# Total UK Armored",4)
		end	
	end	

	-- ITALIAN NEMBO AIRBORNE DIVISION ARRIVES	
	if turn >= 42 and flag.value("Italy Surrendered") == true then -- not before Dec I, 1943
		gen.justOnce("Nembo Airborne Division arrives",function()
			text.simple("The Italian Monarchist regime deploys its second combat unit in support of the Allied war effort, the Nembo Airborne Division, in Foggia!","Italian Monarchist Reinforcements",object.mItalianMonarchist)
			gen.createUnit(object.uParacadutisto, object.pAllies, {{111,107,0},{120,138,0}}, {count=2, randomize=false, veteran=false})
		end)
	end	

	-- SPECIAL BULLETIN NOTICE THAT 3 US DIVISIONS AND 2 RANGER REGIMENTS TO BE WITHDRAWN AT THE START OF TURN 45
	if turn == 43 then -- May III, 1944 (#43)
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Over the next two turns, 3 American Divisions, all IV Corps supporting\n^units and all Rangers will be withdrawn from the theater and reassigned to\n^the US 6th Army Group in preparation for Operation Dragoon, the invasion\n^of southern France. The units will be departing in the following order:\n^\r^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 3 " .. object.uGIRegular.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 1 " .. object.uUSBazooka.name .. " regiment\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 2 " .. object.uRanger.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 2 " .. object.uM2A1105mmArt.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 1 " .. object.uM1155mmArt.name .. " regiment\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+2)) .. ": 6 " .. object.uGIVeteran.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+2)) .. ": 1 " .. object.uM10Wolverine.name .. " battalion\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+2)) .. ": 1 " .. object.uM4A1ShermanInd.name .. " battalion\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+2)) .. ": 1 " .. object.uM8Greyhound.name .. " battalion\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+2)) .. ": 1 " .. object.u6pdrAT.name .. " regiment\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+2)) .. ": 1 " .. object.uLibertyShip.name .. " squadron\n^\r^Be forwarned, that up to " .. param.witdrawalMechanizedPenalty .. " supplies will be subracted for 'each' unit\n^that is missing from the total at the time of withdrawal.\n^\r^As such, you should plan to adjust your frontlines accordingly\n^in the next turns, if so required!"))
	end		

	-- On turn #44 36TH INFANTRY DIVISION, 1 X US BAZOOKA, 2 X 105MM ART. AND 1 X 155MM ART. REGIMENT WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)
	
	-- On turn #44 1ST SSF AND 6615 RANGER REGIMENTS WITDRAW (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)

	-- OPERATION OVERLORD - JUNE 6TH 1944
	if turn == 45 then -- Jun I, 1944 (#45)
		civ.playSound("ddayaddressshort.wav")
		text.setLinesPerWindow(20)
			text.simple("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^It is June 6th, 1944 and Operation Overlord, the long awaited\n^invasion of northern France, has begun.\n^\r^The endeavor is led by General Dwight D. Eisenhower,\n^commander of Supreme Headquarters Allied Expeditionary\n^Force, and General Bernard Montgomery as commander of\n^the 21st Army Group, which comprises all the land forces.\n^\r^As commander of 15th Army Group, it is hoped that your\n^continued efforts in Italy will tie down has many German\n^forces in the Mediterranean as possible. Failure to do so\n^could have dramatic repercussions elsewhere.","Operation Overlord Begins!",object.mOperationOverlord)
		text.setLinesPerWindow(12)
	end

	-- On turn #45 3RD AND 45TH INFANTRY DIVISIONS WITHDRAW (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)
  
	-- 2ND BRITISH TANK BRIGADE ARRIVES
	if turn == 45 then -- Jun I, 1944
		if flag.value("Sea Mines removed from Napoli") == true and flag.value("Roma is captured") == true then
			text.simple("The British 2nd Tank Brigade deploys in Napoli!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uShermanIIaBde, object.pAllies, {{88,106,0}}, {count=2, randomize=false, veteran=false})
			counter.add("# Sherman IIa Bde",2)
			counter.add("# Total UK Armored",2)
		else
			text.simple("The British 2nd Tank Brigade deploys in Messina!","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uShermanIIaBde, object.pAllies, {{73,155,0}}, {count=2, randomize=false, veteran=false})
			counter.add("# Sherman IIa Bde",2)
			counter.add("# Total UK Armored",2)
		end	
	end	

	-- SPECIAL BULLETIN NOTICE THAT THE FREE FRENCH CONTINGENT TO BE WITHDRAWN OVER THE COURSE OF FIVE TURNS
	if turn == 46 then -- Jun II, 1944 (#46)
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Over the next five turns, the entire Free French contingent will\n^be withdrawn from the theater and reassigned to the US 6th Army\n^Group in preparation for Operation Dragoon, the invasion of\n^southern France. The units will be departing in the following order:		\n^\r^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 3 " .. object.uFreeFrenchInfantry.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 2 " .. object.uFreeFrenchM10.name .. " battalions\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 1 " .. object.uFreeFrench105Art.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 1 " .. object.uLibertyShip.name .. " squadron\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+3)) .. ": 4 " .. object.uGoumierMarocain.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+4)) .. ": 6 " .. object.uTirailleurAlgerien.name .. " and\n^1 " .. object.uFreeFrench105Art.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+5)) .. ": 3 " .. object.uTirailleurAlgerien.name .. ",\n^3 " .. object.uInfanterieColoniale.name .. " and 1 " .. object.uFreeFrench105Art.name .. " regiments\n^\r^Be forwarned, that up to " .. param.witdrawalMechanizedPenalty .. " supplies will be subracted for 'each' unit\n^that is missing from the total at the time of withdrawal.\n^\r^As such, you should plan to adjust your frontlines accordingly\n^in the coming month!"))
	end		
	
	-- On turn #47 1ST FREE FRENCH MECHANIZED DIVISION WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)

	-- US 442ND INFANTRY REGIMENT ARRIVES
	if turn == 47 then -- Jun III, 1944
		if flag.value("Sea Mines removed from Napoli") == true then
			text.simple("The US 442nd 'Nisei' Infantry Regiment, which deployed to Napoli, was composed of American soldiers of Japanese descent and would go on to become the most decorated American unit of the war. As such, any American unit stacked in the same tile as this unit receives an extra +1 AF to any combat!","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uUS442ndRegt, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=false})
			counter.add("# US 442nd Regt",1)
			counter.add("# Total US Infantry",1)
		else
			text.simple("The US 442nd 'Nisei' Infantry Regiment, which deployed to Messina, was composed of American soldiers of Japanese descent and would go on to become the most decorated American unit of the war. As such, any American unit stacked in the same tile as this unit receives an extra +1 AF to any combat!","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uUS442ndRegt, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
			counter.add("# US 442nd Regt",1)
			counter.add("# Total US Infantry",1)
		end	
	end	

	-- on turn #49 4 GOUMIERS MAROCAIN REGIMENTS WITHDRAW (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)
	
	-- 91ST US INFANTRY DIVISION ARRIVES
	if turn == 49 then -- Jul I, 1944
		if flag.value("Sea Mines removed from Napoli") == true and flag.value("Roma is captured") == true then
			text.simple("The US 91st Infantry Division deploys in Napoli","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{88,106,0}}, {count=3, randomize=false, veteran=false})
			counter.add("# GI Regular",3)
			counter.add("# Total US Infantry",3)
		else
			text.simple("The US 91st Infantry Division deploys in Palermo","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{44,140,0}}, {count=3, randomize=false, veteran=false})
			counter.add("# GI Regular",3)
			counter.add("# Total US Infantry",3)
		end	
	end	

	-- On turn #50 2ND MAROCCAN AND 3RD ALGERIAN INFANTRY DIVISIONS WITHDRAW (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section) 
	
	-- 56TH BRITISH INFANTRY DIVISION RETURNS
	if turn == 50 then -- Jul II, 1944
		if flag.value("Sea Mines removed from Napoli") == true and flag.value("Roma is captured") == true then
			gen.createUnit(object.uBritishRegular, object.pAllies, {{88,106,0}}, {count=3, randomize=false, veteran=false})
			text.simple("The British 56th Infantry Division returns to duty in Italy and deploys in Napoli!","British Reinforcements",object.mAFHQ)
			counter.add("# British Regular",3)
			counter.add("# Total UK Infantry",3)
		else
			gen.createUnit(object.uBritishRegular, object.pAllies, {{73,155,0}}, {count=3, randomize=false, veteran=false})
			text.simple("The British 56th Infantry Division to duty in Italy and deploys in Messina!","British Reinforcements",object.mAFHQ)
			counter.add("# British Regular",3)
			counter.add("# Total UK Infantry",3)
		end	
	end		
	
	-- On turn #51 4TH MAROCCAN AND 9TH COLONIALE INFANTRY DIVISIONS WITHDRAW (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section) 
	
	-- SPECIAL BULLETIN NOTICE THAT 2 FIGHTER AND 4 BOMBER WINGS TO BE WITHDRAWN STARTING ON TURN 52
	if turn == 51 then -- July III, 1944 (#51)
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Over the next three turns, 2 American fighter and 4 ground support\n^air wings will be withdrawn from the theater and reassigned to the\n^US 6th Army Group in preparation for Operation Dragoon, the invasion\n^of southern France. The units will be departing in the following order:\n^\r^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 1 " .. object.uP40Warhawk.name .. " wing\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 1 " .. object.uP38Lightning.name .. " wing\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 1 " .. object.uA36Apache.name .. " wing\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+2)) .. ": 1 " .. object.uB25Mitchell.name .. " wing\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+2)) .. ": 1 " .. object.uB26Marauder.name .. " wing\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+3)) .. ": 1 " .. object.uB17Fortress.name .. " wing\n^\r^Be forwarned, that up to " .. param.witdrawalAirPenalty .. " supplies will be subracted for 'each'\n^unit that is missing from the total at the time of withdrawal.\n^\r^As such, you should consider adjusting your air plans accordingly\n^in the next turns, if so required!"))
	end		
	
	-- On turn #52 US AIR FORCE FIGHTER WITHRAWALS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)
	
	-- On turn #53 US AIR FORCE 1ST GROUP OF BOMBER WITHRAWALS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)
	
	-- 3RD GREEK MOUNTAIN BRIGADE ARRIVES
	if turn == 53 then -- Aug I, 1944
		if flag.value("Sea Mines removed from Pescara") == true then
			text.simple("The 3rd Greek Mountain Brigade deploys in Pescara!","Greek Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGreekMtnBde, object.pAllies, {{107,83,0}}, {count=1, randomize=false, veteran=false})
			flag.setTrue("Greek Infantry units arrive") 
			counter.add("# Greek Mtn Bde",1)
			counter.add("# Total Brazil et AL",1)	
		else
			text.simple("The 3rd Greek Mountain Brigade deploys in Bari!","Greek Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGreekMtnBde, object.pAllies, {{123,125,0}}, {count=1, randomize=false, veteran=false})
			flag.setTrue("Greek Infantry units arrive") 
			counter.add("# Greek Mtn Bde",1)
			counter.add("# Total Brazil et AL",1)
		end	
	end		

	-- On turn #54 US AIR FORCE 2ND GROUP OF BOMBER WITHRAWALS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)

	-- OPERATION DRAGOON - AUGUST 15TH 1944
	if turn == 55 then -- Aug III, 1944 (#55)
		civ.playSound("Telegraph.wav")
		text.setLinesPerWindow(22)
			text.simple("************ SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^It is August 15th, 1944 and Operation Dragoon, the\n^invasion of Provence in southern France, has begun.\n^\r^The endeavor is led by General Jacob L. Devers,\n^commander of the Sixth Allied Army Group, which\n^consists of the US 7th Army of General Alexander Patch\n^and the French 1st Army of Jean de Tassigny.\n^\r^The Sixth Army Group's primary mission is to aid in the\n^liberation of France and as such its activity with regards\n^the Italian campaign will be strictly limited to securing\n^the frontlines along the Franco-Italian Alps border.","Operation Dragoon Begins!",object.mOperationDragoon1)
		text.setLinesPerWindow(12)
	end


	-- FIRST ITALIAN PARTISAN UNITS MAY ONLY BEGIN TO APPEAR 8 TURNS AFTER THE CAPTURE OF ROMA (historically Rome fell June 4th, 1944 (turn #45))
	-- Also see italianPartisansActivated function in onCityTaken.lua
	if flag.value("Italian CLN Front Activated") == true then 
		gen.justOnce("First Italian CLN Partisan",function()
			civ.playSound("Telegraph.wav")
			text.simple("The first independent Italian partisan formations begin to rise up in northern Italy under the banner of the 'Comitato di Liberazione Nazionale'! They come from a wide range of political groups and as such can often be at odds with one another in terms of the direction or actions to take which can lead to paralysis of command.","Italian 'CLN' Partisan Cell",object.mCLN)
			gen.createUnit(object.uPartigiani, object.pAllies, {{87,13,0},{90,20,0},{127,13,0},{98,36,0},{106,10,0},{101,5,0},{68,8,0},{75,7,0},{114,14,0}}, {count=1, randomize=true, veteran=false})
			flag.setTrue("Italian CLN Partisans Activated")	
		end)
	end

	-- Once activated the Allies may randomly begin receiving Partisans 
	if flag.value("Italian CLN Partisans Activated") == true then 
		if math.random(1,4) == 4 then -- There's a 25% chance per turn that a partisan unit will be generated once the CLN has been activated
			text.simple("A new Italian partisan cell is formed in northern Italy!","Partigiani Cell Formed!",object.mCLN)
			gen.createUnit(object.uPartigiani, object.pAllies, {{87,13,0},{90,20,0},{127,13,0},{98,36,0},{106,10,0},{101,5,0},{68,8,0},{75,7,0},{114,14,0}}, {count=1, randomize=true, veteran=false})
		end
	end	
	-------------------------------------------------

	-- UPCOMING OPTION TO RECRUIT BRITISH 52ND LOWLAND (MOUNTAIN) DIVISION ANNOUNCED
	if turn == 60 then -- Sept IV, 1944 (#60)
		civ.playSound("Telegraph.wav")
		text.setLinesPerWindow(20)
			text.simple("************ SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^You will be given a one time option to recruit the\n^British 52nd Lowland Mountain Division next turn,\n^at a cost of 270 supplies.\n^\r^Should you have the necessary amount of resources\n^in your theater depot and accept the offer the division\n^will arrive in Italy a couple of weeks afterwards.\n^\r^You may, of course, decline the offer but if you do\n^you will not be given a second opportunity later on\n^to change your mind.","Recruiting 52nd Mountain Division Option!",object.mBritish52ndMtn)
		text.setLinesPerWindow(12)
	end
	
	-- OPTION TO RECRUIT BRITISH 52ND LOWLAND (MOUNTAIN) DIVISION
	if turn == 61 then -- October I, 1944 (#61)
			local recruitBritish52ndMtn = param.recruitBritish52ndMtn
			civ.playSound("Telegraph.wav")
			if object.pAllies.money >= recruitBritish52ndMtn then
				local menuOptions = {[1]="Yes, recruit the division (costs ".. param.recruitBritish52ndMtn .. " supplies).", [2]="No, do not recruit the division."}
				local menuText = object.british52ndLowlandDivision
				local menuTitle = "Recruit the British 52nd Mountain Division?"
				
				local menuImage = civ.ui.loadImage("Images\\British52ndMtn.bmp")
				local choice = text.menu(menuOptions,menuText,menuTitle,menuImage) -- If you want to include an image in the pop up box
			
				if choice == 1 then
					civ.playSound("Cheers2.wav")
					text.simple("General ".. commanderAFHQ_Base .. " you opted to accept the offer\n^to recruit the British 52nd Mountain Division and as such it is\n^assigned to the Italian theater of operations and should arrive\n^in a couple weeks time.","British 52nd Lowland Division Assigned to Italy!",object.mBritish52ndMtn)
					object.pAllies.money = object.pAllies.money - recruitBritish52ndMtn
								
					delayedAction.doInFuture("recruitBritish52ndMtnDivision",{},civ.getTurn()+ 2 ,object.pAllies.id)
				elseif choice == 2 then
					text.simple("General ".. commanderAFHQ_Base .. " you opted to decline the\n^offer to recruit the British 52nd Mountain Division and as\n^such it is assigned to the French theater of operations.","British 52nd Lowland Division Assigned to France!",object.mBritish52ndMtn)
				end
			
			elseif object.pAllies.money <= recruitBritish52ndMtn then
				text.simple("General ".. commanderAFHQ_Base .. " you could have had the\n^opportunity to recruit the British 52nd Mountain Division\n^but the cost of ".. recruitBritish52ndMtn .." supplies exceeded the amount\n^currently in your theater depots and therefore it was\n^assigned to the French theater of operations!","British 52nd Lowland Division Unavailable!",object.mBritish52ndMtn)
			end
	
	end			

	-- 92ST US INFANTRY DIVISION ARRIVES
	if turn == 65 then -- Nov I, 1944
		if flag.value("Sea Mines removed from Livorno") == true and flag.value("Roma is captured") == true then
			text.simple("The US 92nd Infantry Division and the attached units of the US IV Corps deploy in Livorno","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{75,35,0}}, {count=3, randomize=false, veteran=false})
			gen.createUnit(object.uM8Greyhound, object.pAllies, {{75,35,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uM10Wolverine, object.pAllies, {{75,35,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uM4A1ShermanInd, object.pAllies, {{75,35,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uM2A1105mmArt, object.pAllies, {{75,35,0}}, {count=2, randomize=false, veteran=false})
			gen.createUnit(object.uM1155mmArt, object.pAllies, {{75,35,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uM157mmAT, object.pAllies, {{75,35,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uUSBazooka, object.pAllies, {{75,35,0}}, {count=1, randomize=false, veteran=false})

			counter.add("# GI Regular",3)
			counter.add("# US Bazooka",1)
			counter.add("# M8 Greyhound",1)
			counter.add("# M10 Wolverine",1)
			counter.add("# M4A1 Ind",1)
			counter.add("# M2A1 105mm Art",2)
			counter.add("# M1 155mm Art",1)
			counter.add("# M1 57mm AT",1)
			
			counter.add("# Total US Infantry",4)
			counter.add("# Total US Armored",3)
			counter.add("# Total US Artillery",4)
			
		elseif flag.value("Sea Mines removed from Napoli") == true and flag.value("Roma is captured") == true then
			text.simple("The US 92nd Infantry Division and the attached units of the US IV Corps deploy in Napoli","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{88,106,0}}, {count=3, randomize=false, veteran=false})
			gen.createUnit(object.uM8Greyhound, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uM10Wolverine, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uM4A1ShermanInd, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uM2A1105mmArt, object.pAllies, {{88,106,0}}, {count=2, randomize=false, veteran=false})
			gen.createUnit(object.uM1155mmArt, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uM157mmAT, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uUSBazooka, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=false})

			counter.add("# GI Regular",3)
			counter.add("# US Bazooka",1)
			counter.add("# M8 Greyhound",1)
			counter.add("# M10 Wolverine",1)
			counter.add("# M4A1 Ind",1)
			counter.add("# M2A1 105mm Art",2)
			counter.add("# M1 155mm Art",1)
			counter.add("# M1 57mm AT",1)
			
			counter.add("# Total US Infantry",4)
			counter.add("# Total US Armored",3)
			counter.add("# Total US Artillery",4)
		else
			text.simple("The US 92nd Infantry Division and the attached units of the US IV Corps deploy in Palermo","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uGIRegular, object.pAllies, {{44,140,0}}, {count=3, randomize=false, veteran=false})
			gen.createUnit(object.uM10Wolverine, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uM4A1ShermanInd, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uM8Greyhound, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uM2A1105mmArt, object.pAllies, {{44,140,0}}, {count=2, randomize=false, veteran=false})
			gen.createUnit(object.uM1155mmArt, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uM157mmAT, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
			gen.createUnit(object.uUSBazooka, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})

			counter.add("# GI Regular",3)
			counter.add("# US Bazooka",1)
			counter.add("# M8 Greyhound",1)
			counter.add("# M10 Wolverine",1)
			counter.add("# M4A1 Ind",1)
			counter.add("# M2A1 105mm Art",2)
			counter.add("# M1 155mm Art",1)
			counter.add("# M1 57mm AT",1)
			
			counter.add("# Total US Infantry",4)
			counter.add("# Total US Armored",3)
			counter.add("# Total US Artillery",4)
		end	
	end	

	-- 1ST BRAZIL INFANTRY DIVISION ARRIVES
	if turn == 65 then -- Nov I, 1944
		if flag.value("Sea Mines removed from Livorno") == true and flag.value("Roma is captured") == true then
			text.simple("The Brazilian 1st Infantry Division deploys in Livorno","Brazilian Reinforcements",object.mAFHQ)
			gen.createUnit(object.uBraz1stInf, object.pAllies, {{75,35,0}}, {count=3, randomize=false, veteran=false})
			flag.setTrue("Brazil Infantry units arrive")
			counter.add("# Braz 1st Inf",3)
			counter.add("# Total Brazil et AL",3)
		elseif flag.value("Sea Mines removed from Napoli") == true and flag.value("Roma is captured") == true then
			text.simple("The Brazilian 1st Infantry Division deploys in Napoli","Brazilian Reinforcements",object.mAFHQ)
			gen.createUnit(object.uBraz1stInf, object.pAllies, {{88,106,0}}, {count=3, randomize=false, veteran=false})
			flag.setTrue("Brazil Infantry units arrive")
			counter.add("# Braz 1st Inf",3)
			counter.add("# Total Brazil et AL",3)
		else
			text.simple("The US 92nd Infantry Division deploys in Palermo","Brazilian Reinforcements",object.mAFHQ)
			gen.createUnit(object.uBraz1stInf, object.pAllies, {{44,140,0}}, {count=3, randomize=false, veteran=false})
			flag.setTrue("Brazil Infantry units arrive")
			counter.add("# Braz 1st Inf",3)
			counter.add("# Total Brazil et AL",3)
		end	
	end	

	-- JEWISH INFANTRY BRIGADE ARRIVES
	if turn == 66 then -- Nov II, 1944
		if flag.value("Sea Mines removed from Pescara") == true then
			text.simple("The Jewish Infantry Brigade deploys in Pescara","Jewish Reinforcements",object.mAFHQ)
			gen.createUnit(object.uJewishBde, object.pAllies, {{107,83,0}}, {count=1, randomize=false, veteran=false})
			flag.setTrue("Jewish Infantry units arrive") 
			counter.add("# Jewish Bde",1)
			counter.add("# Total Brazil et AL",1)
		else
			text.simple("The 3rd Greek Mountain Brigade deploys in Bari!","Greek Reinforcements",object.mAFHQ)
			gen.createUnit(object.uJewishBde, object.pAllies, {{123,125,0}}, {count=1, randomize=false, veteran=false})
			flag.setTrue("Jewish Infantry units arrive") 
			counter.add("# Jewish Bde",1)
			counter.add("# Total Brazil et AL",1)
		end	
	end	

	-- ITALIAN CREMONA INFANTRY BRIGADE ARRIVES	
	if turn >= 69 and flag.value("Italy Surrendered") == true then -- not before Dec I, 1944
		gen.justOnce("Cremona Infantry Brigade arrives",function()
			text.simple("The Italian Monarchist regime deploys another combat unit in support of the Allied war effort, the Cremona Infantry Brigade, in Foggia!","Italian Monarchist Reinforcements",object.mItalianMonarchist)
			gen.createUnit(object.uFanteriaVeterana, object.pAllies, {{111,107,0},{120,138,0}}, {count=1, randomize=false, veteran=false})
		end)
	end	
	
	-- SPECIAL BULLETIN NOTICE THAT GENERAL TRUSCOTT REPLACES GENERAL CLARK and GEN ALEXANDER REPLACES GEN WILSON
	if turn == 69 and flag.value("Patton dismissed") == true then -- Dec I, 1944 (#69)	
		-- object.pAllies.leader.name = "Rupert H. Alexander"
		commanderAFHQ_Base = "Harold H. Alexander"
		commander15thArmy_Base = "Mark W. Clark"

		civ.playSound("Telegraph.wav")
		text.simple("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Effective immediately, General Mark W. Clark is to take over\n^command of 15th Army Group from General Alexander who\n^becomes Supreme Commander of the Allied Forces Headquarters,\n^responsible for all military operations in the Mediterranean theatre,\n^from General Wilson who is promoted to be Chief of the British\n^Joint Staff Mission in Washington.\n^\r^As a consequence, General Lucien K. Truscott takes command\n^of U.S. 5th Army.","Harold H. Alexander",object.mHaroldAlexander)
		
		for unit in civ.iterateUnits() do 
			if unit.type == object.uGenClark then	
				civ.deleteUnit(unit)
			end
		end	
		gen.createUnit(object.uGenTruscott, object.pAllies, {{75,35,0},{80,76,0},{88,106,0}}, {randomize=false, veteran=false})
	end
	
	-- SPECIAL BULLETIN NOTICE THAT 1 BRITISH INFANTRY, 1 ARMORED BDE AND 1 INDIAN DIVISION ARE TO BE WITHDRAWN
	if turn == 69 then -- Dec I, 1944 (#69)
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Over the next four turns, one British Infantry Division,\n^one British Armored Brigade, the Greek Mtn Brigade and the\n^4th Indian Division will be withdrawn from the Italy and\n^sent to Greece to help bring order following the withdrawal\n^of German troops there. The units will be departing in the\n^following order:\n^\r^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 3 " .. object.uBritishRegular.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+2)) .. ": 1 " .. object.uGreekMtnBde.name .. " regiment\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+3)) .. ": 3 " .. object.u4thIndianDiv.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+4)) .. ": 2 " .. object.uShermanIIaBde.name .. " regiments\n^\r^Be forwarned, that " .. param.witdrawalMechanizedPenalty .. " supplies will be subracted for 'each' unit\n^that is missing from the total at the time of withdrawal.\n^\r^As such, you should plan to adjust your frontlines accordingly\n^in the coming turns!"))
	end	

    -- On turn #70 4TH BRITISH INFANTRY DIVISIONS WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)

    -- On turn #72 4TH INDIAN INFANTRY DIVISIONS WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)

	-- SPECIAL BULLETIN NOTICE THAT GEN. LEESE IS TO BE REPLACED BY GEN. McCREERY THIS TURN IF DON'T CONTROL LIVORNO, FIRENZE AND RIMINI
	if turn == 72
	and not (object.cLivorno.owner == object.pAllies
	and object.cFirenze.owner == object.pAllies
	and object.cRimini.owner == object.pAllies)
	then -- Dec IV, 1944 (#72)
		gen.justOnce("Gen. Leese is Replaced",function()
			civ.playSound("Telegraph.wav")
			civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Having failed to make significant enough progress against the\n^Gothic Line, the Allied High Command believes a change of\n^leadership of the British Eigth Army is required.\n^\r^Thus " .. object.uGenLeese.name .. " is to be reassigned to Burma to become\n^Commander-in-Chief (C-in-C) of the Eleventh Army Group.\n^\r^He is to be suceeded by " .. object.uGenLeese.name .. " who will take immediate\n^command of the Eight Army in Italy!"))

			for unit in civ.iterateUnits() do 
				if unit.type == object.uGenLeese then	
					civ.deleteUnit(unit)
					-- gen.replaceUnit(unit,object.uGenMcCreery)
				end
			end		
			flag.setTrue("General Leese Reassigned")
			gen.createUnit(object.uGenMcCreery, object.pAllies, {{107,83,0},{88,106,0},{73,155,0}}, {randomize=false, veteran=false})
		end)
	end	

	
	-- On turn #73 25TH BRITISH TANK BRIGADE WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)

	-- SPECIAL BULLETIN NOTICE THAT THE ENTIRE CANADIAN CONTINGENT ALONG WITH BRITISH 1ST INFANTRY DIVISION TO BE WITHDRAWN OVER THE COURSE OF THREE TURNS
	if turn == 73 then -- Jan I, 1945 (#73)
		civ.playSound("Telegraph.wav")
		civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. " (15th Army Group)\n^\r^Over the next three turns, one British Infantry Division\n^along with the entire Canadian contingent will be withdrawn\n^from the theater and reassigned to 21st Army Group in France\n^in preparation for the final invasion of Germany. The units\n^will be departing in the following order:\n^\r^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ": 2 " .. object.uCdnShermanIII.name .. " battalions\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+2)) .. ": 3 " .. object.uBritishVeteran.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+2)) .. ": 3 " .. object.uCdnInfantry.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+3)) .. ": 1 " .. object.uCdnMotInf.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+3)) .. ": 1 " .. object.uCdnMotorTroops.name .. " regiments\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+3)) .. ": 3 " .. object.uCdnShermanIII.name .. " battalions\n^" .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+3)) .. ": 1 " .. object.uCdnM7105mm.name .. " regiments\n^\r^Be forwarned, that up to " .. param.witdrawalMechanizedPenalty .. " supplies will be subracted for 'each'\n^unit that is missing from the total at the time of withdrawal.\n^\r^As such, you should plan to adjust your frontlines accordingly\n^in the coming turns!"))
	end		

	-- ITALIAN LEGANO INFANTRY BRIGADE ARRIVES	
	if turn >= 73 and flag.value("Italy Surrendered") == true then -- not before Jan I, 1945
		gen.justOnce("Legano Infantry Brigade arrives",function()
			text.simple("The Italian Monarchist regime deploys another combat unit in support of the Allied war effort, the Legano Infantry Brigade, in Firenze!","Italian Monarchist Reinforcements",object.mItalianMonarchist)
			gen.createUnit(object.uFanteriaVeterana, object.pAllies, {{89,39,0},{80,76,0}}, {count=1, randomize=false, veteran=false})
		end)
	end	

	-- On turn #74 1ST CANADIAN TANK BRIGADE WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)
	
	-- 25TH BRITISH ROYAL ENGINEER TANK BRIGADE ARRIVES
	if turn == 74 then -- Jan II, 1945
		if object.ltownAncona.defender and object.ltownAncona.defender == object.pAllies then
			text.simple("The British 25th Royal Engineers Tank Brigade deploys in Ancona! The Churchill AVRE tanks are well suited for attacking bunker type fortifications.","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uChurchillAVRE, object.pAllies, {{111,61,0}}, {count=2, randomize=false, veteran=false})
			flag.setTrue("Churchill AVRE enters UK armor inventory")
			counter.add("# Churchill AVRE",2)
			counter.add("# Total UK Armored",2)
		elseif flag.value("Sea Mines removed from Pescara") == true and flag.value("Roma is captured") == true then
			text.simple("The British 25th Royal Engineers Tank Brigade deploys in Pescara! The Churchill AVRE tanks are well suited for attacking bunker type fortifications.","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uChurchillAVRE, object.pAllies, {{107,83,0}}, {count=2, randomize=false, veteran=false})
			flag.setTrue("Churchill AVRE enters UK armor inventory")
			counter.add("# Churchill AVRE",2)
			counter.add("# Total UK Armored",2)
		elseif flag.value("Sea Mines removed from Livorno") == true and flag.value("Roma is captured") == true then
			text.simple("The British 25th Royal Engineers Tank Brigade deploys in Livorno! The Churchill AVRE tanks are well suited for attacking bunker type fortifications.","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uChurchillAVRE, object.pAllies, {{75,35,0}}, {count=2, randomize=false, veteran=false})
			flag.setTrue("Churchill AVRE enters UK armor inventory")
			counter.add("# Churchill AVRE",2)
			counter.add("# Total UK Armored",2)
		elseif flag.value("Sea Mines removed from Napoli") == true and flag.value("Roma is captured") == true then
			text.simple("The British 25th Royal Engineers Tank Brigade deploys in Napoli! The Churchill AVRE tanks are well suited for attacking bunker type fortifications.","British Reinforcements",object.mAFHQ)
			gen.createUnit(object.uChurchillAVRE, object.pAllies, {{88,106,0}}, {count=2, randomize=false, veteran=false})
			flag.setTrue("Churchill AVRE enters UK armor inventory")
			counter.add("# Churchill AVRE",2)
			counter.add("# Total UK Armored",2)
		end	
	end	

	-- On turn #75 1ST CANADIAN AND 1ST BRITISH INFANTRY DIVISIONS WITDRAW (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section)

	-- 10TH US MOUNTAIN DIVISION ARRIVES
	if turn == 75 then -- Jan III, 1945
		if flag.value("Sea Mines removed from Livorno") == true and flag.value("Roma is captured") == true then
			text.simple("The US 10th Mountain Division deploys in Livorno","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uUS10thMountain, object.pAllies, {{75,35,0}}, {count=3, randomize=false, veteran=false})
			flag.setTrue("10th Mountain units arrive")
			counter.add("# 10th Mountain",3)
			counter.add("# Total US Infantry",3)
		else
			text.simple("The US 10th Mountain Division deploys in Napoli","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uUS10thMountain, object.pAllies, {{88,106,0}}, {count=3, randomize=false, veteran=false})
			flag.setTrue("10th Mountain units arrive")
			counter.add("# 10th Mountain",3)
			counter.add("# Total US Infantry",3)
		end	
	end	

	-- On turn #76 5TH CANADIAN ARMORED DIVISIONS WITHDRAWS (see onTribeTurnBegin.lua "function register.onTribeTurnBegin(turn,tribe)" section) 

	-- ITALIAN FRIULI AND FOLGORE INFANTRY BRIGADES ARRIVES	
	if turn >= 77 and flag.value("Italy Surrendered") == true then -- not before Feb I, 1945
		gen.justOnce("Friuli and Folgore Infantry Brigades arrives",function()
			text.simple("The Italian Monarchist regime deploys 2 other combat units in support of the Allied war effort, the Friuli and Folgore Infantry Brigades, in Firenze!","Italian Monarchist Reinforcements",object.mItalianMonarchist)
			gen.createUnit(object.uFanteriaVeterana, object.pAllies, {{89,39,0},{80,76,0}}, {count=2, randomize=false, veteran=false})
		end)
	end	

	-- RESULTS OF YALTA CONFERENCE DEMORALIZE POLISH TROOPS	
	if turn == 78 then -- not before Feb II, 1945 (#78)
		gen.justOnce("Yalta Conference",function()
			civ.playSound("Telegraph.wav")
			local randomNumber = math.random(3)
			
			-- SEE 'Yalta Conference and Polish Troops' in section BELOW
			-- Essentially, Polish Troops temporarily lose their combat ability after Yalta Conference
			if randomNumber == 1 then
				delayedAction.doInFuture("polishTroopsDemoralized",{},civ.getTurn()+ 1 ,object.pAllies.id)
			elseif randomNumber == 2 then
				delayedAction.doInFuture("polishTroopsDemoralized",{},civ.getTurn()+ 2 ,object.pAllies.id)
			elseif randomNumber == 3 then
				delayedAction.doInFuture("polishTroopsDemoralized",{},civ.getTurn()+ 3 ,object.pAllies.id)
			end

			text.simple("Once they are made known, the terms of the Yalta Conference demoralizes the Polish troops in Italy and as a consequence you will not be able to use them in an offensive capacity for an undetermined number of weeks, while their officers work to regain their spirits, though they will still defend and move in the interim!","Yalta Conference Demoralizes Polish II Corps",object.mPolishCorps)
			print("The Yalta Conference demoralizes Polish Troops for " .. randomNumber .. " weeks!")
			civ.giveTech(object.pAllies, object.aYaltaConference)
			flag.setTrue("Polish Troops Demoralized After Yalta")
		end)
	end	

	-- ITALIAN MANTUA AND PICENO INFANTRY BRIGADES ARRIVES	
	if turn >= 81 and flag.value("Italy Surrendered") == true then -- not before Mar I, 1945
		gen.justOnce("Mantua and Piceno Infantry Brigades arrives",function()
			text.simple("The Italian Monarchist regime deploys 2 other combat units in support of the Allied war effort, the Mantua and Piceno Infantry Brigades, in Firenze!","Italian Monarchist Reinforcements",object.mItalianMonarchist)
			gen.createUnit(object.uFanteriaVeterana, object.pAllies, {{89,39,0},{80,76,0}}, {count=2, randomize=false, veteran=false})
		end)	
	end	

	-- US 442ND INFANTRY REGIMENT RETURNS
	if turn == 83 then -- Mar III, 1945
		if flag.value("Sea Mines removed from Livorno") == true and flag.value("Roma is captured") == true then
			text.simple("The US 442nd 'Nisei' Infantry Regiment returns to Italy and deploys in Livorno","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uUS442ndRegt, object.pAllies, {{75,35,0}}, {count=1, randomize=false, veteran=true})
			counter.add("# US 442nd Regt",1)
			counter.add("# Total US Infantry",1)
		else
			text.simple("The US 442nd 'Nisei' Infantry Regiment returns to Italy and deploys in Napoli","American Reinforcements",object.mAFHQ)
			gen.createUnit(object.uUS442ndRegt, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=true})
			counter.add("# US 442nd Regt",1)
			counter.add("# Total US Infantry",1)
		end	
	end	
	
	-- 46TH BRITISH INFANTRY DIVISION RETURNS
	if turn == 85 then -- Apr I, 1945
		if flag.value("Sea Mines removed from Rimini") == true and flag.value("Roma is captured") == true then
			gen.createUnit(object.uBritishRegular, object.pAllies, {{106,44,0}}, {count=3, randomize=false, veteran=false})
			text.simple("The British 46th Infantry Division returns to duty in Italy and deploys in Rimini!","British Reinforcements",object.mAFHQ)
			counter.add("# British Regular",3)
			counter.add("# Total UK Infantry",3)
		elseif flag.value("Sea Mines removed from Livorno") == true and flag.value("Roma is captured") == true then
			gen.createUnit(object.uBritishRegular, object.pAllies, {{75,35,0}}, {count=3, randomize=false, veteran=false})
			text.simple("The British 46th Infantry Division returns to duty in Italy and deploys in Livorno!","British Reinforcements",object.mAFHQ)
			counter.add("# British Regular",3)
			counter.add("# Total UK Infantry",3)
		else
			gen.createUnit(object.uBritishRegular, object.pAllies, {{88,106,0}}, {count=3, randomize=false, veteran=false})
			text.simple("The British 46th Infantry Division to duty in Italy and deploys in Napoli!","British Reinforcements",object.mAFHQ)
			counter.add("# British Regular",3)
			counter.add("# Total UK Infantry",3)
		end	
	end	

	-- TURN TRIESTE OVER TO COMMUNISTS IF NOT PREVIOUSLY CAPTURED BY ALLIES
	if turn == 89 and flag.value("Trieste is captured") == false then -- May I, 1945
		civ.playSound("Telegraph.wav")
		object.cTrieste.owner = object.pBarbarians
		text.simple("Marshall Tito's 4th Yugoslavian Army has captured the city of Trieste ahead of your troops, thereby securing parts of the Italian Venezia Giulia province for the communist bloc. As such, you will be unable to achieve a decisive victory in your campaign and any of your forces activated while situated on or south of the river Isonzo will be forced to retreat north.","Trieste Falls to Tito's Forces",object.mAFHQ)	
		flag.setTrue("Trieste Falls To Communists")
	end	
	
	-- ====================================================================
	-- AXIS 
	-- ====================================================================

	-- AXIS INITIAL RAILYARD PLACEMENT	
	-- Randomly place a maximum of 5 railyard in one of the 8 possible cities:
	-- Alessandria, Bologna, Brescia, Firenze, Modena, Pavia, Udine, Verona
	if turn == 1 then
		local a = 0
		while( a < 5 )	do
			local placementLocations = {{98,4,0},{106,14,0},{136,20,0},{84,2,0},{100,30,0},{89,39,0},{96,24,0},{75,1,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uRailroadYard) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations)
			gen.createUnit(object.uRailroadYard, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})
			-- civ.ui.text("Created Railroad Yard")
			a = a + 1
		end
	end	
	
	-- Add MSVN units to guard Railroad Yards
	if turn == 1 then
		local a = 0
		while( a < 8 )	do
			local placementLocations = {{98,4,0},{106,14,0},{136,20,0},{84,2,0},{100,30,0},{89,39,0},{96,24,0},{75,1,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uMSVN) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations)
			local unit = gen.createUnit(object.uMSVN, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a uMSVN, but we have to check the unit was actually created, and is not nil
				unit.moveSpent = 255
			end
			-- civ.ui.text("Created uMSVN")
			a = a + 1
		end
	end		

	-- AXIS - INITIAL DEFENSIVE PLACEMENTS
	-- Widerstandsnest, Bunkers, Minenfelds 

	if turn == 1 then -- July I, 1943
	-- ISLAND OF SICILY 
	
		-- Blackshirts
		local a = 0
		while( a < 4 )	do
			local placementLocations = {{73,153,0},{59,161,0},{49,165,0},{33,139,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uMSVN) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations)
			local unit = gen.createUnit(object.uMSVN, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a uMSVN, but we have to check the unit was actually created, and is not nil
				gen.setToNoOrders(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
			end
			-- civ.ui.text("Created MSVN")
			a = a + 1
		end

		-- Random Italian artillery placements in Sicily
		-- 75/27 Art.
		local a = 0
		while( a < 2 )	do
			-- local placementLocations = {{41,139,0},{31,139,0},{36,148,0},{43,161,0},{46,164,0},{56,172,0}}
			local placementLocations = {{32,136,0},{62,162,0},{44,140,0},{59,169,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u7527Art) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations)
			local unit = gen.createUnit(object.u7527Art, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a u7527Art, but we have to check the unit was actually created, and is not nil
				gen.setToNoOrders(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
			end
			-- civ.ui.text("Created MSVN")
			a = a + 1
		end		

		-- 105/28 Art.
		local a = 0
		while( a < 2 )	do
			local placementLocations = {{73,155,0},{62,162,0},{44,140,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u10528Art) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations)
			local unit = gen.createUnit(object.u10528Art, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a 105/28 Art., but we have to check the unit was actually created, and is not nil
				gen.setToNoOrders(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
			end
			-- civ.ui.text("Created 105/28 Art.")
			a = a + 1
		end

		-- 149/40 Art.
		local a = 0
		while( a < 2 )	do
			local placementLocations = {{54,170,0},{51,167,0},{46,160,0},{33,141,0},{41,141,0},{56,168,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u14940Art) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations)
			local unit = gen.createUnit(object.u14940Art, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a 149/40 Art., but we have to check the unit was actually created, and is not nil
				gen.setToNoOrders(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
			end
			-- civ.ui.text("Created 149/40 Art.")
			a = a + 1
		end
		
		-- 20/65 AA
		local a = 0
		while( a < 3 )	do
			local placementLocations = {{33,139,0},{59,161,0},{47,155,0},{49,165,0},{59,169,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u2065AA) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations)
			local unit = gen.createUnit(object.u2065AA, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a 20/65 AA, but we have to check the unit was actually created, and is not nil
				gen.setToNoOrders(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
			end
			-- civ.ui.text("Created 20/65 AA")
			a = a + 1
		end

		-- 75/46 AA
		local a = 0
		while( a < 1 )	do
			local placementLocations = {{32,136,0},{44,140,0},{62,162,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u7546AA) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations)
			local unit = gen.createUnit(object.u7546AA, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a 75/46 AA, but we have to check the unit was actually created, and is not nil
				gen.setToNoOrders(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
			end
			-- civ.ui.text("Created 75/46 AA")
			a = a + 1
		end	


		-- Straits of Messina
		gen.createUnit(object.u7546AA, object.pAxis, {{73,155,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		
		-- Bunkers
		gen.createUnit(object.uBunker, object.pAxis,{{55,151,0},{66,154,0},{70,154,0}}, {count=1, randomize=true, scatter=true, veteran=true})
		gen.createUnit(object.uBunker, object.pAxis, {{60,158,0},{66,158,0},{63,155,0}}, {count=1, randomize=true, scatter=true, veteran=true})
		-- In middle of Sicily around Caltanissetta
		-- local unit = gen.createUnit(object.uBunker, object.pAxis, {{47,155,0},{48,152,0},{50,158,0}}, {count=1, randomize=true, scatter=true, veteran=true})[1]
		-- if unit then -- if the unit was created, it will be a Bunker, but we have to check the unit was actually created, and is not nil
			-- gen.setToNoOrders(unit)
			-- gen.setToFortified(unit)
			-- gen.placeFortress(unit.location)
			-- unit.moveSpent = 255
		-- end	

		-- Küstenbatterie
		gen.createUnit(object.uKustenbatterie, object.pAxis, {{59,169,0},{32,136,0}}, {count=1, randomize=true, scatter=true, veteran=false})

	-- ISLAND OF SARDINIA 
		gen.createUnit(object.u2065AA, object.pAxis, {{15,89,0},{35,65,0},{22,64,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		local unit = gen.createUnit(object.u7527Art, object.pAxis, {{22,64,0},{36,68,0},{14,90,0}}, {count=1, randomize=true, scatter=true, veteran=false})[1]
		if unit then
			unit.moveSpent = 255
		end
		
	-- Crotone airbase
		local unit = gen.createUnit(object.uReggimentoCostiero, object.pAxis, {{102,156,0}}, {count=1, randomize=false, veteran=false})[1]
		if unit then
			gen.setToFortified(unit)
			unit.moveSpent = 255
		end
		local unit = gen.createUnit(object.u2065AA, object.pAxis, {{102,156,0}}, {count=1, randomize=false, veteran=true})[1]
		if unit then
			gen.setToFortified(unit)
			unit.moveSpent = 255
		end

	end

	if turn >= 6 and flag.value("Sicily is Conquered") == true then
		gen.justOnce("Salerno bunker deployed",function()
		-- Salerno zone bunker
			local unit = gen.createUnit(object.uBunker, object.pAxis, {{94,122,0},{94,116,0},{91,111,0},{94,110,0},{94,120,0}}, {count=1, randomize=true, scatter=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a Bunker, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
		end)	
	end
	
	-- SOUTHERN ITALY (Calabria and Puglia) ========================================

	-- CIVITAVECCHIA INVASION ZONE early defensive positions ========================================
	if flag.value("2nd Invasion Available") == true then
		gen.justOnce("Civitavecchia zone defenses deployed",function()
			-- Talamone
			-- First remove any Allied unit that may be located in Talamone
			-- OPTION 1: using teleport
			-- for unit in civ.iterateUnits() do
				-- if unit.owner == object.pAllies and unit.location == object.ltownTalamone then 
				-- print("Teleported " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
				-- civ.teleportUnit(unit, object.loceanTalamone)
				-- end
			-- end
			-- OPTION 2: using moveUnitAdjacent (is better because it will displace the unit(s) into any adjacent tile)
			if object.ltownTalamone.defender and object.ltownTalamone.defender ~= object.pAxis then -- (tile 74,56,0)
				for unit in object.ltownTalamone.units do
					gen.moveUnitAdjacent(unit)
				end
			end
			
			local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{74,56,0}}, {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			gen.createUnit(object.u105cmleFH18, object.pAxis, {{74,56,0}}, {randomize=true, veteran=false})
			gen.createUnit(object.u37cmFlak36, object.pAxis, {{74,56,0}}, {randomize=true, veteran=false})
			local unit = gen.createUnit(object.uBunker, object.pAxis, {{74,56,0}}, {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end

			-- Civitavecchia hill
			-- First remove any Allied unit that may be located in hill due east of Civitavecchia 
			if object.lhillCivitavecchia.defender and object.lhillCivitavecchia.defender ~= object.pAxis then -- (tile 77,67,0)
				for unit in object.lhillCivitavecchia.units do
					gen.moveUnitAdjacent(unit)
				end
			end
			
			local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{77,67,0}}, {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			gen.createUnit(object.uKustenbatterie, object.pAxis, {{77,67,0}}, {randomize=true, veteran=false})
			gen.createUnit(object.u37cmFlak36, object.pAxis, {{77,67,0}}, {randomize=true, veteran=false})
			local unit = gen.createUnit(object.uBunker, object.pAxis, {{77,67,0}}, {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end			
			
			-- Roma airfield
			-- First remove any Allied unit that may be located on airbase north of Roma 
			-- for unit in civ.iterateUnits() do
				-- if unit.owner == object.pAllies and unit.location == object.lAirbaseNW_Roma then 
				-- print("Teleported " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
				-- civ.teleportUnit(unit, object.loceanCivitavecchia2)
				-- end
			-- end
			if object.lAirbaseNW_Roma.defender and object.lAirbaseNW_Roma.defender ~= object.pAxis then -- (tile 78,72,0)
				for unit in object.lAirbaseNW_Roma.units do
					gen.moveUnitAdjacent(unit)
				end
			end

			-- Then place German defensive unit
			local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{78,72,0}}, {randomize=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
		end)
	end

	-- BARBARA TO CIVITAVECCHIA DEFENSIVE LINE OPTIONS ========================================
	-- Axis Defensive Dispositions depends on Allied invasion zone (north or south of Gustav Line)
	
	-- if turn == 1 then -- FOR TESTING ONLY
	if turn >= 6 and flag.value("2nd Invasion Available") == true 
	and ((invasionUnitsInCivitavecchia() == true)
	or (invasionUnitsInAnzio() == true or invasionUnitsInTerracina() == true)
	or	(object.ltownBarietta.owner == object.pAllies or
		object.ltownLagonegro.owner == object.pAllies or
		object.ltownLavello.owner == object.pAllies or
		object.ltownPompei.owner == object.pAllies or
		object.ltownPotenza.owner == object.pAllies or
		object.ltownSalerno.owner == object.pAllies or
		object.ltownSapri.owner == object.pAllies or
		object.ltownTermoli.owner == object.pAllies))
	then
	
		gen.justOnce("Barbara Line alternatives deployed 2",function()
			-- CIVITAVECCHIA ZONE (if 2nd invasion north of Gustav Line)
			if invasionUnitsInCivitavecchia() == true then
				local a = 0
				-- Widerstandsnest (5)
				while( a < 5 )	do
					local placementLocations = {{74,52,0},{77,59,0},{80,62,0},{82,66,0},{80,68,0},{78,58,0},{81,61,0},{84,64,0},{83,69,0}}
					for key,tileCoord in pairs(placementLocations) do
						if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uWiderstandsnest) then
							placementLocations[key] = nil
						end
					end
					gen.makeArrayOneToN(placementLocations)
					local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis,placementLocations , {count=1, randomize=false, scatter=true, veteran=true})[1]
					if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end
					a = a + 1
				end	
				-- Bunker
				gen.createUnit(object.uBunker, object.pAxis, {{77,59,0},{82,66,0},{78,72,0}}, {count=1, randomize=true, scatter=true, veteran=false})
				
				flag.setTrue("2nd Invasion Above Gustav")
				if flag.value("Display Axis Testing Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					text.simple("AXIS MESSAGE:\n^\r^German defensive positions are established around Civitavecchia!","Geman Defensive Positions Established!",object.m10thArmy)	
				end

			-- ANZIO AND TERRACINA ZONE (if 2nd invasion north of Gustav Line)
			elseif invasionUnitsInAnzio() == true or invasionUnitsInTerracina() == true then		
				if object.ltownFiumicino.defender and object.ltownFiumicino.defender ~= object.pAxis then
					for unit in object.ltownFiumicino.units do
						gen.moveUnitAdjacent(unit)
						-- gen.moveUnitAdjacent(unit,rankDestinations)
					end
				end
				if object.lhillSSEROMA.defender and object.lhillSSEROMA.defender ~= object.pAxis then
					for unit in object.lhillSSEROMA.units do
						gen.moveUnitAdjacent(unit)
						-- gen.moveUnitAdjacent(unit,rankDestinations)
					end
				end
				if object.ltownValmontone.defender and object.ltownValmontone.defender ~= object.pAxis then
					for unit in object.ltownValmontone.units do
						gen.moveUnitAdjacent(unit)
						-- gen.moveUnitAdjacent(unit,rankDestinations)
					end
				end

				local a = 0
				-- Widerstandsnest (5)
				while( a < 5 )	do
					local placementLocations = {{77,75,0},{82,80,0},{84,82,0},{83,73,0},{86,80,0},{87,75,0},}
					for key,tileCoord in pairs(placementLocations) do
						if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uWiderstandsnest) then
							placementLocations[key] = nil
						end
					end
					gen.makeArrayOneToN(placementLocations)
					local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis,placementLocations , {count=1, randomize=false, scatter=true, veteran=true})[1]
					if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end
					a = a + 1
				end	
				-- Bunker
				gen.createUnit(object.uBunker, object.pAxis, {{82,80,0},{84,82,0},{77,75,0}}, {count=1, randomize=true, scatter=true, veteran=true})
				
				flag.setTrue("2nd Invasion Above Gustav")
				
				if flag.value("Display Axis Testing Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					text.simple("AXIS MESSAGE:\n^\r^German defensive positions are established around Anzio!","Geman Defensive Positions Established!",object.m10thArmy)	
				end

			-- BARBARA LINE ZONE (if 2nd invasion south of Gustav Line)
			elseif 
			(object.ltownBarietta.owner == object.pAllies or
			object.ltownLagonegro.owner == object.pAllies or
			object.ltownLavello.owner == object.pAllies or
			object.ltownPompei.owner == object.pAllies or
			object.ltownPotenza.owner == object.pAllies or
			object.ltownSalerno.owner == object.pAllies or
			object.ltownSapri.owner == object.pAllies or
			object.ltownTermoli.owner == object.pAllies)
			then

				local a = 0
				-- Widerstandsnest (7)
				while( a < 7 )	do
					local placementLocations = {{94,116,0},{92,102,0},{109,91,0},{91,97,0},{95,95,0},{111,95,0},{106,92,0},{99,93,0},{88,100,0}}
					for key,tileCoord in pairs(placementLocations) do
						if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uWiderstandsnest) then
							placementLocations[key] = nil
						end
					end
					gen.makeArrayOneToN(placementLocations) -- This line seems necessary
					local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis,placementLocations , {randomize=true, veteran=false})[1]
					if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end
					a = a + 1
				end
			
				if flag.value("Display Axis Testing Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					text.simple("AXIS MESSAGE:\n^\r^German defensive positions are established around the Barbara Line!","Geman Defensive Positions Established!",object.m10thArmy)	
					-- civ.ui.text("Print Potenza: " .. tostring(object.ltownPotenza.owner))
				end
	
			end
	
		end)
	end		
	
	-- GUSTAV LINE - Cassino ========================================
	-- Widerstandsnest (8)
	if (turn >= 1 and turn <= 11) and (flag.value("Napoli is captured") == false or flag.value("Roma is captured") == false) then
		local placementLocations = {{88,96,0},{107,87,0},{98,90,0},{100,90,0},{102,90,0},{105,87,0},{100,86,0},{105,85,0}}
		for key,tileCoord in pairs(placementLocations) do
			if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uWiderstandsnest) then
				placementLocations[key] = nil
			end
		end
		gen.makeArrayOneToN(placementLocations) -- This line seems necessary
		local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis,placementLocations , {randomize=true, veteran=true})[1]
		if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
			gen.setToFortified(unit)
			gen.placeFortress(unit.location)
			unit.moveSpent = 255
			gen.setMoved(unit)
		end
	end

	-- Widerstandsnest (3)
	if (turn >= 6 and turn <= 10) and (flag.value("Napoli is captured") == false or flag.value("Roma is captured") == false) then
		local placementLocations = {{83,85,0},{84,88,0},{86,92,0}}
		for key,tileCoord in pairs(placementLocations) do
			if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uWiderstandsnest) then
				placementLocations[key] = nil
			end
		end
		gen.makeArrayOneToN(placementLocations) -- This line seems necessary
		local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis,placementLocations , {randomize=true, veteran=true})[1]
		if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
			gen.setToFortified(unit)
			gen.placeFortress(unit.location)
			unit.moveSpent = 255
			gen.setMoved(unit)
		end
	end	

	-- Widerstandsnest (2) - Protect German 10th Army cutoff towns
	if (turn >= 10 and turn <= 12) then
		if math.random(1,2) == 2 then -- 3 turns / 2 = 1.5 * Widerstandsnest
			local placementLocations = {{84,82,0},{85,77,0},{92,72,0},{104,72,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uWiderstandsnest) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis,placementLocations , {randomize=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
		end	
	end		
	
	-- Widerstandsnest Bunker (11)
	if (turn >= 4 and turn <= 14) and (flag.value("Napoli is captured") == false or flag.value("Roma is captured") == false) then
		local placementLocations = {{88,96,0},{107,87,0},{83,85,0},{84,88,0},{86,92,0},{98,90,0},{100,90,0},{102,90,0},{105,87,0},{100,86,0},{105,85,0}}
		for key,tileCoord in pairs(placementLocations) do
			if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uBunker) then
				placementLocations[key] = nil
			end
		end
		gen.makeArrayOneToN(placementLocations) -- This line seems necessary
		local unit = gen.createUnit(object.uBunker, object.pAxis,placementLocations , {randomize=true, veteran=false})[1]
		if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
			gen.setToFortified(unit)
			-- gen.placeFortress(unit.location)
			unit.moveSpent = 255
			gen.setMoved(unit)
		end		
	end		

	-- Fallschirmjager (4)
	if (turn >= 6 and turn <= 10) and (flag.value("Napoli is captured") == false or flag.value("Roma is captured") == false) then
		local placementLocations = {{91,93,0},{94,92,0},{88,94,0}}
		for key,tileCoord in pairs(placementLocations) do
			if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uFallschirmjager) then
				placementLocations[key] = nil
			end
		end
		gen.makeArrayOneToN(placementLocations) -- This line seems necessary
		local unit = gen.createUnit(object.uFallschirmjager, object.pAxis,placementLocations , {randomize=false, veteran=true})[1]
		if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
			gen.setToFortified(unit)
			gen.placeFortress(unit.location)
			unit.moveSpent = 255
			gen.setMoved(unit)
		end
	end 	
	
	-- Fallschirmjager Bunkers (4)
	if (turn >= 10 and turn <= 14) and (flag.value("Napoli is captured") == false or flag.value("Roma is captured") == false) then
		local placementLocations = {{91,93,0},{94,92,0},{90,90,0},{88,86,0}}
		for key,tileCoord in pairs(placementLocations) do
			if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uBunker) then
				placementLocations[key] = nil
			end
		end
		gen.makeArrayOneToN(placementLocations) -- This line seems necessary
		local unit = gen.createUnit(object.uBunker, object.pAxis,placementLocations , {randomize=false, veteran=true})[1]
		if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
			gen.setToFortified(unit)
			-- gen.placeFortress(unit.location)
			unit.moveSpent = 255
			gen.setMoved(unit)
		end
	end 

	-- 10.5cm leFH18 Art.	
	if (turn >= 5 and turn <= 15) and (flag.value("Napoli is captured") == false or flag.value("Roma is captured") == false) then
		if math.random(1,2) == 2 then -- 11 turns / 2 = 5.5 * 10.5cm leFH18 Art.
			local placementLocations = {{77,77,0},{83,83,0},{86,92,0},{88,96,0},{99,83,0},{107,87,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u105cmleFH18) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			local unit = gen.createUnit(object.u105cmleFH18, object.pAxis,placementLocations , {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
		end	
	end	
	
	-- 15cm sIG33 Art.
	if (turn >= 6 and turn <= 16) and (flag.value("Napoli is captured") == false or flag.value("Roma is captured") == false) then
		if math.random(1,3) == 3 then -- 11 turns / 3 =  3.6 * 15cm sIG33 Art.
			local placementLocations = {{81,79,0},{88,94,0},{94,92,0},{100,90,0},{105,85,0},{83,85,0},{102,90,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u15cmsIG33) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			local unit = gen.createUnit(object.u15cmsIG33, object.pAxis,placementLocations , {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
		end	
	end

	-- 3.7cm Flak 36 AA	
	if (turn >= 5 and turn <= 15) and (flag.value("Napoli is captured") == false or flag.value("Roma is captured") == false) then
		if math.random(1,2) == 2 then -- 11 turns / 2 = 5.5 * 3.7cm Flak 36 AA
			local placementLocations = {{77,77,0},{84,88,0},{88,94,0},{106,80,0},{88,86,0},{98,90,0},{105,87,0},{85,77,0},{90,90,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u37cmFlak36) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			local unit = gen.createUnit(object.u37cmFlak36, object.pAxis,placementLocations , {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end	
		end	
	end	
	
	-- 8.8cm Flak 37 AA
	if (turn >= 6 and turn <= 16) and (flag.value("Napoli is captured") == false or flag.value("Roma is captured") == false) then
		if math.random(1,3) == 3 then -- 11 turns / 3 = 3.6 * 8.8cm Flak 37 AA
			local placementLocations = {{80,76,0},{91,93,0},{100,86,0},{107,83,0},{81,79,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u88cmFlak37) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			local unit = gen.createUnit(object.u88cmFlak37, object.pAxis,placementLocations , {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end				
		end	
	end		

	-- Miscellaneous Bunker (7)
	if (turn >= 8 and turn <= 21) 
	and (flag.value("Roma is captured") == false 
	or object.ltownAnzio.owner ~= object.pAllies
	or object.ltownCivitavecchia.owner ~= object.pAllies
	or object.ltownTerracina.owner ~= object.pAllies)
	then
		if math.random(1,2) == 2 then -- 14 turns / 2 = 7 possible bunkers
			local placementLocations = {{85,91,0},{91,91,0},{85,77,0},{97,85,0},{99,83,0},{106,80,0},{108,72,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uBunker) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			local unit = gen.createUnit(object.uBunker, object.pAxis,placementLocations , {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end			
		end	
	end	
	
 	-- CAESAR C LINE - Avezzano =====================================	

 	-- TRASIMENE LINE - Grosseto to Ancona ==========================	
	-- See onEnterTile.lua

	-- GOTHIC LINE - Bologna ========================================
	-- Trigger beginning of construction of Gothic Line
	if turn >= 9
	and flag.value("Sicily is Conquered") == true
	and (object.ltownAnzio.owner == object.pAllies
	or object.ltownCassino.owner == object.pAllies
	or object.ltownCecina.owner == object.pAllies
	or object.ltownCivitavecchia.owner == object.pAllies
	or object.ltownFano.owner == object.pAllies
	or object.ltownMassa.owner == object.pAllies
	or object.ltownTerracina.owner == object.pAllies
	or object.ltownFormia.owner == object.pAllies
	or object.lfortNE_Isernia.owner == object.pAllies
	or object.ltownOrtona.owner == object.pAllies)
	then
		gen.justOnce("Set Gothic Line Build Status",function()
			flag.setTrue("Gothic Line Construction Begun")
			if flag.value("Display Axis Testing Messages") == true then
				civ.playSound("Bldspcsh.wav")
				text.simple("AXIS TESTING MESSAGE:\n^The Gothic Line building status is now set to begin!","Gothic Line Construction to Begin!",object.monexone)
			end
			gen.placeFortressForce({95,35,0})
			gen.placeFortressForce({96,32,0})
		end)	
	end

	-- Kasematte (9) - main line
	if (turn >= 9 and turn <= 60)
	and flag.value("Gothic Line Construction Begun") == true
	and counter.value("# Gothic Line Kasematte Count") < 11	
	and flag.value("Firenze is captured") == false
	and flag.value("Livorno is captured") == false
	and object.ltownArezzo.owner ~= object.pAllies
	and object.ltownFano.owner ~= object.pAllies
	and object.ltownPisa.owner ~= object.pAllies
	and object.ltownUrbino.owner ~= object.pAllies	
	then
		if math.random(1,2) == 2 then
			local placementLocations = {{80,26,0},{84,24,0},{87,29,0},{88,34,0},{91,37,0},{96,40,0},{99,39,0},{101,43,0},{104,46,0},{106,48,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uKasematte) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			-- local unit = gen.createUnit(object.uKasematte, object.pAxis,placementLocations , {randomize=true, veteran=true})[1]
			local unit = gen.createUnit(object.uKasematte, object.pAxis, filterTilesByOwner(placementLocations,object.pAllies), {randomize=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			counter.add("# Gothic Line Kasematte Count",1)
		end	
	end 

	-- Widerstandsnest (6) - front of main line
	if  (turn >= 9 and turn <= 60)
	and flag.value("Gothic Line Construction Begun") == true
	and counter.value("# Gothic Line WSN front line Count") < 7		
	and flag.value("Firenze is captured") == false
	and flag.value("Livorno is captured") == false
	and object.ltownArezzo.owner ~= object.pAllies
	and object.ltownFano.owner ~= object.pAllies
	and object.ltownPisa.owner ~= object.pAllies
	and object.ltownUrbino.owner ~= object.pAllies	
	then
		if math.random(1,2) == 2 then
			local placementLocations = {{79,29,0},{82,30,0},{94,42,0},{97,45,0},{101,47,0},{80,26,0},{101,43,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uWiderstandsnest) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			-- local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis,placementLocations , {randomize=true, veteran=true})[1]
			local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, filterTilesByOwner(placementLocations,object.pAllies), {randomize=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			counter.add("# Gothic Line WSN front line Count",1)
		end	
	end

	-- Fallschirmjager (4) - front of main line
	if  (turn >= 9 and turn <= 60)
	and flag.value("Gothic Line Construction Begun") == true
	and counter.value("# Gothic Line Fallschirmjager Count") < 7	
	and flag.value("Firenze is captured") == false
	and flag.value("Livorno is captured") == false
	and object.ltownArezzo.owner ~= object.pAllies
	and object.ltownFano.owner ~= object.pAllies
	and object.ltownPisa.owner ~= object.pAllies
	and object.ltownUrbino.owner ~= object.pAllies	
	then
		-- Firenze sector - 4th Fallschirmjäger Division
		if math.random(1,2) == 2 then
		local placementLocations = {{91,37,0},{91,33,0},{93,39,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uFallschirmjager) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			-- local unit = gen.createUnit(object.uFallschirmjager, object.pAxis, placementLocations, {randomize=true, veteran=true})[1]
			local unit = gen.createUnit(object.uFallschirmjager, object.pAxis, filterTilesByOwner(placementLocations,object.pAllies), {randomize=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a Fallschirmjager, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			counter.add("# Gothic Line Fallschirmjager Count",1)
		end	

		-- Rimini sector - 1st Fallschirmjäger Division
		if math.random(1,2) == 2 then
		local placementLocations = {{106,48,0},{104,46,0},{103,39,0},{107,37,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uFallschirmjager) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			-- local unit = gen.createUnit(object.uFallschirmjager, object.pAxis, placementLocations, {randomize=true, veteran=true})[1]
			local unit = gen.createUnit(object.uFallschirmjager, object.pAxis, filterTilesByOwner(placementLocations,object.pAllies), {randomize=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a Fallschirmjager, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			counter.add("# Gothic Line Fallschirmjager Count",1)
		end	
		
	end

	-- Bunker - front of main line (9)
	if (turn >= 9 and turn <= 60)
	and flag.value("Gothic Line Construction Begun") == true
	and counter.value("# Gothic Line BKR front line Count") < 9		
	and flag.value("Firenze is captured") == false
	and flag.value("Livorno is captured") == false
	and object.ltownArezzo.owner ~= object.pAllies
	and object.ltownFano.owner ~= object.pAllies
	and object.ltownPisa.owner ~= object.pAllies
	and object.ltownUrbino.owner ~= object.pAllies	
	then
		if math.random(1,2) == 2 then 
			local placementLocations = {{79,29,0},{82,30,0},{87,31,0},{91,33,0},{93,39,0},{94,42,0},{95,35,0},{97,45,0},{101,47,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uBunker) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			-- local unit = gen.createUnit(object.uBunker, object.pAxis,placementLocations , {randomize=true, veteran=true})[1]
			local unit = gen.createUnit(object.uBunker, object.pAxis, filterTilesByOwner(placementLocations,object.pAllies), {randomize=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a uBunker, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			counter.add("# Gothic Line BKR front line Count",1)
		end	
	end	
	
	-- 10.5cm leFH18 Art. 	
	if  (turn >= 9 and turn <= 60)
	and flag.value("Gothic Line Construction Begun") == true
	and counter.value("# Gothic Line 105mm Art Count") < 6		
	and (object.lfortS_Firenzuola.owner ~= object.pAllies
	or object.lfortSE_Firenzuola.owner ~= object.pAllies
	or object.lfortS_Rimini.owner ~= object.pAllies
	or object.lfortSW_Rimini.owner ~= object.pAllies
	or object.ltownLucca.owner ~= object.pAllies
	or object.ltownMassa.owner ~= object.pAllies
	or object.ltownPistoia.owner ~= object.pAllies
	or object.ltownPracchia.owner ~= object.pAllies)	
	then
		if math.random(1,4) == 4 then 
			local placementLocations = {{79,29,0},{82,30,0},{87,31,0},{91,33,0},{91,37,0},{93,39,0},{94,42,0},{97,45,0},{101,47,0},{106,48,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u105cmleFH18) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			-- local unit = gen.createUnit(object.u105cmleFH18, object.pAxis,placementLocations , {randomize=true, veteran=false})[1]
			local unit = gen.createUnit(object.u105cmleFH18, object.pAxis, filterTilesByOwner(placementLocations,object.pAllies), {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a u105cmleFH18, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			counter.add("# Gothic Line 105mm Art Count",1)
		end	
	end	
	
	-- 15cm sIG33 Art.
	if  (turn >= 9 and turn <= 60)
	and flag.value("Gothic Line Construction Begun") == true
	and counter.value("# Gothic Line 155mm Art Count") < 4		
	and (object.lfortS_Firenzuola.owner ~= object.pAllies
	or object.lfortSE_Firenzuola.owner ~= object.pAllies
	or object.lfortS_Rimini.owner ~= object.pAllies
	or object.lfortSW_Rimini.owner ~= object.pAllies
	or object.ltownLucca.owner ~= object.pAllies
	or object.ltownMassa.owner ~= object.pAllies
	or object.ltownPistoia.owner ~= object.pAllies
	or object.ltownPracchia.owner ~= object.pAllies)	
	then
		if math.random(1,5) == 5 then 
			local placementLocations = {{80,26,0},{84,24,0},{88,34,0},{95,35,0},{96,40,0},{101,43,0},{104,46,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u15cmsIG33) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			-- local unit = gen.createUnit(object.u15cmsIG33, object.pAxis,placementLocations , {randomize=true, veteran=false})[1]
			local unit = gen.createUnit(object.u15cmsIG33, object.pAxis, filterTilesByOwner(placementLocations,object.pAllies), {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a u15cmsIG33, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			counter.add("# Gothic Line 155mm Art Count",1)
		end	
	end	

	-- 3.7cm Flak 36 AA	
	if  (turn >= 9 and turn <= 60)
	and flag.value("Gothic Line Construction Begun") == true
	and counter.value("# Gothic Line 37mm AA Count") < 7		
	and (object.lfortS_Firenzuola.owner ~= object.pAllies
	or object.lfortSE_Firenzuola.owner ~= object.pAllies
	or object.lfortS_Rimini.owner ~= object.pAllies
	or object.lfortSW_Rimini.owner ~= object.pAllies
	or object.ltownLucca.owner ~= object.pAllies
	or object.ltownMassa.owner ~= object.pAllies
	or object.ltownPistoia.owner ~= object.pAllies
	or object.ltownPracchia.owner ~= object.pAllies)	
	then
		if math.random(1,4) == 4 then 
			local placementLocations = {{79,29,0},{80,26,0},{82,30,0},{91,37,0},{93,39,0},{94,42,0},{97,45,0},{99,39,0},{101,47,0},{104,46,0},{106,48,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u37cmFlak36) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			-- local unit = gen.createUnit(object.u37cmFlak36, object.pAxis,placementLocations , {randomize=true, veteran=false})[1]
			local unit = gen.createUnit(object.u37cmFlak36, object.pAxis, filterTilesByOwner(placementLocations,object.pAllies), {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a u37cmFlak36, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			counter.add("# Gothic Line 37mm AA Count",1)
		end	
	end	
	
	-- 8.8cm Flak 37 AA
	if  (turn >= 9 and turn <= 60)
	and flag.value("Gothic Line Construction Begun") == true
	and counter.value("# Gothic Line 88mm AA Count") < 3		
	and (object.lfortS_Firenzuola.owner ~= object.pAllies
	or object.lfortSE_Firenzuola.owner ~= object.pAllies
	or object.lfortS_Rimini.owner ~= object.pAllies
	or object.lfortSW_Rimini.owner ~= object.pAllies
	or object.ltownLucca.owner ~= object.pAllies
	or object.ltownMassa.owner ~= object.pAllies
	or object.ltownPistoia.owner ~= object.pAllies
	or object.ltownPracchia.owner ~= object.pAllies)		
	then
		if math.random(1,5) == 5 then 
			local placementLocations = {{77,23,0},{84,24,0},{87,31,0},{88,34,0},{91,33,0},{95,35,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u88cmFlak37) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			-- local unit = gen.createUnit(object.u88cmFlak37, object.pAxis,placementLocations , {randomize=true, veteran=false})[1]
			local unit = gen.createUnit(object.u88cmFlak37, object.pAxis, filterTilesByOwner(placementLocations,object.pAllies), {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a u88cmFlak37, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			counter.add("# Gothic Line 88mm AA Count",1)
		end	
	end		

	-- ========================================================
	-- Bunker - back of main line (10)
	if turn >= 9
	and flag.value("Gothic Line Construction Begun") == true
	and counter.value("# Gothic Line BKR back line Count") < 9	
	and (object.lfortS_Firenzuola.owner ~= object.pAllies
	or object.lfortSE_Firenzuola.owner ~= object.pAllies
	or object.lfortS_Rimini.owner ~= object.pAllies
	-- or object.lfortSW_Rimini.owner ~= object.pAllies
	or object.ltownLucca.owner ~= object.pAllies
	or object.ltownMassa.owner ~= object.pAllies
	or object.ltownPistoia.owner ~= object.pAllies
	or object.ltownPracchia.owner ~= object.pAllies)	
	then
		if math.random(1,2) == 2 then -- 32 turns / 4 = 8 possible bunkers
			local placementLocations = {{77,19,0},{81,21,0},{88,24,0},{90,28,0},{93,31,0},{98,36,0},{101,35,0},{103,43,0},{107,31,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uBunker) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			-- local unit = gen.createUnit(object.uBunker, object.pAxis,placementLocations , {randomize=true, veteran=true})[1]
			local unit = gen.createUnit(object.uBunker, object.pAxis, filterTilesByOwner(placementLocations,object.pAllies) , {randomize=true, veteran=true})[1]
			if unit then -- if the unit was created, it will be a uBunker, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			counter.add("# Gothic Line BKR back line Count",1)
		end
	end 	

	-- Widerstandsnest - back of main line (3)
	if turn >= 9
	and flag.value("Gothic Line Construction Begun") == true
	and counter.value("# Gothic Line WSN back line Count") < 4	
	and (object.lfortS_Firenzuola.owner ~= object.pAllies
	or object.lfortSE_Firenzuola.owner ~= object.pAllies
	or object.lfortS_Rimini.owner ~= object.pAllies
	-- or object.lfortSW_Rimini.owner ~= object.pAllies
	or object.ltownLucca.owner ~= object.pAllies
	or object.ltownMassa.owner ~= object.pAllies
	or object.ltownPistoia.owner ~= object.pAllies
	or object.ltownPracchia.owner ~= object.pAllies)	
	then
		if math.random(1,2) == 2 then
			-- local placementLocations = {{79,29,0},{82,30,0},{85,29,0},{87,31,0},{88,34,0},{91,37,0},{93,39,0},{94,42,0},{97,45,0},{101,47,0},{104,46,0},{106,48,0},{99,41,0}}
			local placementLocations = {{83,19,0},{81,21,0},{95,35,0},{96,32,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uWiderstandsnest) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			-- local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis,placementLocations , {randomize=true, veteran=false})[1]
			local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, filterTilesByOwner(placementLocations,object.pAllies), {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			counter.add("# Gothic Line WSN back line Count",1)
		end	
	end

	-- ERSATZ CITY REINFORCEMENTS BEHIND GOTHIC LINE
	-- Widerstandsnest - (8)
	if turn >= 20 -- 62 -- Oct II, 1944
	and flag.value("Gothic Line Construction Begun") == true
	and counter.value("# Gothic Line Ersatz Count") < 8	
	and (object.lfortS_Firenzuola.owner == object.pAllies
	or object.lfortSE_Firenzuola.owner == object.pAllies
	or object.lfortS_Rimini.owner == object.pAllies
	-- or object.lfortSW_Rimini.owner ~= object.pAllies
	or object.ltownLucca.owner == object.pAllies
	or object.ltownMassa.owner == object.pAllies
	or object.ltownPistoia.owner == object.pAllies
	or object.ltownPracchia.owner == object.pAllies)	
	then
		
		gen.justOnce("Ersatz City Reinforcements Triggered",function()
			flag.setTrue("Ersatz Reinforcements Triggered")
		end)

		if math.random(1,2) == 2 then -- 
			-- local placementLocations = {{76,16,0},{83,19,0},{86,24,0},{90,28,0},{92,30,0},{95,31,0},{101,35,0},{107,37,0}}
			local placementLocations = {{75,1,0},{91,9,0},{93,21,0},{96,24,0},{100,30,0},{105,27,0},{106,14,0},{137,31,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uWiderstandsnest) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, placementLocations , {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			counter.add("# Gothic Line Ersatz Count",1)
		end
	end 

	-- ADRIATIC AND LIGURIAN COMMANDS ========================================
	-- ADRIATIC: Schützen + Gebirgsjäger + 10.5cm leFH18 + 3.7cm Flak 36
	if turn >= 20 -- 57 -- Sep I, 1944 
	and (object.lFirenze.owner == object.pAllies
	-- or object.ltownArezzo.owner == object.pAllies
	-- or object.ltownFano.owner == object.pAllies
	-- or object.ltownUrbino.owner == object.pAllies
	or object.lfortSE_Firenzuola.owner == object.pAllies
	or object.ltownLeccio.owner == object.pAllies
	or object.lfortSW_Rimini.owner == object.pAllies
	or object.ltownSanMarino.owner == object.pAllies	
	)	
	then
		-- Confirmation Message
		if flag.value("Display Axis Testing Messages") == true then
			gen.justOnce("German Adriatic Command",function()
				civ.playSound("Cheers2.wav")
				text.simple("AXIS TESTING MESSAGE:\n^The German Adriatic Command is activated!","German Adriatic Command Activated!",object.monexone)
				-- flag.setTrue("Adriatic Command triggered")
			end)	
		end	

		gen.justOnce("German Adriatic Command Flag",function()
			flag.setTrue("Adriatic Command triggered")
		end)	

		-- Gebirgsjäger (just once)
		if math.random(1,8) == 8 then -- 2 possible Gebirgsjäger
			gen.justOnce("182nd Gebirgsjäger",function()
				gen.createUnit(object.uGebirgsjager, object.pAxis, {{106,14,0},{105,27,0},{115,21,0},{119,25,0},{136,20,0},{137,31,0},{101,17,0},{107,37,0}}, {count=2, randomize=true, veteran=false})
			end)
		end
		-- Infanterie Veteranen
		if math.random(1,4) == 4 then -- 32 turns / 8 = 4*2 possible Schützen
			gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{106,14,0},{105,27,0},{115,21,0},{119,25,0},{136,20,0},{137,31,0},{101,17,0},{107,37,0}}, {count=1, randomize=true, veteran=false})
		end
	
		-- 105cm leFH18
		if math.random(1,9) == 9 then -- 32 turns / 9 = 3.5 possible 10.5cm leFH18
		--	gen.createUnit(object.u105cmleFH18, object.pAxis, {{106,14,0},{105,27,0},{115,21,0},{119,25,0},{136,20,0},{137,31,0},{101,17,0},{107,37,0}}, {count=1, randomize=true, veteran=false})
			local placementLocations = {{106,14,0},{105,27,0},{115,21,0},{119,25,0},{136,20,0},{137,31,0},{101,17,0},{103,39,0},{107,37,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u105cmleFH18) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			local unit = gen.createUnit(object.u105cmleFH18, object.pAxis, placementLocations , {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
		end

		-- 37cm Flak36
		if math.random(1,9) == 9 then -- 32 turns / 9 = 3.5 possible 3.7cm Flak 36
		--	gen.createUnit(object.u37cmFlak36, object.pAxis, {{106,14,0},{105,27,0},{115,21,0},{119,25,0},{136,20,0},{137,31,0},{101,17,0},{107,37,0}}, {count=1, randomize=true, veteran=false})
			local placementLocations = {{106,14,0},{105,27,0},{115,21,0},{119,25,0},{136,20,0},{137,31,0},{101,17,0},{103,39,0},{107,37,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u37cmFlak36) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			local unit = gen.createUnit(object.u37cmFlak36, object.pAxis, placementLocations , {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
		end

	end		
	
	-- LIGURIAN: Schützen + Gebirgsjäger + 10.5cm leFH18 + 3.7cm Flak 36
	if turn >= 20 -- 57 -- Sep I, 1944
	and (object.lLaSpezia.owner == object.pAllies
	or object.ltownLucca.owner == object.pAllies
	or object.ltownMassa.owner == object.pAllies
	-- or object.ltownPisa.owner == object.pAllies
	or object.ltownPistoia.owner == object.pAllies
	or object.ltownPracchia.owner == object.pAllies
	-- or object.lRimini.owner == object.pAllies
	)		
	then
		-- Confirmation Message
		if flag.value("Display Axis Testing Messages") == true then
			gen.justOnce("German Ligurian Command",function()
				civ.playSound("Cheers2.wav")
				text.simple("AXIS TESTING MESSAGE:\n^The German Ligurian Command is activated!","German Ligurian Command Activated!",object.monexone)
				-- flag.setTrue("Ligurian Command triggered")
			end)	
		end			

		gen.justOnce("German Ligurian Command Flag",function()
			flag.setTrue("Ligurian Command triggered")
		end)
			
		-- Gebirgsjäger (just once)
		if math.random(1,8) == 8 then -- 2 possible Gebirgsjäger
			gen.justOnce("157th Gebirgsjäger",function()
				gen.createUnit(object.uGebirgsjager, object.pAxis, {{77,23,0},{72,12,0},{66,10,0},{87,9,0},{84,2,0},{75,1,0}}, {count=2, randomize=true, veteran=false})
			end)
		end
		-- Infanterie Veteranen
		if math.random(1,5) == 5 then -- 32 turns / 5 = 6.4 possible Schützen
			gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{77,23,0},{72,12,0},{66,10,0},{87,9,0},{84,2,0},{75,1,0}}, {count=1, randomize=true, veteran=false})
		end
		-- 105cm leFH18
		if math.random(1,11) == 11 then -- 32 turns / 11 = 2.9 possible 10.5cm leFH18
		-- gen.createUnit(object.u105cmleFH18, object.pAxis, {{91,9,0},{72,12,0},{66,10,0},{87,9,0},{84,2,0},{75,1,0}}, {count=1, randomize=true, veteran=false})
			local placementLocations = {{91,9,0},{72,12,0},{66,10,0},{87,9,0},{84,2,0},{75,1,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u105cmleFH18) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			local unit = gen.createUnit(object.u105cmleFH18, object.pAxis, placementLocations , {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
		end

		-- 37cm Flak36
		if math.random(1,11) == 11 then -- 12 turns / 11 = 2.9 possible 3.7cm Flak 36
		-- 	gen.createUnit(object.u37cmFlak36, object.pAxis, {{77,23,0},{72,12,0},{66,10,0},{87,9,0},{84,2,0},{75,1,0}}, {count=1, randomize=true, veteran=false})
			local placementLocations = {{72,12,0},{66,10,0},{87,9,0},{84,2,0},{75,1,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u37cmFlak36) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations) -- This line seems necessary
			local unit = gen.createUnit(object.u37cmFlak36, object.pAxis, placementLocations , {randomize=true, veteran=false})[1]
			if unit then -- if the unit was created, it will be a Widerstandsnest, but we have to check the unit was actually created, and is not nil
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
		end

	end	
	
	-- ====================================================================	
	-- AXIS - REINFORCEMENTS PLACEMENT
	-- ====================================================================

	-- ITALIANS
	-- SICILY
	if turn == 1 then -- July I, 1943	
		-- Random Axis placements in Sicily
		gen.justOnce("Axis Sicilian Reaction Force #1 deployed",function()
			local unit = gen.createUnit(object.uFanteriaMotorizzata, object.pAxis, {{33,139,0},{37,141,0},{32,136,0}}, {count=1, randomize=true, veteran=false})[1]
			-- Italian Mechanized units
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uM1340, object.pAxis, {{33,139,0},{37,141,0},{32,136,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uSemovente7518, object.pAxis, {{33,139,0},{37,141,0},{32,136,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uFanteriaVeterana, object.pAxis, {{31,139,0},{36,138,0},{36,148,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uFanteriaVeterana, object.pAxis, {{55,167,0},{54,170,0},{52,166,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uFanteriaMotorizzata, object.pAxis, {{47,155,0},{51,155,0},{50,160,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uM1340, object.pAxis, {{47,155,0},{51,155,0},{50,160,0},{59,169,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uAB41, object.pAxis, {{51,155,0},{44,140,0},{32,136,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uSemovente7518, object.pAxis, {{47,155,0},{51,155,0},{50,160,0}}, {count=1, randomize=true, veteran=true})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uBersaglieri, object.pAxis, {{32,136,0},{44,140,0},{47,155,0}}, {count=1, randomize=true, veteran=true})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uBersaglieri, object.pAxis, {{50,160,0},{73,155,0},{59,169,0}}, {count=1, randomize=true, veteran=true})[1]
			if unit then
				unit.moveSpent = 255
			end

			-- Herman Goering Deploys in Central Sicily 
			local unit = gen.createUnit(object.uMotInfanterie, object.pAxis, {{47,155,0},{51,155,0},{50,162,0},{55,167,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uMotInfanterie, object.pAxis, {{47,155,0},{51,155,0},{50,162,0},{55,167,0}}, {count=1, randomize=true, veteran=true})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uPanzerGrenadier, object.pAxis, {{47,155,0},{51,155,0},{50,162,0},{55,167,0}}, {count=1, randomize=true, veteran=true})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uSdkFz72AA, object.pAxis, {{47,155,0},{51,155,0},{50,162,0},{55,167,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uPzkwIVF, object.pAxis, {{47,155,0},{51,155,0},{50,162,0},{55,167,0}}, {count=1, randomize=true, veteran=true})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uPzkwIVF, object.pAxis, {{47,155,0},{51,155,0},{50,162,0},{55,167,0}}, {count=1, randomize=true, veteran=true})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{47,155,0},{51,155,0},{50,162,0},{55,167,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uPzkwVIE, object.pAxis, {{47,155,0},{51,155,0},{50,162,0},{55,167,0}}, {count=1, randomize=true, veteran=true})[1]
			if unit then
				unit.moveSpent = 255
				-- To reflect the fact that the Germans had only deployed 1 Tiger (PzkwVIE) company to Sicily have the unit start partially 'damaged'
				unit.damage = unit.damage + 15
			end
			
			-- Random placement of 15th PanzerGrenadier Division in Sicily
			local unit = gen.createUnit(object.uSdkFz231, object.pAxis, {{33,139,0},{37,141,0},{36,148,0},{44,144,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uMotInfanterie, object.pAxis, {{33,139,0},{37,141,0},{36,148,0},{44,144,0}}, {count=1, randomize=true, veteran=true})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uMotInfanterie, object.pAxis, {{33,139,0},{37,141,0},{36,148,0},{44,144,0}}, {count=1, randomize=true, veteran=true})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uMotInfanterie, object.pAxis, {{33,139,0},{37,141,0},{36,148,0},{44,144,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uSdkFz72AA, object.pAxis, {{33,139,0},{37,141,0},{36,148,0},{44,144,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uStugIIIG, object.pAxis, {{33,139,0},{37,141,0},{36,148,0},{44,144,0}}, {count=1, randomize=true, veteran=true})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uPzkwIVF, object.pAxis, {{33,139,0},{37,141,0},{36,148,0},{44,144,0}}, {count=1, randomize=true, veteran=true})[1]
			if unit then
				unit.moveSpent = 255
			end
			local unit = gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{33,139,0},{37,141,0},{36,148,0},{44,144,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				unit.moveSpent = 255
			end
		end)	
	end


	-- GERMANS
	-- SICILY
	if turn == 2 then -- July II, 1943
		-- Random placement of Herman Goering Panzer Division in Sicily
		-- local hgSicilyLocationList = {{47,155,0},{51,155,0},{50,162,0},{55,167,0}}
		-- local hgLocationList = hgSicilyLocationList
		-- gen.justOnce("Herman Goering Division is deployed",function()
			-- gen.createUnit(object.uMotInfanterie, object.pAxis, hgLocationList, {count=2, randomize=true, scatter=true, veteran=false})
			-- gen.createUnit(object.uPanzerGrenadier, object.pAxis, hgLocationList, {count=1, randomize=true, scatter=true, veteran=false})	
			-- gen.createUnit(object.uSdkFz72AA, object.pAxis, hgLocationList, {count=1, randomize=true, scatter=true, veteran=false})
			-- gen.createUnit(object.uPzkwIVF, object.pAxis, hgLocationList, {count=2, randomize=true, scatter=true, veteran=false})
			-- gen.createUnit(object.uGrilleHsIG33, object.pAxis, hgLocationList, {count=1, randomize=true, scatter=true, veteran=false})
			-- gen.createUnit(object.uPzkwVIE, object.pAxis, hgLocationList, {count=1, randomize=true, scatter=true, veteran=true})
			-- -- To reflect the fact that the Germans had only deployed 1 Tiger (PzkwVIE) company to Sicily have the unit start partially 'damaged'
			-- for unit in civ.iterateUnits() do
				-- if unit.type == object.uPzkwVIE then
					-- unit.damage = unit.damage + 20
				-- end
			-- end
			-- for unit in civ.iterateUnits() do
				-- if (unit.location.x >= 47 and unit.location.y >= 155 and unit.location.x <= 51 and unit.location.y <= 161 and unit.location.z == 0) then
					-- unit.gotoTile = civ.getTile(46,164,0) 
				-- end
			-- end	
		-- end)
		
		-- Herman Goering Division moves towards Gela beaches
		for unit in civ.iterateUnits() do
			if unit.owner == object.pAxis then
				if (unit.location.x >= 47 and unit.location.y >= 155 and unit.location.x <= 51 and unit.location.y <= 161 and unit.location.z == 0) then
					unit.gotoTile = civ.getTile(46,164,0) 
				end
			end	
		end	
	
		-- Random placement of 15th PanzerGrenadier Division in Sicily
		-- local pzg15SicilyLocationList = {{33,139,0},{37,141,0},{36,148,0},{44,144,0}}
		-- local pzg15LocationList = pzg15SicilyLocationList
		-- gen.justOnce("15th PanzerGrenadier Division is deployed",function()
			-- gen.createUnit(object.uSdkFz231, object.pAxis, pzg15LocationList, {count=1, randomize=true, scatter=true, veteran=false})
			-- gen.createUnit(object.uMotInfanterie, object.pAxis, pzg15LocationList, {count=3, randomize=true, scatter=true, veteran=false})	
			-- gen.createUnit(object.uSdkFz72AA, object.pAxis, pzg15LocationList, {count=1, randomize=true, scatter=true, veteran=false})
			-- gen.createUnit(object.uStugIIIG, object.pAxis, pzg15LocationList, {count=1, randomize=true, scatter=true, veteran=false})		
			-- gen.createUnit(object.uPzkwIVF, object.pAxis, pzg15LocationList, {count=1, randomize=true, scatter=true, veteran=false})
			-- gen.createUnit(object.uGrilleHsIG33, object.pAxis, pzg15LocationList, {count=1, randomize=true, scatter=true, veteran=false})
			-- for unit in civ.iterateUnits() do
				-- if (unit.location.x >= 33 and unit.location.y >= 137 and unit.location.x <= 37 and unit.location.y <= 141 and unit.location.z == 0) then
					-- unit.gotoTile = civ.getTile(31,139,0) 
				-- end
			-- end	
		-- end)

		-- 15th PanzerGrenadier Division moves towards Trapani beaches
		for unit in civ.iterateUnits() do
			if unit.owner == object.pAxis then
				if (unit.location.x >= 33 and unit.location.y >= 137 and unit.location.x <= 37 and unit.location.y <= 141 and unit.location.z == 0) then
					unit.gotoTile = civ.getTile(31,139,0) 
				end
			end	
		end
		
		-- Random placement of 1st Fallschirmjäger Division in Sicily
		local a = 0
		while( a < 2 )	do
			local placementLocations = {{60,162,0},{60,158,0},{62,162,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uFallschirmjager) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations)
			local unit = gen.createUnit(object.uFallschirmjager, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
			if unit then
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
				a = a + 1
			end
		end
	
		local unit = gen.createUnit(object.uFallschirmjager, object.pAxis, {{62,162,0}}, {count=1, randomize=false, veteran=true})	[1]
		if unit then
			gen.setToFortified(unit)
			-- gen.placeFortress(unit.location)
			unit.moveSpent = 255
			gen.setMoved(unit)
		end	

		local unit = gen.createUnit(object.uFallschirmjager, object.pAxis, {{73,155,0},{70,154,0},{59,161,0}}, {count=1, randomize=true, veteran=true})	[1]
		if unit then
			gen.setToFortified(unit)
			gen.placeFortress(unit.location)
			unit.moveSpent = 255
			gen.setMoved(unit)
			text.simple("Allied intelligence has detected the arrival of the 1st " .. unit.type.name .. " Division in Sicily!","Axis Reinforcements",object.monexone)
		end

		-- Random selection of German AA unit for Reggio Calabria
		local randomNumber = math.random(2)
		if randomNumber == 1 then
			gen.createUnit(object.u37cmFlak36, object.pAxis, {{73,159,0}}, {randomize=false, veteran=false})
		elseif randomNumber == 2 then
			gen.createUnit(object.u88cmFlak37, object.pAxis, {{73,159,0}}, {randomize=false, veteran=false})
		end

	end	
	
	if turn == 3 then -- July III, 1943	
		-- Random placement of 29th PanzerGrenadier Division in Sicily
		local pzg29SicilyLocationList = {{73,155,0},{67,153,0},{66,158,0}}
		local pzg29LocationList = pzg29SicilyLocationList
		gen.justOnce("29th PanzerGrenadier Division is deployed",function()
			gen.createUnit(object.uSdkFz231, object.pAxis, pzg29LocationList, {count=1, randomize=true, scatter=true, veteran=false})
			gen.createUnit(object.uMotInfanterie, object.pAxis, pzg29LocationList, {count=2, randomize=true, scatter=true, veteran=true})	
			gen.createUnit(object.uSdkFz72AA, object.pAxis, pzg29LocationList, {count=1, randomize=true, scatter=true, veteran=false})
			gen.createUnit(object.uStugIIIG, object.pAxis, pzg29LocationList, {count=1, randomize=true, scatter=true, veteran=true})		
			gen.createUnit(object.uGrilleHsIG33, object.pAxis, pzg29LocationList, {count=1, randomize=true, scatter=true, veteran=false})
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis then
					if (unit.location.x >= 65 and unit.location.y >= 157 and unit.location.x <= 67 and unit.location.y <= 159 and unit.location.z == 0) then
						unit.gotoTile = civ.getTile(60,164,0) 
					end
				end	
			end	
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis then
					if (unit.location.x >= 66 and unit.location.y >= 152 and unit.location.x <= 68 and unit.location.y <= 154 and unit.location.z == 0) then
						unit.gotoTile = civ.getTile(56,150,0) 
					end
				end
			end	
			text.simple("Allied intelligence has detected the arrival of the 29th PanzerGrenadier Division in Sicily!","Axis Reinforcements",object.monexone)	
		end)
	end

	-- GERMAN U-BOAT REINFORCEMENTS	
	if turn == 2 then
		local a = 0
		while( a < 5 )	do
			local placementLocations = {{60,84,0},{62,22,0},{62,58,0},{69,107,0},{62,122,0},{38,118,0},{127,69,0},{111,169,0}}
			for key,tileCoord in pairs(placementLocations) do
				if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uTypeIXC) then
					placementLocations[key] = nil
				end
			end
			gen.makeArrayOneToN(placementLocations)
			gen.createUnit(object.uTypeIXC, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})
			-- civ.ui.text("Created U-Boat TypeIXC")
			a = a + 1
		end
	end	
	
	-- AXIS AIR REINFORCEMENTS
	if turn == 2 then
		-- Italians
		gen.createUnit(object.uC200Saetta, object.pAxis, {{88,106,0},{102,156,0},{73,159,0},{59,161,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		gen.createUnit(object.uSM79Sparviero, object.pAxis, {{88,106,0},{102,156,0},{73,159,0},{59,161,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		-- Germans
		gen.createUnit(object.uMe109G, object.pAxis, {{94,116,0},{102,156,0},{73,155,0},{44,140,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		gen.createUnit(object.uFW190A, object.pAxis, {{94,116,0},{102,156,0},{73,155,0},{44,140,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		gen.createUnit(object.uDo217E, object.pAxis, {{94,116,0},{102,156,0},{73,155,0},{44,140,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
	end			

	if turn == 3 then
		-- Italians
		gen.createUnit(object.uC200Saetta, object.pAxis, {{88,106,0},{102,156,0},{73,159,0},{59,161,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		gen.createUnit(object.uC202Folgore, object.pAxis, {{88,106,0},{102,156,0},{73,159,0},{59,161,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		gen.createUnit(object.uSM79Sparviero, object.pAxis, {{88,106,0},{102,156,0},{73,159,0},{59,161,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		-- Germans
		gen.createUnit(object.uMe109G, object.pAxis, {{94,116,0},{102,156,0},{73,155,0},{44,140,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		gen.createUnit(object.uFW190A, object.pAxis, {{94,116,0},{102,156,0},{73,155,0},{44,140,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		gen.createUnit(object.uDo217E, object.pAxis, {{94,116,0},{102,156,0},{73,155,0},{44,140,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		gen.createUnit(object.uJu88, object.pAxis, {{94,116,0},{102,156,0},{73,155,0},{44,140,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})	
	end	

	if turn == 4 then
		-- Italians
		gen.createUnit(object.uC202Folgore, object.pAxis, {{88,106,0},{102,156,0},{73,159,0},{59,161,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		gen.createUnit(object.uZ1007Alcione, object.pAxis, {{88,106,0},{102,156,0},{73,159,0},{59,161,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		-- Germans
		gen.createUnit(object.uMe110, object.pAxis, {{94,116,0},{102,156,0},{73,155,0},{44,140,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		gen.createUnit(object.uDo217E, object.pAxis, {{94,116,0},{102,156,0},{73,155,0},{44,140,0},{33,139,0}}, {count=1, randomize=true, scatter=true, veteran=false})
	end	
	
	if turn >= 5 and flag.value("Sicily is Conquered") == true then -- Aug II, 1943
	-- ITALIAN MAINLAND
		gen.justOnce("Italian Mainland Reinforcements",function()
			if flag.value("Display Axis Testing Messages") == true then
				civ.ui.text("With the conquest of Sicily, the Axis deploy Italian Mainland reinforcements!")
			end
			-- Random Italian infantry and armored placements
			gen.createUnit(object.uFanteriaVeterana, object.pAxis, {{103,157,0}}, {count=2, randomize=false, veteran=false}) -- Crotone
			gen.createUnit(object.uFanteriaVeterana, object.pAxis, {{111,107,0}}, {count=2, randomize=false, veteran=false}) -- Foggia
			gen.createUnit(object.uFanteriaVeterana, object.pAxis, {{88,106,0}},  {count=2, randomize=false, veteran=false}) -- Napoli
			gen.createUnit(object.uFanteriaVeterana, object.pAxis, {{92,104,0}},  {count=2, randomize=false, veteran=false}) -- Caserta
			gen.createUnit(object.uFanteriaVeterana, object.pAxis, {{80,76,0}},   {count=2, randomize=false, veteran=false}) -- Roma
			gen.createUnit(object.uFanteriaVeterana, object.pAxis, {{89,39,0}},   {count=2, randomize=false, veteran=false}) -- Firenze
			gen.createUnit(object.uFanteriaVeterana, object.pAxis, {{75,35,0}},   {count=2, randomize=false, veteran=false}) -- Livorno
			gen.createUnit(object.uFanteriaVeterana, object.pAxis, {{106,14,0}},  {count=2, randomize=false, veteran=false}) -- Verona
			
			gen.createUnit(object.uParacadutisto, object.pAxis, {{80,76,0},{107,83,0}},   {count=2, randomize=true, veteran=false}) -- Verona
		
			gen.createUnit(object.uFanteriaMotorizzata, object.pAxis, {{80,76,0}},   {count=2, randomize=false, veteran=false}) -- Roma
			gen.createUnit(object.uFanteriaMotorizzata, object.pAxis, {{91,67,0}},   {count=2, randomize=false, veteran=false}) -- Terni
			gen.createUnit(object.uFanteriaMotorizzata, object.pAxis, {{100,30,0}},  {count=2, randomize=false, veteran=false}) -- Bologna
			
			gen.createUnit(object.uM1340, object.pAxis, {{80,76,0}},   {count=2, randomize=false, veteran=false}) -- Roma	
			gen.createUnit(object.uSemovente7518, object.pAxis, {{80,76,0}},   {count=1, randomize=false, veteran=false}) -- Roma
			gen.createUnit(object.uBersaglieri, object.pAxis, {{80,76,0}},   {count=1, randomize=false, veteran=false}) -- Roma
			
			gen.createUnit(object.uM1340, object.pAxis, {{84,82,0}},   {count=2, randomize=false, veteran=false}) -- Valmontone	
			gen.createUnit(object.uSemovente7518, object.pAxis, {{84,82,0}},   {count=1, randomize=false, veteran=false}) -- Valmontone
			gen.createUnit(object.uBersaglieri, object.pAxis, {{84,82,0}},   {count=1, randomize=false, veteran=false}) -- Valmontone

			gen.createUnit(object.uC200Saetta, object.pAxis, {{75,1,0},{98,4,0},{136,20,0}}, {count=1, randomize=true, scatter=true, veteran=false})
			gen.createUnit(object.uC202Folgore, object.pAxis, {{75,1,0},{98,4,0},{136,20,0}}, {count=2, randomize=true, scatter=true, veteran=false})
			gen.createUnit(object.uSM79Sparviero, object.pAxis, {{75,1,0},{98,4,0},{136,20,0}}, {count=2, randomize=true, scatter=true, veteran=false})

			-- Blackshirts
			local a = 0
			while( a < 6 )	do
				local placementLocations = {{98,4,0},{111,107,0},{88,106,0},{107,83,0},{80,76,0},{89,39,0},{75,35,0},{106,14,0}}
				for key,tileCoord in pairs(placementLocations) do
					if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uMSVN) then
						placementLocations[key] = nil
					end
				end
				gen.makeArrayOneToN(placementLocations)
				local unit = gen.createUnit(object.uMSVN, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
				-- civ.ui.text("Created Divisione Costiera")
				a = a + 1
			end	
			
			-- Random Italian artillery placements on Mainland
			-- 75/27 Art.
			local a = 0
			while( a < 4 )	do
				local placementLocations = {{92,104,0},{82,66,0},{117,117,0},{105,123,0},{99,121,0},{74,52,0}}
				for key,tileCoord in pairs(placementLocations) do
					if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u7527Art) then
						placementLocations[key] = nil
					end
				end
				gen.makeArrayOneToN(placementLocations)
				local unit = gen.createUnit(object.u7527Art, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
				if unit then -- if the unit was created, it will be a 75/27 Art., but we have to check the unit was actually created, and is not nil
					gen.setToFortified(unit)
					-- gen.placeFortress(unit.location)
					unit.moveSpent = 255
					gen.setMoved(unit)
				end
				-- civ.ui.text("Created Divisione Costiera")
				a = a + 1
			end

			-- 105/28 Art.
			local a = 0
			while( a < 2 )	do
				local placementLocations = {{111,107,0},{88,106,0},{107,83,0},{80,76,0}}
				for key,tileCoord in pairs(placementLocations) do
					if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u10528Art) then
						placementLocations[key] = nil
					end
				end
				gen.makeArrayOneToN(placementLocations)
				local unit = gen.createUnit(object.u10528Art, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
				if unit then -- if the unit was created, it will be a 105/28 Art., but we have to check the unit was actually created, and is not nil
					gen.setToFortified(unit)
					-- gen.placeFortress(unit.location)
					unit.moveSpent = 255
					gen.setMoved(unit)
				end
				-- civ.ui.text("Created Divisione Costiera")
				a = a + 1
			end

			-- 149/40 Art.
			local a = 0
			while( a < 3 )	do
				local placementLocations = {{73,159,0},{88,106,0},{80,76,0},{107,83,0},{94,122,0}}
				for key,tileCoord in pairs(placementLocations) do
					if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u14940Art) then
						placementLocations[key] = nil
					end
				end
				gen.makeArrayOneToN(placementLocations)
				local unit = gen.createUnit(object.u14940Art, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
				if unit then -- if the unit was created, it will be a 149/40 Art., but we have to check the unit was actually created, and is not nil
					gen.setToFortified(unit)
					gen.placeFortress(unit.location)
					unit.moveSpent = 255
					gen.setMoved(unit)
				end
				-- civ.ui.text("Created Divisione Costiera")
				a = a + 1
			end
			
			-- 20/65 AA
			local a = 0
			while( a < 4 )	do
				local placementLocations = {{102,156,0},{94,116,0},{111,107,0},{82,92,0},{77,77,0},{111,61,0}}
				for key,tileCoord in pairs(placementLocations) do
					if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u2065AA) then
						placementLocations[key] = nil
					end
				end
				gen.makeArrayOneToN(placementLocations)
				local unit = gen.createUnit(object.u2065AA, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
				if unit then -- if the unit was created, it will be a 20/65 AA, but we have to check the unit was actually created, and is not nil
					gen.setToNoOrders(unit)
					gen.placeFortress(unit.location)
					unit.moveSpent = 255
				end
				-- civ.ui.text("Created Divisione Costiera")
				a = a + 1
			end

			-- 75/46 AA
			local a = 0
			while( a < 2 )	do
				local placementLocations = {{73,159,0},{88,106,0},{80,76,0}}
				for key,tileCoord in pairs(placementLocations) do
					if gen.unitTypeOnTile(gen.toTile(tileCoord),object.u7546AA) then
						placementLocations[key] = nil
					end
				end
				gen.makeArrayOneToN(placementLocations)
				local unit = gen.createUnit(object.u7546AA, object.pAxis,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
				if unit then -- if the unit was created, it will be a 75/46 AA, but we have to check the unit was actually created, and is not nil
					gen.setToFortified(unit)
					-- gen.placeFortress(unit.location)
					unit.moveSpent = 255
					gen.setMoved(unit)
				end
				-- civ.ui.text("Created Divisione Costiera")
				a = a + 1
			end	
		end)
	end
	
	-- Placed German assets to defend town of Crotone once Messina is captured
	-- if turn >= 4 and flag.value("Sicily is Conquered") == true then
	if turn >= 4 and object.lMessina.owner == object.pAllies then
		gen.justOnce("Defend Crotone",function()
			local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{103,157,0},{102,156,0}}, {count=1, randomize=false, veteran=false})[1]
			if unit then
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end	
			local unit = gen.createUnit(object.u37cmFlak36, object.pAxis, {{103,157,0},{102,156,0}}, {count=1, randomize=false, veteran=true})[1]
			if unit then
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			gen.createUnit(object.uKustenbatterie, object.pAxis, {{103,157,0},{102,156,0}}, {randomize=false, veteran=false})
		end)	
	end
	
	if turn >= 9 and flag.value("2nd Invasion Available") == true then
		gen.justOnce("Grille H sIG33 Reinforcements",function()
			gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{88,106,0}}, {randomize=false, veteran=false})
		end)	
	end	

	-- ITALIAN BLACKSHIRTS ARE PERIODICALLY RECRUITED ONCE REPUBLIC OF SALO IS CREATED
	-- SEE Mussolini Liberated from Captivity in onCityTaken.lua
	if flag.value("Republic of Salo Created") == true and flag.value("Republic of Salo Conquered") == false then
		if math.random(1,6) == 6 then
			gen.createUnit(object.uMSVN, object.pAxis, {{72,12,0},{98,4,0},{106,14,0},{115,21,0},{137,3,0},{100,30,0},{75,1,0}}, {count=1, randomize=true, scatter=true, veteran=false})
		end
	end	
	
	-- 334TH INFANTRY DIVISION
	if turn == 21 and flag.value("2nd Invasion Available") == true then
		gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{100,90,0},{102,90,0},{107,83,0}}, {count=3, randomize=true, veteran=true})
	end	
	

	-- ====================================================================
	-- AXIS INVASION REACTION FORCES
	-- ====================================================================
	-- List of invasion reaction force zones:
	-- invasionUnitsInAnzio()
	-- invasionUnitsInCassino()
	-- invasionUnitsInCecina()
	-- invasionUnitsInCivitavecchia()
	-- invasionUnitsInFano()
	-- invasionUnitsInFirenze()
	-- invasionUnitsInFirenzuola()
	-- invasionUnitsInFoggia()
	-- invasionUnitsInForli()
	-- invasionUnitsInMassa()
	-- invasionUnitsInNapoli()
	-- invasionUnitsInOrtona()
	-- invasionUnitsInPracchia()
	-- invasionUnitsInRimini()
	-- invasionUnitsInSalerno()
	-- invasionUnitsInTerracina()
	

	-- KESSELRING'S COMMAND ========================================
	-- 3RD PANZER GRENADIER DIVISION + FRAS. REGIMENT
	-- 16TH PANZER DIVISION
	-- 26TH PANZER DIVISION + NEMBO REGIMENT
	
	-- 2nd INVASION REACTION FORCE -- 3rd Panzer Grenadier Divsion	
	if turn >= 9 and flag.value("2nd Invasion Available") == true and flag.value("3rd PzGr Div triggered") == false then
		local pzg3AnzioLocationList = {{80,76,0},{84,82,0},{85,77,0}}
		local pzg3CivitavecchiaLocationList = {{80,76,0},{74,52,0},{82,66,0}}
		local pzg3TerracinaLocationList = {{82,84,0},{82,82,0},{88,94,0}}
		local pzg3CassinoLocationList = {{88,86,0},{87,95,0},{91,93,0}}
		local pzg3LocationList = nil
		local pzg3LocationDesc = ""
		if invasionUnitsInAnzio() == true then
			pzg3LocationList = pzg3AnzioLocationList
			pzg3LocationDesc = "in Anzio"
		elseif invasionUnitsInCassino() == true then
			pzg3LocationList = pzg3CassinoLocationList
			pzg3LocationDesc = "in Cassino"
		elseif invasionUnitsInCivitavecchia() == true then
			pzg3LocationList = pzg3CivitavecchiaLocationList
			pzg3LocationDesc = "in Civitavecchia"
		elseif invasionUnitsInTerracina() == true then
			pzg3LocationList = pzg3TerracinaLocationList
			pzg3LocationDesc = "in Terracina"
		end
		if pzg3LocationList ~= nil then
			gen.justOnce("2nd Invasion Reaction Force - 3rd PzGr Div, Fras. PzGr Regt and Panzer and Infantry replacements", function()
				gen.createUnit(object.uSdkFz231, object.pAxis, pzg3LocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uMotInfanterie, object.pAxis, pzg3LocationList, {count=2, randomize=true, scatter=true, veteran=true})
				gen.createUnit(object.uPanzerGrenadier, object.pAxis, pzg3LocationList, {count=1, randomize=true, veteran=true}) -- Fras. PzGr Regt	
				gen.createUnit(object.uSdkFz72AA, object.pAxis, pzg3LocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uStugIIIG, object.pAxis, pzg3LocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uPzkwIVF, object.pAxis, pzg3LocationList, {count=2, randomize=true, scatter=true, veteran=false}) -- Panzer replacements
				gen.createUnit(object.uGrilleHsIG33, object.pAxis, pzg3LocationList, {count=1, randomize=true, scatter=true, veteran=false})

				civ.playSound("InfantryGer.wav")
				civ.ui.text("German units react to the Allied moves near " .. pzg3LocationDesc .. "!")
				flag.setTrue("3rd PzGr Div triggered")
				-- GoTo Commands that push reaction force towards beachhead
				if invasionUnitsInAnzio() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 80 and unit.location.y >= 76 and unit.location.x <= 86 and unit.location.y <= 82 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(78,84,0) 
							end
						end	
					end	

					-- If 2nd invasion above Gustav line
					if flag.setTrue("2nd Invasion Above Gustav") == false then				
						local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{82,80,0},{84,82,0}}, {count=1, randomize=true, veteran=false})	[1]
						if unit then
							gen.setToNoOrders(unit)
							gen.setToFortified(unit)
							-- gen.placeFortress(unit.location)
							unit.moveSpent = 255
						end	
						local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{77,75,0},{79,73,0}}, {count=1, randomize=true, veteran=false})	[1]
						if unit then
							gen.setToNoOrders(unit)
							gen.setToFortified(unit)
							-- gen.placeFortress(unit.location)
							unit.moveSpent = 255
						end	
					else
						gen.createUnit(object.uInfanterieVeteranen, object.pAxis, pzg3LocationList, {count=4, randomize=true, scatter=true, veteran=true})
					end	
					
				elseif invasionUnitsInCivitavecchia() == true then
					-- If 2nd invasion above Gustav line
					-- flag.setTrue("2nd Invasion Above Gustav")				
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 74 and unit.location.y >= 52 and unit.location.x <= 82 and unit.location.y <= 76 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(75,67,0) 
							end
						end	
					end
					
					-- If 2nd invasion above Gustav line
					if flag.setTrue("2nd Invasion Above Gustav") == false then	
						local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{74,56,0},{74,52,0},{75,55,0}}, {count=1, randomize=true, veteran=false})	[1]
						if unit then
							gen.setToNoOrders(unit)
							gen.setToFortified(unit)
							-- gen.placeFortress(unit.location)
							unit.moveSpent = 255
						end	
						local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{82,66,0},{82,70,0},{79,73,0}}, {count=1, randomize=true, veteran=false})	[1]
						if unit then
							gen.setToNoOrders(unit)
							gen.setToFortified(unit)
							-- gen.placeFortress(unit.location)
							unit.moveSpent = 255
						end	
					else
						gen.createUnit(object.uInfanterieVeteranen, object.pAxis, pzg3LocationList, {count=4, randomize=true, scatter=true, veteran=true})
					end						
				elseif invasionUnitsInTerracina() == true then
					-- If 2nd invasion above Gustav line
					flag.setTrue("2nd Invasion Above Gustav")				
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 82 and unit.location.y >= 82 and unit.location.x <= 88 and unit.location.y <= 94 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(82,92,0) 
							end
						end	
					end
					
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, pzg3LocationList, {count=4, randomize=true, scatter=true, veteran=true})
					-- local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{83,87,0},{85,87,0}}, {count=1, randomize=true, veteran=false})	[1]
					-- if unit then
						-- gen.setToNoOrders(unit)
						-- gen.setToFortified(unit)
						-- -- gen.placeFortress(unit.location)
						-- unit.moveSpent = 255
					-- end	
					-- local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{86,90,0},{87,91,0},{87,93,0}}, {count=1, randomize=true, veteran=false})	[1]
					-- if unit then
						-- gen.setToNoOrders(unit)
						-- gen.setToFortified(unit)
						-- -- gen.placeFortress(unit.location)
						-- unit.moveSpent = 255
					-- end	
				elseif invasionUnitsInCassino() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then	
							if (unit.location.x >= 87 and unit.location.y >= 85 and unit.location.x <= 91 and unit.location.y <= 95 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(92,100,0) 
							end
						end	
					end
					
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, pzg3LocationList, {count=4, randomize=true, scatter=true, veteran=true})
					-- local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{92,72,0},{92,82,0},{87,75,0}}, {count=1, randomize=true, veteran=false})	[1]
					-- if unit then
						-- gen.setToNoOrders(unit)
						-- gen.setToFortified(unit)
						-- -- gen.placeFortress(unit.location)
						-- unit.moveSpent = 255
					-- end	
					-- local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{85,91,0},{84,82,0},{91,91,0},{85,77,0}}, {count=1, randomize=true, veteran=false})	[1]
					-- if unit then
						-- gen.setToNoOrders(unit)
						-- gen.setToFortified(unit)
						-- -- gen.placeFortress(unit.location)
						-- unit.moveSpent = 255
					-- end	
				end	
			end)
		end
	end	

	-- FOGGIA REACTION FORCE - 16th Panzer Division
	if turn >= 9 and flag.value("2nd Invasion Available") == true and flag.value("16th Pz Div triggered") == false then
		if invasionUnitsInFoggia() == true then
			gen.justOnce("16th Panzer Division is deployed", function()
				local pz16FoggiaLocationList = {{111,107,0},{111,95,0},{101,99,0}}
				local pz16LocationList = pz16FoggiaLocationList
				local pz16LocationDesc = "near Foggia"
				gen.createUnit(object.uSdkFz231, object.pAxis, pz16LocationList, {count=1, randomize=false, veteran=false})
				gen.createUnit(object.uMotInfanterie, object.pAxis, pz16LocationList, {count=1, randomize=false, veteran=true})
				gen.createUnit(object.uPanzerGrenadier, object.pAxis, pz16LocationList, {count=1, randomize=false, veteran=true})	
				gen.createUnit(object.uSdkFz72AA, object.pAxis, pz16LocationList, {count=1, randomize=false, scatter=false, veteran=false})
				gen.createUnit(object.uPzkwIVF, object.pAxis, pz16LocationList, {count=2, randomize=false, scatter=false, veteran=false})
				gen.createUnit(object.uStugIIIG, object.pAxis, pz16LocationList, {count=1, randomize=false, scatter=false, veteran=false})
				gen.createUnit(object.uGrilleHsIG33, object.pAxis, pz16LocationList, {count=1, randomize=false, scatter=false, veteran=false})
				-- civ.playSound("InfantryGer.wav")
				text.simple("German units react to the Allies moves " .. pz16LocationDesc .. "!","German 10th Army Command Reacts!",object.m10thArmy)
				flag.setTrue("16th Pz Div triggered")
			end)	
		end
	end	
	
	-- 2nd INVASION REACTION FORCE -- 26th Panzer Divsion	
	if turn >= 9 and flag.value("2nd Invasion Available") == true and flag.value("26th Pz Div triggered") == false then
		local pz26SalernoLocationList = {{88,106,0},{96,110,0},{99,121,0},{95,129,0}}
		local pz26NapoliLocationList = {{88,106,0},{92,102,0},{98,106,0},{94,116,0}}
		local pz26AnzioLocationList = {{80,76,0},{84,82,0},{85,77,0}}
		local pz26CivitavecchiaLocationList = {{80,76,0},{74,52,0},{82,66,0}}
		local pz26TerracinaLocationList = {{82,84,0},{82,82,0},{88,94,0}}
		local pz26CassinoLocationList = {{88,86,0},{87,95,0},{91,93,0}}
		local pz26LocationList = nil
		local pz26LocationDesc = ""
		if invasionUnitsInSalerno() == true then
			pz26LocationList = pz26SalernoLocationList
			pz26LocationDesc = "near Salerno"
		elseif invasionUnitsInNapoli() == true then
			pz26LocationList = pz26NapoliLocationList
			pz26LocationDesc = "near Napoli"
		elseif invasionUnitsInAnzio() == true then
			pz26LocationList = pz26AnzioLocationList
			pz26LocationDesc = "near Anzio"
		elseif invasionUnitsInCivitavecchia() == true then
			pz26LocationList = pz26CivitavecchiaLocationList
			pz26LocationDesc = "near Civitavecchia"
		elseif invasionUnitsInTerracina() == true then
			pz26LocationList = pz26TerracinaLocationList
			pz26LocationDesc = "near Terracina"
		elseif invasionUnitsInCassino() == true then
			pz26LocationList = pz26CassinoLocationList
			pz26LocationDesc = "near Cassino"
		end
		if pz26LocationList ~= nil then
			gen.justOnce("2nd Invasion Reaction Force - 26th Panzer Divsion + Nembo Regt", function()
				gen.createUnit(object.uSdkFz231, object.pAxis, pz26LocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uParacadutisto, object.pAxis, pz26LocationList, {count=1, randomize=true, scatter=true, veteran=true}) -- Nembo Regt
				gen.createUnit(object.uMotInfanterie, object.pAxis, pz26LocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uPanzerGrenadier, object.pAxis, pz26LocationList, {count=1, randomize=true, scatter=true, veteran=false})	
				gen.createUnit(object.uSdkFz72AA, object.pAxis, pz26LocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uPzkwIVF, object.pAxis, pz26LocationList, {count=2, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uStugIIIG, object.pAxis, pz26LocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uGrilleHsIG33, object.pAxis, pz26LocationList, {count=1, randomize=true, scatter=true, veteran=false})

				civ.playSound("InfantryGer.wav")
				text.simple("German units react to the Allied moves " .. pz26LocationDesc .. "!","German 10th Army Command Reacts!",object.m10thArmy)
				flag.setTrue("26th Pz Div triggered")
				-- GoTo Commands that push reaction force towards beachhead
					if invasionUnitsInSalerno() == true then
						for unit in civ.iterateUnits() do
							if unit.owner == object.pAxis then
								if (unit.location.x >= 88 and unit.location.y >= 106 and unit.location.x <= 100 and unit.location.y <= 130 and unit.location.z == 0) then
									unit.gotoTile = civ.getTile(93,113,0) 
								end
							end	
						end	
					elseif invasionUnitsInNapoli() == true then
						for unit in civ.iterateUnits() do
							if unit.owner == object.pAxis then
								if (unit.location.x >= 88 and unit.location.y >= 102 and unit.location.x <= 98 and unit.location.y <= 116 and unit.location.z == 0) then
									unit.gotoTile = civ.getTile(90,110,0) 
								end
							end	
						end	
					elseif invasionUnitsInAnzio() == true then
						for unit in civ.iterateUnits() do
							if unit.owner == object.pAxis then
								if (unit.location.x >= 80 and unit.location.y >= 76 and unit.location.x <= 86 and unit.location.y <= 82 and unit.location.z == 0) then
									unit.gotoTile = civ.getTile(78,84,0) 
								end
							end	
						end	
					elseif invasionUnitsInCivitavecchia() == true then
						for unit in civ.iterateUnits() do
							if unit.owner == object.pAxis then
								if (unit.location.x >= 74 and unit.location.y >= 52 and unit.location.x <= 82 and unit.location.y <= 76 and unit.location.z == 0) then
									unit.gotoTile = civ.getTile(75,67,0) 
								end
							end	
						end
					elseif invasionUnitsInTerracina() == true then
						for unit in civ.iterateUnits() do
							if unit.owner == object.pAxis then
								if (unit.location.x >= 82 and unit.location.y >= 82 and unit.location.x <= 88 and unit.location.y <= 94 and unit.location.z == 0) then
									unit.gotoTile = civ.getTile(82,92,0) 
								end
							end	
						end
					elseif invasionUnitsInCassino() == true then
						for unit in civ.iterateUnits() do
							if unit.owner == object.pAxis then
								if (unit.location.x >= 87 and unit.location.y >= 85 and unit.location.x <= 91 and unit.location.y <= 95 and unit.location.z == 0) then
									unit.gotoTile = civ.getTile(92,100,0) 
								end
							end	
						end
					end
			end)
		end
	end	
	
	-- ROMMEL'S COMMAND ========================================
	-- 2ND FALLSCHIRMJAGER DIVISION
	-- 24TH PANZER DIVISION
	-- 90TH PANZERGRENADIER DIVISION
	-- 1ST SS LEIBSTANDARTE ADOLF HITLER (LAH) PANZER DIVISION
	-- 2ND SS DAS REICH (DR) PANZER DIVISION
	-- 16TH SS REICHSFUHRER PANZERGRENADIER DIVISION
	
	-- 2nd Fallschirmjäger Division deploys near Roma
	if turn >= 7 and flag.value("2nd Invasion Available") == true then -- August II, 1943
		gen.justOnce("2nd Fallschirmjager Division is deployed", function()

			if object.ltownLidoDiRoma.defender and object.ltownLidoDiRoma.defender ~= object.pAxis then -- (tile 77,77,0)
				for unit in object.ltownLidoDiRoma.units do
					gen.moveUnitAdjacent(unit)
				end
			end
			local unit = gen.createUnit(object.uFallschirmjager, object.pAxis, {{77,77,0}}, {randomize=false, veteran=true})[1]
			if unit then
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			gen.createUnit(object.uBunker, object.pAxis, {{77,77,0}}, {randomize=false, veteran=true})
			
			local unit = gen.createUnit(object.uFallschirmjager, object.pAxis, {{80,76,0}}, {randomize=false, veteran=true})[1]
			if unit then
				gen.setToFortified(unit)
				-- gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			
			if object.lroughSE_ROMA.defender and object.lroughSE_ROMA.defender ~= object.pAxis then -- (tile 81,79,0)
				for unit in object.lroughSE_ROMA.units do
					gen.moveUnitAdjacent(unit)
				end
			end
			local unit = gen.createUnit(object.uFallschirmjager, object.pAxis, {{81,79,0}}, {randomize=false, veteran=true})[1]
			if unit then
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			gen.createUnit(object.uBunker, object.pAxis, {{81,79,0}}, {randomize=false, veteran=true})
			
			if object.lhillSWValmontone.defender and object.lhillSWValmontone.defender ~= object.pAxis then -- (tile 83,83,0)
				for unit in object.lhillSWValmontone.units do
					gen.moveUnitAdjacent(unit)
				end
			end
			local unit = gen.createUnit(object.uFallschirmjager, object.pAxis, {{83,83,0}}, {randomize=false, veteran=true})[1]
			if unit then
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
				gen.setMoved(unit)
			end
			gen.createUnit(object.uBunker, object.pAxis, {{83,83,0}}, {randomize=false, veteran=true})			

		end)		
	end

	-- FIRENZE REACTION FORCE - 24th Panzer Division
	if turn >= 11 and flag.value("2nd Invasion Available") == true and flag.value("24th Pz Div triggered") == false then
		if invasionUnitsInFirenze() == true then
			gen.justOnce("24th Panzer Division is deployed", function()
				local pz24FirenzeLocationList = {{89,39,0},{88,34,0},{96,40,0}}
				local pz24LocationList = pz24FirenzeLocationList
				local pz24LocationDesc = "near Firenze"
				gen.createUnit(object.uMotInfanterie, object.pAxis, pz24LocationList, {count=2, randomize=true, scatter=true, veteran=true})
				gen.createUnit(object.uPanzerGrenadier, object.pAxis, pz24LocationList, {count=1, randomize=true, scatter=true, veteran=true})	
				gen.createUnit(object.uSdkFz72AA, object.pAxis, pz24LocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uPzkwIVF, object.pAxis, pz24LocationList, {count=2, randomize=true, scatter=true, veteran=false})
				-- gen.createUnit(object.uStugIIIG, object.pAxis, pz24LocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uGrilleHsIG33, object.pAxis, pz24LocationList, {count=1, randomize=true, scatter=true, veteran=false})
				civ.playSound("InfantryGer.wav")
				civ.ui.text("German units react to the Allies moves " .. pz24LocationDesc .. "!")
				flag.setTrue("24th Pz Div triggered")
			end)	
		end
	end	
	
	-- LIVORNO REACTION FORCE - 90th PanzerGrenadier Division	
	if turn >= 11 and flag.value("2nd Invasion Available") == true and flag.value("90th PzGr Div triggered") == false then
		local pzg90CecinaLocationList = {{75,35,0},{78,34,0},{75,49,0}}
		local pzg90MassaLocationList = {{77,23,0},{82,32,0},{81,21,0}}
		local pzg90PracchiaLocationList = {{96,24,0},{86,24,0}}
		local pzg90LocationList = nil
		local pzg90LocationDesc = ""
		if invasionUnitsInCecina() == true then
			pzg90LocationList = pzg90CecinaLocationList
			pzg90LocationDesc = "near Cecina"
		elseif invasionUnitsInMassa() == true then
			pzg90LocationList = pzg90MassaLocationList
			pzg90LocationDesc = "near Massa"
		elseif invasionUnitsInPracchia() == true then
			pzg90LocationList = pzg90PracchiaLocationList
			pzg90LocationDesc = "near Pracchia"
		end
		if pzg90LocationList ~= nil then
			gen.justOnce("90th Panzergrenadier Division is deployed", function()
				gen.createUnit(object.uSdkFz231, object.pAxis, pzg90LocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uMotInfanterie, object.pAxis, pzg90LocationList, {count=2, randomize=true, scatter=true, veteran=true})	
				gen.createUnit(object.uSdkFz72AA, object.pAxis, pzg90LocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uStugIIIG, object.pAxis, pzg90LocationList, {count=1, randomize=true, scatter=true, veteran=false})		
				gen.createUnit(object.uGrilleHsIG33, object.pAxis, pzg90LocationList, {count=1, randomize=true, scatter=true, veteran=false})

				civ.playSound("InfantryGer.wav")
				civ.ui.text("German units react to the Allied moves " .. pzg90LocationDesc .. "!")
				flag.setTrue("90th PzGr Div triggered")
				-- GoTo Commands that push reaction force towards beachhead
				if invasionUnitsInCecina() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 75 and unit.location.y >= 33 and unit.location.x <= 79 and unit.location.y <= 49 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(75,39,0) 
							end
						end	
					end	
				elseif invasionUnitsInMassa() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 77 and unit.location.y >= 21 and unit.location.x <= 83 and unit.location.y <= 33 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(78,28,0) 
							end
						end	
					end
				elseif invasionUnitsInPracchia() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then	
							if (unit.location.x >= 86 and unit.location.y >= 24 and unit.location.x <= 96 and unit.location.y <= 28 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(87,31,0) 
							end
						end	
					end
				end
			end)
		end
	end	

	-- LA SPEZIA REACTION FORCE - 1st SS Liebstandarde Adolf Hilter (LAH) Panzer Division	
	if turn <= 30 and flag.value("2nd Invasion Available") == true and flag.value("1st SS Pz Div triggered") == false then
		local pz1SSLAHCecinaLocationList = {{75,35,0},{78,34,0},{75,49,0}}
		local pz1SSLAHMassaLocationList = {{77,23,0},{82,32,0},{81,21,0}}
		local pz1SSLAHPracchiaLocationList = {{96,24,0},{86,24,0}}
		local pz1SSLAHLocationList = nil
		local pz1SSLAHLocationDesc = ""
		if invasionUnitsInCecina() == true then
			pz1SSLAHLocationList = pz1SSLAHCecinaLocationList
			pz1SSLAHLocationDesc = "near Cecina"
		elseif invasionUnitsInMassa() == true then
			pz1SSLAHLocationList = pz1SSLAHMassaLocationList
			pz1SSLAHLocationDesc = "near Massa"
		elseif invasionUnitsInPracchia() == true then
			pz1SSLAHLocationList = pz1SSLAHPracchiaLocationList
			pz1SSLAHLocationDesc = "near Pracchia"
		end
		if pz1SSLAHLocationList ~= nil then
			gen.justOnce("1st SS LAH Panzer Division is deployed", function()
				gen.createUnit(object.uSSMotInfanterie, object.pAxis, pz1SSLAHLocationList, {count=2, randomize=true, scatter=true, veteran=true})
				gen.createUnit(object.uSSPanzergrenadier, object.pAxis, pz1SSLAHLocationList, {count=1, randomize=true, scatter=true, veteran=true})	
				gen.createUnit(object.uSSSdkFz72AA, object.pAxis, pz1SSLAHLocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uSSSdkFz2342, object.pAxis, pz1SSLAHLocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uSSPzkwIV, object.pAxis, pz1SSLAHLocationList, {count=2, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uSSPanther, object.pAxis, pz1SSLAHLocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uSSHummel, object.pAxis, pz1SSLAHLocationList, {count=1, randomize=true, scatter=true, veteran=false})

				civ.playSound("SS.wav")
				civ.ui.text("The 1st Liebstandarde Adolf Hilter (LAH) Panzer Division reacts to the Allied moves " .. pz1SSLAHLocationDesc .. "!")
				flag.setTrue("1st SS Pz Div triggered")
				-- GoTo Commands that push reaction force towards beachhead
				if invasionUnitsInCecina() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 75 and unit.location.y >= 33 and unit.location.x <= 79 and unit.location.y <= 49 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(75,39,0) 
							end
						end	
					end	
				elseif invasionUnitsInMassa() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 77 and unit.location.y >= 21 and unit.location.x <= 83 and unit.location.y <= 33 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(78,28,0) 
							end
						end	
					end
				elseif invasionUnitsInPracchia() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 86 and unit.location.y >= 24 and unit.location.x <= 96 and unit.location.y <= 28 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(87,31,0) 
							end
						end	
					end
				end	
			end)
		end
	end			
	
	-- BOLOGNA REACTION FORCE - 2nd SS Das Reich (DR) Panzer Division	
	if turn <= 30 and flag.value("2nd Invasion Available") == true and flag.value("2nd SS Pz Div triggered") == false then
		local pz2SSDRRiminiLocationList = {{106,44,0},{103,39,0}}
		local pz2SSDRFirenzuolaLocationList = {{100,30,0},{96,24,0}}
		local pz2SSDRForliLocationList = {{103,39,0},{100,30,0}}
		local pz2SSDRLocationList = nil
		local pz2SSDRLocationDesc = ""
		if invasionUnitsInRimini() == true then
			pz2SSDRLocationList = pz2SSDRRiminiLocationList
			pz2SSDRLocationDesc = "near Rimini"
		elseif invasionUnitsInFirenzuola() == true then
			pz2SSDRLocationList = pz2SSDRFirenzuolaLocationList
			pz2SSDRLocationDesc = "near Firenzuola"
		elseif invasionUnitsInForli() == true then
			pz2SSDRLocationList = pz2SSDRForliLocationList
			pz2SSDRLocationDesc = "near Forli"
		end
		if pz2SSDRLocationList ~= nil then
			gen.justOnce("2nd SS Das Reich Panzer Division is deployed", function()
				gen.createUnit(object.uSSMotInfanterie, object.pAxis, pz2SSDRLocationList, {count=2, randomize=true, scatter=true, veteran=true})
				gen.createUnit(object.uSSPanzergrenadier, object.pAxis, pz2SSDRLocationList, {count=1, randomize=true, scatter=true, veteran=true})	
				gen.createUnit(object.uSSSdkFz72AA, object.pAxis, pz2SSDRLocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uSSSdkFz2342, object.pAxis, pz2SSDRLocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uSSPzkwIV, object.pAxis, pz2SSDRLocationList, {count=2, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uSSPanther, object.pAxis, pz2SSDRLocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uSSHummel, object.pAxis, pz2SSDRLocationList, {count=1, randomize=true, scatter=true, veteran=false})

				civ.playSound("SS.wav")
				civ.ui.text("The 2nd SS Das Reich (DR) Panzer Division reacts to the Allied moves " .. pz2SSDRLocationDesc .. "!")
				flag.setTrue("2nd SS Pz Div triggered")
				-- GoTo Commands that push reaction force towards beachhead
				if invasionUnitsInRimini() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then	
							if (unit.location.x >= 102 and unit.location.y >= 38 and unit.location.x <= 106 and unit.location.y <= 44 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(109,51,0) 
							end
						end	
					end	
				elseif invasionUnitsInFirenzuola() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 96 and unit.location.y >= 24 and unit.location.x <= 100 and unit.location.y <= 30 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(89,39,0) 
							end
						end	
					end
				elseif invasionUnitsInForli() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 100 and unit.location.y >= 30 and unit.location.x <= 104 and unit.location.y <= 40 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(98,40,0) 
							end
						end	
					end
				end	
			end)
		end
	end	

	-- REICHSFUHRER-SS REACTION FORCE - 16th SS Reichsführer-SS PanzerGrenadier Division	
	if turn >= 24 and flag.value("2nd Invasion Available") == true and flag.value("16th SS PzGr Div triggered") == false then
		local pzg16SSCecinaLocationList = {{75,35,0},{78,34,0},{75,49,0}}
		local pzg16SSMassaLocationList = {{77,23,0},{82,32,0},{81,21,0}}
		local pzg16SSAnzioLocationList = {{80,76,0},{84,82,0},{85,77,0}}
		local pzg16SSCivitavecchiaLocationList = {{80,76,0},{74,52,0},{82,66,0}}
		local pzg16SSCassinoLocationList = {{88,86,0},{87,95,0},{91,93,0}}		
		local pzg16SSLocationList = nil
		local pzg16SSLocationDesc = ""
		if invasionUnitsInCecina() == true then
			pzg16SSLocationList = pzg16SSCecinaLocationList
			pzg16SSLocationDesc = "near Cecina"
		elseif invasionUnitsInMassa() == true then
			pzg16SSLocationList = pzg16SSMassaLocationList
			pzg16SSLocationDesc = "near Massa"
		elseif invasionUnitsInAnzio() == true then
			pzg16SSLocationList = pzg16SSAnzioLocationList
			pzg16SSLocationDesc = "near Anzio"
		elseif invasionUnitsInCivitavecchia() == true then
			pzg16SSLocationList = pzg16SSCivitavecchiaLocationList
			pzg16SSLocationDesc = "near Civitavecchia"
		elseif invasionUnitsInCassino() == true then
			pzg16SSLocationList = pzg16SSCassinoLocationList
			pzg16SSLocationDesc = "near Cassino"
		end
		if pzg16SSLocationList ~= nil then
			gen.justOnce("16th SS Panzergrenadier Division is deployed", function()
				gen.createUnit(object.uSSMotInfanterie, object.pAxis, pzg16SSLocationList, {count=2, randomize=true, scatter=true, veteran=true})
				gen.createUnit(object.uSSSdkFz72AA, object.pAxis, pzg16SSLocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uSSSdkFz2342, object.pAxis, pzg16SSLocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uSSPzkwIV, object.pAxis, pzg16SSLocationList, {count=1, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uSSHummel, object.pAxis, pzg16SSLocationList, {count=1, randomize=true, scatter=true, veteran=false})

				civ.playSound("SS.wav")
				civ.ui.text("The 16th Reichsfuhrer-SS Panzergrenadier Division unit reacts to the Allied moves " .. pzg16SSLocationDesc .. "!")
				flag.setTrue("16th SS PzGr Div triggered")
				-- GoTo Commands that push reaction force towards beachhead
				if invasionUnitsInCecina() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 75 and unit.location.y >= 33 and unit.location.x <= 79 and unit.location.y <= 49 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(75,39,0) 
							end
						end	
					end	
				elseif invasionUnitsInMassa() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 77 and unit.location.y >= 21 and unit.location.x <= 83 and unit.location.y <= 33 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(78,28,0) 
							end
						end	
					end
				elseif invasionUnitsInAnzio() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 80 and unit.location.y >= 76 and unit.location.x <= 86 and unit.location.y <= 82 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(78,84,0) 
							end
						end	
					end	
				elseif invasionUnitsInCivitavecchia() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 74 and unit.location.y >= 52 and unit.location.x <= 82 and unit.location.y <= 76 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(75,67,0) 
							end
						end	
					end
				elseif invasionUnitsInCassino() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 87 and unit.location.y >= 85 and unit.location.x <= 91 and unit.location.y <= 95 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(92,100,0) 
							end
						end	
					end
				end	
			end)
		end
	end	
	
	-- 3rd INVASION REACTION FORCE ========================================
	-- IMMEDIATE REACTION
	-- 71st Infantry Division
	-- 362nd Infantry Division
	-- 2 Ersatz (aka Widerstandsnest)
	-- 2 Panther Batt's

	-- NEXT TURN
	---- 715th Motorized Infantry Division
	---- 3 Panzer Grenadier Regt's + 1 Schützen Regt
	---- 188th Gebirgsjäger Division
	---- 1 Krupp K5 Guns
	-- Plus assorted artillery and Flak guns
	
	-- 2ND TURN AFTER
	------ 60th Panzer Grenadier Divsion
	------ 1 Tiger Batt
	
	-- OPTION 1 = If Allies proceed with an actual 3rd invasion
	if (turn >= 17 and turn <= 28) and flag.value("3rd Invasion Available") == true and (flag.value("3rd Invasion Launched") == true or flag.value("2nd Invasion Above Gustav") == true) then
		local invReact3rdT1_MassaLocationList = {{77,23,0},{82,32,0},{81,21,0}}
		local invReact3rdT1_CecinaLocationList = {{75,35,0},{78,34,0},{75,49,0}}
		local invReact3rdT1_FanoLocationList = {{106,44,0},{103,51,0},{104,54,0}}
		local invReact3rdT1_CivitavecchiaLocationList = {{80,76,0},{74,52,0},{82,66,0}}	
		local invReact3rdT1_AnzioLocationList = {{80,76,0},{84,82,0},{85,77,0}}	
		local invReact3rdT1_TerracinaLocationList = {{80,76,0},{84,82,0},{85,77,0}}	
		local invReact3rdT1_FirenzeLocationList = {{89,39,0},{88,34,0},{93,41,0}}		
		local invReact3rdT1_LocationList = nil
		local invReact3rdT1_LocationDesc = ""
		if invasionUnitsInMassa() == true then
			invReact3rdT1_LocationList = invReact3rdT1_MassaLocationList
			invReact3rdT1_LocationDesc = "near Massa"		
		elseif invasionUnitsInCecina() == true then
			invReact3rdT1_LocationList = invReact3rdT1_CecinaLocationList
			invReact3rdT1_LocationDesc = "near Cecina"		
		elseif invasionUnitsInFano() == true then
			invReact3rdT1_LocationList = invReact3rdT1_FanoLocationList
			invReact3rdT1_LocationDesc = "near Fano"		
		elseif invasionUnitsInCivitavecchia() == true then
			invReact3rdT1_LocationList = invReact3rdT1_CivitavecchiaLocationList
			invReact3rdT1_LocationDesc = "near Civitavecchia"		
		elseif invasionUnitsInAnzio() == true then
			invReact3rdT1_LocationList = invReact3rdT1_AnzioLocationList
			invReact3rdT1_LocationDesc = "near Anzio"
		elseif invasionUnitsInTerracina() == true then
			invReact3rdT1_LocationList = invReact3rdT1_TerracinaLocationList
			invReact3rdT1_LocationDesc = "near Terracina"
		elseif invasionUnitsInFirenze() == true then
			invReact3rdT1_LocationList = invReact3rdT1_FirenzeLocationList
			invReact3rdT1_LocationDesc = "near Firenze"
		end

		if invReact3rdT1_LocationList ~= nil then
			gen.justOnce("3rd Invasion Units, 1st turn, deployed", function()
				gen.createUnit(object.uInfanterieVeteranen, object.pAxis, invReact3rdT1_LocationList, {count=5, randomize=true, scatter=true, veteran=false})
				gen.createUnit(object.uPzkwVG, object.pAxis, invReact3rdT1_LocationList, {count=2, randomize=true, scatter=true, veteran=false})
				civ.playSound("InfantryGer.wav")
				civ.ui.text("3rd Invasion: German units react to the Allied moves " .. invReact3rdT1_LocationDesc .. "!")
				flag.setTrue("3rd Invasion Units, 1st turn, triggered")

				-- GoTo Commands that push reaction forces towards beachhead
				if invasionUnitsInMassa() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 77 and unit.location.y >= 21 and unit.location.x <= 83 and unit.location.y <= 33 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(78,28,0) 
							end
						end	
					end
					-- Place extra blocking Widerstandsnest units
					local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{77,23,0},{81,23,0},{82,25,0}}, {count=1, randomize=true, veteran=false})[1]
					if unit then
						gen.setToNoOrders(unit)
						gen.setToFortified(unit)
						gen.placeFortress(unit.location)
						unit.moveSpent = 255
					end	
					local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{80,36,0},{82,34,0},{87,31,0}}, {count=1, randomize=true, veteran=false})[1]
					if unit then
						gen.setToNoOrders(unit)
						gen.setToFortified(unit)
						gen.placeFortress(unit.location)
						unit.moveSpent = 255
					end							
					-- Follow up 3rd reaction force units are delayed 1 and 2 turns
					delayedAction.doInFuture("invReact3rdT2_UnitsMassa",{},civ.getTurn()+ 1 ,object.pAxis.id)
					delayedAction.doInFuture("invReact3rdT3_UnitsMassa",{},civ.getTurn()+ 2 ,object.pAxis.id)

				elseif invasionUnitsInCecina() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 75 and unit.location.y >= 33 and unit.location.x <= 79 and unit.location.y <= 49 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(75,39,0) 
							end
						end	
					end	
					-- Place extra blocking Widerstandsnest units
					local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{75,35,0},{77,39,0},{77,37,0}}, {count=1, randomize=true, veteran=false})[1]
					if unit then
						gen.setToNoOrders(unit)
						gen.setToFortified(unit)
						gen.placeFortress(unit.location)
						unit.moveSpent = 255
					end	
					local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{80,46,0},{73,49,0},{76,48,0}}, {count=1, randomize=true, veteran=false})[1]
					if unit then
						gen.setToNoOrders(unit)
						gen.setToFortified(unit)
						gen.placeFortress(unit.location)
						unit.moveSpent = 255
					end						
					-- Follow up 3rd reaction force units are delayed 1 and 2 turns
					delayedAction.doInFuture("invReact3rdT2_UnitsCecina",{},civ.getTurn()+ 1 ,object.pAxis.id)
					delayedAction.doInFuture("invReact3rdT3_UnitsCecina",{},civ.getTurn()+ 2 ,object.pAxis.id)

				elseif invasionUnitsInFano() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 102 and unit.location.y >= 44 and unit.location.x <= 106 and unit.location.y <= 54 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(109,51,0) 
							end
						end	
					end	
					-- Place extra blocking Widerstandsnest units
					local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{105,51,0},{104,54,0},{108,56,0}}, {count=1, randomize=true, veteran=false})[1]
					if unit then
						gen.setToNoOrders(unit)
						gen.setToFortified(unit)
						gen.placeFortress(unit.location)
						unit.moveSpent = 255
					end	
					local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{105,47,0},{102,46,0},{103,43,0}}, {count=1, randomize=true, veteran=false})[1]
					if unit then
						gen.setToNoOrders(unit)
						gen.setToFortified(unit)
						gen.placeFortress(unit.location)
						unit.moveSpent = 255
					end						
					-- Follow up 3rd reaction force units are delayed 1 and 2 turns
					delayedAction.doInFuture("invReact3rdT2_UnitsFano",{},civ.getTurn()+ 1 ,object.pAxis.id)
					delayedAction.doInFuture("invReact3rdT3_UnitsFano",{},civ.getTurn()+ 2 ,object.pAxis.id)					
			
				elseif invasionUnitsInCivitavecchia() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 74 and unit.location.y >= 52 and unit.location.x <= 82 and unit.location.y <= 76 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(75,67,0) 
							end
						end	
					end
					-- Place extra blocking Widerstandsnest units
					local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{85,57,0},{81,49,0},{75,55,0}}, {count=1, randomize=true, veteran=false})	[1]
					if unit then
						gen.setToNoOrders(unit)
						gen.setToFortified(unit)
						gen.placeFortress(unit.location)
						unit.moveSpent = 255
					end	
					local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{82,64,0},{82,70,0},{79,73,0}}, {count=1, randomize=true, veteran=false})	[1]
					if unit then
						gen.setToNoOrders(unit)
						gen.setToFortified(unit)
						gen.placeFortress(unit.location)
						unit.moveSpent = 255
					end
					-- Follow up 3rd reaction force units are delayed 1 and 2 turns
					delayedAction.doInFuture("invReact3rdT2_UnitsCivitavecchia",{},civ.getTurn()+ 1 ,object.pAxis.id)
					delayedAction.doInFuture("invReact3rdT3_UnitsCivitavecchia",{},civ.getTurn()+ 2 ,object.pAxis.id)	

				elseif invasionUnitsInAnzio() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 80 and unit.location.y >= 76 and unit.location.x <= 86 and unit.location.y <= 82 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(78,84,0) 
							end
						end	
					end	
					
					-- Place extra blocking Widerstandsnest units
					-- local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{82,80,0},{84,82,0}}, {count=1, randomize=true, veteran=false})	[1]
					-- if unit then
						-- gen.setToNoOrders(unit)
						-- gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						-- unit.moveSpent = 255
					-- end	
					-- local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{77,75,0},{79,73,0}}, {count=1, randomize=true, veteran=false})	[1]
					-- if unit then
						-- gen.setToNoOrders(unit)
						-- gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						-- unit.moveSpent = 255
					-- end							
	
					-- Replaced WSN with Infanterie Veteranen
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, invReact3rdT1_LocationList, {count=4, randomize=true, scatter=true, veteran=true})						
					
					-- Follow up 3rd reaction force units are delayed 1 and 2 turns
					delayedAction.doInFuture("invReact3rdT2_UnitsAnzio",{},civ.getTurn()+ 1 ,object.pAxis.id)
					delayedAction.doInFuture("invReact3rdT3_UnitsAnzio",{},civ.getTurn()+ 2 ,object.pAxis.id)
					
				--end	
				
				elseif invasionUnitsInTerracina() == true then
					for unit in civ.iterateUnits() do
						if unit.owner == object.pAxis then
							if (unit.location.x >= 80 and unit.location.y >= 76 and unit.location.x <= 86 and unit.location.y <= 82 and unit.location.z == 0) then
								unit.gotoTile = civ.getTile(82,92,0) 
							end
						end	
					end	
					
					-- Replaced WSN with Infanterie Veteranen
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, invReact3rdT1_LocationList, {count=4, randomize=true, scatter=true, veteran=true})						
					
					-- Follow up 3rd reaction force units are delayed 1 and 2 turns
					delayedAction.doInFuture("invReact3rdT2_UnitsTerracina",{},civ.getTurn()+ 1 ,object.pAxis.id)
					delayedAction.doInFuture("invReact3rdT3_UnitsTerracina",{},civ.getTurn()+ 2 ,object.pAxis.id)
					


				elseif invasionUnitsInFirenze() == true then
					-- Place extra blocking Widerstandsnest units
					-- local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{89,41,0},{88,34,0}}, {count=1, randomize=true, veteran=false})	[1]
					-- if unit then
						-- gen.setToNoOrders(unit)
						-- gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						-- unit.moveSpent = 255
					-- end	
					-- local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{91,37,0},{86,40,0}}, {count=1, randomize=true, veteran=false})	[1]
					-- if unit then
						-- gen.setToNoOrders(unit)
						-- gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						-- unit.moveSpent = 255
					-- end							

					-- Replaced WSN with Infanterie Veteranen
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, invReact3rdT1_LocationList, {count=4, randomize=true, scatter=true, veteran=true})

					-- Follow up 3rd reaction force units are delayed 1 and 2 turns
					delayedAction.doInFuture("invReact3rdT2_UnitsFirenze",{},civ.getTurn()+ 1 ,object.pAxis.id)
					delayedAction.doInFuture("invReact3rdT3_UnitsFirenze",{},civ.getTurn()+ 2 ,object.pAxis.id)
					
				end	
			end)
		end	
	
	end
	
	-- OPTION 2 = If the Allies opt not to proceed with an actual 3rd invasion then deploy Axis units after turn 29 (Feb I, 1944)
	if turn >= 29 and flag.value("3rd Invasion Available") == true and flag.value("3rd Invasion Not-Launched") == true then
		gen.justOnce("No 3rd Invasion Launched, 1st turn, deployed", function()
			civ.playSound("InfantryGer.wav")
			civ.ui.text("New German units deploy to Italy!")
			flag.setTrue("No 3rd Invasion Units, 1st turn, triggered")
			gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=5, randomize=true, veteran=false})
			gen.createUnit(object.uPzkwVG, object.pAxis, {{80,76,0},{88,106,0},{107,83,0},{88,96,0},{91,93,0},{100,90,0}}, {count=2, randomize=true, veteran=false})
			local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				gen.setToNoOrders(unit)
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
			end	
			local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=1, randomize=true, veteran=false})[1]
			if unit then
				gen.setToNoOrders(unit)
				gen.setToFortified(unit)
				gen.placeFortress(unit.location)
				unit.moveSpent = 255
			end	
			-- Follow up reaction force units are delayed 1 and 2 turns
			delayedAction.doInFuture("no3rdInvasionT1_reactionUnits",{},civ.getTurn()+ 1 ,object.pAxis.id)
			delayedAction.doInFuture("no3rdInvasionT2_reactionUnits",{},civ.getTurn()+ 2 ,object.pAxis.id)	
	
		end)
	end
	
	-- Relocate Krupp K5 rail gun if any of three conditions below apply
	if turn >= 17 and (flag.value("German 10th Army Cutoff") == true or flag.value("Gustav Line is breached") == true or object.cROMA.owner == object.pAllies) then
		gen.justOnce("Relocate Krupp K5 rail gun",function()
			for unit in civ.iterateUnits() do
				if unit.type == object.uKruppK5 then
					civ.deleteUnit(unit)
				end
			end	
			
			gen.createUnit(object.uKruppK5, object.pAxis, {{80,26,0},{87,29,0},{91,33,0},{96,40,0},{103,43,0}}, {count=1, randomize=true, veteran=false})
			if flag.value("Display Axis Testing Messages") == true then
				text.simple("AXIS TESTING MESSAGE:\n^The Germans relocate the Krupp K5 rail gun somewhere along the Gothic Line.","Krupp K5 Gun Relocated!",object.m10thArmy)
			end
			
		end)
	end
	
	-- ====================================================================
	-- AXIS WEEKLY REPLACEMENTS	
	-- ====================================================================
	-- INFANTRY (1 per turn)
		-- object.uInfanterieVeteranen 66% chance per turn
		-- object.uPanzerschreck 33% chance per turn
	-- ARMORED (1 every 3 turns)
		-- object.uStugIIIG 11% chance per turn
		-- object.uPzkwIVF 11% chance per turn
		-- object.uPzkwIVH 11% chance per turn
	-- ARTILLERY (1 every 9 turns)
		-- object.u105cmleFH18 11% chance per turn
		-- object.u15cmsIG33 0% chance per turn
		-- object.u37cmFlak36 0% chance per turn
		-- object.u88cmFlak37 0% chance per turn				
	-- AIR (1 every 3 turns)
		-- object.uMe109G 11% chance per turn	
		-- object.uFW190A 22% chance per turn
		-- object.uJu88 11% chance per turn
		-- object.uDo217E 11% chance per turn
		-- object.uFritzX 22% chance per turn
	-- U-BOATS (1 every 8 turns)	
		
	-- ONLY FOR TESTING RAILROAD YARD # PURPOSES
	-- if turn == 1 then 
		-- gen.createUnit(object.uRailroadYard, object.pAxis, {{98,4,0}}, {count=1, veteran=false})	
		-- gen.createUnit(object.uRailroadYard, object.pAxis, {{106,14,0}}, {count=1, veteran=false})
		-- gen.createUnit(object.uRailroadYard, object.pAxis, {{115,21,0}}, {count=1, veteran=false})
		-- gen.createUnit(object.uRailroadYard, object.pAxis, {{100,30,0}}, {count=1, veteran=false})
		-- gen.createUnit(object.uRailroadYard, object.pAxis, {{89,39,0}}, {count=1, veteran=false})
		-- gen.createUnit(object.uM12155mmHMC, object.pAllies, {{100,32,0}}, {count=1, veteran=true})
	-- end
	
	if turn >= 4 then
	-- ===========================================================
	-- COUNT RAILROAD YARD UNIT EACH TURN
	-- Each active Railroad Yard determines Axis infantry, armored and air unit replacement percentage rates for each turn
		local totalRailroadYard = 0
		for unit in civ.iterateUnits() do
			if unit.owner == object.pAxis then
				if unit.type == object.uRailroadYard then
					totalRailroadYard = totalRailroadYard + 1
				end
			end
		end	
		
		-- AXIS REPLACEMENTS ALONG GUSTAV LINE (Central Italy)
		if (flag.value("Gustav Line is breached") == false and flag.value("German 10th Army Cutoff") == false) then 
			
			if totalRailroadYard == 5 then -- (100%)
				local randomNumber = math.random(9)
				
				if randomNumber <= 3 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					-- gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{80,76,0},{88,106,0},{107,83,0},{105,87,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}}, {randomize=true, veteran=false})
				elseif randomNumber <= 4 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {count=2, randomize=true, veteran=false})				
				elseif randomNumber <= 5 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {count=2, randomize=true, veteran=false})					
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber == 6 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uMe109G, object.pAxis, {{80,76,0},{91,65,0},{107,83,0},{111,107,0}}, {randomize=true, veteran=false})
				elseif randomNumber == 7 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uSdkFz72AA, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uFW190A, object.pAxis, filterTilesByOwner({{80,76,0},{91,65,0},{107,83,0},{111,107,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber == 8 then
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerGrenadier, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=true})
					gen.createUnit(object.uPzkwIVF, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPzkwVG, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uGrilleHsIG33, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uFW190A, object.pAxis, filterTilesByOwner({{80,76,0},{91,65,0},{107,83,0},{111,107,0}},object.pAllies), {randomize=true, veteran=false})
					-- gen.createUnit(object.uJu88, object.pAxis, filterTilesByOwner({{80,76,0},{91,65,0},{107,83,0},{111,107,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber == 9 then
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})					
					gen.createUnit(object.uMotInfanterie, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=true})					
					gen.createUnit(object.uPanzerGrenadier, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=true})					
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPzkwVIE, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uGrilleHsIG33, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uMe110, object.pAxis, filterTilesByOwner({{80,76,0},{91,65,0},{107,83,0},{111,107,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					-- gen.createUnit(object.uDo217E, object.pAxis, filterTilesByOwner({{80,76,0},{91,65,0},{107,83,0},{111,107,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
				end
				
				if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					if randomNumber <=3 then
						-- text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^- For numbers 1 to 5 they receive one replacement\n^- For number 6 they receive 2 replacements\n^- For number 7 they receive 3 replacements\n^- For number 8 they receive 5 replacements\n^- For number 9 they receive 6 replacements","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 1 replacement.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",1)
					elseif randomNumber == 4 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 2 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",2)
					elseif randomNumber == 5 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 3 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",3)
					elseif randomNumber == 6 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 4 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",4)
					elseif randomNumber == 7 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 5 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",5)
					elseif randomNumber == 8 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 6 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",6)
					elseif randomNumber == 9 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 7 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",7)
					end
				end
				print("Gustav Line: There are " .. totalRailroadYard .. " RR yards and the random number selected was " .. randomNumber .. " (of 9)")
			
			elseif totalRailroadYard == 4 then -- (90%)
				local randomNumber = math.random(9)
				
				if randomNumber <= 4 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					-- gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{80,76,0},{88,106,0},{107,83,0},{105,87,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}}, {randomize=true, veteran=false})
				elseif randomNumber <= 5 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {count=2, randomize=true, veteran=false})				
				elseif randomNumber <= 6 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {count=2, randomize=true, veteran=false})					
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber == 7 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uMe109G, object.pAxis, {{80,76,0},{91,65,0},{107,83,0},{111,107,0}}, {randomize=true, veteran=false})
				elseif randomNumber == 8 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uSdkFz72AA, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uFW190A, object.pAxis, filterTilesByOwner({{80,76,0},{91,65,0},{107,83,0},{111,107,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber == 9 then
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerGrenadier, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=true})
					gen.createUnit(object.uPzkwIVF, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPzkwVG, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uGrilleHsIG33, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uFW190A, object.pAxis, filterTilesByOwner({{80,76,0},{91,65,0},{107,83,0},{111,107,0}},object.pAllies), {randomize=true, veteran=false})
					-- gen.createUnit(object.uJu88, object.pAxis, filterTilesByOwner({{80,76,0},{91,65,0},{107,83,0},{111,107,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
				end
				
				if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					if randomNumber <=4 then
						-- text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^- For numbers 1 to 5 they receive one replacement\n^- For number 6 they receive 2 replacements\n^- For number 7 they receive 3 replacements\n^- For number 8 they receive 5 replacements\n^- For number 9 they receive 6 replacements","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 1 replacement.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",1)
					elseif randomNumber == 5 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 2 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",2)
					elseif randomNumber == 6 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 3 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",3)
					elseif randomNumber == 7 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 4 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",4)
					elseif randomNumber == 8 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 5 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",5)
					elseif randomNumber == 9 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 6 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",6)
					end
				end
				print("Gustav Line: There are " .. totalRailroadYard .. " RR yards and the random number selected was " .. randomNumber .. " (of 9)")
				
			elseif totalRailroadYard == 3 then -- (80%)
				local randomNumber = math.random(9)
				
				if randomNumber <= 5 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					-- gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{80,76,0},{88,106,0},{107,83,0},{105,87,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}}, {randomize=true, veteran=false})
				elseif randomNumber <= 6 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {count=2, randomize=true, veteran=false})				
				elseif randomNumber <= 7 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {count=2, randomize=true, veteran=false})					
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber == 8 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uMe109G, object.pAxis, {{80,76,0},{91,65,0},{107,83,0},{111,107,0}}, {randomize=true, veteran=false})
				elseif randomNumber == 9 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uSdkFz72AA, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uFW190A, object.pAxis, filterTilesByOwner({{80,76,0},{91,65,0},{107,83,0},{111,107,0}},object.pAllies), {randomize=true, veteran=false})
				end
				
				if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					if randomNumber <=5 then
						-- text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^- For numbers 1 to 5 they receive one replacement\n^- For number 6 they receive 2 replacements\n^- For number 7 they receive 3 replacements\n^- For number 8 they receive 5 replacements\n^- For number 9 they receive 6 replacements","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 1 replacement.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",1)
					elseif randomNumber == 6 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 2 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",2)
					elseif randomNumber == 7 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 3 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",3)
					elseif randomNumber == 8 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 4 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",4)
					elseif randomNumber == 9 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 5 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",5)
					end
				end
				print("Gustav Line: There are " .. totalRailroadYard .. " RR yards and the random number selected was " .. randomNumber .. " (of 9)")
				
			elseif totalRailroadYard == 2 then -- (70%)
				local randomNumber = math.random(9)
				
				if randomNumber <= 6 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					-- gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{80,76,0},{88,106,0},{107,83,0},{105,87,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}}, {randomize=true, veteran=false})
				elseif randomNumber <= 7 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {count=2, randomize=true, veteran=false})				
				elseif randomNumber <= 8 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {count=2, randomize=true, veteran=false})					
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber == 9 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uMe109G, object.pAxis, {{80,76,0},{91,65,0},{107,83,0},{111,107,0}}, {randomize=true, veteran=false})
				end
				
				if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					if randomNumber <=6 then
						-- text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^- For numbers 1 to 5 they receive one replacement\n^- For number 6 they receive 2 replacements\n^- For number 7 they receive 3 replacements\n^- For number 8 they receive 5 replacements\n^- For number 9 they receive 6 replacements","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 1 replacement.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",1)
					elseif randomNumber == 7 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 2 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",2)
					elseif randomNumber == 8 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 3 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",3)
					elseif randomNumber == 9 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 4 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",4)
					end
				end
				print("Gustav Line: There are " .. totalRailroadYard .. " RR yards and the random number selected was " .. randomNumber .. " (of 9)")
				
			elseif totalRailroadYard <= 1 then -- (60%)
				local randomNumber = math.random(9)
				
				if randomNumber <= 7 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
					-- gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{80,76,0},{88,106,0},{107,83,0},{105,87,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}}, {randomize=true, veteran=false})
				elseif randomNumber <= 8 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {count=2, randomize=true, veteran=false})				
				elseif randomNumber <= 9 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {count=2, randomize=true, veteran=false})					
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0},{105,85,0},{88,96,0},{91,93,0},{100,86,0},{89,39,0}},object.pAllies), {randomize=true, veteran=false})
				end
				
				if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					if randomNumber <=7 then
						-- text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^- For numbers 1 to 5 they receive one replacement\n^- For number 6 they receive 2 replacements\n^- For number 7 they receive 3 replacements\n^- For number 8 they receive 5 replacements\n^- For number 9 they receive 6 replacements","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 1 replacement.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",1)
					elseif randomNumber == 8 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 2 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",2)
					elseif randomNumber == 9 then
						text.simple("AXIS MESSAGE:\n^\r^Gustav Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 3 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",3)
					end
				end
				print("Gustav Line: There are " .. totalRailroadYard .. " RR yards and the random number selected was " .. randomNumber .. " (of 9)")
				
			end		
		
		-- AXIS REPLACEMENTS ALONG GOTHIC LINE (Northern Italy)
		elseif (flag.value("Gustav Line is breached") == true or flag.value("German 10th Army Cutoff") == true) then -- if Gustav Line is breached than begin receiving replacements along Gothic Line instead
		
			if totalRailroadYard == 5 then -- (100%)
				local randomNumber = math.random(9)
				
				if randomNumber <= 3 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber <= 4 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {count=2, randomize=true, veteran=false})
				elseif randomNumber <= 5 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {count=2, randomize=true, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber == 6 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})					
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uMe109G, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
				elseif randomNumber == 7 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})						
					gen.createUnit(object.uSdkFz72AA, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uFW190A, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
				elseif randomNumber == 8 then
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})															
					gen.createUnit(object.uPanzerGrenadier, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=true})
					gen.createUnit(object.uPzkwIVF, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPzkwVG, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uGrilleHsIG33, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uFW190A, object.pAxis, {{89,39,0},{96,24,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
					-- gen.createUnit(object.uJu88, object.pAxis, {{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
				elseif randomNumber == 9 then
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})															
					gen.createUnit(object.uMotInfanterie, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=true})
					gen.createUnit(object.uPanzerGrenadier, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=true})															
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPzkwVIE, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uGrilleHsIG33, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uMe110, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
					-- gen.createUnit(object.uDo217E, object.pAxis, {{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
				end
				
				if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					if randomNumber <=3 then
						-- text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^- For numbers 1 to 5 they receive one replacement\n^- For number 6 they receive 2 replacements\n^- For number 7 they receive 3 replacements\n^- For number 8 they receive 5 replacements\n^- For number 9 they receive 6 replacements","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 1 replacement.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",1)
					elseif randomNumber == 4 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 2 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",2)
					elseif randomNumber == 5 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 3 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",3)
					elseif randomNumber == 6 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 4 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",4)
					elseif randomNumber == 7 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 5 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",5)
					elseif randomNumber == 8 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 6 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",6)
					elseif randomNumber == 9 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 7 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",7)
					end
				end
				print("Gothic Line: There are " .. totalRailroadYard .. " RR yards and the random number selected was " .. randomNumber .. " (of 9)")
			
			elseif totalRailroadYard == 4 then -- (90%)
				local randomNumber = math.random(9)
				
				if randomNumber <= 4 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber <= 5 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {count=2, randomize=true, veteran=false})
				elseif randomNumber <= 6 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {count=2, randomize=true, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber == 7 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})					
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uMe109G, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
				elseif randomNumber == 8 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})						
					gen.createUnit(object.uSdkFz72AA, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uFW190A, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
				elseif randomNumber == 9 then
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})															
					gen.createUnit(object.uPanzerGrenadier, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=true})
					gen.createUnit(object.uPzkwIVF, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPzkwVG, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uGrilleHsIG33, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uFW190A, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
					-- gen.createUnit(object.uJu88, object.pAxis, {{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
				end
				
				if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					if randomNumber <=4 then
						-- text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^- For numbers 1 to 5 they receive one replacement\n^- For number 6 they receive 2 replacements\n^- For number 7 they receive 3 replacements\n^- For number 8 they receive 5 replacements\n^- For number 9 they receive 6 replacements","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 1 replacement.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",1)
					elseif randomNumber == 5 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 2 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",2)
					elseif randomNumber == 6 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 3 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",3)
					elseif randomNumber == 7 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 4 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",4)
					elseif randomNumber == 8 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 5 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",5)
					elseif randomNumber == 9 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 6 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",6)
					end
				end
				print("Gothic Line: There are " .. totalRailroadYard .. " RR yards and the random number selected was " .. randomNumber .. " (of 9)")
				
			elseif totalRailroadYard == 3 then -- (80%)
				local randomNumber = math.random(9)
				
				if randomNumber <= 5 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber <= 6 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {count=2, randomize=true, veteran=false})
				elseif randomNumber <= 7 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {count=2, randomize=true, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber == 8 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})					
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uMe109G, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
				elseif randomNumber == 9 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})						
					gen.createUnit(object.uSdkFz72AA, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uFW190A, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
				end
				
				if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					if randomNumber <=5 then
						-- text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^- For numbers 1 to 5 they receive one replacement\n^- For number 6 they receive 2 replacements\n^- For number 7 they receive 3 replacements\n^- For number 8 they receive 5 replacements\n^- For number 9 they receive 6 replacements","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 1 replacement.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",1)
					elseif randomNumber == 6 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 2 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",2)
					elseif randomNumber == 7 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 3 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",3)
					elseif randomNumber == 8 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 4 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",4)
					elseif randomNumber == 9 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 5 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",5)
					end
				end
				print("Gothic Line: There are " .. totalRailroadYard .. " RR yards and the random number selected was " .. randomNumber .. " (of 9)")
				
			elseif totalRailroadYard == 2 then -- (70%)
				local randomNumber = math.random(9)
				
				if randomNumber <= 6 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber <= 7 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {count=2, randomize=true, veteran=false})
				elseif randomNumber <= 8 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {count=2, randomize=true, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber == 9 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})					
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
					gen.createUnit(object.uMe109G, object.pAxis, {{89,39,0},{100,30,0},{115,21,0},{72,12,0},{98,4,0}}, {randomize=true, veteran=false})
				end
				
				if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					if randomNumber <=6 then
						-- text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^- For numbers 1 to 5 they receive one replacement\n^- For number 6 they receive 2 replacements\n^- For number 7 they receive 3 replacements\n^- For number 8 they receive 5 replacements\n^- For number 9 they receive 6 replacements","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 1 replacement.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",1)
					elseif randomNumber == 7 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 2 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",2)
					elseif randomNumber == 8 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 3 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",3)
					elseif randomNumber == 9 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 4 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",4)
					end
				end
				print("Gothic Line: There are " .. totalRailroadYard .. " RR yards and the random number selected was " .. randomNumber .. " (of 9)")
				
			elseif totalRailroadYard <= 1 then -- (60%)
				local randomNumber = math.random(9)
				
				if randomNumber <= 7 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
				elseif randomNumber <= 8 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {count=2, randomize=true, veteran=false})
				elseif randomNumber <= 9 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {count=2, randomize=true, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{77,23},{89,39,0},{100,30,0},{103,39,0},{106,44,0},{82,30,0},{88,34,0}},object.pAllies), {randomize=true, veteran=false})
				end
				
				if flag.value("Display Axis Messages") == true then -- Set to false to turn Axis messages off for actual gameplay 
					if randomNumber <=7 then
						-- text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^- For numbers 1 to 5 they receive one replacement\n^- For number 6 they receive 2 replacements\n^- For number 7 they receive 3 replacements\n^- For number 8 they receive 5 replacements\n^- For number 9 they receive 6 replacements","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 1 replacement.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",1)
					elseif randomNumber == 8 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 2 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",2)
					elseif randomNumber == 9 then
						text.simple("AXIS MESSAGE:\n^\r^Gothic Line: There are " .. totalRailroadYard .. " active RR yards and the\n^Germans randomly selected number " .. randomNumber .. " (of 9)\n^\r^As a result, they received 3 replacements.","Heeresgruppe Sud Replacements!",object.mArmyGroupSud)
						counter.add("# Total German Replacements Received",3)
					end
				end
				print("Gothic Line: There are " .. totalRailroadYard .. " RR yards and the random number selected was " .. randomNumber .. " (of 9)")
				
			end			
		end
				
	end
	
	-- ====================================================================
	-- GERMAN U-BOAT AND FRITZ-X REINFORCEMENTS
	-- ====================================================================		
	-- U-BOAT REINFORCEMENTS
	if turn >= 4 then
		if math.random(1,6) == 6 then
			gen.createUnit(object.uTypeIXC, object.pAxis, {{60,84,0},{62,22,0},{62,58,0},{69,107,0},{62,122,0},{38,118,0},{127,69,0},{111,169,0}}, {randomize=true, veteran=false})
			-- gen.createUnit(object.uTypeIXC, object.pAxis, {{88,106,0},{53,47,0},{75,35,0},{72,12,0},{106,44,0}}, {randomize=true, veteran=false})
			if flag.value("Display Axis Testing Messages") == true then
				text.simple("AXIS TESTING MESSAGE:\n^The Kriegsmarine deploys a new U-Boat flotilla in the Mediterranean","New U-Boat Flotilla!",object.m10thArmy)
			end
			
			-- Apply GoTo commands
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis and unit.type == object.uTypeIXC then
					-- U-Boat (62,22)
					if (unit.location.x >= 56 and unit.location.y >= 16 and unit.location.x <= 68 and unit.location.y <= 28 and unit.location.z == 0) then
						unit.gotoTile = civ.getTile(69,43,0) 
					end
					-- U-Boat (62,58)
					if (unit.location.x >= 56 and unit.location.y >= 52 and unit.location.x <= 68 and unit.location.y <= 64 and unit.location.z == 0) then
						unit.gotoTile = civ.getTile(72,64,0) 
					end
					-- U-Boat (60,84)
					if (unit.location.x >= 54 and unit.location.y >= 78 and unit.location.x <= 66 and unit.location.y <= 90 and unit.location.z == 0) then
						unit.gotoTile = civ.getTile(63,87,0) 
					end
					-- U-Boat (69,107)
					if (unit.location.x >= 63 and unit.location.y >= 101 and unit.location.x <= 75 and unit.location.y <= 113 and unit.location.z == 0) then
						unit.gotoTile = civ.getTile(81,99,0) 
					end
					-- U-Boat (62,122)
					if (unit.location.x >= 56 and unit.location.y >= 116 and unit.location.x <= 68 and unit.location.y <= 128 and unit.location.z == 0) then
						unit.gotoTile = civ.getTile(74,142,0) 
					end
					-- U-Boat (38,118)
					if (unit.location.x >= 32 and unit.location.y >= 112 and unit.location.x <= 44 and unit.location.y <= 124 and unit.location.z == 0) then
						unit.gotoTile = civ.getTile(45,135,0) 
					end
					-- U-Boat (111,169)
					if (unit.location.x >= 105 and unit.location.y >= 163 and unit.location.x <= 117 and unit.location.y <= 175 and unit.location.z == 0) then
						unit.gotoTile = civ.getTile(114,142,0) 
					end
				end	
			end	
		end
	end

	-- FRITZ-X REINFORCEMENTS
	if turn >= 4 then
		if math.random(1,5) == 5 then
			if (flag.value("Gustav Line is breached") == false and flag.value("German 10th Army Cutoff") == false) then 
				gen.createUnit(object.uFritzX, object.pAxis, filterTilesByOwner({{80,76,0},{88,106,0},{107,83,0}},object.pAllies), {randomize=true, veteran=false})
			elseif (flag.value("Gustav Line is breached") == true or flag.value("German 10th Army Cutoff") == true) then 
				gen.createUnit(object.uFritzX, object.pAxis, {{75,35,0},{77,23,0},{72,12,0},{107,37,0}}, {randomize=true, veteran=false})
			end
		end
	end
	
	-- ====================================================================
	-- ITALIAN NAVY	AND FORCE 'H'
	-- ====================================================================	
	-- Regia Marina OOB in Summer 1943
	-- object.uLittorioClassBB 	= 6 BB (At La Spezia: Roma, Vittorio Veneto and Italia (formerly Littorio), At Taranto: Andrea Doria, Dulio and Giulio Cesare)
	-- object.uGiusannoClassCL 	= 9 CL + 2 CA
	-- object.uSellaClassDD = 3 DD
	-- object.uSottomarino 	= 5 SS
	
	-- Regia Marina deploys 3 destroyers, 2 cruisers and 5 submarines within first 4 turns as part of its commitment to the Italian campaign
	if turn == 1 then  
		local randomNumber = math.random(3)
		if randomNumber == 1 then
			delayedAction.doInFuture("regiaMarinaSquadrons",{},civ.getTurn()+ 1 ,object.pAxis.id)
			print("Initial Regia Marina activation random number is " .. randomNumber .. " (vs. 3)")
		elseif randomNumber == 2 then
			delayedAction.doInFuture("regiaMarinaSquadrons",{},civ.getTurn()+ 2 ,object.pAxis.id)
			print("Initial Regia Marina activation random number is " .. randomNumber .. " (vs. 3)")
		elseif randomNumber == 3 then
			delayedAction.doInFuture("regiaMarinaSquadrons",{},civ.getTurn()+ 3 ,object.pAxis.id)
			print("Initial Regia Marina activation random number is " .. randomNumber .. " (vs. 3)")
		end	
	end
	
	-- If Allies fail to capture island of Sicily prior to turn 8, part of Italian main battle fleet makes a sortie
	if turn >= 8 and flag.value("Sicily is Conquered") == false then
		gen.justOnce("Regia Marina Main Battle Fleet",function()
			local randomNumber = math.random(3)
			
			-- Random number and placement of main battle fleet units 
			if randomNumber == 1 then
				print("Regia Marina Main Battle Fleet activation random number is " .. randomNumber .. " (vs. 3)")
				civ.playSound("warship.wav")
				text.simple("Feeling bolstered by the tenacious defense of Sicily the Italian Commando Supremo has ordered elements of the Regia Marina main battle fleet to sortie in order to repel the Allied naval forces arrayed around the island!","Regia Marina Main Battle Fleet Sorties!", object.mRegiaMarina)
				gen.createUnit(object.uLittorioClassBB, object.pAxis, {{88,106},{61,139,0},{72,148,0},{75,155,0}}, {count=3, randomize=true, veteran=false})
				gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{88,106},{61,139,0},{72,148,0},{75,155,0}}, {count=3, randomize=true, veteran=false})
				gen.createUnit(object.uLittorioClassBB, object.pAxis, {{71,159,0},{69,171,0},{81,163,0}}, {count=1, randomize=true, veteran=false})
				gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{71,159,0},{69,171,0},{81,163,0}}, {count=1, randomize=true, veteran=false})		
				-- Remainder of Regia Marina Naval Units deployed in LaSpezia for potential capture by Germans
				-- ALSO SEE onActivateUnit.lua "PREVENT ITALIAN NAVAL UNITS STATIONED IN LA SPEZIA FROM MOVING OUT OF CITY" section
				gen.createUnit(object.uLittorioClassBB, object.pAxis, {{77,23,0}}, {count=2, randomize=false, veteran=false})
				gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{77,23,0}}, {count=5, randomize=false, veteran=false})
				flag.setTrue("Regia Marina Main Battle Fleet Sorties")
				
			elseif randomNumber == 2 then
				print("Regia Marina Main Battle Fleet activation random number is " .. randomNumber .. " (vs. 3)")
				civ.playSound("warship.wav")
				text.simple("Feeling bolstered by the tenacious defense of Sicily the Italian Commando Supremo has ordered elements of the Regia Marina main battle fleet to sortie in order to repel the Allied naval forces arrayed around the island!","Regia Marina Main Battle Fleet Sorties!", object.mRegiaMarina)
				gen.createUnit(object.uLittorioClassBB, object.pAxis, {{88,106},{61,139,0},{72,148,0},{75,155,0}}, {count=2, randomize=true, veteran=false})
				gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{88,106},{61,139,0},{72,148,0},{75,155,0}}, {count=3, randomize=true, veteran=false})
				gen.createUnit(object.uLittorioClassBB, object.pAxis, {{71,159,0},{69,171,0},{81,163,0},{120,138,0}}, {count=1, randomize=true, veteran=false})
				gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{71,159,0},{69,171,0},{81,163,0},{120,138,0}}, {count=2, randomize=true, veteran=false})						
				-- Remainder of Regia Marina Naval Units deployed in LaSpezia for potential capture by Germans
				-- ALSO SEE onActivateUnit.lua "PREVENT ITALIAN NAVAL UNITS STATIONED IN LA SPEZIA FROM MOVING OUT OF CITY" section
				gen.createUnit(object.uLittorioClassBB, object.pAxis, {{77,23,0}}, {count=3, randomize=false, veteran=false})
				gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{77,23,0}}, {count=4, randomize=false, veteran=false})
				flag.setTrue("Regia Marina Main Battle Fleet Sorties")
				
			elseif randomNumber == 3 then
				print("Regia Marina Main Battle Fleet activation random number is " .. randomNumber .. " (vs. 3)")
				civ.playSound("warship.wav")
				text.simple("Feeling bolstered by the tenacious defense of Sicily the Italian Commando Supremo has ordered elements of the Regia Marina main battle fleet to sortie in order to repel the Allied naval forces arrayed around the island!","Regia Marina Main Battle Fleet Sorties!", object.mRegiaMarina)
				gen.createUnit(object.uLittorioClassBB, object.pAxis, {{61,139,0},{72,148,0},{75,155,0}}, {count=1, randomize=true, veteran=false})
				gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{88,106},{61,139,0},{72,148,0},{75,155,0}}, {count=3, randomize=true, veteran=false})
				gen.createUnit(object.uLittorioClassBB, object.pAxis, {{71,159,0},{69,171,0},{81,163,0},{120,138,0}}, {count=1, randomize=true, veteran=false})
				gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{71,159,0},{69,171,0},{81,163,0},{120,138,0}}, {count=3, randomize=true, veteran=false})	
				-- Remainder of Regia Marina Naval Units deployed in LaSpezia for potential capture by Germans
				-- ALSO SEE onActivateUnit.lua "PREVENT ITALIAN NAVAL UNITS STATIONED IN LA SPEZIA FROM MOVING OUT OF CITY" section
				gen.createUnit(object.uLittorioClassBB, object.pAxis, {{77,23,0}}, {count=4, randomize=false, veteran=false})
				gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{77,23,0}}, {count=3, randomize=false, veteran=false})	
				flag.setTrue("Regia Marina Main Battle Fleet Sorties")
				
			end
		end)
	end	

	-- Royal Navy Force 'H' Squadron Reacts to Regia Marina Battle Fleet Sortie
	if flag.value("Regia Marina Main Battle Fleet Sorties") == true then
		gen.justOnce("Royal Navy Reacts to Regia Marina Sortie",function()
			text.simple("Admiral Cunningham's Force 'H' located in Malta is ready to make sail in response to the Regia Marina's Main Battle Fleet sortie! The battlegroup is composed of 2 battleships, 2 heavy and one light cruiser and 1 destroyer squadron.","Royal Navy Force H Responds", object.mForceH)
			gen.createUnit(object.uQEClassBB, object.pAllies, {{40,180,0}}, {count=2, randomize=false, veteran=true})
			gen.createUnit(object.uYorkClassCA, object.pAllies, {{40,180,0}}, {count=2, randomize=false, veteran=true})
			gen.createUnit(object.uLeanderClassCL, object.pAllies, {{40,180,0}}, {count=1, randomize=false, veteran=true})
			gen.createUnit(object.uTribalClassDD, object.pAllies, {{40,180,0}}, {count=1, randomize=false, veteran=true})
			
			counter.add("# Tribal Class DD",1)
			counter.add("# Leander Class CL",1)
			counter.add("# York Class CA",2)
			counter.add("# QE Class BB",2) -- BB's don't count against Total Naval units
			
			counter.add("# Total Naval Units",6) -- BB's don't count against Total Naval units			
		end)	
	end	
	
	-- Force 'H' is withdrawn if the Italian navy is reduced to 4 or less units
	if turn >= 1 and flag.value("Regia Marina Main Battle Fleet Sorties") == true and flag.value("Italy Surrendered") == true then
		gen.justOnce("Force 'H' is withdrawn",function()
			local totalItalianNavalunit = 0
			-- Count the number of Italian vessels at the start of a turn
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis and tables.italianNavyToBeDestroyed[unit.type.id] then
					totalItalianNavalunit = totalItalianNavalunit + 1
				end
			end	

			-- If the count equals 4 or less than withdraw the corresponding Royal Navy 2 BB, 2 CA, 1 CL and 1 DD
			if totalItalianNavalunit <= 4 then
				local a = 0
				local b = 0
				local c = 0
				local d = 0						
				-- QE Class BB
				for unit in civ.iterateUnits() do 
					if unit.type == object.uQEClassBB then
						if a < 2 then
							civ.deleteUnit(unit)
							a = a + 1
							counter.subtract("# QE Class BB",1)
							counter.subtract("# Total Naval Units",1) -- BB's don't count against Total Naval units
						end
					end    
				end
				-- York Class CA
				for unit in civ.iterateUnits() do 
					if unit.type == object.uYorkClassCA then
						if b < 2 then
							civ.deleteUnit(unit)
							b = b + 1
							counter.subtract("# York Class CA",1)
							counter.subtract("# Total Naval Units",1)
						end
					end    
				end 			
				-- Leander Class CL
				for unit in civ.iterateUnits() do 
					if unit.type == object.uLeanderClassCL then
						if c < 1 then
							civ.deleteUnit(unit)
							c = c + 1
							counter.subtract("# Leander Class CL",1)
							counter.subtract("# Total Naval Units",1)
						end
					end    
				end 
				-- Tribal Class DD
				for unit in civ.iterateUnits() do 
					if unit.type == object.uTribalClassDD then
						if d < 1 then
							civ.deleteUnit(unit)
							d = d + 1
							counter.subtract("# Tribal Class DD",1)
							counter.subtract("# Total Naval Units",1)
						end
					end    
				end 

				civ.playSound("Shipwhistle.wav")
				text.simple("The British Admiralty considers that the Regia Marina has been sufficently neutralized and therefore that Admiral Cumningham's Force 'H' is no longer required in this theather and thus its 6 previously committed units are withdrawn.","Force 'H' is Withdrawn!",object.mForceH)
			end	
		end)	
	end

	-- ====================================================================
	-- GERMAN UNITS EVACUATE SICILY	
	-- ====================================================================	
	if turn >= 1 then
		local germansEvacuateSicily = 0
		if object.cCatania.owner == object.pAllies then
			germansEvacuateSicily = germansEvacuateSicily + 1
		end
		if object.cPalermo.owner == object.pAllies then
			germansEvacuateSicily = germansEvacuateSicily + 1
		end
		if object.cSiracusa.owner == object.pAllies then
			germansEvacuateSicily = germansEvacuateSicily + 1
		end
		if object.cTrapani.owner == object.pAllies then
			germansEvacuateSicily = germansEvacuateSicily + 1
		end

		print("Conditions met for German evacuation from Sicily: " .. germansEvacuateSicily .. " out of 4 possible")
		
		if germansEvacuateSicily >= 4 then
			gen.justOnce("Sicily evacuated by Axis forces 2",function()
				civ.playSound("Cathedrl.wav")
				for unit in civ.iterateUnits() do
					if unit.owner == object.pAxis and tables.germansToBeEvacuatedSicily[unit.type.id] and gen.inPolygon(unit.location,tables.sicilyPolygon) then 
					-- and unit.location.x > 27 and unit.location.x < 75 and unit.location.y > 131 and unit.location.y < 177 and unit.location.z == 0 then
						print("Destroyed " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
						civ.deleteUnit(unit)
					end
				end
				for unit in civ.iterateUnits() do
					if unit.owner == object.pAxis and tables.germansAirToBeEvacuated[unit.type.id] and gen.inPolygon(unit.location,tables.sicilyPolygon) then 
					-- and unit.location.x > 27 and unit.location.x < 75 and unit.location.y > 131 and unit.location.y < 177 and unit.location.z == 0 then
						print("Teleported " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
						civ.teleportUnit(unit, object.lFoggia)
					end
				end
				
				text.simple("With the fall of Catania and Palermo, Generalfeldmarschall Albert\n^Kesselring, Oberbefehlshaber Sud, understands it is no longer\n^possible to hold onto Sicily and orders the withdrawal of all\n^remaining Geman forces situated on the island save a possible\n^rearguard.\n^\r^The road to Messina should now be open and the capture of the\n^city within your grasp!\n^\r^As such, you may want to begin regrouping your forces around the\n^cities and airbases of Catania, Palermo and Messina (once captured),\n^in preparation for the invasion of the Italian mainland.","Axis forces evacuate Sicily!!",object.mAlbertKesselring)
				flag.setTrue("Germans evacuate Sicily")
			end)
		end
	
	end	

	-- ====================================================================
	-- SICILY IS CONQUERED	
	-- ====================================================================	
	if turn >= 1 then
		local sicilyIsConquered = 0
		if object.cCatania.owner == object.pAllies then
			sicilyIsConquered = sicilyIsConquered + 1
		end
		if object.cMessina.owner == object.pAllies then
			sicilyIsConquered = sicilyIsConquered + 1
		end
		if object.cPalermo.owner == object.pAllies then
			sicilyIsConquered = sicilyIsConquered + 1
		end
		if object.cSiracusa.owner == object.pAllies then
			sicilyIsConquered = sicilyIsConquered + 1
		end
		if object.cTrapani.owner == object.pAllies then
			sicilyIsConquered = sicilyIsConquered + 1
		end

		print("Conditions met for conquest of Sicily: " .. sicilyIsConquered .. " out of 5 possible")
		if sicilyIsConquered == 5 then
			gen.justOnce("Sicily conquered by Allies",function()
				civ.playSound("Cathedrl.wav")
				flag.setTrue("Sicily is Conquered")
				flag.setTrue("2nd Invasion Available")
				object.pAllies.money = object.pAllies.money + param.captureSicily
				civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^The conquest of the island of Sicily is now complete along\n^with the US 7th Army's mission. The Allied High Command\n^has now given you permission to proceed with the plans for the\n^invasion of the Italian mainland, which will be assigned to the\n^newly constituted US 5th Army.\n^\r^As a consequence a further ".. param.captureSicily .. " supplies have been allocated\n^to your theater and ".. counter.value("# Landing Craft Available for 2nd Invasion") .. " Landing Craft to help in your preparations.\n^\r^You can initiate your Second Invasion by reviewing your\n^Command Decisions (menu #5) and selecting the invasion\n^options that best meet your strategic goals..."))
				civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.value("Special Bulletin #") .. " ******************\n^\r^Secret negotiations have been ongoing with the government\n^of Marshall Badoglio in view of signing an armistice.\n^\r^The agreement is that Italian forces will lay down their arms\n^provided the Allies land in sufficient force on the mainland\n^in order to secure as much of the Italian territory as possible\n^before the inevitable German counter moves and reprisals.\n^\r^To complete this task you must have at least 21 Allied ground\n^combat units above the 'Italian Surrender Line', and have\n^seized any 2 major cities south of the Roma - San Benedetto\n^line. For these purposes, both Napoli and Roma are\n^considered worth two city points.\n^\r^As soon as this is accomplished all Italian air, ground and naval\n^forces loyal to the king and the Badoglio government will surrender.\n^\r^The capture of Foggia, Napoli or Roma will not only provide you\n^with extra supplies, but access to airbases and new air units..."))
				civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.value("Special Bulletin #") .. " ******************\n^\r^Once you select the 'Activate 2nd Invasion' box of the Command\n^Decision menu, you will be presented with 3 distinct options:\n^Activate Baytown, Activate Slapstick and Activate 2nd Invasion.\n^\r^Activate Baytown will allow you to automatically deploy a small\n^contingent of up to 7 British units in the town of Giola Tauro for a\n^cost of ".. param.baytownOpsCost .. " supplies.\n^\r^Activate Slapstick will allow you to tranport up to 2 British Para's\n^aboard a York Class CA and go land these unopposed, by secret\n^agreement with the Italians, near the port of Taranto for a cost\n^of ".. param.slapstickOpsCost .. " supplies.\n^\r^Activate 2nd Invasion will allow you to immediately deploy your\n^" .. counter.value("# Landing Craft Available for 2nd Invasion") .. " landing craft either for an invasion along the Adriatic (eastern)\n^or Tyrrhenian (western) Italian coasts.\n^\r^Time remains of the essence if the Allies are to keep their French\n^cross channel operation on schedule and thus these 3 options may\n^not be activated any later than October I, 1943. Failure to do so within\n^this time limit will only leave you with the reduced 3rd invasion option."))
				civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.value("Special Bulletin #") .. " ******************\n^\r^Finally, the Allied High Command expects you, given all the\n^resources placed at your disposal, to make progress in this campaign.\n^\r^As such, you must capture 14 city objective points by no later then the\n^end of Week IV of June, 1944 (turn #48). Should you be unsuccessful\n^in this endeavor, they will consider your mission a failure and order\n^all future operations in this theather stopped and most of your forces\n^redirected to France.\n^\r^You can keep track of your progress in the Status Report screen\n^(menu #1). Be aware, that in this undertaking, that wonders are\n^included in the objectives point calculations."))
				
				-- Run bat file to switch to US 5th Army badges
				text.simple("At this time, save and exit your game. Then run the BOI_1943.bat file and select option '2. Load 5th Army Graphics'.","US 5th Army Constitued!",object.m5thArmy)		
				
				-- Convert all remaining Italian Reggimento Costiero, not in Corsica/Sardinia, to POW units
				for unit in civ.iterateUnits() do 
					if unit.type == object.uReggimentoCostiero and ( (not gen.inPolygon(unit.location,tables.corsicaPolygon)) and (not gen.inPolygon(unit.location,tables.sardiniaPolygon)) ) then	
						-- gen.createUnit(object.uPrigionieridiguerra, object.pAxis, unit.location, {count=1, randomize=true, veteran=false})
						-- gen.deleteUnit(unit)
						gen.replaceUnit(unit,object.uPrigionieridiguerra)
					end 
				end
				-- civ.ui.text("The " .. object.uReggimentoCostiero.name .. " were disbanded!")				
			end)
		end		
	end
	

	-- ====================================================================
	-- ITALIAN SURRENDER CONDITIONS	
	-- ====================================================================	
	
	-- ONLY FOR ITALIAN SURRENDER TESTING PURPOSES
	-- if turn == 1 then
		-- gen.createUnit(object.uBritishRegular, object.pAllies, {{80,78,0}}, {count=4, randomize=false, veteran=false})
	-- end
	-- if turn == 1 then
		-- gen.createUnit(object.uBritishRegular, object.pAllies, {{69,47,0}}, {count=1, randomize=false, veteran=false})
		-- gen.createUnit(object.uLandingCraft, object.pAllies, {{68,48,0}}, {count=1, randomize=false, veteran=false})
	-- end
	
	-- Italy surrenders if there are 21 or more Allied ground units above the 'Italian Surrender Line' situated south of the cities of Salerno and Foggia
	-- and the Allies have captured at least the equivalent of 2 major cities south of and including Roma (note that both Napoli and Roma are worth 2 points)
	-- If Italy surrenders its air, ground and naval units will be removed from play depending on its SURRENDER LEVEL below and 
	-- you will be required to run the bat file to swap the cities.bmp file which will display the German colors and flag over former italian cities.
		
		local citiesForItalianSurrender = 0
		if object.cBari.owner == object.pAllies then
			citiesForItalianSurrender = citiesForItalianSurrender + 1
			counter.add("# Cities For Italian Surrender",1)
		end
		if object.cBrindisi.owner == object.pAllies then
			citiesForItalianSurrender = citiesForItalianSurrender + 1
			counter.add("# Cities For Italian Surrender",1)
		end
		if object.cFoggia.owner == object.pAllies then
			citiesForItalianSurrender = citiesForItalianSurrender + 1
			counter.add("# Cities For Italian Surrender",1)
		end
		if object.cNapoli.owner == object.pAllies then
			citiesForItalianSurrender = citiesForItalianSurrender + 2
			counter.add("# Cities For Italian Surrender",2)
		end
		if object.cPescara.owner == object.pAllies then
			citiesForItalianSurrender = citiesForItalianSurrender + 1
			counter.add("# Cities For Italian Surrender",1)
		end
		if object.cReggioCalabria.owner == object.pAllies then
			citiesForItalianSurrender = citiesForItalianSurrender + 1
			counter.add("# Cities For Italian Surrender",1)
		end
		if object.cTaranto.owner == object.pAllies then
			citiesForItalianSurrender = citiesForItalianSurrender + 1
			counter.add("# Cities For Italian Surrender",1)
		end
		if object.cROMA.owner == object.pAllies then
			citiesForItalianSurrender = citiesForItalianSurrender + 2
			counter.add("# Cities For Italian Surrender",2)
		end

	if turn >= 1 and flag.value("Sicily is Conquered") == true then
		if invasionUnitsInItalianSurrender() == true and citiesForItalianSurrender >= 2 then
			
			-- civ.ui.text("Italy Surrenders!")
			-- text.simple("Italy Surrenders!","Italy Surrenders!", object.mItalySurrenders)
			-- civ.ui.text("Italy has surrendered! Save your game, run the 'Battle of Italy' batch file, and select option 5, 'Italy surrenders'.")
			
			-- Depending on which turn Italy surrenders there can be more fascist diehards that continue the fight on the German side
			gen.justOnce("Italy Surrender Levels",function()
			
				flag.setTrue("Italy Surrendered")
				-- flag.setFalse("Regia Marina Main Battle Fleet Sorties")
				object.pAxis.leader.name = "Generalfeldmarschall Albert Kesselring"	
				
				-------------------------------
				-- ITALIAN SURRENDER LEVEL ONE: occurs within historical timeframe and just a few diehard fascist (Blakshirts and Paracadutisto) Italian units remain
				-------------------------------
				if (turn >= 1 and turn <= 10) then
					civ.playSound("Cathedrl.wav")
					-- text.simple("Italy Surrenders Level 1!","Italy Surrenders!", object.mItalySurrenders)
					text.setLinesPerWindow(22)
						text.simple("As per the terms of the armistice of Cassibile, all Italian\n^forces loyal to the monarchy lay down their arms, though\n^a few hardline Italian fascist units vow to continue the\n^fight on the German side.\n^\r^According to intercepted radio broadcasts, the Germans\n^move quickly to seize all Italian territory not yet within\n^Allied control and attack and disarm any loyalist forces\n^that attempt to resist them.\n^\r^As per these same intercept, Generalfeldmarschall Wilhelm\n^Keitel, confirms the Germans are able to appropriate a large\n^bounty of surplus Italian weapons, which they will no doubt\n^put to good use.\n^Finally, the Italian commander on the island of Sardinia,\n^Generallisimo Antonio Basso, agrees to surrender. All cities\n^on that island revert to your control and troops from the\n^garrison are placed under your command.\n^\r^The German forces previoulsy on the island evacuate to,\n^Corsica which you will need to liberate if you wish to\n^capture the objective city of Bastia...\n^","Italy Surrenders - Level 1!", object.mItalySurrenders)
						text.simple("With Sardinia under your control, you may transport (aka\n^teleport), at a cost of " .. param.teleportCostSardegna .. " supplies each, any American G.I.,\n^Rangers or French Goumiers units located in Palermo, Trapani\n^or Tunis to Cagliari or Olbia provided they are in your possession.\n^\r^To do so, simply press on the 'shift' + '2' keys simultaneouly\n^when the infantry unit is active and select the appropriate\n^destination.\n^\r^You may, of course, also use your Liberty Ship's to transport\n^any troops to a liberated port city on that island.\n^\r^The island of Corsica itself can only, initially, be invaded\n^by American G.I., Rangers or French Goumiers units, provided\n^they are located in the port city of Olbia.\n^\r^To do so, simply press on the 'shift' and '2' keys simultaneouly\n^when the infantry unit is active and select the 'send unit to\n^Bonifacio' option, which will teleport the unit to the beach tile\n^just northeast of the town, provided there isn't an enemy defender\n^on it.\n^\r^Liberating that town will activate Corsican partisan cells on the\n^island.","Operation Firebrand!",object.mGoumierMarocain)
					text.setLinesPerWindow(12)
					object.pAxis.money = object.pAxis.money + 1280
					
					-- Delete all Level 1, 2 and 3 italian units
					for activeUnit in civ.iterateUnits() do
						if activeUnit.owner == object.pAxis and (tables.italiansToBeDestroyed_Level1[activeUnit.type.id] or tables.italiansToBeDestroyed_Level2[activeUnit.type.id] or tables.italiansToBeDestroyed_Level3[activeUnit.type.id]) then 
							print("Destroyed " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z)
							civ.deleteUnit(activeUnit)
						end
					end
					-- If Regia Marina Main Battle Fleet never sortied then deploy remaining naval vessels in La Spezia for German capture attempt
					if flag.value("Regia Marina Main Battle Fleet Sorties") == false then
						gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{72,12,0}}, {count=9, randomize=false, veteran=false})
						gen.createUnit(object.uLittorioClassBB, object.pAxis, {{72,12,0}}, {count=4, randomize=false, veteran=false})
					end
					
					-- Possibility of Germany capturing Italian naval units = 11% chance (1 of 9)
					local italianNavalUnit = 0
					for activeUnit in civ.iterateUnits() do
						if activeUnit.owner == object.pAxis and tables.italianNavyToBeDestroyed[activeUnit.type.id] then
							local randomNumber = math.random(9)
							if (randomNumber >= 1 and randomNumber <= 8) then
								print("Destroyed " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z .. ", RandomNumber" .. randomNumber)
								civ.deleteUnit(activeUnit)
							else
								-- text.simple("Germans and pro-Fascist Italians managed to seize an\n^Italian " .. activeUnit.type.name .. " naval vessel before it could \n^flee or be scuttled!\n^\r^It will take sometime for the ship to become fully operational\n^again as a new crew complement will need to be recruited!","Italian Ships Seized by Germans",object.monexone)
								civ.teleportUnit(activeUnit, object.lLaSpezia)
								print("Captured " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z .. ", RandomNumber" .. randomNumber)
								-- Inflict damage on vessel to reflect possible loss of non pro-fascist ship's crew
								if activeUnit.type == object.uSellaClassDD then
									activeUnit.damage = activeUnit.damage + 15
									italianNavalUnit = italianNavalUnit + 1
								elseif activeUnit.type == object.uGiusannoClassCL then
									activeUnit.damage = activeUnit.damage + 20
									italianNavalUnit = italianNavalUnit + 1
								elseif  activeUnit.type == object.uLittorioClassBB then
									activeUnit.damage = activeUnit.damage + 25
									italianNavalUnit = italianNavalUnit + 1
								elseif  activeUnit.type == object.uSottomarino then
									activeUnit.damage = activeUnit.damage + 15
									italianNavalUnit = italianNavalUnit + 1
								end
							end
						end
					end
					text.simple("The Germans along with some pro-Fascist Italians managed to seize " .. italianNavalUnit .. " Regia Marina naval vessel(s) before they could flee or be scuttled!","Italian Ships Seized by Germans",object.mRegiaMarina)		
					
					-- SARDINIA SURRENDERS TO ALLIES
					-- All island's cities switch to Allied control
					object.cCagliari.owner = object.pAllies
					object.cSassari.owner = object.pAllies
					object.cOlbia.owner = object.pAllies
					
					-- Place all German/Pro-fascist italians units in Sardinia to Corsica
					gen.createUnit(object.uMotInfanterie, object.pAxis, {{40,48,0},{38,58,0},{53,47,0}}, {count=2, randomize=true, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, {{40,48,0},{38,58,0},{53,47,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{40,48,0},{38,58,0},{53,47,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uParacadutisto, object.pAxis, {{40,48,0},{38,58,0},{53,47,0}}, {count=1, randomize=true, veteran=false})

					text.simple("AXIS MESSAGE:\n^\r^5 German units are activated in Corsica","Total Axis Units Report!",object.m10thArmy)

					-- SEE SECTION BELOW: 'Once ALL 3 Sardinian cities are in Allied control, teleport any remaining German units on island to Corsica'
					
					-- Deploy 6 newly recruited Allied-Italian garrison units in Sardinia
					local a = 0
					while( a < 6 )	do
						local placementLocations = {{14,90,0},{22,64,0},{36,68,0},{25,85,0},{6,86,0},{13,75,0}}
						for key,tileCoord in pairs(placementLocations) do
							if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uReggimentoCostiero) then
								placementLocations[key] = nil
							end
						end
						gen.makeArrayOneToN(placementLocations)
						local unit = gen.createUnit(object.uReggimentoCostiero, object.pAllies,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
						if unit then -- if the unit was created, it will be a Divisione Costiera, but we have to check the unit was actually created, and is not nil
							gen.placeFortress(unit.location)
							unit.moveSpent = 255
						end
						-- civ.ui.text("Created Divisione Costiera")
						a = a + 1
					end

				-------------------------------	
				-- ITALIAN SURRENDER LEVEL TWO: occurs up to three weeks past historical timeframe and a few more diehard fascist Italian units remain
				-------------------------------
				elseif (turn >= 11 and turn <= 12) then
					civ.playSound("Cathedrl.wav")
					-- text.simple("Italy Surrenders Level 2!","Italy Surrenders!", object.mItalySurrenders)	
					text.setLinesPerWindow(22)
						text.simple("As per the terms of the armistice of Cassibile, all Italian\n^forces loyal to the monarchy lay down their arms, though\n^more than a few hardline Italian fascist units vow to continue\n^the fight on the German side.\n^\r^According to intercepted radio broadcasts, the Germans\n^move quickly to seize all Italian territory not yet within\n^Allied control and attack and disarm any loyalist forces that\n^attempt to resist them.\n^\r^As per these same intercept, Generalfeldmarschall Wilhelm\n^Keitel, confirms the Germans are able to appropriate a large\n^bounty of surplus Italian weapons, which they will no doubt\n^put to good use.\n^Finally, the Italian commander on the island of Sardinia,\n^Generallisimo Antonio Basso, agrees to revert the cities\n^of Cagliari and Sassari and the garrison troops within to\n^your control.\n^\r^On the other hand, the German forces on the island still\n^control Ostia and you will need to capture it to get them\n^to evacuate to Corsica, which you will need to liberate if\n^you wish to capture the objective city of Bastia...\n^","Italy Surrenders - Level 2!", object.mItalySurrenders)
						text.simple("With southern Sardinia under your control, you may transport\n^(aka teleport), at a cost of " .. param.teleportCostSardegna .. " supplies each, any American G.I.,\n^Rangers or French Goumiers units located in Palermo, Trapani\n^or Tunis to Cagliari or Olbia provided they are in your possession.\n^\r^To do so, simply press on the 'shift' + '2' keys simultaneouly\n^when the infantry unit is active and select the appropriate\n^destination.\n^\r^You may, of course, also use your Liberty Ship's to transport\n^any troops to a liberated port city on that island.\n^\r^The island of Corsica itself can only, initially, be invaded\n^by American G.I., Rangers or French Goumiers units, provided\n^they are located in the port city of Olbia.\n^To do so, simply press on the 'shift' + '2' keys simultaneouly\n^when the infantry unit is active and select the 'send unit to\n^Bonifacio' option, which will teleport the unit to the beach tile\n^just northeast of the town, provided there isn't an enemy\n^defender on it.\n^\r^Liberating that town will activate Corsican partisan cells on the\n^island.","Operations Brimstone and Firebrand!",object.mGoumierMarocain)					
					text.setLinesPerWindow(12)
					object.pAxis.money = object.pAxis.money + 1640
					
					-- If Regia Marina Main Battle Fleet never sortied then deploy remaining naval vessels in La Spezia for German capture attempt
					if flag.value("Regia Marina Main Battle Fleet Sorties") == false then
						gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{77,23,0}}, {count=9, randomize=false, veteran=false})
						gen.createUnit(object.uLittorioClassBB, object.pAxis, {{77,23,0}}, {count=4, randomize=false, veteran=false})
						print("Italian CL and BB added to LaSpezia")
					end						
					-- Delete all Level 1 and 3 italian units
					for activeUnit in civ.iterateUnits() do	
						if activeUnit.owner == object.pAxis and (tables.italiansToBeDestroyed_Level1[activeUnit.type.id] or tables.italiansToBeDestroyed_Level3[activeUnit.type.id]) then 
							print("Destroyed " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z)
							civ.deleteUnit(activeUnit)
						end
					end
				
					-- Possibility of Germany capturing Italian naval units = 22% chance (2 of 9)
					local italianNavalUnit = 0
					for activeUnit in civ.iterateUnits() do
						if activeUnit.owner == object.pAxis and tables.italianNavyToBeDestroyed[activeUnit.type.id] then
							local randomNumber = math.random(9)
							if (randomNumber >= 1 and randomNumber <= 7) then
								print("Destroyed " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z .. ", RandomNumber" .. randomNumber)
								civ.deleteUnit(activeUnit)
							else
								-- text.simple("Germans and pro-Fascist Italians managed to seize an\n^Italian " .. activeUnit.type.name .. " naval vessel before it could \n^flee or be scuttled!\n^\r^It will take sometime for the ship to become fully operational\n^again as a new crew complement will need to be recruited!","Italian Ships Seized by Germans",object.monexone)
								civ.teleportUnit(activeUnit, object.lLaSpezia)
								-- Inflict damage on vessel to reflect possible loss of non pro-fascist ship's crew
								if activeUnit.type == object.uSellaClassDD then
									activeUnit.damage = activeUnit.damage + 10
									italianNavalUnit = italianNavalUnit + 1
								elseif activeUnit.type == object.uGiusannoClassCL then
									activeUnit.damage = activeUnit.damage + 15
									italianNavalUnit = italianNavalUnit + 1
								elseif  activeUnit.type == object.uLittorioClassBB then
									activeUnit.damage = activeUnit.damage + 20
									italianNavalUnit = italianNavalUnit + 1
								elseif  activeUnit.type == object.uSottomarino then
									activeUnit.damage = activeUnit.damage + 10
									italianNavalUnit = italianNavalUnit + 1
								end
							end
						end
					end
					text.simple("The Germans along with some pro-Fascist Italians managed to seize " .. italianNavalUnit .. " Regia Marina naval vessel(s) before they could flee or be scuttled!","Italian Ships Seized by Germans",object.mRegiaMarina)		

					-- SARDINIA SURRENDERS TO ALLIES
					-- Two of island's cities revert to Allied control
					object.cCagliari.owner = object.pAllies
					object.cSassari.owner = object.pAllies
					-- object.cOlbia.owner = object.pAllies
					
					-- Place half German/Pro-fascist italians units in Sardinia and other half in Corsica
					-- Corsica
					gen.createUnit(object.uMotInfanterie, object.pAxis, {{40,48,0},{38,58,0},{53,47,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{40,48,0},{38,58,0},{53,47,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uParacadutisto, object.pAxis, {{40,48,0},{38,58,0},{53,47,0}}, {count=1, randomize=true, veteran=false})
					-- Sardinia
					gen.createUnit(object.uMotInfanterie, object.pAxis, {{36,68,0}}, {count=1, randomize=false, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, {{36,68,0}}, {count=1, randomize=false, veteran=false})
	
					text.simple("AXIS MESSAGE:\n^\r^3 German units are activated in Corsica and 2 in Sardinia","Total Axis Units Report!",object.m10thArmy)
	
					-- SEE SECTION BELOW: 'Once ALL 3 Sardinian cities are in Allied control, teleport any remaining German units on island to Corsica'
					
					-- Deploy 5 newly recruited Allied-Italian garrison units in Sardinian cities
					local a = 0
					while( a < 5 )	do
						local placementLocations = {{14,90,0},{22,64,0},{25,85,0},{6,86,0},{13,75,0}}
						for key,tileCoord in pairs(placementLocations) do
							if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uReggimentoCostiero) then
								placementLocations[key] = nil
							end
						end
						gen.makeArrayOneToN(placementLocations)
						local unit = gen.createUnit(object.uReggimentoCostiero, object.pAllies,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
						if unit then -- if the unit was created, it will be a Divisione Costiera, but we have to check the unit was actually created, and is not nil
							gen.placeFortress(unit.location)
							unit.moveSpent = 255
						end
						-- civ.ui.text("Created Divisione Costiera")
						a = a + 1
					end
					
				-------------------------------
				-- ITALIAN SURRENDER LEVEL THREE: occurs over four weeks past historical timeframe and numerous diehard fascist Italian units remain
				-------------------------------
				elseif turn >= 13 then
					civ.playSound("Cathedrl.wav")
					-- text.simple("Italy Surrenders Level 3!","Italy Surrenders!", object.mItalySurrenders)
					text.setLinesPerWindow(22)
						text.simple("As per the terms of the armistice of Cassibile, all Italian\n^forces loyal to the monarchy lay down their arms, though\n^many hardline Italian fascist units vow to continue the\n^fight on the German side.\n^\r^According to intercepted radio broadcasts, the Germans\n^move quickly to seize all Italian territory not yet within\n^Allied control and attack and disarm any loyalist forces\n^that attempt to resist them.\n^\r^As per these same intercept, Generalfeldmarschall Wilhelm\n^Keitel, confirms the Germans are able to appropriate a large\n^bounty of surplus Italian weapons, which they will no doubt\n^put to good use.\n^Finally, the Italian commander on the island of Sardinia,\n^Generallisimo Antonio Basso, agrees to surrender the city\n^of Cagliari and the garrisons still under his control to your\n^command.\n^\r^On the other hand, the German forces on the island still\n^control Ostia and Sassari and you will need to capture these\n^to get them to evacuate to Corsica, which you will need to\n^liberate if you wish to capture the objective city of Bastia...\n^","Italy Surrenders - Level 3!", object.mItalySurrenders)
						text.simple("With southern Sardinia under your control, you may transport\n^(aka teleport), at a cost of " .. param.teleportCostSardegna .. " supplies each, any American G.I.,\n^Rangers or French Goumiers units located in Palermo, Trapani\n^or Tunis to Cagliari or Olbia provided they are in your possession.\n^\r^To do so, simply press on the 'shift' and '2' keys simultaneouly\n^when the infantry unit is active and select the appropriate\n^destination.\n^\r^You may, of course, also use your Liberty Ship's to transport\n^any troops to a liberated port city on that island.\n^\r^The island of Corsica itself can only, initially, be invaded\n^by American G.I., Rangers or French Goumiers units, provided\n^they are located in the port city of Olbia.\n^To do so, simply press on the 'shift' and '2' keys simultaneouly\n^when the infantry unit is active and select the 'send unit to\n^Bonifacio' option, which will teleport the unit to the beach tile\n^just northeast of the town, provided there isn't an enemy\n^defender on it.\n^\r^Liberating that town will activate Corsican partisan cells on the\n^island.","Operations Brimstone and Firebrand!",object.mGoumierMarocain)
					text.setLinesPerWindow(12)
					object.pAxis.money = object.pAxis.money + 2000
					
				-- Delete all Level 1 italian units
					for activeUnit in civ.iterateUnits() do	
						if activeUnit.owner == object.pAxis and (tables.italiansToBeDestroyed_Level1[activeUnit.type.id]) then 
							print("Destroyed " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z)
							civ.deleteUnit(activeUnit)
						end
					end
					-- If Regia Marina Main Battle Fleet never sortied then deploy remaining naval vessels in La Spezia for German capture attempt
					if flag.value("Regia Marina Main Battle Fleet Sorties") == false then
						gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{72,12,0}}, {count=9, randomize=false, veteran=false})
						gen.createUnit(object.uLittorioClassBB, object.pAxis, {{72,12,0}}, {count=4, randomize=false, veteran=false})
					end	
					
					-- Possibility of Germany capturing Italian naval units = 33% chance (3 of 9)
					local italianNavalUnit = 0
					for activeUnit in civ.iterateUnits() do
						if activeUnit.owner == object.pAxis and tables.italianNavyToBeDestroyed[activeUnit.type.id] then
							local randomNumber = math.random(9)
							if (randomNumber >= 1 and randomNumber <= 6) then
								print("Destroyed " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z .. ", RandomNumber" .. randomNumber)
								civ.deleteUnit(activeUnit)
							else
								-- text.simple("Germans and pro-Fascist Italians managed to seize an\n^Italian " .. activeUnit.type.name .. " naval vessel before it could \n^flee or be scuttled!\n^\r^It will take sometime for the ship to become fully operational\n^again as a new crew complement will need to be recruited!","Italian Ships Seized by Germans",object.monexone)
								civ.teleportUnit(activeUnit, object.lLaSpezia)
								print("Captured " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z .. ", RandomNumber" .. randomNumber)
								-- Inflict damage on vessel to reflect possible loss of non pro-fascist ship's crew
								if activeUnit.type == object.uSellaClassDD then
									activeUnit.damage = activeUnit.damage + 5
									italianNavalUnit = italianNavalUnit + 1
								elseif activeUnit.type == object.uGiusannoClassCL then
									activeUnit.damage = activeUnit.damage + 10
									italianNavalUnit = italianNavalUnit + 1
								elseif  activeUnit.type == object.uLittorioClassBB then
									activeUnit.damage = activeUnit.damage + 15
									italianNavalUnit = italianNavalUnit + 1
								elseif  activeUnit.type == object.uSottomarino then
									activeUnit.damage = activeUnit.damage + 5
									italianNavalUnit = italianNavalUnit + 1
								end
							end
						end
					end	
					text.simple("The Germans along with some pro-Fascist Italians managed to seize " .. italianNavalUnit .. " Regia Marina naval vessel(s) before they could flee or be scuttled!","Italian Ships Seized by Germans",object.mRegiaMarina)		
					
					-- SARDINIA SURRENDERS TO ALLIES
					-- Only one of island's cities revert to Allied control
					object.cCagliari.owner = object.pAllies
					-- object.cSassari.owner = object.pAllies
					-- object.cOlbia.owner = object.pAllies
					
					-- Place majority German/Pro-fascist italians units in Sardinia and rest in Corsica
					-- Corsica
					gen.createUnit(object.uParacadutisto, object.pAxis, {{53,47,0}}, {count=1, randomize=true, veteran=false})
					-- Sardinia
					gen.createUnit(object.uMotInfanterie, object.pAxis, {{22,64,0},{36,68,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, {{22,64,0},{36,68,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{22,64,0},{36,68,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uMotInfanterie, object.pAxis, {{22,64,0},{36,68,0}}, {count=1, randomize=true, veteran=false})

					text.simple("AXIS MESSAGE:\n^\r^1 German unit is activated in Corsica and 4 in Sardinia","Total Axis Units Report!",object.m10thArmy)
					
					-- SEE SECTION BELOW: 'Once ALL 3 Sardinian cities are in Allied control, teleport any remaining German units on island to Corsica'
					
					-- Deploy 4 newly recruited Allied-Italian garrison units in Sardinian cities
					local a = 0
					while( a < 4 )	do
						local placementLocations = {{14,90,0},{25,85,0},{6,86,0},{13,75,0}}
						for key,tileCoord in pairs(placementLocations) do
							if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uReggimentoCostiero) then
								placementLocations[key] = nil
							end
						end
						gen.makeArrayOneToN(placementLocations)
						local unit = gen.createUnit(object.uReggimentoCostiero, object.pAllies,placementLocations , {count=1, randomize=true, scatter=true, veteran=true})[1]
						if unit then -- if the unit was created, it will be a Divisione Costiera, but we have to check the unit was actually created, and is not nil
							gen.placeFortress(unit.location)
							unit.moveSpent = 255
						end
						-- civ.ui.text("Created Divisione Costiera")
						a = a + 1
					end					
					
				end
				-- END LEVEL THREE SURRENDER SECTION ==================
				
				-- Run bat file to switch to to German city Graphics
				text.simple("At this time, save and exit your game. Then run the BOI_1943.bat file and select option '3. Load German Cities Graphics'.","Germany Takes Control!",object.m10thArmy)		

			end)
		end	
	end
	
	-- Once ALL 3 Sardinian cities are in Allied control, teleport any remaining German units on island to Corsica 
	if turn >= 1 then
		if object.cCagliari.owner == object.pAllies 
		and object.cOlbia.owner == object.pAllies
		and object.cSassari.owner == object.pAllies then
			-- Teleport all German units in Sardinia to Corsica
			gen.justOnce("Axis abandons Sardinia",function()
				for unit in civ.iterateUnits() do
					if unit.owner == object.pAxis and object.ltownBonifacio.owner == object.pAxis and gen.inPolygon(unit.location,tables.sardiniaPolygon) then 
						-- Inflict damage to reflect chaotic nature of evacuation from Sardinia
						if unit and math.random(1,1) == 1  then
							unit.damage = unit.damage + 5
						end	
						print("Teleported " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
						civ.teleportUnit(unit, object.ltownBonifacio)
						-- civ.ui.text("Axis forces move to Bonifacio!")
					elseif unit.owner == object.pAxis and object.lAjaccio.owner == object.pAxis and gen.inPolygon(unit.location,tables.sardiniaPolygon) then
						-- Inflict damage to reflect chaotic nature of evacuation from Sardinia
						if unit and math.random(1,1) == 1  then
							unit.damage = unit.damage + 5
						end	
						print("Teleported " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
						civ.teleportUnit(unit, object.lAjaccio)
						-- civ.ui.text("Axis forces move to Ajaccio!")
					end
				end
				text.simple("Since the Allies control all major Sardinian cities, the remaining Axis forces on the island flee to Corsica!","Sardinia Conquered!",object.monexone)
				
			end)		
		end	
	
	end	
	

	-- ====================================================================
	-- GERMAN 10TH ARMY CUTOFF CONDITIONS	
	-- ====================================================================	
	if turn >= 9 and flag.value("2nd Invasion Available") == true 
	and flag.value("Gustav Line is breached") == false
	-- and flag.value("German 10th Army Escaped") == false
	and	(object.ltownAscoli.owner == object.pAllies and
		object.ltownRieti.owner == object.pAllies and
		object.ltownSanBenedetto.owner == object.pAllies and
		object.ltownTivoli.owner == object.pAllies and
		object.ltownValmontone.owner == object.pAllies)
	then
		gen.justOnce("Allies Cutoff German 10th Army",function()
			for unit in civ.iterateUnits() do 
				if unit.owner == object.pAxis and tables.germanFortifiedStatus[unit.type.id] and (gen.inPolygon(unit.location,tables.gustavLinePolygon)) then	
					civ.deleteUnit(unit)
				end 
			end
			for unit in civ.iterateUnits() do 
				if unit.owner == object.pAxis and unit.type == object.uTypeIXC and unit.location == object.lPescara then	
					civ.teleportUnit(unit,object.lRimini)
				end 
			end
			for unit in civ.iterateUnits() do 
				if unit.owner == object.pAxis and (not tables.germanFortifiedStatus[unit.type.id]) and unit.location == object.lPescara then	
					civ.teleportUnit(unit,object.lFirenze)
				end 
			end
			
			civ.playSound("Cathedrl.wav")
			text.simple("The Allies have successfully cutoff the supply route of\n^the German 10th Army located along the Gustav Line and\n^as such most units caught in the trap are either forced\n^to surrender or find an escape route as best they can.\n^\r^The Gustav Line is now breached and besides some\n^mopping up operations the Allies can move on to their next\n^objective, the conquest of the Po Valley!","German Tenth Army Cutoff!",object.m10thArmy)
			flag.setTrue("Gustav Line is breached") 
			flag.setTrue("German 10th Army Cutoff") 
		end)
	end
	
	-- If control Fiumicino	and Roma can force surrender of Axis foces in Lido di Roma
	if turn >= 9 and object.ltownFiumicino.owner == object.pAllies and object.ltownLidoDiRoma.owner ~= object.pAllies and flag.value("Roma is captured") == true then
		gen.justOnce("Capture of Fiumicino",function()
			-- Delete all German units in town of Lido di Roma
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAxis and unit.type.domain == 0  and unit.location == object.ltownLidoDiRoma then 
					print("Destroyed " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
					civ.deleteUnit(unit)
				end
			end

			civ.playSound("Cheers2.wav")
			text.simple("The Germans troops located in Lido di Roma are cut off and forced to surrender!","Lido di Roma surrenders!",object.monexone)
		end)		
	end
	
	-- ====================================================================
	-- GUSTAV LINE BREACHED CONDITIONS	
	-- ====================================================================	
	if turn >= 9 and flag.value("2nd Invasion Available") == true 
	and flag.value("Gustav Line is breached") == false
	then
		local gustavLineIsBreached = 0
		if object.ltownCassino.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end
		if object.lfortNE_Isernia.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end
		if object.lfortNNE_Isernia.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end
		if object.lfortNE_Formia.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end
		if object.lfortNW_Cassino.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end
		if object.lfortNW_Ortona.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end
		if object.lPescara.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end		

		if gustavLineIsBreached >= 4 and flag.value("German 10th Army Cutoff") == false then
			gen.justOnce("Allies Breach Gustav Line",function()
				for unit in civ.iterateUnits() do 
					if unit.owner == object.pAxis and tables.germanFortifiedStatus[unit.type.id] and (gen.inPolygon(unit.location,tables.gustavLinePolygon)) then	
						civ.deleteUnit(unit)
					end 
				end
				for unit in civ.iterateUnits() do 
					if unit.owner == object.pAxis and unit.type == object.uTypeIXC and unit.location == object.lPescara then	
						civ.teleportUnit(unit,object.lRimini)
					end 
				end
				for unit in civ.iterateUnits() do 
					if unit.owner == object.pAxis and (not tables.germanFortifiedStatus[unit.type.id]) and unit.location == object.lPescara then	
						civ.teleportUnit(unit,object.lRimini)
					end 
				end	

				civ.playSound("Cathedrl.wav")
				text.simple("The Allies have successfully breached the Gustav Line and\n^therefore Generalfeldmarschall Kesselring orders the German\n^10th Army to abandon its remaining defensive positions and\n^leave a rearguard so that the bulk of his troops can begin\n^an orderly retreat northward.\n^\r^You order your troops to begin mopping any nests of resistance\n^and to advance as quickly as possible into central Italy.\n^\r^All the same, the Allied High Command is very pleased\n^with your progress and therefore allocates an extra " .. param.gustavLineBreached .. "\n^supplies to your theater.","Gustav Defensive Line Breached!",object.m10thArmy)
				object.pAllies.money = object.pAllies.money + param.gustavLineBreached
				flag.setTrue("German 10th Army Escaped")
				flag.setTrue("Gustav Line is breached")
			end)
		end
	end
	
	-- ====================================================================
	-- CUTOFF TALAMONE TROOPS CONDITIONS	
	-- ====================================================================	
	if turn >= 9 and flag.value("2nd Invasion Available") == true 
	and	(object.ltownBolsena.owner == object.pAllies and
		object.ltownCecina.owner == object.pAllies and
		object.ltownCivitavecchia.owner == object.pAllies and
		object.ltownMontalcino.owner == object.pAllies and
		object.ltownSiena.owner == object.pAllies and
		object.ltownViterbo.owner == object.pAllies)
	then
		gen.justOnce("Talamone Pocket Surrenders",function()
			-- Delete all German fort type units in pocket around town of Talamone
			for unit in civ.iterateUnits() do 
				if unit.owner == object.pAxis and tables.germanFortifiedStatus[unit.type.id] and (gen.inPolygon(unit.location,tables.talamonePocketpolygon)) then	
					civ.deleteUnit(unit)
				end 
			end
	
			civ.playSound("Cheers2.wav")
			text.simple("The Germans static troops located in the pocket around the town of Talamone are cut off and forced to surrender!","Talamone Pocket Surrenders!",object.monexone)
		end)	
	end
	
	-- ====================================================================
	-- GOTHIC LINE BREACHED CONDITIONS	
	-- ====================================================================	
	if turn >= 9 and flag.value("2nd Invasion Available") == true 
	and flag.value("Gothic Line is breached") == false
	then
		local gothicLineIsBreached = 0
		if object.cBologna.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end
		if object.cForli.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end
		if object.cLaSpezia.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end
		if object.cModena.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end
		if object.cParma.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end
		if object.cRavenna.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end
		if object.cRimini.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end	

		if gothicLineIsBreached >= 6 then
			gen.justOnce("Allies Breach Gothic Line",function()
				for unit in civ.iterateUnits() do 
					if unit.owner == object.pAxis and (tables.germanFortifiedStatus[unit.type.id] and unit.type ~= object.uBunker) and (gen.inPolygon(unit.location,tables.gothicLineTotalPolygon)) then	
						civ.deleteUnit(unit)
					end 
				end

				-- Inflict damage on bunkers to reflect weakened state
				for unit in civ.iterateUnits() do
					if unit.owner == object.pAxis and unit.type == object.uBunker and gen.inPolygon(unit.location,tables.gothicLineTotalPolygon) then 
						if math.random(1,2) == 2 then
							print("Destroyed " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
							civ.deleteUnit(unit)
						else
							unit.damage = unit.damage + 24
						end
					end
				end

				civ.playSound("Cathedrl.wav")
				text.simple("The Allies have successfully breached the Gothic Line!\n^\r^Generalfeldmarschall Kesselring orders those few units\n^that can to conduct rearguard actions in the hope that\n^some units might still escape to fight another day but\n^the bulk of the 10th Army caught in the  pocket have no\n^other option but to surrender!\n^\r^You order your troops to begin mopping any nests of\n^resistance and to advance as quickly as possible into\n^the Po Valley.\n^\r^The Allied High Command is very pleased as the end\n^of this bitter campaign is finally in sight!","Gothic Defensive Line Breached!",object.m10thArmy)

				flag.setTrue("Gothic Line is breached")
			end)
		end
	end	

	-- ===========================================================	
	-- GAME ENDS ON TURN 81 (RATHER THAN 89) IF GEN. PATTON ISN'T DISMISSED BY EISENHOWER (SEE Slapping Incident above)
	if turn == 81 and flag.value("Game ends March I, 1945") == true then
		civ.ui.text("Game ends!")
		civ.endGame(true)
	end

	-- ===========================================================		
	-- ALLIED AUTOMATIC VICTORY 
	-- If the Allies control all 10 primary objective cities they win an automatic victory and the game ends

	-- ALLIED SUDDEN DEATH LOSS 
	-- If the Allies FAIL to control at least x primary objective cities by turns 48 and 62, they lose and the game ends
	-- ===========================================================	
	if flag.value("Allied automatic victory") == false then
		local conditionsAlliedVictory = 0
		local conditionsAlliedSuddenDeath = 0
		local conditionsAlliedSuddenDeath_GoL = 0
		if object.cROMA.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 4
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 4
		end
		if object.cBari.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cBastia.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cBologna.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 2
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 2
		end
		if object.cFirenze.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cFoggia.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cGenova.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cLivorno.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cMilano.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 2
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 2
		end
		if object.cNapoli.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 2
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 2
		end
		if object.cPalermo.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cPescara.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cReggioCalabria.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cRimini.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cVerona.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cTurino.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 2
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 2
		end	
		if object.cTrieste.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 3
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 3
		end
		if object.cValetta.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 2
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 2
		end

		-- For 2nd Sudden Death Conditions along Gothic Line
		if object.ltownLucca.owner == object.pAllies then
			conditionsAlliedSuddenDeath_GoL = conditionsAlliedSuddenDeath_GoL + 1
		end
		if object.ltownMassa.owner == object.pAllies then
			conditionsAlliedSuddenDeath_GoL = conditionsAlliedSuddenDeath_GoL + 1
		end
		if object.ltownSanMarino.owner == object.pAllies then
			conditionsAlliedSuddenDeath_GoL = conditionsAlliedSuddenDeath_GoL + 1
		end
		if object.ltownPistoia.owner == object.pAllies then
			conditionsAlliedSuddenDeath_GoL = conditionsAlliedSuddenDeath_GoL + 1
		end
		if object.ltownLeccio.owner == object.pAllies then
			conditionsAlliedSuddenDeath_GoL = conditionsAlliedSuddenDeath_GoL + 1
		end		


		-- 1944 NEW YEAR MESSAGE
		if turn == 25 then -- Jan I, 1944 (#25)
			civ.playSound("Cathedrl.wav")
			if conditionsAlliedSuddenDeath >= 14 then
				civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^It is now, January 1st of the New Year 1944 and the campaign\n^is entering its 7th month.\n^\r^You currently control ".. conditionsAlliedSuddenDeath .." objective city points of the required\n^14 needed by the week of June IV, 1944 and thus have\n^already achieved your first milestone as set by the\n^Allied High Command, which remind you of the importance\n^of keeping the momentum and pressure on the enemy.\n^\r^Finally, if you haven't already done so, they advise you that\n^the final date limit for launching the 3rd invasion is week\n^IV of January 1944 (#28), after which all remaining Landing\n^Craft in the Mediterranean are set to leave for England.\n^\r^They wish you and your troops continued success and a\n^Happy New Year!"))
			else
				civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^It is now, January 1st of the New Year 1944 and the campaign\n^is entering its 7th month.\n^\r^You currently control ".. conditionsAlliedSuddenDeath .." objective city points of the required\n^14 needed by the week of June IV, 1944 to remain in the battle,\n^and thus the Allied High Command reminds you of the importance\n^of keeping the momentum and pressure on the enemy.\n^\r^Finally, if you haven't already done so, they advise you that\n^the final date limit for launching the 3rd invasion is week\n^IV of January 1944 (#28), after which all remaining Landing\n^Craft in the Mediterranean are set to leave for England.\n^\r^They wish you and your troops much success and a Happy\n^New Year!"))
			end
		end

		-- 1945 NEW YEAR MESSAGE
		if turn == 73 then -- Jan I, 1945 (#73)
			civ.playSound("Cathedrl.wav")
			if conditionsAlliedSuddenDeath < 28 then
				civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^It is now, January 1st of the New Year 1945 and the campaign\n^is entering its 19th month.\n^\r^You currently control ".. conditionsAlliedVictory .." objective city points of the required\n^28 needed to achieve a decisive victory before the end\n^of the campaign.\n^\r^If you haven't already done so, they remind you of the\n^importance of breaching and isolating the Gothic line\n^defenses as soon as possible to be able to capture the\n^final objective cities that lie just beyond.\n^\r^They wish you and your troops continued success and a\n^Happy New Year!"))
			end
		end

		-- Sudden Death Conditions
			-- FIRST SD MILESTONE TO ACHIEVE BY END TURN 49
			-- The Allied player requires 14 objectives ...
			if turn <= 49 and conditionsAlliedSuddenDeath > 13 then
				gen.justOnce("Allies complete first milestone",function()
					civ.playSound("Cheers2.wav")
					print("Conditions met for automatic Allied victory: " .. conditionsAlliedSuddenDeath .. " out of 8 possible")
					civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Under your leadership, you have successfully met the first major milestone\n^as set out by the Allied Command, i.e. secured a sufficient number of Axis\n^controlled cities to warrant the continuation of your Italian campaign.\n^\r^Unfortunately, you cannot wrest upon your laurels as it is imperative\n^that you maintain the pressure on the German defenders so that they may\n^not divert resources to go aid their beleaguered forces elsewhere.\n^\r^As such, the High Command has issued a second milestone for you,\n^control a total of 19 objective points before the end of the week of\n^October II, 1944 (#62).\n^\r^In this instance, capturing the small towns of Leccio, Lucca, Massa,\n^Pistoia and San Marino (whose names are in 'green') within the Gothic\n^Line defensive zone will also each count as an objective point, in addition\n^to any city objective point you may control."))	
					flag.setTrue("Allies completed first SD milestone")
				end)	
			end
			-- Otherwise if failed to capture 14 objectives by beginning of turn 49 end the game
			if turn == 50 and flag.value("Allies completed first SD milestone") == false then
				civ.playSound("defeat.wav")
				print("Conditions met for automatic Allied victory: " .. conditionsAlliedSuddenDeath .. " out of 8 possible")
				civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Unfortunately under your leadership, Allied Forces have been unable\n^to significantly pierce the Axis defensive barriers that would have\n^have allowed you to gain victory in this Italian campaign.\n^\r^In effect, the Allied High Command feels that an unbreakable stalemate\n^has taken hold and that allocating further resources to this theater would\n^prove unfruitfull and therefore have ordered your forces to assume a defensive\n^posture moving forward.\n^\r^The major offensive operations in the Mediterranean have come to a close\n^and the ultimate surrender of Germany and the end of the war in Europe\n^will have to be achieved through the campaign in France."))
				civ.endGame(true)
			end

			-- SECOND SD MILESTONE TO ACHIEVE BY END OF TURN 62
			-- The Allied player requires 19 objectives (including any of the 5 Gothic Line 'green' towns) ...
			if (turn >= 50 and turn <= 62) and (conditionsAlliedSuddenDeath + conditionsAlliedSuddenDeath_GoL) > 18 then
			-- if flag.value("Allies completed first SD milestone") == true and flag.value("Allies completed first second milestone") == false and conditionsAlliedSuddenDeath > 18 then
				gen.justOnce("Allies complete second milestone",function()
					civ.playSound("Cheers2.wav")
					print("Conditions met for automatic Allied victory: " .. conditionsAlliedSuddenDeath .. " out of 8 possible")
					civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Under your leadership, you have successfully met the second major\n^milestone as set out by the Allied Command, i.e. secured a sufficient\n^number of Axis objective cities and Gothic Line positions to warrant the\n^continuation of your operations in Italy.\n^\r^Once again, you cannot wrest upon your laurels as it is imperative\n^that you maintain the pressure on the German defenders so that they\n^may not divert resources to go aid their beleaguered forces in France.\n^\r^As such, the High Command has issued a final milestone for you,\n^control 28 CITY objective points to achieve a decisive victory\n^before this campaign comes to an end.\n^\r^In the west, you may gain control of the major objective cities\n^of Milano and Turino by capturing their respective road junctions.\n^In the east, you must seize Trieste before Marshall Tito's communist\n^forces do so. Good luck!"))	
					flag.setTrue("Allies completed second SD milestone")
				end)
			end
			-- Otherwise if failed to capture 19 objectives by beginning of turn 63 end the game
			if turn == 63 and flag.value("Allies completed second SD milestone") == false then
				civ.playSound("defeat.wav")
				print("Conditions met for automatic Allied victory: " .. conditionsAlliedSuddenDeath .. " out of 11 possible")
				civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Unfortunately under your leadership, Allied Forces have been unable\n^to significantly pierce the Axis defensive barriers that would have\n^have allowed you to gain victory in this Italian campaign.\n^\r^In effect, the Allied High Command feels that an unbreakable stalemate\n^has taken hold and that allocating further resources to this theater would\n^prove unfruitfull and therefore have ordered your forces to assume a defensive\n^posture moving forward.\n^\r^The major offensive operations in the Mediterranean have come to a close\n^and the ultimate surrender of Germany and the end of the war in Europe\n^will have to be achieved through the campaign in France."))
				civ.endGame(true)
			end	


		-- Automatic Victory Conditions
		if conditionsAlliedVictory >= 28 then
			civ.playSound("Cathedrl.wav")
			print("Conditions met for automatic Allied victory: " .. conditionsAlliedVictory .. " out of 15 possible")
				-- If FDR is still alive
				if turn <= 86 then
					civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Through your remarkable leadership, your forces have captured the\n^last significant Fascist bastion that stood in your way to ultimate\n^victory in this Italian campaign.\n^\r^As such, the theater German commander has sent a delegation to\n^negotiate an armistice in view of bringing the conflict to an end in the\n^Italian theater.\n^\r^The ultimate surrender of Germany and the end of the war in Europe\n^and the return of peace is finally in sight.\n^\r^The British Prime Minister, Winston Churchill, and American President,\n^Franklin Roosevelt, send you their heartfelt thanks for your oustanding\n^services to the Allied cause!"))
				-- If Harry Truman is President
				else
					civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. commander15thArmy_Base .. ", (15th Army Group)\n^\r^Through your remarkable leadership, your forces have captured the\n^last significant Fascist bastion that stood in your way to ultimate\n^victory in this Italian campaign.\n^\r^As such, the theater German commander has sent a delegation to\n^negotiate an armistice in view of bringing the conflict to an end in the\n^Italian theater.\n^\r^The ultimate surrender of Germany and the end of the war in Europe\n^and the return of peace is finally in sight.\n^\r^The British Prime Minister, Winston Churchill, and American President,\n^Harry Truman, send you their heartfelt thanks for your oustanding\n^services to the Allied cause!"))
				end
			civ.endGame(true)
		end
	end	

	-- ===========================================================	
	-- RUN BAT FILES 
	-- ===========================================================
	-- IF ITALY SURRENDERS
	-- See ITALIAN SURRENDER CONDITIONS ABOVE

	
	
end -- onTurn.onTurn(turn) ============================================
---&endAutoDoc

-- ============================================
-- DELAYED ACTIONS SECTION
-- ============================================

--DELAYED 3RD INVASION REACTION FORCES 
---------------------
-- OPTION I SCENARIO: If Allies proceed with 3rd Amphibious invasion
---------------------
-- MASSA REACTION FORCE
-- 2nd turn after invasion
local function unitsMassaT2_Deployed(argTable)
	gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{77,23,0},{82,32,0},{81,21,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGebirgsjager, object.pAxis, {{77,23,0},{82,32,0},{81,21,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{77,23,0},{82,32,0},{81,21,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uPanzerGrenadier, object.pAxis, {{77,23,0},{82,32,0},{81,21,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uKruppK5, object.pAxis, {{77,23,0},{82,30,0},{75,35,0}}, {count=1, randomize=true, veteran=false})
	civ.ui.text("3rd Invasion - Massa Reaction Force turn 2 arrives!")
	flag.setTrue("3rd Invasion Units, 2nd turn, triggered")
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis then
			if (unit.location.x >= 77 and unit.location.y >= 21 and unit.location.x <= 83 and unit.location.y <= 33 and unit.location.z == 0) then
				unit.gotoTile = civ.getTile(78,28,0) 
			end
		end
	end
	
end 
delayedAction.makeFunctionDelayable("invReact3rdT2_UnitsMassa",unitsMassaT2_Deployed)	

-- 3rd turn after invasion
local function unitsMassaT3_Deployed(argTable)
	gen.createUnit(object.uSdkFz231, object.pAxis, {{77,23,0},{82,32,0},{81,21,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{77,23,0},{82,32,0},{81,21,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uSdkFz72AA, object.pAxis, {{77,23,0},{82,32,0},{81,21,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uStugIIIG, object.pAxis, {{77,23,0},{82,32,0},{81,21,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{77,23,0},{82,32,0},{81,21,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uPzkwVIE, object.pAxis, {{77,23,0},{82,32,0},{81,21,0}}, {count=1, randomize=true, veteran=false})	
	civ.ui.text("3rd Invasion - Massa Reaction Force turn 3 arrives!")
	flag.setTrue("3rd Invasion Units, 3rd turn, triggered")
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis then
			if (unit.location.x >= 77 and unit.location.y >= 21 and unit.location.x <= 83 and unit.location.y <= 33 and unit.location.z == 0) then
				unit.gotoTile = civ.getTile(78,28,0) 
			end
		end
	end
end 
delayedAction.makeFunctionDelayable("invReact3rdT3_UnitsMassa",unitsMassaT3_Deployed)

-- CECINA REACTION FORCE
-- 2nd turn after invasion
local function unitsCecinaT2_Deployed(argTable)
	gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{75,35,0},{78,34,0},{75,49,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGebirgsjager, object.pAxis, {{75,35,0},{78,34,0},{75,49,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{75,35,0},{78,34,0},{75,49,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uPanzerGrenadier, object.pAxis, {{75,35,0},{78,34,0},{75,49,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uKruppK5, object.pAxis, {{75,35,0},{80,36,0},{80,42,0}}, {count=1, randomize=true, veteran=false})
	civ.ui.text("3rd Invasion - Cecina Reaction Force turn 2 arrives!")
	flag.setTrue("3rd Invasion Units, 2nd turn, triggered")
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis then
			if (unit.location.x >= 77 and unit.location.y >= 21 and unit.location.x <= 83 and unit.location.y <= 33 and unit.location.z == 0) then
				unit.gotoTile = civ.getTile(78,28,0) 
			end
		end	
	end
	
end 
delayedAction.makeFunctionDelayable("invReact3rdT2_UnitsCecina",unitsCecinaT2_Deployed)	

-- 3rd turn after invasion
local function unitsCecinaT3_Deployed(argTable)
	gen.createUnit(object.uSdkFz231, object.pAxis, {{75,35,0},{78,34,0},{75,49,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{75,35,0},{78,34,0},{75,49,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uSdkFz72AA, object.pAxis, {{75,35,0},{78,34,0},{75,49,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uStugIIIG, object.pAxis, {{75,35,0},{78,34,0},{75,49,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{75,35,0},{78,34,0},{75,49,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uPzkwVIE, object.pAxis, {{75,35,0},{78,34,0},{75,49,0}}, {count=1, randomize=true, veteran=false})	
	civ.ui.text("3rd Invasion - Cecina Reaction Force turn 3 arrives!")
	flag.setTrue("3rd Invasion Units, 3rd turn, triggered")
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis then
			if (unit.location.x >= 77 and unit.location.y >= 21 and unit.location.x <= 83 and unit.location.y <= 33 and unit.location.z == 0) then
				unit.gotoTile = civ.getTile(78,28,0) 
			end
		end	
	end
end 
delayedAction.makeFunctionDelayable("invReact3rdT3_UnitsCecina",unitsCecinaT3_Deployed)

-- FANO REACTION FORCE
-- 2nd turn after invasion
local function unitsFanoT2_Deployed(argTable)
	gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{106,44,0},{103,51,0},{104,54,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGebirgsjager, object.pAxis, {{106,44,0},{103,51,0},{104,54,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{106,44,0},{103,51,0},{104,54,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uPanzerGrenadier, object.pAxis, {{106,44,0},{103,51,0},{104,54,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uKruppK5, object.pAxis, {{106,44,0},{102,46,0},{103,51,0}}, {count=1, randomize=true, veteran=false})
	civ.ui.text("3rd Invasion - Fano Reaction Force turn 2 arrives!")
	flag.setTrue("3rd Invasion Units, 2nd turn, triggered")
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis then
			if (unit.location.x >= 102 and unit.location.y >= 44 and unit.location.x <= 106 and unit.location.y <= 54 and unit.location.z == 0) then
				unit.gotoTile = civ.getTile(109,51,0) 
			end
		end	
	end	
	
end 
delayedAction.makeFunctionDelayable("invReact3rdT2_UnitsFano",unitsFanoT2_Deployed)	

-- 3rd turn after invasion
local function unitsFanoT3_Deployed(argTable)
	gen.createUnit(object.uSdkFz231, object.pAxis, {{106,44,0},{103,51,0},{104,54,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{106,44,0},{103,51,0},{104,54,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uSdkFz72AA, object.pAxis, {{106,44,0},{103,51,0},{104,54,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uStugIIIG, object.pAxis, {{106,44,0},{103,51,0},{104,54,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{106,44,0},{103,51,0},{104,54,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uPzkwVIE, object.pAxis, {{106,44,0},{103,51,0},{104,54,0}}, {count=1, randomize=true, veteran=false})	
	civ.ui.text("3rd Invasion - Fano Reaction Force turn 3 arrives!")
	flag.setTrue("3rd Invasion Units, 3rd turn, triggered")
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis then
			if (unit.location.x >= 102 and unit.location.y >= 44 and unit.location.x <= 106 and unit.location.y <= 54 and unit.location.z == 0) then
				unit.gotoTile = civ.getTile(109,51,0) 
			end
		end	
	end	
end 
delayedAction.makeFunctionDelayable("invReact3rdT3_UnitsFano",unitsFanoT3_Deployed)

-- CIVITAVECCHIA REACTION FORCE
-- 2nd turn after invasion
local function unitsCivitavecchiaT2_Deployed(argTable)
	gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{80,76,0},{74,52,0},{82,66,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGebirgsjager, object.pAxis, {{80,76,0},{74,52,0},{82,66,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{80,76,0},{74,52,0},{82,66,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uPanzerGrenadier, object.pAxis, {{80,76,0},{74,52,0},{82,66,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uKruppK5, object.pAxis, {{74,56,0},{82,66,0},{77,55,0}}, {count=1, randomize=true, veteran=false})
	civ.ui.text("3rd Invasion - Civitavecchia Reaction Force turn 2 arrives!")
	flag.setTrue("3rd Invasion Units, 2nd turn, triggered")
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis then
			if (unit.location.x >= 74 and unit.location.y >= 52 and unit.location.x <= 82 and unit.location.y <= 76 and unit.location.z == 0) then
				unit.gotoTile = civ.getTile(75,67,0) 
			end
		end	
	end
	
end 
delayedAction.makeFunctionDelayable("invReact3rdT2_UnitsCivitavecchia",unitsCivitavecchiaT2_Deployed)	

-- 3rd turn after invasion
local function unitsCivitavecchiaT3_Deployed(argTable)
	gen.createUnit(object.uSdkFz231, object.pAxis, {{80,76,0},{74,52,0},{82,66,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{80,76,0},{74,52,0},{82,66,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uSdkFz72AA, object.pAxis, {{80,76,0},{74,52,0},{82,66,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uStugIIIG, object.pAxis, {{80,76,0},{74,52,0},{82,66,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{80,76,0},{74,52,0},{82,66,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uPzkwVIE, object.pAxis, {{80,76,0},{74,52,0},{82,66,0}}, {count=1, randomize=true, veteran=false})	
	civ.ui.text("3rd Invasion - Civitavecchia Reaction Force turn 3 arrives!")
	flag.setTrue("3rd Invasion Units, 3rd turn, triggered")
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis then
			if (unit.location.x >= 74 and unit.location.y >= 52 and unit.location.x <= 82 and unit.location.y <= 76 and unit.location.z == 0) then
				unit.gotoTile = civ.getTile(75,67,0) 
			end
		end	
	end
end 
delayedAction.makeFunctionDelayable("invReact3rdT3_UnitsCivitavecchia",unitsCivitavecchiaT3_Deployed)

-- ANZIO REACTION FORCE
-- 2nd turn after invasion
local function unitsAnzioT2_Deployed(argTable)
	gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGebirgsjager, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uPanzerGrenadier, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uKruppK5, object.pAxis, {{83,85,0},{82,80,0},{77,77,0}}, {count=1, randomize=true, veteran=false})
	civ.ui.text("3rd Invasion - Anzio Reaction Force turn 2 arrives!")
	flag.setTrue("3rd Invasion Units, 2nd turn, triggered")
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis then
			if (unit.location.x >= 80 and unit.location.y >= 76 and unit.location.x <= 86 and unit.location.y <= 82 and unit.location.z == 0) then
				unit.gotoTile = civ.getTile(78,84,0) 
			end
		end	
	end	
end 
delayedAction.makeFunctionDelayable("invReact3rdT2_UnitsAnzio",unitsAnzioT2_Deployed)	

-- 3rd turn after invasion
local function unitsAnzioT3_Deployed(argTable)
	gen.createUnit(object.uSdkFz231, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uSdkFz72AA, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uStugIIIG, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uPzkwVIE, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=1, randomize=true, veteran=false})	
	civ.ui.text("3rd Invasion - Anzio Reaction Force turn 3 arrives!")
	flag.setTrue("3rd Invasion Units, 3rd turn, triggered")
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis then
			if (unit.location.x >= 80 and unit.location.y >= 76 and unit.location.x <= 86 and unit.location.y <= 82 and unit.location.z == 0) then
				unit.gotoTile = civ.getTile(78,84,0) 
			end
		end	
	end	
end 
delayedAction.makeFunctionDelayable("invReact3rdT3_UnitsAnzio",unitsAnzioT3_Deployed)

-- TERRACINA REACTION FORCE
-- 2nd turn after invasion
local function unitsTerracinaT2_Deployed(argTable)
	gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGebirgsjager, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uPanzerGrenadier, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uKruppK5, object.pAxis, {{83,85,0},{88,86,0},{90,90,0}}, {count=1, randomize=true, veteran=false})
	civ.ui.text("3rd Invasion - Terracina Reaction Force turn 2 arrives!")
	flag.setTrue("3rd Invasion Units, 2nd turn, triggered")
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis then
			if (unit.location.x >= 80 and unit.location.y >= 76 and unit.location.x <= 86 and unit.location.y <= 82 and unit.location.z == 0) then
				unit.gotoTile = civ.getTile(78,84,0) 
			end
		end	
	end	
end 
delayedAction.makeFunctionDelayable("invReact3rdT2_UnitsTerracina",unitsTerracinaT2_Deployed)	

-- 3rd turn after invasion
local function unitsTerracinaT3_Deployed(argTable)
	gen.createUnit(object.uSdkFz231, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uSdkFz72AA, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uStugIIIG, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uPzkwVIE, object.pAxis, {{80,76,0},{84,82,0},{85,77,0}}, {count=1, randomize=true, veteran=false})	
	civ.ui.text("3rd Invasion - Terracina Reaction Force turn 3 arrives!")
	flag.setTrue("3rd Invasion Units, 3rd turn, triggered")
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis then
			if (unit.location.x >= 80 and unit.location.y >= 76 and unit.location.x <= 86 and unit.location.y <= 82 and unit.location.z == 0) then
				unit.gotoTile = civ.getTile(78,84,0) 
			end
		end	
	end	
end 
delayedAction.makeFunctionDelayable("invReact3rdT3_UnitsTerracina",unitsTerracinaT3_Deployed)


-- FIRENZE REACTION FORCE
-- 2nd turn after invasion
local function unitsFirenzeT2_Deployed(argTable)
	gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{89,39,0},{88,34,0},{93,41,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGebirgsjager, object.pAxis, {{89,39,0},{88,34,0},{93,41,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{89,39,0},{88,34,0},{93,41,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uPanzerGrenadier, object.pAxis, {{89,39,0},{88,34,0},{93,41,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uKruppK5, object.pAxis, {{89,39,0},{88,34,0},{91,37,0}}, {count=1, randomize=true, veteran=false})
	civ.ui.text("3rd Invasion - Firenze Reaction Force turn 2 arrives!")
	flag.setTrue("3rd Invasion Units, 2nd turn, triggered")
end 
delayedAction.makeFunctionDelayable("invReact3rdT2_UnitsFirenze",unitsFirenzeT2_Deployed)	

-- 3rd turn after invasion
local function unitsFirenzeT3_Deployed(argTable)
	gen.createUnit(object.uSdkFz231, object.pAxis, {{89,39,0},{88,34,0},{93,41,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{89,39,0},{88,34,0},{93,41,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uSdkFz72AA, object.pAxis, {{89,39,0},{88,34,0},{93,41,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uStugIIIG, object.pAxis, {{89,39,0},{88,34,0},{93,41,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{89,39,0},{88,34,0},{93,41,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uPzkwVIE, object.pAxis, {{89,39,0},{88,34,0},{93,41,0}}, {count=1, randomize=true, veteran=false})	
	civ.ui.text("3rd Invasion - Firenze Reaction Force turn 3 arrives!")
	flag.setTrue("3rd Invasion Units, 3rd turn, triggered")
end 
delayedAction.makeFunctionDelayable("invReact3rdT3_UnitsFirenze",unitsFirenzeT3_Deployed)

---------------------
-- OPTION II SCENARIO: If Allies don't proceed with 3rd Amphibious invasion
---------------------

-- 2nd turn after invasion
local function no3rdInvasionT1_Deployed(argTable)
	gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGebirgsjager, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uPanzerGrenadier, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uKruppK5, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=1, randomize=true, veteran=false})
	civ.ui.text("New German units deploy to Italy - turn 2 arrive!")
	flag.setTrue("No 3rd Invasion Units, 2nd turn, triggered")
	
end 
delayedAction.makeFunctionDelayable("no3rdInvasionT1_reactionUnits",no3rdInvasionT1_Deployed)	


-- 3rd turn after invasion
local function no3rdInvasionT2_Deployed(argTable)
	gen.createUnit(object.uSdkFz231, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uMotInfanterie, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=2, randomize=true, veteran=false})
	gen.createUnit(object.uSdkFz72AA, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uStugIIIG, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uGrilleHsIG33, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=1, randomize=true, veteran=false})
	gen.createUnit(object.uPzkwVIE, object.pAxis, {{80,76,0},{107,83,0},{91,93,0},{75,35,0},{89,39,0},{106,44,0}}, {count=1, randomize=true, veteran=false})	
	civ.ui.text("New German units deploy to Italy - turn 3 arrive!")
	flag.setTrue("No 3rd Invasion Units, 3rd turn, triggered")
end 
delayedAction.makeFunctionDelayable("no3rdInvasionT2_reactionUnits",no3rdInvasionT2_Deployed)
		
--END DELAYED 3RD INVASION REACTION FORCES ==============================	


-- Regia Marina DD and SS Squadrons are activated
local function regiaMarinaSquadrons_Activated(argTable)
	civ.playSound("warship.wav")
	text.simple("Naval vessels of the Regia Marina make a sortie to counter Allied amphibious operations!","Regia Marina Makes a Local Sortie!",object.mRegiaMarinaDestroyers)
	-- print("The Regia Marina makes a sortie to counter Allied amphibious operations!")
	gen.createUnit(object.uSottomarino, object.pAxis, {{22,118},{38,118,0},{81,163,0},{61,139,0},{72,148,0},{75,155,0},{80,180,0}}, {count=5, randomize=true, scatter=true, veteran=false})
	gen.createUnit(object.uSellaClassDD, object.pAxis, {{30,134},{46,138,0},{72,148,0},{87,165,0},{81,163,0}}, {count=3, randomize=true, veteran=false})
	gen.createUnit(object.uGiusannoClassCL, object.pAxis, {{30,134},{46,138,0},{72,148,0},{87,165,0},{81,163,0}}, {count=2, randomize=true, veteran=false})	
end 
delayedAction.makeFunctionDelayable("regiaMarinaSquadrons",regiaMarinaSquadrons_Activated)


-- Recruit British 52nd Mountain Division
local function british52ndMtnDivision_Activated(argTable)
	civ.playSound("Cheers2.wav")
	if flag.value("Sea Mines removed from Livorno") == true and flag.value("Roma is captured") == true then
		gen.createUnit(object.uBritish52ndMtn, object.pAllies, {{75,35,0}}, {count=3, randomize=false, veteran=false})
		text.simple("The British 52nd Mountain Division arrives in Livorno!","British Reinforcements",object.mAFHQ)
		counter.add("# British 52nd Mountain",3)
		counter.add("# Total UK Infantry",3)
	elseif flag.value("Sea Mines removed from Napoli") == true and flag.value("Roma is captured") == true then
		gen.createUnit(object.uBritish52ndMtn, object.pAllies, {{88,106,0}}, {count=3, randomize=false, veteran=false})
		text.simple("The British 52nd Mountain Division arrives in Napoli!","British Reinforcements",object.mAFHQ)
		counter.add("# British 52nd Mountain",3)
		counter.add("# Total UK Infantry",3)
	else
		gen.createUnit(object.uBritish52ndMtn, object.pAllies, {{73,155,0}}, {count=3, randomize=false, veteran=false})
		text.simple("The British 52nd Mountain Division arrives in  Messina!","British Reinforcements",object.mAFHQ)
		counter.add("# British 52nd Mountain",3)
		counter.add("# Total UK Infantry",3)
	end	
	flag.setTrue("British 52nd Mountain Deployed")
end 
delayedAction.makeFunctionDelayable("recruitBritish52ndMtnDivision",british52ndMtnDivision_Activated)

-- Yalta Conference and Polish Troops
local function polishTroops_RegainsSpirits(argTable)
	civ.playSound("Cheers.wav")
	text.simple("After the bitter disappointment of the Yalta Conference, the commander of the Polish II Corps, General Wladyslaw Anders, manages to rebolster the fighting spirit of his troops and confirms you can renew combat operations with them!","Polish Troops Regain Spirits!",object.mPolishCorps)
	civ.takeTech(object.pAllies, object.aYaltaConference)
	flag.setFalse("Polish Troops Demoralized After Yalta")

end 
delayedAction.makeFunctionDelayable("polishTroopsDemoralized",polishTroops_RegainsSpirits)


onTurn.commanderAFHQ_Base = commanderAFHQ_Base
onTurn.commanderAFHQ_1 = commanderAFHQ_1
onTurn.commanderAFHQ_2 = commanderAFHQ_2
onTurn.commanderAFHQ_3 = commanderAFHQ_3
onTurn.commander15thArmy_Base = commander15thArmy_Base
onTurn.commander15thArmy_1 = commander15thArmy_1
onTurn.commander15thArmy_2 = commander15thArmy_2

return onTurn